@@ -116,8 +116,12 @@ namespace Mono.Debugging.Soft
 		string InvokeToString (SoftEvaluationContext ctx, MethodMirror method, object obj)
 		{
 			try {
-				var res = ctx.RuntimeInvoke (method, obj, new Value[0]) as StringMirror;
-				return res != null ? res.Value : null;
+				var res = ctx.RuntimeInvoke (method, obj, new Value [0]) as StringMirror;
+				if (res != null) {
+					return MirrorStringToString (ctx, res);
+				} else {
+					return null;
+				}
 			} catch {
 				return GetDisplayTypeName (GetValueTypeName (ctx, obj));
 			}
@@ -254,19 +258,59 @@ namespace Mono.Debugging.Soft
 			return method.Invoke (value);
 		}
 
+		static bool CanForceCast (EvaluationContext ctx, TypeMirror fromType, TypeMirror toType)
+		{
+			var cx = (SoftEvaluationContext) ctx;
+			MethodMirror method;
+
+			// check for explicit and implicit cast operators in the target type
+			method = OverloadResolve (cx, toType, "op_Explicit", null, new [] { fromType }, false, true, false, false);
+			if (method != null)
+				return true;
+
+			method = OverloadResolve (cx, toType, "op_Implicit", null, new [] { fromType }, false, true, false, false);
+			if (method != null)
+				return true;
+
+			// check for explicit and implicit cast operators on the source type
+			method = OverloadResolve (cx, fromType, "op_Explicit", null, toType, new [] { fromType }, false, true, false, false);
+			if (method != null)
+				return true;
+
+			method = OverloadResolve (cx, fromType, "op_Implicit", null, toType, new [] { fromType }, false, true, false, false);
+			if (method != null)
+				return true;
+
+			method = OverloadResolve (cx, toType, ".ctor", null, new [] { fromType }, true, false, false, false);
+			if (method != null)
+				return true;
+
+			return false;
+		}
+
 		object TryForceCast (EvaluationContext ctx, Value value, TypeMirror fromType, TypeMirror toType)
 		{
 			var cx = (SoftEvaluationContext) ctx;
 			MethodMirror method;
 
-			method = OverloadResolve (cx, toType, "op_Explicit", null, new [] { fromType }, false, true, false);
+			// check for explicit and implicit cast operators in the target type
+			method = OverloadResolve (cx, toType, "op_Explicit", null, new [] { fromType }, false, true, false, false);
 			if (method != null)
 				return cx.RuntimeInvoke (method, toType, new [] { value });
 
-			method = OverloadResolve (cx, toType, "op_Implicit", null, new [] { fromType }, false, true, false);
+			method = OverloadResolve (cx, toType, "op_Implicit", null, new [] { fromType }, false, true, false, false);
 			if (method != null)
 				return cx.RuntimeInvoke (method, toType, new [] { value });
 
+			// check for explicit and implicit cast operators on the source type
+			method = OverloadResolve (cx, fromType, "op_Explicit", null, toType, new [] { fromType }, false, true, false, false);
+			if (method != null)
+				return cx.RuntimeInvoke (method, fromType, new [] { value });
+
+			method = OverloadResolve (cx, fromType, "op_Implicit", null, toType, new [] { fromType }, false, true, false, false);
+			if (method != null)
+				return cx.RuntimeInvoke (method, fromType, new [] { value });
+
 			// Finally, try a ctor...
 			try {
 				return CreateValue (ctx, toType, value);
@@ -288,6 +332,11 @@ namespace Mono.Debugging.Soft
 
 			fromType = valueType as TypeMirror;
 
+			// If we are trying to cast into non-primitive/enum value(e.g. System.nint)
+			// that class might have implicit operator and this must be handled via TypeMirrors
+			if (toType != null && !toType.IsPrimitive  && !toType.IsEnum)
+				fromType = ToTypeMirror (ctx, valueType);
+
 			if (fromType != null) {
 				if (toType != null && toType.IsAssignableFrom (fromType))
 					return val;
@@ -425,32 +474,66 @@ namespace Mono.Debugging.Soft
 			return ((TypeMirror) type).GetTypeObject ();
 		}
 
-		public override object CreateValue (EvaluationContext ctx, object type, params object[] args)
+		public override object CreateValue (EvaluationContext ctx, object type, params object[] argValues)
 		{
 			ctx.AssertTargetInvokeAllowed ();
 			
 			var cx = (SoftEvaluationContext) ctx;
 			var tm = (TypeMirror) type;
 			
-			var types = new TypeMirror [args.Length];
-			var values = new Value[args.Length];
+			var types = new TypeMirror [argValues.Length];
+			var values = new Value[argValues.Length];
 
-			for (int n = 0; n < args.Length; n++) {
-				types[n] = ToTypeMirror (ctx, GetValueType (ctx, args[n]));
-				values[n] = (Value) args[n];
+			for (int n = 0; n < argValues.Length; n++) {
+				types[n] = ToTypeMirror (ctx, GetValueType (ctx, argValues[n]));
 			}
-			
-			var ctor = OverloadResolve (cx, tm, ".ctor", null, types, true, false, false);
 
-			if (ctor != null)
-				return tm.NewInstance (cx.Thread, ctor, values);
+			var method = OverloadResolve (cx, tm, ".ctor", null, types, true, false, false);
 
-			if (args.Length == 0 && tm.VirtualMachine.Version.AtLeast (2, 31))
+			if (method != null) {
+				var mparams = method.GetParameters ();
+
+				for (int n = 0; n < argValues.Length; n++) {
+					var param_type = mparams [n].ParameterType;
+
+					if (param_type.FullName != types [n].FullName && !param_type.IsAssignableFrom (types [n]) && param_type.IsGenericType) {
+						/* TODO: Add genericTypeArgs and handle this
+						bool throwCastException = true;
+
+						if (method.VirtualMachine.Version.AtLeast (2, 15)) {
+							var args = param_type.GetGenericArguments ();
+
+							if (args.Length == genericTypes.Length) {
+								var real_type = soft.Adapter.GetType (soft, param_type.GetGenericTypeDefinition ().FullName, genericTypes);
+
+								values [n] = (Value)TryCast (soft, (Value)argValues [n], real_type);
+								if (!(values [n] == null && argValues [n] != null && !soft.Adapter.IsNull (soft, argValues [n])))
+									throwCastException = false;
+							}
+						}
+
+						if (throwCastException) {
+							string fromType = !IsGeneratedType (types [n]) ? soft.Adapter.GetDisplayTypeName (soft, types [n]) : types [n].FullName;
+							string toType = soft.Adapter.GetDisplayTypeName (soft, param_type);
+
+							throw new EvaluatorException ("Argument {0}: Cannot implicitly convert `{1}' to `{2}'", n, fromType, toType);
+						}*/
+					} else if (param_type.FullName != types [n].FullName && !param_type.IsAssignableFrom (types [n]) && CanForceCast (ctx, types [n], param_type)) {
+						values [n] = (Value)TryCast (ctx, argValues [n], param_type);
+					} else {
+						values [n] = (Value)argValues [n];
+					}
+				}
+
+				return tm.NewInstance (cx.Thread, method, values);
+			}
+
+			if (argValues.Length == 0 && tm.VirtualMachine.Version.AtLeast (2, 31))
 				return tm.NewInstance ();
 
 			string typeName = ctx.Adapter.GetDisplayTypeName (ctx, type);
 
-			throw new EvaluatorException ("Constructor not found for type `{1}'.", typeName);
+			throw new EvaluatorException ("Constructor not found for type `{0}'.", typeName);
 		}
 
 		public override object CreateValue (EvaluationContext ctx, object value)
@@ -461,6 +544,12 @@ namespace Mono.Debugging.Soft
 			if (str != null)
 				return cx.Domain.CreateString (str);
 
+			if (value is decimal) {
+				var bits = decimal.GetBits ((decimal)value);
+
+				return CreateValue (ctx, ToTypeMirror (ctx, typeof (decimal)), CreateValue (ctx, bits [0]), CreateValue (ctx, bits [1]), CreateValue (ctx, bits [2]), CreateValue (ctx, (bits [3] & unchecked((int)0x80000000)) != 0), CreateValue (ctx, (byte)(bits [3] >> 16)));
+			}
+
 			return cx.Session.VirtualMachine.CreateValue (value);
 		}
 
@@ -536,7 +625,7 @@ namespace Mono.Debugging.Soft
 				type = type.BaseType;
 			}
 			
-			var idx = OverloadResolve ((SoftEvaluationContext) ctx, targetType, null, null, types, candidates, true);
+			var idx = OverloadResolve ((SoftEvaluationContext) ctx, targetType, null, null, null, types, candidates, true);
 			int i = candidates.IndexOf (idx);
 
 			var getter = props[i].GetGetMethod (true);
@@ -590,10 +679,14 @@ namespace Mono.Debugging.Soft
 		
 		static bool IsClosureReferenceField (FieldInfoMirror field)
 		{
-			// mcs is "<>f__ref"
+			// mcs is "$locvar"
+			// old mcs is "<>f__ref"
 			// csc is "CS$<>"
+			// roslyn is "<>8__"
 			return field.Name.StartsWith ("CS$<>", StringComparison.Ordinal) ||
-				field.Name.StartsWith ("<>f__ref", StringComparison.Ordinal);
+				        field.Name.StartsWith ("<>f__ref", StringComparison.Ordinal) ||
+				        field.Name.StartsWith ("$locvar", StringComparison.Ordinal) ||
+				        field.Name.StartsWith ("<>8__", StringComparison.Ordinal);
 		}
 		
 		static bool IsClosureReferenceLocal (LocalVariable local)
@@ -631,7 +724,7 @@ namespace Mono.Debugging.Soft
 			return null;
 		}
 
-		IEnumerable<ValueReference> GetHoistedLocalVariables (SoftEvaluationContext cx, ValueReference vthis)
+		IEnumerable<ValueReference> GetHoistedLocalVariables (SoftEvaluationContext cx, ValueReference vthis, HashSet<FieldInfoMirror> alreadyVisited = null)
 		{
 			if (vthis == null)
 				return new ValueReference [0];
@@ -651,7 +744,11 @@ namespace Mono.Debugging.Soft
 					continue;
 
 				if (IsClosureReferenceField (field)) {
-					list.AddRange (GetHoistedLocalVariables (cx, new FieldValueReference (cx, field, val, type)));
+					alreadyVisited = alreadyVisited ?? new HashSet<FieldInfoMirror> ();
+					if (alreadyVisited.Contains (field))
+						continue;
+					alreadyVisited.Add (field);
+					list.AddRange (GetHoistedLocalVariables (cx, new FieldValueReference (cx, field, val, type), alreadyVisited));
 					continue;
 				}
 
@@ -681,15 +778,19 @@ namespace Mono.Debugging.Soft
 			return null;
 		}
 		
-		ValueReference GetHoistedThisReference (SoftEvaluationContext cx, TypeMirror type, object val)
+		ValueReference GetHoistedThisReference (SoftEvaluationContext cx, TypeMirror type, object val, HashSet<FieldInfoMirror> alreadyVisited = null)
 		{
 			foreach (var field in type.GetFields ()) {
 				if (IsHoistedThisReference (field))
 					return new FieldValueReference (cx, field, val, type, "this", ObjectValueFlags.Literal);
 
 				if (IsClosureReferenceField (field)) {
+					alreadyVisited = alreadyVisited ?? new HashSet<FieldInfoMirror> ();
+					if (alreadyVisited.Contains (field))
+						continue;
+					alreadyVisited.Add (field);
 					var fieldRef = new FieldValueReference (cx, field, val, type);
-					var thisRef = GetHoistedThisReference (cx, field.FieldType, fieldRef.Value);
+					var thisRef = GetHoistedThisReference (cx, field.FieldType, fieldRef.Value, alreadyVisited);
 					if (thisRef != null)
 						return thisRef;
 				}
@@ -812,7 +913,6 @@ namespace Mono.Debugging.Soft
 		protected override ValueReference GetMember (EvaluationContext ctx, object t, object co, string name)
 		{
 			var type = t as TypeMirror;
-
 			while (type != null) {
 				var field = FindByName (type.GetFields (), f => f.Name, name, ctx.CaseSensitive);
 
@@ -830,7 +930,7 @@ namespace Mono.Debugging.Soft
 
 					// Backing field not available, so do things the old fashioned way.
 					var getter = prop.GetGetMethod (true);
-					
+
 					return getter != null ? new PropertyValueReference (ctx, prop, co, type, getter, null) : null;
 				}
 
@@ -847,14 +947,14 @@ namespace Mono.Debugging.Soft
 
 			return generated != null;
 		}
-		
+
 		static bool IsStatic (PropertyInfoMirror prop)
 		{
 			var met = prop.GetGetMethod (true) ?? prop.GetSetMethod (true);
 
 			return met.IsStatic;
 		}
-		
+
 		static T FindByName<T> (IEnumerable<T> items, Func<T,string> getName, string name, bool caseSensitive)
 		{
 			T best = default(T);
@@ -1246,6 +1346,30 @@ namespace Mono.Debugging.Soft
 					return tm;
 			}
 
+			var method = cx.Frame.Method;
+			if (method.IsGenericMethod && !method.IsGenericMethodDefinition) {
+				var definition = method.GetGenericMethodDefinition ();
+				var names = definition.GetGenericArguments ();
+				var types = method.GetGenericArguments ();
+
+				for (i = 0; i < names.Length; i++) {
+					if (names[i].FullName == name)
+						return types[i];
+				}
+			}
+
+			var declaringType = method.DeclaringType;
+			if (declaringType.IsGenericType && !declaringType.IsGenericTypeDefinition) {
+				var definition = declaringType.GetGenericTypeDefinition ();
+				var types = declaringType.GetGenericArguments ();
+				var names = definition.GetGenericArguments ();
+
+				for (i = 0; i < names.Length; i++) {
+					if (names[i].FullName == name)
+						return types[i];
+				}
+			}
+
 			return null;
 		}
 
@@ -1282,9 +1406,18 @@ namespace Mono.Debugging.Soft
 			var tm = (TypeMirror) type;
 
 			foreach (var nested in tm.GetNestedTypes ())
+				if (!IsGeneratedType (nested))
+					yield return nested;
+		}
+
+		public override IEnumerable<object> GetImplementedInterfaces (EvaluationContext ctx, object type)
+		{
+			var tm = (TypeMirror) type;
+
+			foreach (var nested in tm.GetInterfaces ())
 				yield return nested;
 		}
-		
+
 		public override string GetTypeName (EvaluationContext ctx, object type)
 		{
 			var tm = type as TypeMirror;
@@ -1323,7 +1456,7 @@ namespace Mono.Debugging.Soft
 				return pv.Value.GetType ();
 			}
 
-			throw new NotSupportedException ();
+			throw new NotSupportedException (val.GetType ().FullName);
 		}
 		
 		public override object GetBaseType (EvaluationContext ctx, object type)
@@ -1583,6 +1716,17 @@ namespace Mono.Debugging.Soft
 		}
 
 		public override object RuntimeInvoke (EvaluationContext ctx, object targetType, object target, string methodName, object[] genericTypeArgs, object[] argTypes, object[] argValues)
+		{
+			object[] outArgs;
+			return RuntimeInvoke (ctx, targetType, target, methodName, genericTypeArgs, argTypes, argValues, false, out outArgs);
+		}
+
+		public override object RuntimeInvoke (EvaluationContext ctx, object targetType, object target, string methodName, object[] genericTypeArgs, object[] argTypes, object[] argValues, out object[] outArgs)
+		{
+			return RuntimeInvoke (ctx, targetType, target, methodName, genericTypeArgs, argTypes, argValues, true, out outArgs);
+		}
+
+		private object RuntimeInvoke (EvaluationContext ctx, object targetType, object target, string methodName, object[] genericTypeArgs, object[] argTypes, object[] argValues,bool enableOutArgs, out object[] outArgs)
 		{
 			var type = ToTypeMirror (ctx, targetType);
 			var soft = (SoftEvaluationContext) ctx;
@@ -1604,7 +1748,7 @@ namespace Mono.Debugging.Soft
 			for (int n = 0; n < argValues.Length; n++) {
 				var param_type = mparams[n].ParameterType;
 
-				if (param_type.FullName != types[n].FullName && !param_type.IsAssignableFrom (types[n]) && param_type.IsGenericType) {
+				if (param_type.FullName != types [n].FullName && !param_type.IsAssignableFrom (types [n]) && param_type.IsGenericType) {
 					bool throwCastException = true;
 
 					if (method.VirtualMachine.Version.AtLeast (2, 15)) {
@@ -1613,24 +1757,33 @@ namespace Mono.Debugging.Soft
 						if (args.Length == genericTypes.Length) {
 							var real_type = soft.Adapter.GetType (soft, param_type.GetGenericTypeDefinition ().FullName, genericTypes);
 
-							values[n] = (Value) TryCast (soft, (Value) argValues[n], real_type);
-							if (!(values[n] == null && argValues[n] != null && !soft.Adapter.IsNull (soft, argValues[n])))
+							values [n] = (Value)TryCast (soft, (Value)argValues [n], real_type);
+							if (!(values [n] == null && argValues [n] != null && !soft.Adapter.IsNull (soft, argValues [n])))
 								throwCastException = false;
 						}
 					}
 
 					if (throwCastException) {
-						string fromType = !IsGeneratedType (types[n]) ? soft.Adapter.GetDisplayTypeName (soft, types[n]) : types[n].FullName;
+						string fromType = !IsGeneratedType (types [n]) ? soft.Adapter.GetDisplayTypeName (soft, types [n]) : types [n].FullName;
 						string toType = soft.Adapter.GetDisplayTypeName (soft, param_type);
 
 						throw new EvaluatorException ("Argument {0}: Cannot implicitly convert `{1}' to `{2}'", n, fromType, toType);
 					}
+				} else if (param_type.FullName != types [n].FullName && !param_type.IsAssignableFrom (types [n]) && CanForceCast (ctx, types [n], param_type)) {
+					values [n] = (Value)TryCast (ctx, argValues [n], param_type);
 				} else {
-					values[n] = (Value) argValues[n];
+					values [n] = (Value)argValues [n];
 				}
 			}
-
-			return soft.RuntimeInvoke (method, target ?? targetType, values);
+			if (enableOutArgs) {
+				Value[] outArgsValue;
+				var result = soft.RuntimeInvoke (method, target ?? targetType, values, out outArgsValue);
+				outArgs = (object[])outArgsValue;
+				return result;
+			} else {
+				outArgs = null;
+				return soft.RuntimeInvoke (method, target ?? targetType, values);
+			}
 		}
 
 		static TypeMirror[] ResolveGenericTypeArguments (MethodMirror method, TypeMirror[] argTypes)
@@ -1661,43 +1814,48 @@ namespace Mono.Debugging.Soft
 			return types;
 		}
 
-		public static MethodMirror OverloadResolve (SoftEvaluationContext ctx, TypeMirror type, string methodName, TypeMirror[] genericTypeArgs, TypeMirror[] argTypes, bool allowInstance, bool allowStatic, bool throwIfNotFound)
+		public static MethodMirror OverloadResolve (SoftEvaluationContext ctx, TypeMirror type, string methodName, TypeMirror[] genericTypeArgs, TypeMirror[] argTypes, bool allowInstance, bool allowStatic, bool throwIfNotFound, bool tryCasting = true)
+		{
+			return OverloadResolve (ctx, type, methodName, genericTypeArgs, null, argTypes, allowInstance, allowStatic, throwIfNotFound, tryCasting);
+		}
+
+		public static MethodMirror OverloadResolve (SoftEvaluationContext ctx, TypeMirror type, string methodName, TypeMirror [] genericTypeArgs, TypeMirror returnType, TypeMirror [] argTypes, bool allowInstance, bool allowStatic, bool throwIfNotFound, bool tryCasting)
 		{
 			const BindingFlags methodByNameFlags = BindingFlags.DeclaredOnly | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static;
 			var cache = ctx.Session.OverloadResolveCache;
 			var candidates = new List<MethodMirror> ();
 			var currentType = type;
-			
+
 			while (currentType != null) {
-				MethodMirror[] methods = null;
-				
+				MethodMirror [] methods = null;
+
 				if (ctx.CaseSensitive) {
 					lock (cache) {
 						cache.TryGetValue (Tuple.Create (currentType, methodName), out methods);
 					}
 				}
-				
+
 				if (methods == null) {
 					if (currentType.VirtualMachine.Version.AtLeast (2, 7)) {
 						methods = currentType.GetMethodsByNameFlags (methodName, methodByNameFlags, !ctx.CaseSensitive);
 					} else {
 						methods = currentType.GetMethods ();
 					}
-					
+
 					if (ctx.CaseSensitive) {
 						lock (cache) {
 							cache [Tuple.Create (currentType, methodName)] = methods;
 						}
 					}
 				}
-				
+
 				foreach (var method in methods) {
 					if (method.Name == methodName || (!ctx.CaseSensitive && method.Name.Equals (methodName, StringComparison.CurrentCultureIgnoreCase))) {
 						MethodMirror actualMethod;
 
 						if (argTypes != null && method.VirtualMachine.Version.AtLeast (2, 24) && method.IsGenericMethod) {
 							var generic = method.GetGenericMethodDefinition ();
-							TypeMirror[] typeArgs;
+							TypeMirror [] typeArgs;
 
 							//Console.WriteLine ("Attempting to resolve generic type args for: {0}", GetPrettyMethodName (ctx, generic));
 
@@ -1725,10 +1883,10 @@ namespace Mono.Debugging.Soft
 
 				if (argTypes == null && candidates.Count > 0)
 					break; // when argTypes is null, we are just looking for *any* match (not a specific match)
-				
+
 				if (methodName == ".ctor")
 					break; // Can't create objects using constructor from base classes
-				
+
 				// Make sure that we always pull in at least System.Object methods (this is mostly needed for cases where 'type' was an interface)
 				if (currentType.BaseType == null && currentType.FullName != "System.Object")
 					currentType = ctx.Adapter.GetType (ctx, "System.Object") as TypeMirror;
@@ -1736,10 +1894,10 @@ namespace Mono.Debugging.Soft
 					currentType = currentType.BaseType;
 			}
 
-			return OverloadResolve (ctx, type, methodName, genericTypeArgs, argTypes, candidates, throwIfNotFound);
+			return OverloadResolve (ctx, type, methodName, genericTypeArgs, returnType, argTypes, candidates, throwIfNotFound, tryCasting);
 		}
 
-		static bool IsApplicable (SoftEvaluationContext ctx, MethodMirror method, TypeMirror[] genericTypeArgs, TypeMirror[] types, out string error, out int matchCount)
+		static bool IsApplicable (SoftEvaluationContext ctx, MethodMirror method, TypeMirror[] genericTypeArgs, TypeMirror returnType, TypeMirror[] types, out string error, out int matchCount, bool tryCasting = true)
 		{
 			var mparams = method.GetParameters ();
 			matchCount = 0;
@@ -1766,18 +1924,32 @@ namespace Mono.Debugging.Soft
 					}
 				}
 
+				if (tryCasting && CanForceCast (ctx, param_type, types [i]))
+					continue;
+
 				string fromType = !IsGeneratedType (types[i]) ? ctx.Adapter.GetDisplayTypeName (ctx, types[i]) : types[i].FullName;
 				string toType = ctx.Adapter.GetDisplayTypeName (ctx, param_type);
 
-				error = String.Format ("Argument {0}: Cannot implicitly convert `{1}' to `{2}'", i, fromType, toType);
+				error = string.Format ("Argument {0}: Cannot implicitly convert `{1}' to `{2}'", i, fromType, toType);
+
+				return false;
+			}
+
+			if (returnType != null && returnType != method.ReturnType) {
+				string actual = ctx.Adapter.GetDisplayTypeName (ctx, method.ReturnType);
+				string expected = ctx.Adapter.GetDisplayTypeName (ctx, returnType);
+
+				error = string.Format ("Return types do not match: `{0}' vs `{1}'", expected, actual);
+
 				return false;
 			}
 
 			error = null;
+
 			return true;
 		}
 
-		static MethodMirror OverloadResolve (SoftEvaluationContext ctx, TypeMirror type, string methodName, TypeMirror[] genericTypeArgs, TypeMirror[] argTypes, List<MethodMirror> candidates, bool throwIfNotFound)
+		static MethodMirror OverloadResolve (SoftEvaluationContext ctx, TypeMirror type, string methodName, TypeMirror[] genericTypeArgs, TypeMirror returnType, TypeMirror[] argTypes, List<MethodMirror> candidates, bool throwIfNotFound, bool tryCasting = true)
 		{
 			if (candidates.Count == 0) {
 				if (throwIfNotFound) {
@@ -1807,7 +1979,7 @@ namespace Mono.Debugging.Soft
 				string error;
 				int matchCount;
 
-				if (IsApplicable (ctx, candidates[0], genericTypeArgs, argTypes, out error, out matchCount))
+				if (IsApplicable (ctx, candidates[0], genericTypeArgs, returnType, argTypes, out error, out matchCount, tryCasting))
 					return candidates[0];
 
 				if (throwIfNotFound)
@@ -1825,7 +1997,7 @@ namespace Mono.Debugging.Soft
 				string error;
 				int matchCount;
 				
-				if (!IsApplicable (ctx, method, genericTypeArgs, argTypes, out error, out matchCount))
+				if (!IsApplicable (ctx, method, genericTypeArgs, returnType, argTypes, out error, out matchCount, tryCasting))
 					continue;
 
 				if (matchCount == bestCount) {
@@ -1865,34 +2037,14 @@ namespace Mono.Debugging.Soft
 		public override object TargetObjectToObject (EvaluationContext ctx, object obj)
 		{
 			if (obj is StringMirror) {
-				var mirror = (StringMirror) obj;
-				string str;
-				
-				if (ctx.Options.EllipsizeStrings) {
-					if (mirror.VirtualMachine.Version.AtLeast (2, 10)) {
-						int length = mirror.Length;
-						
-						if (length > ctx.Options.EllipsizedLength)
-							str = new string (mirror.GetChars (0, ctx.Options.EllipsizedLength)) + EvaluationOptions.Ellipsis;
-						else
-							str = mirror.Value;
-					} else {
-						str = mirror.Value;
-						if (str.Length > ctx.Options.EllipsizedLength)
-							str = str.Substring (0, ctx.Options.EllipsizedLength) + EvaluationOptions.Ellipsis;
-					}
-				} else {
-					str = mirror.Value;
-				}
-				
-				return str;
+				return MirrorStringToString (ctx, (StringMirror)obj);
 			}
 
 			if (obj is PrimitiveValue)
 				return ((PrimitiveValue)obj).Value;
 
 			if (obj is PointerValue)
-				return new IntPtr (((PointerValue)obj).Address);
+				return new Mono.Debugging.Backend.EvaluationResult ("0x" + ((PointerValue)obj).Address.ToString ("x"));
 
 			if (obj is StructMirror) {
 				var sm = (StructMirror) obj;
@@ -1900,7 +2052,7 @@ namespace Mono.Debugging.Soft
 				if (sm.Type.IsPrimitive) {
 					// Boxed primitive
 					if (sm.Type.FullName == "System.IntPtr")
-						return new IntPtr ((long)((PrimitiveValue)sm.Fields[0]).Value);
+						return new Mono.Debugging.Backend.EvaluationResult ("0x" + ((long)((PrimitiveValue)sm.Fields [0]).Value).ToString ("x"));
 					if (sm.Fields.Length > 0 && (sm.Fields[0] is PrimitiveValue))
 						return ((PrimitiveValue)sm.Fields[0]).Value;
 				} else if (sm.Type.FullName == "System.Decimal") {
@@ -1933,11 +2085,35 @@ namespace Mono.Debugging.Soft
 
 			return base.TargetObjectToObject (ctx, obj);
 		}
+
+		static string MirrorStringToString (EvaluationContext ctx, StringMirror mirror)
+		{
+			string str;
+
+			if (ctx.Options.EllipsizeStrings) {
+				if (mirror.VirtualMachine.Version.AtLeast (2, 10)) {
+					int length = mirror.Length;
+
+					if (length > ctx.Options.EllipsizedLength)
+						str = new string (mirror.GetChars (0, ctx.Options.EllipsizedLength)) + EvaluationOptions.Ellipsis;
+					else
+						str = mirror.Value;
+				} else {
+					str = mirror.Value;
+					if (str.Length > ctx.Options.EllipsizedLength)
+						str = str.Substring (0, ctx.Options.EllipsizedLength) + EvaluationOptions.Ellipsis;
+				}
+			} else {
+				str = mirror.Value;
+			}
+
+			return str;
+		}
 	}
 
 	class MethodCall: AsyncOperation
 	{
-		const InvokeOptions options = InvokeOptions.DisableBreakpoints | InvokeOptions.SingleThreaded;
+		readonly InvokeOptions options = InvokeOptions.DisableBreakpoints | InvokeOptions.SingleThreaded;
 
 		readonly ManualResetEvent shutdownEvent = new ManualResetEvent (false);
 		readonly SoftEvaluationContext ctx;
@@ -1946,14 +2122,20 @@ namespace Mono.Debugging.Soft
 		readonly object obj;
 		IAsyncResult handle;
 		Exception exception;
-		Value result;
+		InvokeResult result;
 		
-		public MethodCall (SoftEvaluationContext ctx, MethodMirror function, object obj, Value[] args)
+		public MethodCall (SoftEvaluationContext ctx, MethodMirror function, object obj, Value[] args, bool enableOutArgs)
 		{
 			this.ctx = ctx;
 			this.function = function;
 			this.obj = obj;
 			this.args = args;
+			if (enableOutArgs) {
+				this.options |= InvokeOptions.ReturnOutArgs;
+			}
+			if (function.VirtualMachine.Version.AtLeast (2, 40)) {
+				this.options |= InvokeOptions.Virtual;
+			}
 		}
 		
 		public override string Description {
@@ -1970,7 +2152,7 @@ namespace Mono.Debugging.Soft
 				else if (obj is TypeMirror)
 					handle = ((TypeMirror)obj).BeginInvokeMethod (ctx.Thread, function, args, options, null, null);
 				else if (obj is StructMirror)
-					handle = ((StructMirror)obj).BeginInvokeMethod (ctx.Thread, function, args, options, null, null);
+					handle = ((StructMirror)obj).BeginInvokeMethod (ctx.Thread, function, args, options | InvokeOptions.ReturnOutThis, null, null);
 				else if (obj is PrimitiveValue)
 					handle = ((PrimitiveValue)obj).BeginInvokeMethod (ctx.Thread, function, args, options, null, null);
 				else
@@ -2006,13 +2188,13 @@ namespace Mono.Debugging.Soft
 		{
 			try {
 				if (obj is ObjectMirror)
-					result = ((ObjectMirror)obj).EndInvokeMethod (handle);
+					result = ((ObjectMirror)obj).EndInvokeMethodWithResult (handle);
 				else if (obj is TypeMirror)
-					result = ((TypeMirror)obj).EndInvokeMethod (handle);
+					result = ((TypeMirror)obj).EndInvokeMethodWithResult (handle);
 				else if (obj is StructMirror)
-					result = ((StructMirror)obj).EndInvokeMethod (handle);
+					result = ((StructMirror)obj).EndInvokeMethodWithResult (handle);
 				else
-					result = ((PrimitiveValue)obj).EndInvokeMethod (handle);
+					result = ((PrimitiveValue)obj).EndInvokeMethodWithResult (handle);
 			} catch (InvocationException ex) {
 				if (!Aborting && ex.Exception != null) {
 					string ename = ctx.Adapter.GetValueTypeName (ctx, ex.Exception);
@@ -2066,7 +2248,15 @@ namespace Mono.Debugging.Soft
 			get {
 				if (exception != null)
 					throw new EvaluatorException (exception.Message);
-				return result;
+				return result.Result;
+			}
+		}
+
+		public Value[] OutArgs {
+			get {
+				if (exception != null)
+					throw new EvaluatorException (exception.Message);
+				return result.OutArgs;
 			}
 		}
 	}
