@@ -115,6 +115,12 @@ namespace IKVM.Reflection
 	 *      - Module.__ReadDataFromRVA()
 	 *      - MethodBase.GetMethodBody()
 	 *      - FieldInfo.__GetDataFromRVA()
+	 *
+	 *   DeterministicOutput
+	 *      The generated output file will depend only on the input. In other words,
+	 *      the PE file header time stamp will be set to zero and the module version
+	 *      id will be based on a SHA1 of the contents, instead of a random guid.
+	 *      This option can not be used in combination with PDB file generation.
 	 */
 
 	[Flags]
@@ -128,6 +134,9 @@ namespace IKVM.Reflection
 		MetadataOnly = 16,
 		ResolveMissingMembers = 32,
 		DisableWindowsRuntimeProjection = 64,
+		DecodeVersionInfoAttributeBlobs = 128,
+		DeterministicOutput = 256,
+
 		SupressReferenceTypeIdentityConversion = 1 << 20
 	}
 
@@ -817,6 +826,9 @@ namespace IKVM.Reflection
 			{
 				return asm;
 			}
+#if CORECLR
+			return null;
+#else
 			string fileName;
 			if (throwOnError)
 			{
@@ -847,6 +859,7 @@ namespace IKVM.Reflection
 				}
 			}
 			return LoadFile(fileName);
+#endif
 		}
 
 		public Type GetType(string assemblyQualifiedTypeName)
@@ -919,9 +932,13 @@ namespace IKVM.Reflection
 		// this is equivalent to the Fusion CompareAssemblyIdentity API
 		public bool CompareAssemblyIdentity(string assemblyIdentity1, bool unified1, string assemblyIdentity2, bool unified2, out AssemblyComparisonResult result)
 		{
+#if CORECLR
+			return Fusion.CompareAssemblyIdentityPure(assemblyIdentity1, unified1, assemblyIdentity2, unified2, out result);
+#else
 			return useNativeFusion
 				? Fusion.CompareAssemblyIdentityNative(assemblyIdentity1, unified1, assemblyIdentity2, unified2, out result)
 				: Fusion.CompareAssemblyIdentityPure(assemblyIdentity1, unified1, assemblyIdentity2, unified2, out result);
+#endif
 		}
 
 		public AssemblyBuilder DefineDynamicAssembly(AssemblyName name, AssemblyBuilderAccess access)
@@ -939,6 +956,7 @@ namespace IKVM.Reflection
 			return new AssemblyBuilder(this, name, dir, null);
 		}
 
+#if !CORECLR
 #if NET_4_0
 		[Obsolete]
 #endif
@@ -958,6 +976,7 @@ namespace IKVM.Reflection
 				ab.__AddDeclarativeSecurity(CustomAttributeBuilder.__FromBlob(CustomAttributeBuilder.LegacyPermissionSet, (int)action, Encoding.Unicode.GetBytes(permissionSet.ToXml().ToString())));
 			}
 		}
+#endif
 
 		internal void RegisterDynamicAssembly(AssemblyBuilder asm)
 		{
@@ -1102,7 +1121,11 @@ namespace IKVM.Reflection
 				}
 				return method;
 			}
+#if CORECLR
+			throw new MissingMethodException(declaringType.ToString() + "." + name);
+#else
 			throw new MissingMethodException(declaringType.ToString(), name);
+#endif
 		}
 
 		internal FieldInfo GetMissingFieldOrThrow(Module requester, Type declaringType, string name, FieldSignature signature)
@@ -1116,7 +1139,11 @@ namespace IKVM.Reflection
 				}
 				return field;
 			}
+#if CORECLR
+			throw new MissingFieldException(declaringType.ToString() + "." + name);
+#else
 			throw new MissingFieldException(declaringType.ToString(), name);
+#endif
 		}
 
 		internal PropertyInfo GetMissingPropertyOrThrow(Module requester, Type declaringType, string name, PropertySignature propertySignature)
@@ -1132,7 +1159,11 @@ namespace IKVM.Reflection
 				}
 				return property;
 			}
+#if CORECLR
+			throw new System.MissingMemberException(declaringType.ToString() + "." + name);
+#else
 			throw new System.MissingMemberException(declaringType.ToString(), name);
+#endif
 		}
 
 		internal Type CanonicalizeType(Type type)
@@ -1218,9 +1249,19 @@ namespace IKVM.Reflection
 			get { return (options & UniverseOptions.DisableWindowsRuntimeProjection) == 0; }
 		}
 
+		internal bool DecodeVersionInfoAttributeBlobs
+		{
+			get { return (options & UniverseOptions.DecodeVersionInfoAttributeBlobs) != 0; }
+		}
+
 		internal bool SupressReferenceTypeIdentityConversion
 		{
 			get { return (options & UniverseOptions.SupressReferenceTypeIdentityConversion) != 0; }
 		}
+
+		internal bool Deterministic
+		{
+			get { return (options & UniverseOptions.DeterministicOutput) != 0; }
+		}
 	}
 }
