@@ -46,18 +46,25 @@ namespace macdoc
 				return "Mono Documentation Browser";
 			}
 		}
+
+		public void LoadWithSearch (string searchTerm)
+		{
+			if (!string.IsNullOrEmpty (searchTerm)) {
+				toolbarSearchEntry.StringValue = searchTerm;
+				tabSelector.SelectAt (2);
+				Search (searchTerm);
+				Logger.Log ("Searched: '{0}'", searchTerm);
+			}
+		}
 		
 		public override bool ReadFromUrl (NSUrl url, string typeName, out NSError outError)
 		{
-			Console.WriteLine ("ReadFromUrl : {0}", url.ToString ());
+			Logger.Log ("ReadFromUrl: {0}", url.ToString ());
 			outError = null;
-			const int NSServiceMiscellaneousError = 66800;
-			if (url.Scheme != "monodoc" && url.Scheme != "mdoc") {
-				outError = new NSError (NSError.CocoaErrorDomain,
-				                       	NSServiceMiscellaneousError,
-				                      	NSDictionary.FromObjectAndKey (NSError.LocalizedFailureReasonErrorKey, new NSString (string.Format ("Scheme {0} isn't supported", url.Scheme))));
-				return false;
-			}
+
+			// if scheme is not right, we ignore the url
+			if (url.Scheme != "monodoc" && url.Scheme != "mdoc")
+				return true;
 			
 			// ResourceSpecifier is e.g. "//T:System.String"
 			initialLoadFromUrl = Uri.UnescapeDataString (url.ResourceSpecifier.Substring (2));
@@ -83,8 +90,7 @@ namespace macdoc
 					navigationCells.SetSelected (true, 1);
 			};
 			HideMultipleMatches ();
-			if (!string.IsNullOrEmpty (initialLoadFromUrl))
-				LoadUrl (initialLoadFromUrl);
+			LoadUrl (string.IsNullOrEmpty (initialLoadFromUrl) ? "root:" : initialLoadFromUrl);
 		}
 
 		void HandleAddBookmarkBtnActivated (object sender, EventArgs e)
@@ -116,7 +122,8 @@ namespace macdoc
 		{
 			outlineView.DataSource = new DocTreeDataSource (this);
 			outlineView.Delegate = new OutlineDelegate (this);
-			outlineView.EnclosingScrollView.HorizontalScrollElasticity = outlineView.EnclosingScrollView.VerticalScrollElasticity = NSScrollElasticity.None;
+			if (AppDelegate.IsOnLionOrBetter)
+				outlineView.EnclosingScrollView.HorizontalScrollElasticity = outlineView.EnclosingScrollView.VerticalScrollElasticity = NSScrollElasticity.None;
 		}
 		
 		void SetupSearch ()
@@ -128,6 +135,10 @@ namespace macdoc
 			multipleMatchResults.Source = new MultipleMatchDataSource (this);
 			searchResults.Source = new ResultDataSource ();
 			splitView.Delegate = new SplitViewDelegate ();
+			tabSelector.DidSelect += (sender, e) => {
+				if (e.Item.TabView.IndexOf (e.Item) == 2)
+					WindowForSheet.MakeFirstResponder (toolbarSearchEntry);
+			};
 		}
 		
 		class SplitViewDelegate : NSSplitViewDelegate
@@ -143,7 +154,7 @@ namespace macdoc
 		
 		void SetupBookmarks ()
 		{
-			if (!AppDelegate.IsOnLion){
+			if (!AppDelegate.IsOnLionOrBetter){
 				addBookmarkBtn.Hidden = true;
 				
 				viewBookmarksBtn.Hidden = true;
@@ -192,14 +203,30 @@ namespace macdoc
 
 			if (!manager.IsCreatingSearchIndex) {
 				InvokeOnMainThread (delegate {
+					var indexSpinnerHeight = indexSpinnerView.Frame.Height * 3 /4;
+					var searchSpinnerHeight = spinnerView.Frame.Height * 3 / 4;
+					
 					spinnerWidget.StopAnimation (this);
 					spinnerView.Hidden = true;
 					indexSpinnerWidget.StopAnimation (this);
 					indexSpinnerView.Hidden = true;
+
 					searchIndex = AppDelegate.Root.GetSearchIndex ();
 					indexSearchEntry.Enabled = true;
 					mdocSearch.Index = AppDelegate.Root.GetIndex ();
 					indexResults.ReloadData ();
+					
+					var splitViewFrame = splitView.Frame;
+					splitView.Frame = new RectangleF (splitViewFrame.X,
+					                                  splitViewFrame.Y - indexSpinnerHeight,
+					                                  splitViewFrame.Width,
+					                                  splitViewFrame.Height + indexSpinnerHeight);
+					
+					var searchScrollViewFrame = searchScrollView.Frame;
+					searchScrollView.Frame = new RectangleF (searchScrollViewFrame.X,
+					                                         searchScrollViewFrame.Y - searchSpinnerHeight,
+					                                         searchScrollViewFrame.Width,
+					                                         searchScrollViewFrame.Height + searchSpinnerHeight);
 				});
 			} else {
 				InvokeOnMainThread (delegate {
@@ -214,16 +241,14 @@ namespace macdoc
 		
 		internal void LoadUrl (string url, bool syncTreeView = false, HelpSource source = null, bool addToHistory = true)
 		{
-			if (url.StartsWith ("#")) {
-				Console.WriteLine ("FIXME: Anchor jump");
+			if (url.StartsWith ("#"))
 				return;
-			}
 			// In case user click on an external link e.g. [Android documentation] link at bottom of MonoDroid docs
 			if (url.StartsWith ("http://")) {
 				UrlLauncher.Launch (url);
 				return;
 			}
-			Console.WriteLine ("Loading {0}", url);
+			Logger.Log ("Loading {0}", url);
 			var ts = Interlocked.Increment (ref loadUrlTimestamp);
 			Task.Factory.StartNew (() => {
 				Node node;
@@ -237,7 +262,8 @@ namespace macdoc
 						if (ts < loadUrlTimestamp)
 							return;
 						currentUrl = node == null ? url : node.PublicUrl;
-						InvalidateRestorableState ();
+						if (AppDelegate.IsOnLionOrBetter)
+							InvalidateRestorableState ();
 						if (addToHistory)
 							history.AppendHistory (new LinkPageVisit (this, currentUrl));
 						LoadHtml (res);
@@ -321,13 +347,13 @@ namespace macdoc
 		
 		void HideMultipleMatches ()
 		{
-			splitView.SetPositionofDivider (splitView.MaxPositionOfDivider (0), 0);
+			splitView.SetPositionOfDivider (splitView.MaxPositionOfDivider (0), 0);
 		}
 		
 		void ShowMultipleMatches ()
 		{
 			float middle = (splitView.MaxPositionOfDivider (0) - splitView.MinPositionOfDivider (0))/2;
-			splitView.SetPositionofDivider (middle, 0);
+			splitView.SetPositionOfDivider (middle, 0);
 		}
 		
 		// Action: when the user clicks on the index table view
@@ -358,10 +384,13 @@ namespace macdoc
 		partial void StartSearch (NSSearchField sender)
 		{
 			var contents = sender.StringValue;
-			if (contents == null || contents == "")
+			if (string.IsNullOrEmpty (contents))
 				return;
 			tabSelector.SelectAt (2);
 			Search (contents);
+			// Unselect the search term in case user is typing slowly
+			if (sender.CurrentEditor != null)
+				sender.CurrentEditor.SelectedRange = new NSRange (contents.Length, 0);
 		}
 		
 		// Typing in the index panel
@@ -389,7 +418,7 @@ namespace macdoc
 			}
 		}
 
-		void HandleWebViewDecidePolicyForNavigation (object sender, WebNavigatioPolicyEventArgs e)
+		void HandleWebViewDecidePolicyForNavigation (object sender, WebNavigationPolicyEventArgs e)
 		{
 			if (LoadingFromString){
 				WebView.DecideUse (e.DecisionToken);
@@ -426,11 +455,11 @@ namespace macdoc
 			
 			// Process embedded images coming from doc source
 			// Because WebView doesn't let me answer a NSUrlRequest myself I have to resort to this piece of crap of a solution
-			var imgs = dom.GetElementsByTagName ("img").Where (node => node.Attributes["src"].Value.StartsWith ("source-id"));
+			var imgs = dom.GetElementsByTagName ("img").Where (node => node.Attributes["src"].NodeValue.StartsWith ("source-id"));
 			byte[] buffer = new byte[4096];
 			
 			foreach (var img in imgs) {
-				var src = img.Attributes["src"].Value;
+				var src = img.Attributes["src"].NodeValue;
 				var imgStream = AppDelegate.Root.GetImage (src);
 				if (imgStream == null)
 					continue;
@@ -474,16 +503,20 @@ namespace macdoc
 			outlineView.ExpandItem (item);
 			
 			// Focus the last child, then this child to ensure we show as much as possible
-			if (n.Nodes.Count > 0)
-				ScrollToVisible ((Node) n.Nodes [n.Nodes.Count-1]);
+			if (n.ChildNodes.Count > 0)
+				ScrollToVisible ((Node) n.ChildNodes [n.ChildNodes.Count-1]);
 			var row = ScrollToVisible (n);
 			ignoreSelect = true;
-			outlineView.SelectRows (new NSIndexSet (row), false);
+			if (row > 0)
+				outlineView.SelectRows (new NSIndexSet (row), false);
 			ignoreSelect = false;
 		}
 		
 		int ScrollToVisible (Node n)
 		{
+			if (!nodeToWrapper.ContainsKey (n))
+				return 0;
+
 			var item = nodeToWrapper [n];
 			var row = outlineView.RowForItem (item);
 			outlineView.ScrollRowToVisible (row);
@@ -508,7 +541,7 @@ namespace macdoc
 					return;
 				
 				var node = WrapNode.FromObject (parent.outlineView.ItemAtRow ((int) indexes.FirstIndex));
-				parent.LoadUrl (node.PublicUrl, false, node.tree.HelpSource);
+				parent.LoadUrl (node.PublicUrl, false, node.Tree.HelpSource);
 			}
 		}
 
