@@ -39,7 +39,7 @@ namespace macdoc
 		{
 			switch (product) {
 			case Product.MonoTouch:
-				return "/Developer/MonoTouch/usr/share/doc/MonoTouch/apple-doc-wizard";
+				return "/Library/Frameworks/Xamarin.iOS.framework/Versions/Current/share/doc/MonoTouch/apple-doc-wizard";
 			case Product.MonoMac:
 				return "/Library/Frameworks/Xamarin.Mac.framework/Versions/Current/usr/share/doc/Xamarin.Mac/apple-doc-wizard";
 			default:
