@@ -89,7 +89,7 @@ namespace ICSharpCode.NRefactory.CSharp.Analysis
 		protected TextLocation regionEnd;
 		
 		protected CSharpAstResolver resolver;
-		bool isInAccessorContainingValueParameter;
+		protected bool isInAccessorContainingValueParameter;
 		
 		protected abstract void Colorize(TextLocation start, TextLocation end, TColor color);
 		
@@ -535,7 +535,8 @@ namespace ICSharpCode.NRefactory.CSharp.Analysis
 					return true;
 				case SymbolKind.Constructor:
 				case SymbolKind.Destructor:
-					return TryGetTypeHighlighting (member.DeclaringType.Kind, out color);
+					var declaringType = member.DeclaringType;
+					return TryGetTypeHighlighting (declaringType != null ? declaringType.Kind : TypeKind.Unknown, out color);
 				default:
 					color = default (TColor);
 					return false;
