@@ -24,7 +24,7 @@ using System.Reflection;
 using System.Reflection.Emit;
 #endif
 
-namespace Mono.CSharp {
+namespace ICSharpCode.NRefactory.MonoCSharp {
 
 	public abstract class CompilerGeneratedContainer : ClassOrStruct
 	{
@@ -353,7 +353,7 @@ namespace Mono.CSharp {
 				hoisted_locals.Add (hoisted);
 			}
 
-			if (ec.CurrentBlock.Explicit != localVariable.Block.Explicit && !(hoisted.Storey is StateMachine))
+			if (ec.CurrentBlock.Explicit != localVariable.Block.Explicit && !(hoisted.Storey is StateMachine) && hoisted.Storey != null)
 				hoisted.Storey.AddReferenceFromChildrenBlock (ec.CurrentBlock.Explicit);
 		}
 
@@ -1798,8 +1798,10 @@ namespace Mono.CSharp {
 				// this argument is generated during compilation which speeds up dispatch
 				// by about 25%
 				//
-				method_parameters = ParametersCompiled.Prefix (method_parameters,
-					new Parameter (null, null, 0, null, loc), ec.Module.Compiler.BuiltinTypes.Object);
+				// Unused as it breaks compatibility
+				//
+				// method_parameters = ParametersCompiled.Prefix (method_parameters,
+				//	new Parameter (null, null, 0, null, loc), ec.Module.Compiler.BuiltinTypes.Object);
 			}
 
 			if (storey == null && hoisted_tparams == null)
