@@ -23,7 +23,7 @@ using System.Reflection;
 using System.Reflection.Emit;
 #endif
 
-namespace Mono.CSharp
+namespace ICSharpCode.NRefactory.MonoCSharp
 {
 	public class Await : ExpressionStatement
 	{
@@ -151,7 +151,7 @@ namespace Mono.CSharp
 
 			public bool ProbingMode { get; set; }
 
-			protected override void Error_TypeDoesNotContainDefinition (ResolveContext rc, TypeSpec type, string name)
+			public override void Error_TypeDoesNotContainDefinition (ResolveContext rc, TypeSpec type, string name)
 			{
 				Error_OperatorCannotBeApplied (rc, type);
 			}
@@ -165,7 +165,7 @@ namespace Mono.CSharp
 				if (invocation != null && invocation.MethodGroup != null && (invocation.MethodGroup.BestCandidate.Modifiers & Modifiers.ASYNC) != 0) {
 					rc.Report.Error (4008, loc, "Cannot await void method `{0}'. Consider changing method return type to `Task'",
 						invocation.GetSignatureForError ());
-				} else {
+				} else if (type != InternalType.ErrorType) {
 					rc.Report.Error (4001, loc, "Cannot await `{0}' expression", type.GetSignatureForError ());
 				}
 			}
