@@ -21,7 +21,7 @@ using System.IO;
 using System.Text;
 using System.Linq;
 
-namespace Mono.CSharp
+namespace ICSharpCode.NRefactory.MonoCSharp
 {
 
 	/// <summary>
@@ -159,7 +159,7 @@ namespace Mono.CSharp
 		///
 		///   This is the base class that will host the code
 		///   executed by the Evaluator.  By default
-		///   this is the Mono.CSharp.InteractiveBase class
+		///   this is the ICSharpCode.NRefactory.MonoCSharp.InteractiveBase class
 		///   which is useful for interactive use.
 		///
 		///   By changing this property you can control the
@@ -860,13 +860,19 @@ namespace Mono.CSharp
 
 		public string GetUsing ()
 		{
+			if (source_file == null || source_file.Usings == null)
+				return string.Empty;
+
 			StringBuilder sb = new StringBuilder ();
 			// TODO:
 			//foreach (object x in ns.using_alias_list)
 			//    sb.AppendFormat ("using {0};\n", x);
 
 			foreach (var ue in source_file.Usings) {
-				sb.AppendFormat ("using {0};", ue.ToString ());
+				if (ue.Alias != null || ue.ResolvedExpression == null)
+					continue;
+
+				sb.AppendFormat("using {0};", ue.ToString ());
 				sb.Append (Environment.NewLine);
 			}
 
@@ -877,7 +883,11 @@ namespace Mono.CSharp
 		{
 			var res = new List<string> ();
 
-			foreach (var ue in source_file.Usings) {
+			if (source_file == null || source_file.Usings == null)
+				return res;
+
+			foreach (var ue in source_file.Usings)
+			{
 				if (ue.Alias != null || ue.ResolvedExpression == null)
 					continue;
 
@@ -1274,7 +1284,7 @@ namespace Mono.CSharp
 
 			if (current_container.Containers != null)
 			{
-				var existing = current_container.Containers.FirstOrDefault (l => l.Basename == tc.Basename);
+				var existing = current_container.Containers.FirstOrDefault (l => l.MemberName.Basename == tc.MemberName.Basename);
 				if (existing != null) {
 					current_container.RemoveContainer (existing);
 					undo_actions.Add (() => current_container.AddTypeContainer (existing));
