@@ -26,7 +26,7 @@ using System.Reflection;
 using System.Reflection.Emit;
 #endif
 
-namespace Mono.CSharp
+namespace ICSharpCode.NRefactory.MonoCSharp
 {
 	public class FieldDeclarator
 	{
@@ -424,7 +424,7 @@ namespace Mono.CSharp
 			}
 			
 			// Create nested fixed buffer container
-			string name = String.Format ("<{0}>__FixedBuffer{1}", Name, GlobalCounter++);
+			string name = String.Format ("<{0}>__FixedBuffer{1}", TypeDefinition.FilterNestedName (Name), GlobalCounter++);
 			fixed_buffer_type = Parent.TypeBuilder.DefineNestedType (name,
 				TypeAttributes.NestedPublic | TypeAttributes.Sealed | TypeAttributes.BeforeFieldInit,
 				Compiler.BuiltinTypes.ValueType.GetMetaInfo ());
@@ -609,9 +609,23 @@ namespace Mono.CSharp
 			if (TypeSpec.IsReferenceType (MemberType))
 				return true;
 
-			if (MemberType.IsEnum)
+			if (MemberType.IsPointer)
 				return true;
 
+			if (MemberType.IsEnum) {
+				switch (EnumSpec.GetUnderlyingType (MemberType).BuiltinType) {
+				case BuiltinTypeSpec.Type.SByte:
+				case BuiltinTypeSpec.Type.Byte:
+				case BuiltinTypeSpec.Type.Short:
+				case BuiltinTypeSpec.Type.UShort:
+				case BuiltinTypeSpec.Type.Int:
+				case BuiltinTypeSpec.Type.UInt:
+					return true;
+				default:
+					return false;
+				}
+			}
+
 			return false;
 		}
 
