@@ -22,7 +22,7 @@ using MetaType = System.Type;
 using System.Reflection;
 #endif
 
-namespace Mono.CSharp
+namespace ICSharpCode.NRefactory.MonoCSharp
 {
 	//
 	// Inflated or non-inflated representation of any type. 
@@ -490,17 +490,22 @@ namespace Mono.CSharp
 		//
 		// Text representation of type used by documentation writer
 		//
-		public override string GetSignatureForDocumentation ()
+		public sealed override string GetSignatureForDocumentation ()
+		{
+			return GetSignatureForDocumentation (false);
+		}
+
+		public virtual string GetSignatureForDocumentation (bool explicitName)
 		{
 			StringBuilder sb = new StringBuilder ();
 			if (IsNested) {
-				sb.Append (DeclaringType.GetSignatureForDocumentation ());
-			} else {
-				sb.Append (MemberDefinition.Namespace);
+				sb.Append (DeclaringType.GetSignatureForDocumentation (explicitName));
+			} else if (MemberDefinition.Namespace != null) {
+				sb.Append (explicitName ? MemberDefinition.Namespace.Replace ('.', '#') : MemberDefinition.Namespace);
 			}
 
 			if (sb.Length != 0)
-				sb.Append (".");
+				sb.Append (explicitName ? "#" : ".");
 
 			sb.Append (Name);
 			if (Arity > 0) {
@@ -510,7 +515,7 @@ namespace Mono.CSharp
 				        if (i > 0)
 				            sb.Append (",");
 
-				        sb.Append (TypeArguments[i].GetSignatureForDocumentation ());
+						sb.Append (TypeArguments[i].GetSignatureForDocumentation (explicitName));
 				    }
 				    sb.Append ("}");
 				} else {
@@ -522,33 +527,6 @@ namespace Mono.CSharp
 			return sb.ToString ();
 		}
 
-		public string GetExplicitNameSignatureForDocumentation ()
-		{
-			StringBuilder sb = new StringBuilder ();
-			if (IsNested) {
-				sb.Append (DeclaringType.GetExplicitNameSignatureForDocumentation ());
-			} else if (MemberDefinition.Namespace != null) {
-				sb.Append (MemberDefinition.Namespace.Replace ('.', '#'));
-			}
-
-			if (sb.Length != 0)
-				sb.Append ("#");
-
-			sb.Append (Name);
-			if (Arity > 0) {
-				sb.Append ("{");
-				for (int i = 0; i < Arity; ++i) {
-					if (i > 0)
-						sb.Append (",");
-
-					sb.Append (TypeArguments[i].GetExplicitNameSignatureForDocumentation ());
-				}
-				sb.Append ("}");
-			}
-
-			return sb.ToString ();
-		}
-
 		public override string GetSignatureForError ()
 		{
 			string s;
@@ -1446,6 +1424,7 @@ namespace Mono.CSharp
 		public static readonly InternalType FakeInternalType = new InternalType ("<fake$type>");
 		public static readonly InternalType Namespace = new InternalType ("<namespace>");
 		public static readonly InternalType ErrorType = new InternalType ("<error>");
+		public static readonly InternalType VarOutType = new InternalType ("var out");
 
 		readonly string name;
 
@@ -1658,9 +1637,9 @@ namespace Mono.CSharp
 			return null;
 		}
 
-		public override string GetSignatureForDocumentation ()
+		public override string GetSignatureForDocumentation (bool explicitName)
 		{
-			return Element.GetSignatureForDocumentation () + GetPostfixSignature ();
+			return Element.GetSignatureForDocumentation (explicitName) + GetPostfixSignature ();
 		}
 
 		public override string GetSignatureForError ()
@@ -1893,29 +1872,33 @@ namespace Mono.CSharp
 			return sb.ToString ();
 		}
 
-		public override string GetSignatureForDocumentation ()
+		public override string GetSignatureForDocumentation (bool explicitName)
 		{
 			StringBuilder sb = new StringBuilder ();
-			GetElementSignatureForDocumentation (sb);
+			GetElementSignatureForDocumentation (sb, explicitName);
 			return sb.ToString ();
 		}
 
-		void GetElementSignatureForDocumentation (StringBuilder sb)
+		void GetElementSignatureForDocumentation (StringBuilder sb, bool explicitName)
 		{
 			var ac = Element as ArrayContainer;
 			if (ac == null)
-				sb.Append (Element.GetSignatureForDocumentation ());
+				sb.Append (Element.GetSignatureForDocumentation (explicitName));
 			else
-				ac.GetElementSignatureForDocumentation (sb);
+				ac.GetElementSignatureForDocumentation (sb, explicitName);
 
-			sb.Append ("[");
-			for (int i = 1; i < rank; i++) {
-				if (i == 1)
-					sb.Append ("0:");
+			if (explicitName) {
+				sb.Append (GetPostfixSignature (rank));
+			} else {
+				sb.Append ("[");
+				for (int i = 1; i < rank; i++) {
+					if (i == 1)
+						sb.Append ("0:");
 
-				sb.Append (",0:");
+					sb.Append (",0:");
+				}
+				sb.Append ("]");
 			}
-			sb.Append ("]");
 		}
 
 		public static ArrayContainer MakeType (ModuleContainer module, TypeSpec element)
