@@ -77,7 +77,13 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 		/// Specifies whether the ast builder should add annotations to type references.
 		/// The default value is <c>false</c>.
 		/// </summary>
-		public bool AddAnnotations { get; set; }
+		public bool AddTypeReferenceAnnotations { get; set; }
+		
+		/// <summary>
+		/// Specifies whether the ast builder should add ResolveResult annotations to AST nodes.
+		/// The default value is <c>false</c>.
+		/// </summary>
+		public bool AddResolveResultAnnotations { get; set; }
 		
 		/// <summary>
 		/// Controls the accessibility modifiers are shown.
@@ -122,6 +128,12 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 		/// </summary>
 		public bool ShowConstantValues { get; set; }
 		
+		/// <summary>
+		/// Controls whether to show attributes.
+		/// The default value is <c>false</c>.
+		/// </summary>
+		public bool ShowAttributes { get; set; }
+		
 		/// <summary>
 		/// Controls whether to use fully-qualified type names or short type names.
 		/// The default value is <c>false</c>.
@@ -159,8 +171,10 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			if (type == null)
 				throw new ArgumentNullException("type");
 			AstType astType = ConvertTypeHelper(type);
-			if (AddAnnotations)
+			if (AddTypeReferenceAnnotations)
 				astType.AddAnnotation(type);
+			if (AddResolveResultAnnotations)
+				astType.AddAnnotation(new TypeResolveResult(type));
 			return astType;
 		}
 		
@@ -417,7 +431,10 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			}
 			
 			if (rr is TypeOfResolveResult) {
-				return new TypeOfExpression(ConvertType(rr.Type));
+				var expr = new TypeOfExpression(ConvertType(((TypeOfResolveResult)rr).ReferencedType));
+				if (AddResolveResultAnnotations)
+					expr.AddAnnotation(rr);
+				return expr;
 			} else if (rr is ArrayCreateResolveResult) {
 				ArrayCreateResolveResult acrr = (ArrayCreateResolveResult)rr;
 				ArrayCreateExpression ace = new ArrayCreateExpression();
@@ -438,6 +455,8 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 					initializer.Elements.AddRange(acrr.InitializerElements.Select(ConvertConstantValue));
 					ace.Initializer = initializer;
 				}
+				if (AddResolveResultAnnotations)
+					ace.AddAnnotation(rr);
 				return ace;
 			} else if (rr.IsCompileTimeConstant) {
 				return ConvertConstantValue(rr.Type, rr.ConstantValue);
@@ -451,10 +470,17 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			if (type == null)
 				throw new ArgumentNullException("type");
 			if (constantValue == null) {
-				if (type.IsReferenceType == true)
-					return new NullReferenceExpression();
-				else
-					return new DefaultValueExpression(ConvertType(type));
+				if (type.IsReferenceType == true) {
+					var expr = new NullReferenceExpression();
+					if (AddResolveResultAnnotations)
+						expr.AddAnnotation(new ConstantResolveResult(SpecialType.NullType, null));
+					return expr;
+				} else {
+					var expr = new DefaultValueExpression(ConvertType(type));
+					if (AddResolveResultAnnotations)
+						expr.AddAnnotation(new ConstantResolveResult(type, null));
+					return expr;
+				}
 			} else if (type.Kind == TypeKind.Enum) {
 				return ConvertEnumValue(type, (long)CSharpPrimitiveCast.Cast(TypeCode.Int64, constantValue, false));
 			} else {
@@ -547,6 +573,9 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			} else if (parameter.IsParams) {
 				decl.ParameterModifier = ParameterModifier.Params;
 			}
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (parameter.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+			}
 			decl.Type = ConvertType(parameter.Type);
 			if (this.ShowParameterNames) {
 				decl.Name = parameter.Name;
@@ -605,7 +634,7 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 					return ConvertDestructor((IMethod)entity);
 				case SymbolKind.Accessor:
 					IMethod accessor = (IMethod)entity;
-					return ConvertAccessor(accessor, accessor.AccessorOwner != null ? accessor.AccessorOwner.Accessibility : Accessibility.None);
+					return ConvertAccessor(accessor, accessor.AccessorOwner != null ? accessor.AccessorOwner.Accessibility : Accessibility.None, false);
 				default:
 					throw new ArgumentException("Invalid value for SymbolKind: " + entity.SymbolKind);
 			}
@@ -659,6 +688,12 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			var decl = new TypeDeclaration();
 			decl.ClassType = classType;
 			decl.Modifiers = modifiers;
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (typeDefinition.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new TypeResolveResult(typeDefinition));
+			}
 			decl.Name = typeDefinition.Name;
 			
 			int outerTypeParameterCount = (typeDefinition.DeclaringTypeDefinition == null) ? 0 : typeDefinition.DeclaringTypeDefinition.TypeParameterCount;
@@ -671,12 +706,19 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			
 			if (this.ShowBaseTypes) {
 				foreach (IType baseType in typeDefinition.DirectBaseTypes) {
-					decl.BaseTypes.Add(ConvertType(baseType));
+					if (baseType.IsKnownType (KnownTypeCode.Enum)) {
+						if (!typeDefinition.EnumUnderlyingType.IsKnownType (KnownTypeCode.Int32)) {
+							decl.BaseTypes.Add (ConvertType (typeDefinition.EnumUnderlyingType));
+						}
+					} else if (!baseType.IsKnownType (KnownTypeCode.Object) &&
+						 !baseType.IsKnownType (KnownTypeCode.ValueType)) {
+						decl.BaseTypes.Add (ConvertType (baseType));
+					}
 				}
 			}
 			
 			if (this.ShowTypeParameters && this.ShowTypeParameterConstraints) {
-				foreach (ITypeParameter tp in typeDefinition.TypeParameters) {
+				foreach (ITypeParameter tp in typeDefinition.TypeParameters.Skip(outerTypeParameterCount)) {
 					var constraint = ConvertTypeParameterConstraint(tp);
 					if (constraint != null)
 						decl.Constraints.Add(constraint);
@@ -691,6 +733,12 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			
 			DelegateDeclaration decl = new DelegateDeclaration();
 			decl.Modifiers = modifiers & ~Modifiers.Sealed;
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (d.Attributes.Select (a => new AttributeSection (ConvertAttribute (a))));
+				decl.Attributes.AddRange (invokeMethod.ReturnTypeAttributes.Select ((a) => new AttributeSection (ConvertAttribute (a)) {
+					AttributeTarget = "return"
+				}));
+			}
 			decl.ReturnType = ConvertType(invokeMethod.ReturnType);
 			decl.Name = d.Name;
 			
@@ -729,6 +777,12 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 				}
 				decl.Modifiers = m;
 			}
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (field.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, field));
+			}
 			decl.ReturnType = ConvertType(field.ReturnType);
 			Expression initializer = null;
 			if (field.IsConst && this.ShowConstantValues)
@@ -748,13 +802,27 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			}
 		}
 		
-		Accessor ConvertAccessor(IMethod accessor, Accessibility ownerAccessibility)
+		Accessor ConvertAccessor(IMethod accessor, Accessibility ownerAccessibility, bool addParamterAttribute)
 		{
 			if (accessor == null)
 				return Accessor.Null;
 			Accessor decl = new Accessor();
 			if (this.ShowAccessibility && accessor.Accessibility != ownerAccessibility)
 				decl.Modifiers = ModifierFromAccessibility(accessor.Accessibility);
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (accessor.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+				decl.Attributes.AddRange (accessor.ReturnTypeAttributes.Select ((a) => new AttributeSection (ConvertAttribute (a)) {
+					AttributeTarget = "return"
+				}));
+				if (addParamterAttribute && accessor.Parameters.Count > 0) {
+					decl.Attributes.AddRange (accessor.Parameters.Last ().Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a)) {
+						AttributeTarget = "param"
+					}));
+				}
+			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, accessor));
+			}
 			decl.Body = GenerateBodyBlock();
 			return decl;
 		}
@@ -763,10 +831,17 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 		{
 			PropertyDeclaration decl = new PropertyDeclaration();
 			decl.Modifiers = GetMemberModifiers(property);
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (property.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, property));
+			}
 			decl.ReturnType = ConvertType(property.ReturnType);
 			decl.Name = property.Name;
-			decl.Getter = ConvertAccessor(property.Getter, property.Accessibility);
-			decl.Setter = ConvertAccessor(property.Setter, property.Accessibility);
+			decl.Getter = ConvertAccessor(property.Getter, property.Accessibility, false);
+			decl.Setter = ConvertAccessor(property.Setter, property.Accessibility, true);
+			decl.PrivateImplementationType = GetExplicitInterfaceType (property);
 			return decl;
 		}
 		
@@ -774,12 +849,19 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 		{
 			IndexerDeclaration decl = new IndexerDeclaration();
 			decl.Modifiers = GetMemberModifiers(indexer);
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (indexer.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, indexer));
+			}
 			decl.ReturnType = ConvertType(indexer.ReturnType);
 			foreach (IParameter p in indexer.Parameters) {
 				decl.Parameters.Add(ConvertParameter(p));
 			}
-			decl.Getter = ConvertAccessor(indexer.Getter, indexer.Accessibility);
-			decl.Setter = ConvertAccessor(indexer.Setter, indexer.Accessibility);
+			decl.Getter = ConvertAccessor(indexer.Getter, indexer.Accessibility, false);
+			decl.Setter = ConvertAccessor(indexer.Setter, indexer.Accessibility, true);
+			decl.PrivateImplementationType = GetExplicitInterfaceType (indexer);
 			return decl;
 		}
 		
@@ -788,14 +870,26 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			if (this.UseCustomEvents) {
 				CustomEventDeclaration decl = new CustomEventDeclaration();
 				decl.Modifiers = GetMemberModifiers(ev);
+				if (ShowAttributes) {
+					decl.Attributes.AddRange (ev.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+				}
+				if (AddResolveResultAnnotations) {
+					decl.AddAnnotation(new MemberResolveResult(null, ev));
+				}
 				decl.ReturnType = ConvertType(ev.ReturnType);
 				decl.Name = ev.Name;
-				decl.AddAccessor    = ConvertAccessor(ev.AddAccessor, ev.Accessibility);
-				decl.RemoveAccessor = ConvertAccessor(ev.RemoveAccessor, ev.Accessibility);
+				decl.AddAccessor    = ConvertAccessor(ev.AddAccessor, ev.Accessibility, true);
+				decl.RemoveAccessor = ConvertAccessor(ev.RemoveAccessor, ev.Accessibility, true);
 				return decl;
 			} else {
 				EventDeclaration decl = new EventDeclaration();
 				decl.Modifiers = GetMemberModifiers(ev);
+				if (ShowAttributes) {
+					decl.Attributes.AddRange (ev.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+				}
+				if (AddResolveResultAnnotations) {
+					decl.AddAnnotation(new MemberResolveResult(null, ev));
+				}
 				decl.ReturnType = ConvertType(ev.ReturnType);
 				decl.Variables.Add(new VariableInitializer(ev.Name));
 				return decl;
@@ -808,6 +902,15 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			decl.Modifiers = GetMemberModifiers(method);
 			if (method.IsAsync && ShowModifiers)
 				decl.Modifiers |= Modifiers.Async;
+			if (ShowAttributes) {
+				decl.Attributes.AddRange (method.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
+				decl.Attributes.AddRange (method.ReturnTypeAttributes.Select ((a) => new AttributeSection (ConvertAttribute (a)) {
+					AttributeTarget = "return"
+				}));
+			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, method));
+			}
 			decl.ReturnType = ConvertType(method.ReturnType);
 			decl.Name = method.Name;
 			
@@ -831,6 +934,7 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 				}
 			}
 			decl.Body = GenerateBodyBlock();
+			decl.PrivateImplementationType = GetExplicitInterfaceType (method);
 			return decl;
 		}
 		
@@ -847,6 +951,9 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			foreach (IParameter p in op.Parameters) {
 				decl.Parameters.Add(ConvertParameter(p));
 			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, op));
+			}
 			decl.Body = GenerateBodyBlock();
 			return decl;
 		}
@@ -855,11 +962,16 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 		{
 			ConstructorDeclaration decl = new ConstructorDeclaration();
 			decl.Modifiers = GetMemberModifiers(ctor);
+			if (ShowAttributes)
+				decl.Attributes.AddRange (ctor.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
 			if (ctor.DeclaringTypeDefinition != null)
 				decl.Name = ctor.DeclaringTypeDefinition.Name;
 			foreach (IParameter p in ctor.Parameters) {
 				decl.Parameters.Add(ConvertParameter(p));
 			}
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, ctor));
+			}
 			decl.Body = GenerateBodyBlock();
 			return decl;
 		}
@@ -869,6 +981,9 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			DestructorDeclaration decl = new DestructorDeclaration();
 			if (dtor.DeclaringTypeDefinition != null)
 				decl.Name = dtor.DeclaringTypeDefinition.Name;
+			if (AddResolveResultAnnotations) {
+				decl.AddAnnotation(new MemberResolveResult(null, dtor));
+			}
 			decl.Body = GenerateBodyBlock();
 			return decl;
 		}
@@ -894,18 +1009,33 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			}
 		}
 		
+		bool NeedsAccessibility(IMember member)
+		{
+			var declaringType = member.DeclaringType;
+			if ((declaringType != null && declaringType.Kind == TypeKind.Interface) || member.IsExplicitInterfaceImplementation)
+				return false;
+			switch (member.SymbolKind) {
+				case SymbolKind.Constructor:
+					return !member.IsStatic;
+				case SymbolKind.Destructor:
+					return false;
+				default:
+					return true;
+			}
+		}
+		
 		Modifiers GetMemberModifiers(IMember member)
 		{
-			bool isInterfaceMember = member.DeclaringType.Kind == TypeKind.Interface;
 			Modifiers m = Modifiers.None;
-			if (this.ShowAccessibility && !isInterfaceMember) {
-				m |= ModifierFromAccessibility(member.Accessibility);
+			if (this.ShowAccessibility && NeedsAccessibility(member)) {
+				m |= ModifierFromAccessibility (member.Accessibility);
 			}
 			if (this.ShowModifiers) {
 				if (member.IsStatic) {
 					m |= Modifiers.Static;
 				} else {
-					if (member.IsAbstract && !isInterfaceMember)
+					var declaringType = member.DeclaringType;
+					if (member.IsAbstract && declaringType != null && declaringType.Kind != TypeKind.Interface)
 						m |= Modifiers.Abstract;
 					if (member.IsOverride)
 						m |= Modifiers.Override;
@@ -927,6 +1057,8 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 			TypeParameterDeclaration decl = new TypeParameterDeclaration();
 			decl.Variance = tp.Variance;
 			decl.Name = tp.Name;
+			if (ShowAttributes)
+				decl.Attributes.AddRange (tp.Attributes.Select ((a) => new AttributeSection (ConvertAttribute (a))));
 			return decl;
 		}
 		
@@ -946,7 +1078,7 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 				if (!IsObjectOrValueType(t))
 					c.BaseTypes.Add(ConvertType(t));
 			}
-			if (tp.HasDefaultConstructorConstraint) {
+			if (tp.HasDefaultConstructorConstraint && !tp.HasValueTypeConstraint) {
 				c.BaseTypes.Add(new PrimitiveType("new"));
 			}
 			return c;
@@ -977,5 +1109,15 @@ namespace ICSharpCode.NRefactory.CSharp.Refactoring
 		{
 			return new NamespaceDeclaration(ns.FullName);
 		}
+
+		AstType GetExplicitInterfaceType (IMember member)
+		{
+			if (member.IsExplicitInterfaceImplementation) {
+				var baseMember = member.ImplementedInterfaceMembers.FirstOrDefault ();
+				if (baseMember != null)
+					return ConvertType (baseMember.DeclaringType);
+			}
+			return null;
+		}
 	}
 }
