@@ -11,9 +11,6 @@ namespace WpfTestRunner
 		[STAThread]
 		static void Main (string[] args)
 		{
-			Xwt.Application.Initialize (Xwt.ToolkitType.Wpf);
-			ReferenceImageManager.Init ("WpfTestRunner");
-
 			var list = new List<string> (args);
 			list.Add ("-domain=None");
 			list.Add ("-noshadow");
