@@ -31,6 +31,12 @@ using System.Collections.Generic;
 using System.Reflection;
 using System.Reflection.Emit;
 
+#if XWT_GTK3
+using GtkTreeModel = Gtk.ITreeModel;
+#else
+using GtkTreeModel = Gtk.TreeModel;
+#endif
+
 namespace Xwt.GtkBackend
 {
 	public static class GtkWorkarounds
@@ -103,7 +109,7 @@ namespace Xwt.GtkBackend
 		
 		static System.Reflection.MethodInfo glibObjectGetProp, glibObjectSetProp;
 		
-		public static int GtkMinorVersion = 12, GtkMicroVersion = 0;
+		public static int GtkMajorVersion = 2, GtkMinorVersion = 12, GtkMicroVersion = 0;
 		static bool oldMacKeyHacks = false;
 		
 		static GtkWorkarounds ()
@@ -115,13 +121,19 @@ namespace Xwt.GtkBackend
 			var flags = System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic;
 			glibObjectSetProp = typeof (GLib.Object).GetMethod ("SetProperty", flags);
 			glibObjectGetProp = typeof (GLib.Object).GetMethod ("GetProperty", flags);
-			
+
+			#if XWT_GTK3
+			GtkMajorVersion = (int)Gtk.Global.MajorVersion;
+			GtkMinorVersion = (int)Gtk.Global.MinorVersion;
+			GtkMicroVersion = (int)Gtk.Global.MicroVersion;
+			#else
 			foreach (int i in new [] { 24, 22, 20, 18, 16, 14 }) {
 				if (Gtk.Global.CheckVersion (2, (uint)i, 0) == null) {
 					GtkMinorVersion = i;
 					break;
 				}
 			}
+			#endif
 			
 			for (int i = 1; i < 20; i++) {
 				if (Gtk.Global.CheckVersion (2, (uint)GtkMinorVersion, (uint)i) == null) {
@@ -581,7 +593,7 @@ namespace Xwt.GtkBackend
 			Gdk.ModifierType modifier = evt.State;
 			byte grp = evt.Group;
 			
-			if (GtkMinorVersion >= 20) {
+			if (GtkMajorVersion > 2 || GtkMajorVersion <= 2 && GtkMinorVersion >= 20) {
 				gdk_keymap_add_virtual_modifiers (keymap.Handle, ref modifier);
 			}
 			
@@ -980,7 +992,7 @@ namespace Xwt.GtkBackend
 
 		public static void SetLinkHandler (this Gtk.Label label, Action<string> urlHandler)
 		{
-			if (GtkMinorVersion >= 18)
+			if (GtkMajorVersion > 2 || GtkMajorVersion <= 2 && GtkMinorVersion >= 18)
 				new UrlHandlerClosure (urlHandler).ConnectTo (label);
 		}
 
@@ -1052,10 +1064,9 @@ namespace Xwt.GtkBackend
 		[DllImport (GtkInterop.LIBGTK, CallingConvention = CallingConvention.Cdecl)]
 		static extern bool gtk_tree_view_get_tooltip_context (IntPtr raw, ref int x, ref int y, bool keyboard_tip, out IntPtr model, out IntPtr path, IntPtr iter);
 
-		#if !XWT_GTK3
 		//the GTK# version of this has 'out' instead of 'ref', preventing passing the x,y values in
 		public static bool GetTooltipContext (this Gtk.TreeView tree, ref int x, ref int y, bool keyboardTip,
-			 out Gtk.TreeModel model, out Gtk.TreePath path, out Gtk.TreeIter iter)
+			 out GtkTreeModel model, out Gtk.TreePath path, out Gtk.TreeIter iter)
 		{
 			IntPtr intPtr = Marshal.AllocHGlobal (Marshal.SizeOf (typeof (Gtk.TreeIter)));
 			IntPtr handle;
@@ -1067,14 +1078,13 @@ namespace Xwt.GtkBackend
 			Marshal.FreeHGlobal (intPtr);
 			return result;
 		}
-		#endif
 
 		[DllImport (GtkInterop.LIBGTK, CallingConvention = CallingConvention.Cdecl)]
 		static extern void gtk_image_menu_item_set_always_show_image (IntPtr menuitem, bool alwaysShow);
 
 		public static void ForceImageOnMenuItem (Gtk.ImageMenuItem mi)
 		{
-			if (GtkMinorVersion >= 16)
+			if (GtkMajorVersion > 2 || GtkMajorVersion <= 2 && GtkMinorVersion >= 16)
 				gtk_image_menu_item_set_always_show_image (mi.Handle, true);
 		}
 
@@ -1214,6 +1224,77 @@ namespace Xwt.GtkBackend
 			return new Gtk.ComboBoxEntry ();
 			#endif
 		}
+
+
+		/// <summary>
+		/// Adjusts pointer coordinates to be relative to the widget.
+		/// </summary>
+		/// <returns>The pointer coordinates.</returns>
+		/// <param name="widget">The Widget.</param>
+		/// <param name="eventWindow">The event source window.</param>
+		/// <param name="x">The events pointer x coordinate.</param>
+		/// <param name="y">The events pointer y coordinate.</param>
+		/// <remarks>
+		/// Some widgets have additional child Gdk windows (or real child widgets if
+		/// the widget is a container) on top of their root window.
+		/// In this case pointer events may come from child windows and contain
+		/// wrong coordinates (relative to child window and not to the widget itself).
+		/// 
+		/// CheckPointerCoordinates checks whether the events source window is not
+		/// the widgets root window and adjusts the coordinates to relative
+		/// to the widget and not to its child.
+		///</remarks>
+		public static Xwt.Point CheckPointerCoordinates (this Gtk.Widget widget, Gdk.Window eventWindow, double x, double y)
+		{
+			if (widget.GdkWindow != eventWindow)
+			{
+				int pointer_x, pointer_y;
+				widget.GetPointer (out pointer_x, out pointer_y);
+				return new Xwt.Point (pointer_x, pointer_y);
+			}
+			return new Xwt.Point (x, y);
+		}
+
+
+		[DllImport(GtkInterop.LIBGTK, CallingConvention = CallingConvention.Cdecl)]
+		static extern IntPtr gtk_message_dialog_get_message_area(IntPtr raw);
+		
+		public static Gtk.Box GetMessageArea(this Gtk.MessageDialog dialog)
+		{
+			#if XWT_GTK3
+			// according to Gtk docs MessageArea should always be a Gtk.Box, but we test this
+			// to be on the safe side.
+			var messageArea = dialog.MessageArea as Gtk.Box;
+			return messageArea ?? dialog.ContentArea;
+			#else
+			if (GtkWorkarounds.GtkMajorVersion <= 2 && GtkWorkarounds.GtkMinorVersion < 22) // message area not present before 2.22
+				return dialog.VBox;
+			IntPtr raw_ret = gtk_message_dialog_get_message_area(dialog.Handle);
+			Gtk.Box ret = GLib.Object.GetObject(raw_ret) as Gtk.Box;
+			return ret;
+			#endif
+		}
+
+
+		[DllImport(GtkInterop.LIBGOBJECT, CallingConvention = CallingConvention.Cdecl)]
+		static extern IntPtr g_signal_stop_emission_by_name(IntPtr raw, string name);
+
+		public static void StopSignal (this GLib.Object gobject, string signalid)
+		{
+			g_signal_stop_emission_by_name (gobject.Handle, signalid);
+		}
+
+		[DllImport(GtkInterop.LIBGTK, CallingConvention = CallingConvention.Cdecl)]
+		static extern IntPtr gtk_binding_set_find (string setName);
+		[DllImport(GtkInterop.LIBGTK, CallingConvention = CallingConvention.Cdecl)]
+		static extern void gtk_binding_entry_remove (IntPtr bindingSet, uint keyval, Gdk.ModifierType modifiers);
+
+		public static void RemoveKeyBindingFromClass (GLib.GType gtype, Gdk.Key key, Gdk.ModifierType modifiers)
+		{
+			var bindingSet = gtk_binding_set_find (gtype.ToString ());
+			if (bindingSet != IntPtr.Zero)
+				gtk_binding_entry_remove (bindingSet, (uint)key, modifiers);
+		}
 	}
 	
 	public struct KeyboardShortcut : IEquatable<KeyboardShortcut>
