@@ -707,6 +707,18 @@ namespace MonoDevelop.AspNet.Projects
 			}
 		}
 
+		public bool IsAspWebApiProject {
+			get {
+				return References.Any (r => r.Reference.StartsWith ("System.Web.Http.WebHost", StringComparison.Ordinal));
+			}
+		}
+
+		public virtual bool IsAspWebFormsProject {
+			get {
+				return Files.Any (f => f.Name.EndsWith (".aspx", StringComparison.Ordinal));
+			}
+		}
+
 		class BrowserExecutionTarget : ExecutionTarget
 		{
 			string name, id;
