@@ -40,13 +40,14 @@ using MonoDevelop.Ide;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Gui.Content;
 using MonoDevelop.Ide.CodeCompletion;
-using MonoDevelop.Projects.Dom;
-using MonoDevelop.Projects.Dom.Output;
 using MonoDevelop.Components;
 using MonoDevelop.Components.Commands;
 
 using CBinding.Parser;
 using Mono.TextEditor;
+using MonoDevelop.Ide.TypeSystem;
+using ICSharpCode.NRefactory.TypeSystem;
+using ICSharpCode.NRefactory.Completion;
 
 namespace CBinding
 {
@@ -80,7 +81,12 @@ namespace CBinding
 		};
 		
 		protected Mono.TextEditor.TextEditorData textEditorData{ get; set; }
-		
+
+		public override string CompletionLanguage {
+			get {
+				return "C/C++";
+			}
+		}
 		static bool IsOpenBrace (char c)
 		{
 			return c == '(' || c == '{' || c == '<' || c == '[';
@@ -136,7 +142,7 @@ namespace CBinding
 			return -1;
 		}
 		
-		static int GetClosingBraceForLine (TextEditorData editor, LineSegment line, out int openingLine)
+		static int GetClosingBraceForLine (TextEditorData editor, DocumentLine line, out int openingLine)
 		{
 			int offset = SearchMatchingBracket (editor, line.Offset, '{', '}', -1);
 			if (offset == -1) {
@@ -155,7 +161,7 @@ namespace CBinding
 			int lineCursorIndex = Math.Min (lineText.Length, Editor.Caret.Column);
 			
 			// Smart Indentation
-			if (TextEditorProperties.IndentStyle == IndentStyle.Smart)
+			if (Document.Editor.Options.IndentStyle == IndentStyle.Smart)
 			{
 				if (keyChar == '}') {
 					// Only indent if the brace is preceeded by whitespace.
@@ -163,7 +169,7 @@ namespace CBinding
 						int braceOpeningLine;
 						if(GetClosingBraceForLine(Editor, line, out braceOpeningLine) >= 0)
 						{
-							Editor.Replace (line.Offset, line.EditableLength, GetIndent(Editor, braceOpeningLine, 0) + "}" + lineText.Substring(lineCursorIndex));
+							Editor.Replace (line.Offset, line.Length, GetIndent(Editor, braceOpeningLine, 0) + "}" + lineText.Substring(lineCursorIndex));
 							Editor.Document.CommitLineUpdate (line);
 							return false;
 						}
@@ -195,7 +201,7 @@ namespace CBinding
 									nextChar = lineText[lineCursorIndex];
 	
 								if(finalChar == '{')
-									indent = TextEditorProperties.IndentString;
+									indent = Document.Editor.Options.IndentationString;
 							}
 	
 							// If the next character is an closing brace, indent it appropriately.
@@ -250,31 +256,31 @@ namespace CBinding
 			return null;
 		}
 		
-		public override ICompletionDataList HandleCodeCompletion (
-		    CodeCompletionContext completionContext, char completionChar)
-		{
-			int triggerWordLength = 0;
-			return HandleCodeCompletion (completionContext, completionChar, ref triggerWordLength);
-			
-			string lineText = Editor.GetLineText (completionContext.TriggerLine).TrimEnd();
-			
-			// If the line ends with a matched extension, invoke its handler
-			foreach (KeyValuePair<string, GetMembersForExtension> pair in completionExtensions) {
-				if (lineText.EndsWith(pair.Key)) {
-					lineText = lineText.Substring (0, lineText.Length - pair.Key.Length);
-					
-					int nameStart = lineText.LastIndexOfAny (allowedCharsMinusColon) + 1;
-					string itemName = lineText.Substring (nameStart).Trim ();
-					
-					if (string.IsNullOrEmpty (itemName))
-						return null;
-					
-					return pair.Value (this, pair.Key, itemName);
-				}
-			}
-			
-			return null;
-		}
+//		public override ICompletionDataList HandleCodeCompletion (
+//		    CodeCompletionContext completionContext, char completionChar)
+//		{
+//			int triggerWordLength = 0;
+//			return HandleCodeCompletion (completionContext, completionChar, ref triggerWordLength);
+//			
+//			string lineText = Editor.GetLineText (completionContext.TriggerLine).TrimEnd();
+//			
+//			// If the line ends with a matched extension, invoke its handler
+//			foreach (KeyValuePair<string, GetMembersForExtension> pair in completionExtensions) {
+//				if (lineText.EndsWith(pair.Key)) {
+//					lineText = lineText.Substring (0, lineText.Length - pair.Key.Length);
+//					
+//					int nameStart = lineText.LastIndexOfAny (allowedCharsMinusColon) + 1;
+//					string itemName = lineText.Substring (nameStart).Trim ();
+//					
+//					if (string.IsNullOrEmpty (itemName))
+//						return null;
+//					
+//					return pair.Value (this, pair.Key, itemName);
+//				}
+//			}
+//			
+//			return null;
+//		}
 		
 		public override ICompletionDataList CodeCompletionCommand (
 		    CodeCompletionContext completionContext)
@@ -284,7 +290,8 @@ namespace CBinding
 			
 			foreach (KeyValuePair<string, GetMembersForExtension> pair in completionExtensions) {
 				if(lineText.EndsWith(pair.Key)) {
-					return HandleCodeCompletion (completionContext, Editor.GetCharAt (pos));
+					int triggerWordLength = completionContext.TriggerWordLength;
+					return HandleCodeCompletion (completionContext, Editor.GetCharAt (pos), ref triggerWordLength);
 				}
 			}
 
@@ -560,7 +567,7 @@ namespace CBinding
 			return list;
 		}
 		
-		public override  IParameterDataProvider HandleParameterCompletion (
+		public override  MonoDevelop.Ide.CodeCompletion.ParameterDataProvider HandleParameterCompletion (
 		    CodeCompletionContext completionContext, char completionChar)
 		{
 			if (completionChar != '(')
@@ -585,7 +592,7 @@ namespace CBinding
 			if (string.IsNullOrEmpty (functionName))
 				return null;
 			
-			return new ParameterDataProvider (Document, info, functionName);
+			return new ParameterDataProvider (nameStart, Document, info, functionName);
 		}
 		
 		private bool AllWhiteSpace (string lineText)
@@ -647,7 +654,7 @@ namespace CBinding
 			
 			object tag = path[index].Tag;
 			DropDownBoxListWindow.IListDataProvider provider = null;
-			if (tag is ICompilationUnit) {
+			if (tag is ParsedDocument) {
 				provider = new CompilationUnitDataProvider (Document);
 			} else {
 				provider = new DataProvider (Document, tag, GetAmbience ());
@@ -674,59 +681,41 @@ namespace CBinding
 		// Yoinked from C# binding
 		void UpdatePath (object sender, Mono.TextEditor.DocumentLocationEventArgs e)
 		{
-			var unit = Document.CompilationUnit;
+	/*		var unit = Document.ParsedDocument;
 			if (unit == null)
 				return;
-				
-			var loc = textEditorData.Caret.Location;
-			IType type = unit.GetTypeAt (loc.Line, loc.Column);
-			List<PathEntry> result = new List<PathEntry> ();
-			Ambience amb = GetAmbience ();
-			IMember member = null;
-			INode node = (INode)unit;
 			
-			if (type != null && type.ClassType != ClassType.Delegate) {
-				member = type.GetMemberAt (loc.Line, loc.Column);
+			var loc = Document.Editor.Caret.Location;
+			
+			var result = new List<PathEntry> ();
+			var amb = GetAmbience ();
+			var type = unit.GetInnermostTypeDefinition (loc.Line, loc.Column) ?? unit.TopLevelTypeDefinitions.FirstOrDefault ();
+			var curType = type;
+			object lastTag = unit;
+			while (curType != null) {
+				var markup = amb.GetString ((IEntity)curType, OutputFlags.IncludeGenerics | OutputFlags.IncludeParameters | OutputFlags.ReformatDelegates | OutputFlags.IncludeMarkup);
+				result.Insert (0, new PathEntry (ImageService.GetPixbuf (type.GetStockIcon (), Gtk.IconSize.Menu), curType.IsObsolete () ? "<s>" + markup + "</s>" : markup) { Tag = lastTag });
+				lastTag = curType;
+				curType = curType.DeclaringTypeDefinition;
 			}
 			
-			if (null != member) {
-				node = member;
-			} else if (null != type) {
-				node = type;
-			}
-			
-			while (node != null) {
-				PathEntry entry;
-				if (node is ICompilationUnit) {
-					if (!Document.ParsedDocument.UserRegions.Any ())
-						break;
-					FoldingRegion reg = Document.ParsedDocument.UserRegions.Where (r => r.Region.Contains (loc.Line, loc.Column)).LastOrDefault ();
-					if (reg == null) {
-						entry = new PathEntry (GettextCatalog.GetString ("No region"));
-					} else {
-						entry = new PathEntry (CompilationUnitDataProvider.Pixbuf, reg.Name);
-					}
-					entry.Position = EntryPosition.Right;
-				} else {
-					entry = new PathEntry (ImageService.GetPixbuf (((IMember)node).StockIcon, Gtk.IconSize.Menu), amb.GetString ((IMember)node, OutputFlags.IncludeGenerics | OutputFlags.IncludeParameters | OutputFlags.ReformatDelegates));
-				}
-				entry.Tag = node;
-				result.Insert (0, entry);
-				node = node.Parent;
+			var member = type.Members.FirstOrDefault (m => m.Region.IsInside (loc.Line, loc.Column));
+			if (member != null) {
+				var ctx = Document.ParsedDocument.ParsedFile.GetTypeResolveContext (Document.Compilation, member.Region.Begin);;
+				var markup = amb.GetString (member.CreateResolved (ctx), OutputFlags.IncludeGenerics | OutputFlags.IncludeParameters | OutputFlags.ReformatDelegates | OutputFlags.IncludeMarkup);
+				result.Add (new PathEntry (ImageService.GetPixbuf (member.GetStockIcon (), Gtk.IconSize.Menu), member.IsObsolete () ? "<s>" + markup + "</s>" : markup) { Tag = lastTag });
 			}
 			
 			PathEntry noSelection = null;
 			if (type == null) {
-				noSelection = new PathEntry (GettextCatalog.GetString ("No selection")) { Tag = new CustomNode (Document.CompilationUnit) };
-			} else if (member == null && type.ClassType != ClassType.Delegate) 
-				noSelection = new PathEntry (GettextCatalog.GetString ("No selection")) { Tag = new CustomNode (type) };
-			if (noSelection != null) {
+				noSelection = new PathEntry (GettextCatalog.GetString ("No selection")) { Tag = unit };
+			} else if (member == null && type.Kind != TypeKind.Delegate) 
+				noSelection = new PathEntry (GettextCatalog.GetString ("No selection")) { Tag = type };
+			if (noSelection != null) 
 				result.Add (noSelection);
-			}
-			
 			var prev = CurrentPath;
 			CurrentPath = result.ToArray ();
-			OnPathChanged (new DocumentPathChangedEventArgs (prev));
+			OnPathChanged (new DocumentPathChangedEventArgs (prev));*/
 		}
 		
 		public override void Initialize ()
@@ -738,21 +727,14 @@ namespace CBinding
 			Document.DocumentParsed += delegate { UpdatePath (null, null); };
 		}
 		
-		// Yoinked from C# binding
-		class CustomNode : MonoDevelop.Projects.Dom.AbstractNode
-		{
-			public CustomNode (INode parent)
-			{
-				this.Parent = parent;
-			}
-		}
-		
 		/// <summary>
 		/// Move the completion trigger offset to the beginning of the current token
 		/// </summary>
 		protected virtual int ResetTriggerOffset (CodeCompletionContext completionContext)
 		{
 			int i = completionContext.TriggerOffset;
+			if (i >= Editor.Length)
+				return 0;
 			int accumulator = 0;
 			
 			for (;
@@ -761,7 +743,7 @@ namespace CBinding
 			completionContext.TriggerOffset = i-1;
 			return accumulator+1;
 		}// ResetTriggerOffset
-		
+
 		[CommandHandler (MonoDevelop.Refactoring.RefactoryCommands.GotoDeclaration)]
 		public void GotoDeclaration ()
 		{
@@ -793,15 +775,18 @@ namespace CBinding
 		{
 			int lineOffset = location.Column-1;
 			string line = Editor.GetLineText (location.Line);
-			int first = line.LastIndexOfAny (allowedChars, lineOffset)+1,
-			    last = line.IndexOfAny (allowedChars, lineOffset);
+			if (line.Length == 0)
+				return "";
+			if (lineOffset >= line.Length)
+				lineOffset = line.Length - 1;
+			int first = line.LastIndexOfAny (allowedChars, lineOffset) + 1;
+			int last = line.IndexOfAny (allowedChars, lineOffset);
 			if (last < 0) last = line.Length - 1;
 			string token = string.Empty;
 			    
 			if (first >= 0 && first < last && last < line.Length) {
 				token = line.Substring (first, last-first);
 			}
-			
 			return token.Trim ();
 		}
 	}
