@@ -33,6 +33,7 @@
 using System;
 using System.IO;
 using System.Text;
+using System.Linq;
 using System.Text.RegularExpressions;
 using System.Collections.Generic;
 using System.Threading;
@@ -58,6 +59,7 @@ namespace CBinding.Parser
 		private static TagDatabaseManager instance;
 		private Queue<ProjectFilePair> parsingJobs = new Queue<ProjectFilePair> ();
 		private Thread parsingThread;
+		private CTagsManager ctags;
 		
 		public event ClassPadEventHandler FileUpdated;
 		
@@ -78,15 +80,20 @@ namespace CBinding.Parser
 			}
 		}
 		
-		bool DepsInstalled {
+		public bool DepsInstalled {
 			get {
 				if (!checkedCtagsInstalled) {
 					checkedCtagsInstalled = true;
-					if (IdeApp.Services.PlatformService.Name.Equals ("OSX")) {
-						return false;
-					}
+					
 					try {
-						Runtime.ProcessService.StartProcess ("ctags", "--version", null, null).WaitForOutput ();
+						var output = new StringWriter ();
+						Runtime.ProcessService.StartProcess (CTagsManager.CTagsExecutable, "--version", null, output, null, null).WaitForExit ();
+						if (Platform.IsMac && !output.ToString ().StartsWith ("Exuberant", StringComparison.Ordinal)) {
+							System.Console.WriteLine ("Fallback to OSX ctags");
+							ctags = new BsdCTagsManager ();
+						} else {
+							ctags = new ExuberantCTagsManager ();
+						}
 					} catch {
 						LoggingService.LogWarning ("Cannot update C/C++ tags database because exuberant ctags is not installed.");
 						return false;
@@ -101,7 +108,7 @@ namespace CBinding.Parser
 						ctagsInstalled = true;
 					}
 				}
-				return ctagsInstalled;
+				return ctagsInstalled && ctags != null;
 			}
 			set {
 				//don't assume that the caller is correct :-)
@@ -114,17 +121,31 @@ namespace CBinding.Parser
 		
 		private string[] Headers (Project project, string filename, bool with_system)
 		{
-			string option = (with_system ? "-M" : "-MM");
+			List<string> headers = new List<string> ();
+			CProject cproject = project as CProject;
+			if (cproject == null){ return headers.ToArray(); }
+			
 			StringBuilder output = new StringBuilder ();
+			StringBuilder option = new StringBuilder ("-M");
+			if (!with_system) {
+				option.Append("M");
+			}
+			
+			option.Append (" -MG ");
+			foreach (Package package in cproject.Packages) {
+				package.ParsePackage ();
+				option.AppendFormat ("{0} ", string.Join(" ", package.CFlags.ToArray ()));
+			}
 			
 			ProcessWrapper p = null;
 			try {
-				p = Runtime.ProcessService.StartProcess ("gcc", option + " -MG " + filename.Replace(@"\ ", " ").Replace(" ", @"\ "), null, null);
+				p = Runtime.ProcessService.StartProcess ("gcc", option.ToString () + filename.Replace(@"\ ", " ").Replace(" ", @"\ "), null, null);
 				p.WaitForOutput ();
 
-				// Skip first two lines (.o & .c* files) - WARNING, sometimes this is compacted to 1 line... we need a better way of handling this.
-				if(p.StandardOutput.ReadLine () == null) return new string[0]; // object file
-				if(p.StandardOutput.ReadLine () == null) return new string[0]; // compile file
+				// Doing the below completely breaks header parsing
+				// // Skip first two lines (.o & .c* files) - WARNING, sometimes this is compacted to 1 line... we need a better way of handling this.
+				// if(p.StandardOutput.ReadLine () == null) return new string[0]; // object file
+				// if(p.StandardOutput.ReadLine () == null) return new string[0]; // compile file
 
 				string line;
 				while ((line = p.StandardOutput.ReadLine ()) != null)
@@ -137,8 +158,7 @@ namespace CBinding.Parser
 				if(p != null)
 					p.Dispose();
 			}
-
-			List<string> headers = new List<string> ();
+			
 			MatchCollection files = Regex.Matches(output.ToString().Replace(@" \", String.Empty), @" (?<file>([^ \\]|(\\ ))+)", RegexOptions.IgnoreCase);
 
 			foreach (Match match in files) {
@@ -174,10 +194,12 @@ namespace CBinding.Parser
 				fullpath = findFileInPath (filename, project.BaseDirectory);
 				if (string.Empty != fullpath) return fullpath;
 
-				// Check project's additional configuration includes
-				foreach (string p in conf.Includes) {
-					fullpath = findFileInPath (filename, p);
-					if (string.Empty != fullpath) return fullpath;
+				if (conf != null) {
+					// Check project's additional configuration includes
+					foreach (string p in conf.Includes) {
+						fullpath = findFileInPath (filename, p);
+						if (string.Empty != fullpath) return fullpath;
+					}
 				}
 			}
 			
@@ -217,97 +239,39 @@ namespace CBinding.Parser
 			return string.Empty;
 		}
 		
-		private void UpdateSystemTags (Project project, string filename, string[] includedFiles)
-		{
-			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
-			List<FileInformation> files;
-			
-			if (!info.IncludedFiles.ContainsKey (filename)) {
-				files = new List<FileInformation> ();
-				info.IncludedFiles.Add (filename, files);
-			} else {
-				files = info.IncludedFiles[filename];
-			}
-			
-			foreach (string includedFile in includedFiles) {
-				bool contains = false;
-				
-				foreach (FileInformation fi in files) {
-					if (fi.FileName == includedFile) {
-						contains = true;
-					}
-				}
-				
-				if (!contains) {
-					FileInformation newFileInfo = new FileInformation (project, includedFile);
-					files.Add (newFileInfo);
-					FillFileInformation (newFileInfo);
-				}
-				
-				contains = false;
-			}
-		}
-		
-		private void FillFileInformation (FileInformation fileInfo)
+		private void UpdateSystemTags (Project project, string filename, IEnumerable<string> includedFiles)
 		{
 			if (!DepsInstalled)
 				return;
+			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
+			List<FileInformation> files;
 			
-			string confdir = PropertyService.ConfigPath;
-			string tagFileName = Path.GetFileName (fileInfo.FileName) + ".tag";
-			string tagdir = Path.Combine (confdir, "system-tags");
-			string tagFullFileName = Path.Combine (tagdir, tagFileName);
-			string ctags_kinds = "--C++-kinds=+px";
-			
-			if (PropertyService.Get<bool> ("CBinding.ParseLocalVariables", true))
-				ctags_kinds += "l";
-			
-			string ctags_options = ctags_kinds + " --fields=+aStisk-fz --language-force=C++ --excmd=number --line-directives=yes -f '" + tagFullFileName + "' '" + fileInfo.FileName + "'";
-			
-			if (!Directory.Exists (tagdir))
-				Directory.CreateDirectory (tagdir);
-			
-			if (!File.Exists (tagFullFileName) || File.GetLastWriteTimeUtc (tagFullFileName) < File.GetLastWriteTimeUtc (fileInfo.FileName)) {
-				ProcessWrapper p = null;
-				System.IO.StringWriter output = null;
-				try {
-					output = new System.IO.StringWriter ();
+			lock (info) {
+				if (!info.IncludedFiles.ContainsKey (filename)) {
+					files = new List<FileInformation> ();
+					info.IncludedFiles.Add (filename, files);
+				} else {
+					files = info.IncludedFiles[filename];
+				}
+				
+				foreach (string includedFile in includedFiles) {
+					bool contains = false;
 					
-					p = Runtime.ProcessService.StartProcess ("ctags", ctags_options, null, null, output, null);
-					p.WaitForOutput (10000);
-					if (p.ExitCode != 0 || !File.Exists (tagFullFileName)) {
-						LoggingService.LogError ("Ctags did not successfully populate the tags database '{0}' within ten seconds.\nOutput: {1}", tagFullFileName, output.ToString ());
-						return;
+					foreach (FileInformation fi in files) {
+						if (fi.FileName == includedFile) {
+							contains = true;
+						}
 					}
-				} catch (Exception ex) {
-					throw new IOException ("Could not create tags database (You must have exuberant ctags installed).", ex);
-				} finally {
-					if (output != null)
-						output.Dispose ();
-					if (p != null)
-						p.Dispose ();
+					
+					if (!contains) {
+						FileInformation newFileInfo = new FileInformation (project, includedFile);
+						files.Add (newFileInfo);
+						ctags.FillFileInformation (newFileInfo);
+					}
+					
+					contains = false;
 				}
 			}
-			
-			string ctags_output;
-			string tagEntry;
-			
-			using (StreamReader reader = new StreamReader (tagFullFileName)) {
-				ctags_output = reader.ReadToEnd ();
-			}
-			
-			using (StringReader reader = new StringReader (ctags_output)) {
-				while ((tagEntry = reader.ReadLine ()) != null) {
-					if (tagEntry.StartsWith ("!_")) continue;
-					
-					Tag tag = ParseTag (tagEntry);
-					
-					if (tag != null)
-						AddInfo (fileInfo, tag, ctags_output);
-				}
-			}
-			
-			fileInfo.IsFilled = true;
 		}
 		
 		private void ParsingThread ()
@@ -320,17 +284,21 @@ namespace CBinding.Parser
 						p = parsingJobs.Dequeue ();
 					}
 					
-					DoUpdateFileTags (p.Project, p.File);
+					string[] headers = Headers (p.Project, p.File, false);
+					ctags.DoUpdateFileTags (p.Project, p.File, headers);
+					OnFileUpdated (new ClassPadEventArgs (p.Project));
+					
+					if (PropertyService.Get<bool> ("CBinding.ParseSystemTags", true))
+						UpdateSystemTags (p.Project, p.File, Headers (p.Project, p.File, true).Except (headers));
+					
+					if (cache.Count > cache_size)
+						cache.Clear ();
 				}
 			} catch (Exception ex) {
 				LoggingService.LogError ("Unhandled error updating parser database. Disabling C/C++ parsing.", ex);
 				DepsInstalled = false;
 				return;
 			}
-//			catch {
-//				LoggingService.LogError("Unexpected error while updating parser database. Disabling C/C++ parsing.");
-//				DepsInstalled = false;
-//			}
 		}
 		
 		public void UpdateFileTags (Project project, string filename)
@@ -347,229 +315,13 @@ namespace CBinding.Parser
 			
 			if (parsingThread == null || !parsingThread.IsAlive) {
 				parsingThread = new Thread (ParsingThread);
+				parsingThread.Name = "Tag database parser";
 				parsingThread.IsBackground = true;
 				parsingThread.Priority = ThreadPriority.Lowest;
 				parsingThread.Start();
 			}
 		}
 		
-		private void DoUpdateFileTags (Project project, string filename)
-		{
-			if (!DepsInstalled)
-				return;
-
-			string[] headers = Headers (project, filename, false);
-			string ctags_kinds = "--C++-kinds=+px";
-			
-			if (PropertyService.Get<bool> ("CBinding.ParseLocalVariables", true))
-				ctags_kinds += "+l";
-			
-			// Maybe we should only ask for locals for 'local' files? (not external #includes?)
-			string ctags_options = ctags_kinds + " --fields=+aStisk-fz --language-force=C++ --excmd=number --line-directives=yes -f - '" + filename + "'";// " + string.Join (" ", headers);
-			
-			string[] system_headers = diff (Headers (project, filename, true), headers);
-			
-			string ctags_output = string.Empty;
-
-			ProcessWrapper p = null;
-			System.IO.StringWriter output = null, error = null;
-			try {
-				output = new System.IO.StringWriter ();
-				error = new System.IO.StringWriter ();
-				
-				p = Runtime.ProcessService.StartProcess ("ctags", ctags_options, project.BaseDirectory, output, error, null);
-				p.WaitForOutput (10000);
-				if (p.ExitCode != 0) {
-					LoggingService.LogError ("Ctags did not successfully populate the tags database from '{0}' within ten seconds.\nError output: {1}", filename, error.ToString ());
-					return;
-				}
-				ctags_output = output.ToString ();
-			} catch (Exception ex) {
-				throw new IOException ("Could not create tags database (You must have exuberant ctags installed).", ex);
-			} finally {
-				if (output != null)
-					output.Dispose ();
-				if (error != null)
-					error.Dispose ();
-				if (p != null)
-					p.Dispose ();
-			}
-			
-			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
-			string tagEntry;
-
-			using (StringReader reader = new StringReader (ctags_output)) {
-				while ((tagEntry = reader.ReadLine ()) != null) {
-					if (tagEntry.StartsWith ("!_")) continue;
-					
-					Tag tag = ParseTag (tagEntry);
-					
-					if (tag != null)
-						AddInfo (info, tag, ctags_output);
-				}
-			}			
-			
-			if (FileUpdated != null)
-				FileUpdated (new ClassPadEventArgs (project));
-			
-			if (PropertyService.Get<bool> ("CBinding.ParseSystemTags", true))
-				UpdateSystemTags (project, filename, system_headers);
-			
-			if (cache.Count > cache_size)
-				cache.Clear ();
-		}
-		
-		private void AddInfo (FileInformation info, Tag tag, string ctags_output)
-		{
-			switch (tag.Kind)
-			{
-			case TagKind.Class:
-				Class c = new Class (tag, info.Project, ctags_output);
-				if (!info.Classes.Contains (c))
-					info.Classes.Add (c);
-				break;
-			case TagKind.Enumeration:
-				Enumeration e = new Enumeration (tag, info.Project, ctags_output);
-				if (!info.Enumerations.Contains (e))
-					info.Enumerations.Add (e);
-				break;
-			case TagKind.Enumerator:
-				Enumerator en= new Enumerator (tag, info.Project, ctags_output);
-				if (!info.Enumerators.Contains (en))
-					info.Enumerators.Add (en);
-				break;
-			case TagKind.ExternalVariable:
-				break;
-			case TagKind.Function:
-				Function f = new Function (tag, info.Project, ctags_output);
-				if (!info.Functions.Contains (f))
-					info.Functions.Add (f);
-				break;
-			case TagKind.Local:
-				Local lo = new Local (tag, info.Project, ctags_output);
-				if(!info.Locals.Contains (lo))
-					info.Locals.Add (lo);
-				break;
-			case TagKind.Macro:
-				Macro m = new Macro (tag, info.Project);
-				if (!info.Macros.Contains (m))
-					info.Macros.Add (m);
-				break;
-			case TagKind.Member:
-				Member me = new Member (tag, info.Project, ctags_output);
-				if (!info.Members.Contains (me))
-					info.Members.Add (me);
-				break;
-			case TagKind.Namespace:
-				Namespace n = new Namespace (tag, info.Project, ctags_output);
-				if (!info.Namespaces.Contains (n))
-					info.Namespaces.Add (n);
-				break;
-			case TagKind.Prototype:
-				Function fu = new Function (tag, info.Project, ctags_output);
-				if (!info.Functions.Contains (fu))
-					info.Functions.Add (fu);
-				break;
-			case TagKind.Structure:
-				Structure s = new Structure (tag, info.Project, ctags_output);
-				if (!info.Structures.Contains (s))
-					info.Structures.Add (s);
-				break;
-			case TagKind.Typedef:
-				Typedef t = new Typedef (tag, info.Project, ctags_output);
-				if (!info.Typedefs.Contains (t))
-					info.Typedefs.Add (t);
-				break;
-			case TagKind.Union:
-				Union u = new Union (tag, info.Project, ctags_output);
-				if (!info.Unions.Contains (u))
-					info.Unions.Add (u);
-				break;
-			case TagKind.Variable:
-				Variable v = new Variable (tag, info.Project);
-				if (!info.Variables.Contains (v))
-					info.Variables.Add (v);
-				break;
-			default:
-				break;
-			}
-		}
-		
-		private Tag ParseTag (string tagEntry)
-		{
-			string file;
-			UInt64 line;
-			string name;
-			string tagField;
-			TagKind kind;
-			AccessModifier access = AccessModifier.Public;
-			string _class = null;
-			string _namespace = null;
-			string _struct = null;
-			string _union = null;
-			string _enum = null;
-			string signature = null;
-			
-			int i1 = tagEntry.IndexOf ('\t');
-			name = tagEntry.Substring (0, tagEntry.IndexOf ('\t'));
-			
-			i1 += 1;
-			int i2 = tagEntry.IndexOf ('\t', i1);
-			file = tagEntry.Substring (i1, i2 - i1);
-			
-			i1 = i2 + 1;
-			i2 = tagEntry.IndexOf (";\"", i1);
-			line = UInt64.Parse(tagEntry.Substring (i1, i2 - i1));
-
-			i1 = i2 + 3;	
-			kind = (TagKind)tagEntry[i1];
-			
-			i1 += 2;
-			tagField = (tagEntry.Length > i1? tagField = tagEntry.Substring(i1) : String.Empty);
-			
-			string[] fields = tagField.Split ('\t');
-			int index;
-			
-			foreach (string field in fields) {
-				index = field.IndexOf (':');
-				
-				// TODO: Support friend modifier
-				if (index > 0) {
-					string key = field.Substring (0, index);
-					string val = field.Substring (index + 1);
-					
-					switch (key) {
-						case "access":
-							try {
-								access = (AccessModifier)System.Enum.Parse (typeof(AccessModifier), val, true);
-							} catch (ArgumentException) {
-							}
-							break;
-						case "class":
-							_class = val;
-							break;
-						case "namespace":
-							_namespace = val;
-							break;
-						case "struct":
-							_struct = val;
-							break;
-						case "union":
-							_union = val;
-							break;
-						case "enum":
-							_enum = val;
-							break;
-						case "signature":
-							signature = val;
-							break;
-					}
-				}
-			}
-			
-			return new Tag (name, file, line, kind, access, _class, _namespace, _struct, _union, _enum, signature);
-		}
-		
 		Tag BinarySearch (string[] ctags_lines, TagKind kind, string name)
 		{
 			int low;
@@ -602,7 +354,7 @@ namespace CBinding.Parser
 					bool eof = false;
 					
 					while (true) {
-						Tag tag = ParseTag (entry);
+						Tag tag = ctags.ParseTag (entry);
 						
 						if (tag == null)
 							return null;
@@ -682,17 +434,28 @@ namespace CBinding.Parser
 			}
 		}
 		
-		private static string[] diff (string[] a1, string[] a2)
+		/// <summary>
+		/// Remove a file's parse information from the database.
+		/// </summary>
+		/// <param name="project">
+		/// A <see cref="Project"/>: The project to which the file belongs.
+		/// </param>
+		/// <param name="filename">
+		/// A <see cref="System.String"/>: The file.
+		/// </param>
+		public void RemoveFileInfo(Project project, string filename)
 		{
-			List<string> res = new List<string> ();
-			List<string> right = new List<string> (a2);
-			
-			foreach (string s in a1) {
-				if (!right.Contains (s))
-					res.Add (s);
-			}
-			
-			return res.ToArray ();
+			ProjectInformation info = ProjectInformationManager.Instance.Get (project);
+			lock (info){ info.RemoveFileInfo(filename); }
+			OnFileUpdated(new ClassPadEventArgs (project));
+		}
+		
+		/// <summary>
+		/// Wrapper method for the FileUpdated event.
+		/// </summary>
+		void OnFileUpdated (ClassPadEventArgs args)
+		{
+			if (null != FileUpdated){ FileUpdated(args); }
 		}
 		
 		private class ProjectFilePair
