@@ -32,37 +32,35 @@
 using System;
 using System.IO;
 using System.Xml;
-using System.Collections;
 using System.Collections.Generic;
 using System.ComponentModel;
-using System.CodeDom.Compiler;
 
 using Mono.Addins;
 
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.Projects;
 using MonoDevelop.Core.Serialization;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Deployment;
 using MonoDevelop.Deployment.Linux;
-
 using CBinding.Parser;
+using MonoDevelop.Ide;
 
 namespace CBinding
 {
 	public enum Language {
 		C,
-		CPP
+		CPP,
+		OBJC,
+		OBJCPP
 	}
 	
 	public enum CProjectCommands {
 		AddPackage,
 		UpdateClassPad,
-		SwapSourceHeader,
-		ShowPackageDetails
+		ShowPackageDetails,
+		GotoDeclaration,
 	}
 	
 	[DataInclude(typeof(CProjectConfiguration))]
@@ -85,7 +83,7 @@ namespace CBinding
 		/// <summary>
 		/// Extensions for C/C++ source files
 		/// </summary>
-		public static string[] SourceExtensions = { ".C", ".CC", ".CPP", ".CXX" };
+		public static string[] SourceExtensions = { ".C", ".CC", ".CPP", ".CXX", ".M", ".MM" };
 		
 		/// <summary>
 		/// Extensions for C/C++ header files
@@ -95,8 +93,9 @@ namespace CBinding
 		private void Init ()
 		{
 			packages.Project = this;
-			
-			IdeApp.Workspace.ItemAddedToSolution += OnEntryAddedToCombine;
+
+			if (IdeApp.IsInitialized)
+				IdeApp.Workspace.ItemAddedToSolution += OnEntryAddedToCombine;
 		}
 		
 		public CProject ()
@@ -123,6 +122,12 @@ namespace CBinding
 			case "CPP":
 				this.language = Language.CPP;
 				break;
+			case "Objective C":
+				this.language = Language.OBJC;
+				break;
+			case "Objective C++":
+				this.language = Language.OBJCPP;
+				break;
 			}
 			
 			Compiler = null; // use default compiler depending on language
@@ -154,6 +159,10 @@ namespace CBinding
 						    typeof(CBinding.CompileTarget),
 						    projectOptions.Attributes["Target"].InnerText);
 					}
+					if (projectOptions.GetAttribute ("ExternalConsole") == "True") {
+						c.ExternalConsole = true;
+						c.PauseConsoleOutput = true;
+					}
 					if (projectOptions.Attributes["PauseConsoleOutput"] != null) {
 						c.PauseConsoleOutput = bool.Parse (
 							projectOptions.Attributes["PauseConsoleOutput"].InnerText);
@@ -167,13 +176,14 @@ namespace CBinding
 				}
 			}			
 		}
-		
-		public override string ProjectType {
-			get { return "Native"; }
+
+		public override IEnumerable<string> GetProjectTypes ()
+		{
+			yield return "Native";
 		}
-		
+
 		public override string[] SupportedLanguages {
-			get { return new string[] { "C", "CPP" }; }
+			get { return new string[] { "C", "CPP", "Objective C", "Objective C++" }; }
 		}
 		
 		public CompileTarget CompileTarget {
@@ -192,19 +202,19 @@ namespace CBinding
 			}
 		}
 		
-		public override IEnumerable<SolutionItem> GetReferencedItems (string configuration)
+		public override IEnumerable<SolutionItem> GetReferencedItems (ConfigurationSelector configuration)
 		{
-			List<string> project_names = new List<string> ();
-			
+			foreach (var p in base.GetReferencedItems (configuration))
+				yield return p;
+
+			if (ParentSolution == null)
+				yield break;
+
 			foreach (Package p in Packages) {
 				if (p.IsProject && p.Name != Name) {
-					project_names.Add (p.Name);
-				}
-			}
-			
-			foreach (SolutionItem e in ParentFolder.Items) {
-				if (e is CProject && project_names.Contains (e.Name)) {
-					yield return e;
+					var cp = ParentSolution.FindProjectByName (p.Name) as CProject;
+					if (cp != null)
+						yield return cp;
 				}
 			}
 		}
@@ -217,7 +227,7 @@ namespace CBinding
 		/// <summary>
 		/// Ths pkg-config package is for internal MonoDevelop use only, it is not deployed.
 		/// </summary>
-		public void WriteMDPkgPackage (string configuration)
+		public void WriteMDPkgPackage (ConfigurationSelector configuration)
 		{
 			string pkgfile = Path.Combine (BaseDirectory, Name + ".md.pc");
 			
@@ -239,9 +249,11 @@ namespace CBinding
 				writer.WriteLine ("Name: {0}", Name);
 				writer.WriteLine ("Description: {0}", Description);
 				writer.WriteLine ("Version: {0}", Version);
-				writer.WriteLine ("Libs: -L{0} -l{1}", config.OutputDirectory, config.Output);
+				writer.WriteLine ("Libs: -L\"{0}\" -l{1}", config.OutputDirectory, config.Output.StartsWith ("lib", StringComparison.OrdinalIgnoreCase)?
+				                                                                                                config.Output.Substring (3):
+				                                                                                                config.Output);
 //				writer.WriteLine ("Cflags: -I{0}", BaseDirectory);
-				writer.WriteLine ("Cflags: -I{0}", string.Join (" -I", headerDirectories.ToArray ()));
+				writer.WriteLine ("Cflags: -I\"{0}\"", string.Join ("\" -I\"", headerDirectories.ToArray ()));
 			}
 			
 			// If this project compiles into a shared object we need to
@@ -280,7 +292,9 @@ namespace CBinding
 				// TODO: How should I get this?
 				writer.WriteLine ("Conflicts: {0}", string.Empty);
 				writer.Write ("Libs: -L${libdir} ");
-				writer.WriteLine ("-l{0}", config.Output);
+				writer.WriteLine ("-l{0}", config.Output.StartsWith ("lib", StringComparison.OrdinalIgnoreCase)?
+				                                                            config.Output.Substring (3):
+				                                                            config.Output);
 				writer.Write ("Cflags: -I${includedir}/");
 				writer.WriteLine ("{0} {1}", Name, Compiler.GetDefineFlags (project, config));
 			}
@@ -288,7 +302,7 @@ namespace CBinding
 			return pkgfile;
 		}
 		
-		protected override BuildResult DoBuild (IProgressMonitor monitor, string configuration)
+		protected override BuildResult DoBuild (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
 			CProjectConfiguration pc = (CProjectConfiguration) GetConfiguration (configuration);
 			pc.SourceDirectory = BaseDirectory;
@@ -298,20 +312,27 @@ namespace CBinding
 				pc,
 			    monitor);
 		}
+		
+		protected virtual ExecutionCommand CreateExecutionCommand (CProjectConfiguration conf)
+		{
+			string app = Path.Combine (conf.OutputDirectory, conf.Output);
+			NativeExecutionCommand cmd = new NativeExecutionCommand (app);
+			cmd.Arguments = conf.CommandLineParameters;
+			cmd.WorkingDirectory = Path.GetFullPath (conf.OutputDirectory);
+			cmd.EnvironmentVariables = conf.EnvironmentVariables;
+			return cmd;
+		}
 
-		protected override bool OnGetCanExecute (ExecutionContext context, string solutionConfiguration)
+		protected override bool OnGetCanExecute (ExecutionContext context, ConfigurationSelector solutionConfiguration)
 		{
 			CProjectConfiguration conf = (CProjectConfiguration) GetConfiguration (solutionConfiguration);
-			string cmd = Path.Combine (conf.OutputDirectory, conf.Output);
+			ExecutionCommand cmd = CreateExecutionCommand (conf);
 			return (target == CBinding.CompileTarget.Bin) && context.ExecutionHandler.CanExecute (cmd);
 		}
 
-		protected override void DoExecute (IProgressMonitor monitor, ExecutionContext context, string configuration)
+		protected override void DoExecute (IProgressMonitor monitor, ExecutionContext context, ConfigurationSelector configuration)
 		{
 			CProjectConfiguration conf = (CProjectConfiguration) GetConfiguration (configuration);
-			string command = conf.Output;
-			string args = conf.CommandLineParameters;
-			string dir = Path.GetFullPath (conf.OutputDirectory);
 			bool pause = conf.PauseConsoleOutput;
 			IConsole console;
 			
@@ -330,30 +351,31 @@ namespace CBinding
 			AggregatedOperationMonitor operationMonitor = new AggregatedOperationMonitor (monitor);
 			
 			try {
-				string cmd = Path.Combine (dir, command);
+				ExecutionCommand cmd = CreateExecutionCommand (conf);
 				if (!context.ExecutionHandler.CanExecute (cmd)) {
-					monitor.ReportError ("Cannot execute \"" + command + "\". The selected execution mode is not supported for C projects.", null);
+					monitor.ReportError ("Cannot execute \"" + conf.Output + "\". The selected execution mode is not supported for C projects.", null);
 					return;
 				}
 				
-				IProcessAsyncOperation op = context.ExecutionHandler.Execute (cmd, args, dir, conf.EnvironmentVariables, console);
+				IProcessAsyncOperation op = context.ExecutionHandler.Execute (cmd, console);
 				
 				operationMonitor.AddOperation (op);
 				op.WaitForCompleted ();
 				
 				monitor.Log.WriteLine ("The operation exited with code: {0}", op.ExitCode);
 			} catch (Exception ex) {
-				monitor.ReportError ("Cannot execute \"" + command + "\"", ex);
+				LoggingService.LogError (string.Format ("Cannot execute \"{0}\"", conf.Output), ex);
+				monitor.ReportError ("Cannot execute \"" + conf.Output + "\"", ex);
 			} finally {			
 				operationMonitor.Dispose ();			
 				console.Dispose ();
 			}
 		}
 		
-		protected override string OnGetOutputFileName (string configuration)
+		public override FilePath GetOutputFileName (ConfigurationSelector configuration)
 		{
 			CProjectConfiguration conf = (CProjectConfiguration) GetConfiguration (configuration);
-			return Path.Combine (conf.OutputDirectory, conf.CompiledOutputName);
+			return conf.OutputDirectory.Combine (conf.CompiledOutputName);
 		}
 		
 		public override SolutionItemConfiguration CreateConfiguration (string name)
@@ -404,26 +426,38 @@ namespace CBinding
 			}
 		}
 		
-		protected override void OnFileAddedToProject (ProjectFileEventArgs e)
+		protected override void OnFileAddedToProject (ProjectFileEventArgs args)
 		{
-			base.OnFileAddedToProject (e);
+			base.OnFileAddedToProject (args);
 			
-			if (!Loading && !IsCompileable (e.ProjectFile.Name) &&
-			    e.ProjectFile.BuildAction == BuildAction.Compile) {
-				e.ProjectFile.BuildAction = BuildAction.None;
+			foreach (ProjectFileEventInfo e in args) {
+				if (!Loading && !IsCompileable (e.ProjectFile.Name) &&
+				    e.ProjectFile.BuildAction == BuildAction.Compile) {
+					e.ProjectFile.BuildAction = BuildAction.None;
+				}
+				
+				if (e.ProjectFile.BuildAction == BuildAction.Compile)
+					TagDatabaseManager.Instance.UpdateFileTags (this, e.ProjectFile.Name);
 			}
-			
-			if (e.ProjectFile.BuildAction == BuildAction.Compile)
-				TagDatabaseManager.Instance.UpdateFileTags (this, e.ProjectFile.Name);
 		}
 		
 		protected override void OnFileChangedInProject (ProjectFileEventArgs e)
 		{
 			base.OnFileChangedInProject (e);
 			
-			TagDatabaseManager.Instance.UpdateFileTags (this, e.ProjectFile.Name);
+			foreach (ProjectFileEventInfo fe in e)
+				TagDatabaseManager.Instance.UpdateFileTags (this, fe.ProjectFile.Name);
 		}
 		
+		protected override void OnFileRemovedFromProject (ProjectFileEventArgs e)
+		{
+			base.OnFileRemovedFromProject (e);
+			
+			foreach (ProjectFileEventInfo fe in e)
+				TagDatabaseManager.Instance.RemoveFileInfo (this, fe.ProjectFile.Name);
+		}
+
+		
 		private static void OnEntryAddedToCombine (object sender, SolutionItemEventArgs e)
 		{
 			CProject p = e.SolutionItem as CProject;
@@ -445,11 +479,11 @@ namespace CBinding
 			PackageAddedToProject (this, new ProjectPackageEventArgs (this, package));
 		}
 
-		public DeployFileCollection GetDeployFiles (string solutionConfiguration)
+		public DeployFileCollection GetDeployFiles (ConfigurationSelector configuration)
 		{
 			DeployFileCollection deployFiles = new DeployFileCollection ();
 			
-			CProjectConfiguration conf = (CProjectConfiguration) GetActiveConfiguration (solutionConfiguration);
+			CProjectConfiguration conf = (CProjectConfiguration) GetConfiguration (configuration);
 			CompileTarget target = conf.CompileTarget;
 			
 			// Headers and resources
@@ -458,12 +492,12 @@ namespace CBinding
 					string targetDirectory =
 						(IsHeaderFile (f.Name) ? TargetDirectory.Include : TargetDirectory.ProgramFiles);
 					
-					deployFiles.Add (new DeployFile (this, f.FilePath, f.RelativePath, targetDirectory));
+					deployFiles.Add (new DeployFile (this, f.FilePath, f.ProjectVirtualPath, targetDirectory));
 				}
 			}
 			
 			// Output
-			string output = GetOutputFileName (solutionConfiguration);		
+			string output = GetOutputFileName (configuration);		
 			if (!string.IsNullOrEmpty (output)) {
 				string targetDirectory = string.Empty;
 				
