@@ -77,8 +77,9 @@ namespace MonoDevelop.CSharp.Highlighting
 		CancellationTokenSource src;
 
 		public bool SemanticHighlightingEnabled {
-			get;
-			set;
+			get {
+				return true;
+			}
 		}
 
 		internal class StyledTreeSegment : TreeSegment
@@ -281,6 +282,41 @@ namespace MonoDevelop.CSharp.Highlighting
 				}
 				tree.AddStyle (startOffset, endOffset, color);
 			}
+
+			public override void VisitSimpleType (SimpleType simpleType)
+			{
+				var identifierToken = simpleType.IdentifierToken;
+				VisitChildrenUntil(simpleType, identifierToken);
+				var resolveResult = resolver.Resolve (simpleType, cancellationToken);
+				if (resolveResult.Type.Namespace == "System") {
+					switch (resolveResult.Type.Name) {
+					case "nfloat":
+					case "nint":
+					case "nuint":
+						Colorize(identifierToken, "Keyword(Type)");
+						break;
+					default:
+						Colorize (identifierToken, resolveResult);
+						break;
+					}
+				} else {
+					Colorize (identifierToken, resolveResult);
+				}
+				VisitChildrenAfter(simpleType, identifierToken);
+			}
+
+			public override void VisitIdentifierExpression (IdentifierExpression identifierExpression)
+			{
+				var identifier = identifierExpression.IdentifierToken;
+				VisitChildrenUntil(identifierExpression, identifier);
+				if (isInAccessorContainingValueParameter && identifierExpression.Identifier == "value") {
+					Colorize(identifier, valueKeywordColor);
+				} else {
+					var resolveResult = resolver.Resolve (identifierExpression, cancellationToken);
+					Colorize (identifier, resolveResult);
+				}
+				VisitChildrenAfter(identifierExpression, identifier);
+			}
 		}
 
 		class QuickTaskVisitor : DepthFirstAstVisitor
@@ -442,8 +478,6 @@ namespace MonoDevelop.CSharp.Highlighting
 		{
 			this.guiDocument = document;
 			guiDocument.DocumentParsed += HandleDocumentParsed;
-			SemanticHighlightingEnabled = PropertyService.Get ("EnableSemanticHighlighting", true);
-			PropertyService.PropertyChanged += HandlePropertyChanged;
 			if (guiDocument.ParsedDocument != null)
 				HandleDocumentParsed (this, EventArgs.Empty);
 
@@ -513,119 +547,21 @@ namespace MonoDevelop.CSharp.Highlighting
 			if (src != null)
 				src.Cancel ();
 			guiDocument.DocumentParsed -= HandleDocumentParsed;
-			PropertyService.PropertyChanged -= HandlePropertyChanged;
 		}
 
 		#endregion
 
-		void HandlePropertyChanged (object sender, PropertyChangedEventArgs e)
-		{
-			if (e.Key == "EnableSemanticHighlighting")
-				SemanticHighlightingEnabled = PropertyService.Get ("EnableSemanticHighlighting", true);
-		}
 
-		public override SpanParser CreateSpanParser (DocumentLine line, CloneableStack<Span> spanStack)
-		{
-			return new CSharpSpanParser (this, spanStack ?? line.StartSpan.Clone ());
-		}
+//		public override SpanParser CreateSpanParser (DocumentLine line, CloneableStack<Span> spanStack)
+//		{
+//			return new CSharpSpanParser (this, spanStack ?? line.StartSpan.Clone ());
+//		}
 		
 		public override ChunkParser CreateChunkParser (SpanParser spanParser, ColorScheme style, DocumentLine line)
 		{
 			return new CSharpChunkParser (this, spanParser, style, line);
 		}
-		
-		abstract class AbstractBlockSpan : Span
-		{
-			public bool IsValid {
-				get;
-				private set;
-			}
-			
-			bool disabled;
-			
-			public bool Disabled {
-				get { return disabled; }
-				set { disabled = value; SetColor (); }
-			}
-			
-			
-			public AbstractBlockSpan (bool isValid)
-			{
-				IsValid = isValid;
-				SetColor ();
-				StopAtEol = false;
-			}
-			
-			protected void SetColor ()
-			{
-				TagColor = "Preprocessor";
-				if (disabled || !IsValid) {
-					Color = "Excluded Code";
-					Rule = "PreProcessorComment";
-				} else {
-					Color = "Plain Text";
-					Rule = "<root>";
-				}
-			}
-		}
 
-		class DefineSpan : Span
-		{
-			string define;
-
-			public string Define { 
-				get { 
-					return define;
-				}
-			}
-
-			public DefineSpan (string define)
-			{
-				this.define = define;
-				StopAtEol = false;
-				Color = "Plain Text";
-				Rule = "<root>";
-			}
-		}
-
-		class IfBlockSpan : AbstractBlockSpan
-		{
-			public IfBlockSpan (bool isValid) : base (isValid)
-			{
-			}
-			
-			public override string ToString ()
-			{
-				return string.Format("[IfBlockSpan: IsValid={0}, Disabled={3}, Color={1}, Rule={2}]", IsValid, Color, Rule, Disabled);
-			}
-		}
-		
-		class ElseIfBlockSpan : AbstractBlockSpan
-		{
-			public ElseIfBlockSpan (bool isValid) : base (isValid)
-			{
-				base.Begin = new Regex ("#elif");
-			}
-			
-			public override string ToString ()
-			{
-				return string.Format("[ElseIfBlockSpan: IsValid={0}, Disabled={3}, Color={1}, Rule={2}]", IsValid, Color, Rule, Disabled);
-			}
-		}
-		
-		class ElseBlockSpan : AbstractBlockSpan
-		{
-			public ElseBlockSpan (bool isValid) : base (isValid)
-			{
-				base.Begin = new Regex ("#else");
-			}
-			
-			public override string ToString ()
-			{
-				return string.Format("[ElseBlockSpan: IsValid={0}, Disabled={3}, Color={1}, Rule={2}]", IsValid, Color, Rule, Disabled);
-			}
-		}
-		
 		protected class CSharpChunkParser : ChunkParser, IResolveVisitorNavigator
 		{
 
@@ -719,349 +655,6 @@ namespace MonoDevelop.CSharp.Highlighting
 			}
 		}
 		
-		protected class CSharpSpanParser : SpanParser
-		{
-			CSharpSyntaxMode CSharpSyntaxMode {
-				get {
-					return (CSharpSyntaxMode)mode;
-				}
-			}
-			class ConditinalExpressionEvaluator : DepthFirstAstVisitor<object, object>
-			{
-				HashSet<string> symbols;
-
-				MonoDevelop.Projects.Project GetProject (TextDocument doc)
-				{
-					// There is no reference between document & higher level infrastructure,
-					// therefore it's a bit tricky to find the right project.
-					
-					MonoDevelop.Projects.Project project = null;
-					var view = doc.Annotation<MonoDevelop.SourceEditor.SourceEditorView> ();
-					if (view != null)
-						project = view.Project;
-					
-					if (project == null) {
-						var ideDocument = IdeApp.Workbench.GetDocument (doc.FileName);
-						if (ideDocument != null)
-							project = ideDocument.Project;
-					}
-					
-					if (project == null)
-						project = IdeApp.Workspace.GetProjectContainingFile (doc.FileName);
-					
-					return project;
-				}
-
-
-
-				public ConditinalExpressionEvaluator (TextDocument doc, IEnumerable<string> symbols)
-				{
-					this.symbols = new HashSet<string> (symbols);
-					var project = GetProject (doc);
-					
-					if (project == null) {
-						var ideDocument = IdeApp.Workbench.GetDocument (doc.FileName);
-						if (ideDocument != null)
-							project = ideDocument.Project;
-					}
-					
-					if (project == null)
-						project = IdeApp.Workspace.GetProjectContainingFile (doc.FileName);
-					
-					if (project != null) {
-						foreach (var ss in GetDefinedSymbols (project)) {
-							if (!symbols.Contains (ss))
-								this.symbols.Add (ss);
-						}
-					}
-/*					var parsedDocument = TypeSystemService.ParseFile (document.ProjectContent, doc.FileName, doc.MimeType, doc.Text);
-					if (parsedDocument == null)
-						parsedDocument = TypeSystemService.ParseFile (dom, doc.FileName ?? "a.cs", delegate { return doc.Text; });
-					if (parsedDocument != null) {
-						foreach (PreProcessorDefine define in parsedDocument.Defines) {
-							symbols.Add (define.Define);
-						}
-						
-					}*/
-				}
-				
-				public override object VisitIdentifierExpression (IdentifierExpression identifierExpression, object data)
-				{
-					return symbols.Contains (identifierExpression.Identifier);
-				}
-				
-				public override object VisitUnaryOperatorExpression (UnaryOperatorExpression unaryOperatorExpression, object data)
-				{
-					bool result = (bool)(unaryOperatorExpression.Expression.AcceptVisitor (this, data) ?? false);
-					if (unaryOperatorExpression.Operator ==  UnaryOperatorType.Not)
-						return !result;
-					return result;
-				}
-
-
-				public override object VisitPrimitiveExpression (PrimitiveExpression primitiveExpression, object data)
-				{
-					if (primitiveExpression.Value is bool)
-						return primitiveExpression.Value;
-					return false;
-				}
-
-				public override object VisitBinaryOperatorExpression (BinaryOperatorExpression binaryOperatorExpression, object data)
-				{
-					bool left = (bool)(binaryOperatorExpression.Left.AcceptVisitor (this, data) ?? false);
-					bool right = (bool)(binaryOperatorExpression.Right.AcceptVisitor (this, data) ?? false);
-					switch (binaryOperatorExpression.Operator) {
-					case BinaryOperatorType.InEquality:
-						return left != right;
-					case BinaryOperatorType.Equality:
-						return left == right;
-					case BinaryOperatorType.ConditionalOr:
-						return left || right;
-					case BinaryOperatorType.ConditionalAnd:
-						return left && right;
-					}
-					
-					Console.WriteLine ("Unknown operator:" + binaryOperatorExpression.Operator);
-					return left;
-				}
-
-				public override object VisitParenthesizedExpression (ParenthesizedExpression parenthesizedExpression, object data)
-				{
-					return parenthesizedExpression.Expression.AcceptVisitor (this, data);
-				}
-
-			}
-			
-			void ScanPreProcessorElse (ref int i)
-			{
-				if (!spanStack.Any (s => s is IfBlockSpan || s is ElseIfBlockSpan)) {
-					base.ScanSpan (ref i);
-					return;
-				}
-				bool previousResult = false;
-				foreach (Span span in spanStack) {
-					if (span is IfBlockSpan) {
-						previousResult = ((IfBlockSpan)span).IsValid;
-					}
-					if (span is ElseIfBlockSpan) {
-						previousResult |= ((ElseIfBlockSpan)span).IsValid;
-					}
-				}
-				//					LineSegment line = doc.GetLineByOffset (i);
-				//					int length = line.Offset + line.EditableLength - i;
-				while (spanStack.Count > 0 && !(CurSpan is IfBlockSpan || CurSpan is ElseIfBlockSpan)) {
-					spanStack.Pop ();
-				}
-				var ifBlock = CurSpan as IfBlockSpan;
-				var elseIfBlock = CurSpan as ElseIfBlockSpan;
-				var elseBlockSpan = new ElseBlockSpan (!previousResult);
-				if (ifBlock != null) {
-					elseBlockSpan.Disabled = ifBlock.Disabled;
-				} else if (elseIfBlock != null) {
-					elseBlockSpan.Disabled = elseIfBlock.Disabled;
-				}
-				FoundSpanBegin (elseBlockSpan, i, "#else".Length);
-				i += "#else".Length;
-					
-				// put pre processor eol span on stack, so that '#elif' gets the correct highlight
-				Span preprocessorSpan = CreatePreprocessorSpan ();
-				FoundSpanBegin (preprocessorSpan, i, 0);
-			}
-			IEnumerable<string> Defines {
-				get {
-					if (SpanStack == null)
-						yield break;
-					foreach (var span in SpanStack) {
-						if (span is DefineSpan) {
-							var define = ((DefineSpan)span).Define;
-							if (define != null)
-								yield return define;
-						}
-					}
-				}
-			}
-			void ScanPreProcessorIf (int textOffset, ref int i)
-			{
-				var end = CurText.Length;
-				int idx = 0;
-				while ((idx = CurText.IndexOf ('/', idx)) >= 0 && idx + 1 < CurText.Length) {
-					var next = CurText [idx + 1];
-					if (next == '/') {
-						end = idx - 1;
-						break;
-					}
-					idx++;
-				}
-
-				int length = end - textOffset;
-				string parameter = CurText.Substring (textOffset + 3, length - 3);
-				AstNode expr = new CSharpParser ().ParseExpression (parameter);
-				bool result = false;
-				if (expr != null && !expr.IsNull) {
-					object o = expr.AcceptVisitor (new ConditinalExpressionEvaluator (doc, Defines), null);
-					if (o is bool)
-						result = (bool)o;
-				}
-					
-				foreach (Span span in spanStack) {
-					if (span is ElseBlockSpan) {
-						result &= ((ElseBlockSpan)span).IsValid;
-						break;
-					}
-					if (span is IfBlockSpan) {
-						result &= ((IfBlockSpan)span).IsValid;
-						break;
-					}
-					if (span is ElseIfBlockSpan) {
-						result &= ((ElseIfBlockSpan)span).IsValid;
-						break;
-					}
-				}
-					
-				var ifBlockSpan = new IfBlockSpan (result);
-					
-				foreach (Span span in spanStack) {
-					if (span is AbstractBlockSpan) {
-						var parentBlock = (AbstractBlockSpan)span;
-						ifBlockSpan.Disabled = parentBlock.Disabled || !parentBlock.IsValid;
-						break;
-					}
-				}
-					
-				FoundSpanBegin (ifBlockSpan, i, length);
-				i += length - 1;
-			}
-
-			void ScanPreProcessorElseIf (ref int i)
-			{
-				DocumentLine line = doc.GetLineByOffset (i);
-				int length = line.Offset + line.Length - i;
-				string parameter = doc.GetTextAt (i + 5, length - 5);
-				AstNode expr= new CSharpParser ().ParseExpression (parameter);
-				bool result;
-				if (expr != null && !expr.IsNull) {
-					var visitResult = expr.AcceptVisitor (new ConditinalExpressionEvaluator (doc, Defines), null);
-					result = visitResult != null ? (bool)visitResult : false;
-				} else {
-					result = false;
-				}
-					
-				IfBlockSpan containingIf = null;
-				if (result) {
-					bool previousResult = false;
-					foreach (Span span in spanStack) {
-						if (span is IfBlockSpan) {
-							containingIf = (IfBlockSpan)span;
-							previousResult = ((IfBlockSpan)span).IsValid;
-							break;
-						}
-						if (span is ElseIfBlockSpan) {
-							previousResult |= ((ElseIfBlockSpan)span).IsValid;
-						}
-					}
-						
-					result = !previousResult;
-				}
-					
-				var elseIfBlockSpan = new ElseIfBlockSpan (result);
-				if (containingIf != null)
-					elseIfBlockSpan.Disabled = containingIf.Disabled;
-					
-				FoundSpanBegin (elseIfBlockSpan, i, 0);
-					
-				// put pre processor eol span on stack, so that '#elif' gets the correct highlight
-				var preprocessorSpan = CreatePreprocessorSpan ();
-				FoundSpanBegin (preprocessorSpan, i, 0);
-			}
-
-			protected override bool ScanSpan (ref int i)
-			{
-				if (CSharpSyntaxMode.DisableConditionalHighlighting) {
-					return base.ScanSpan (ref i);
-				}
-				int textOffset = i - StartOffset;
-
-				if (textOffset < CurText.Length && CurRule.Name != "Comment" && CurRule.Name != "String" && CurRule.Name != "VerbatimString" && CurText [textOffset] == '#' && IsFirstNonWsChar (textOffset)) {
-
-					if (CurText.IsAt (textOffset, "#define") && (spanStack == null || !spanStack.Any (span => span is IfBlockSpan && !((IfBlockSpan)span).IsValid))) {
-						int length = CurText.Length - textOffset;
-						string parameter = CurText.Substring (textOffset + "#define".Length, length - "#define".Length).Trim ();
-						var defineSpan = new DefineSpan (parameter);
-						FoundSpanBegin (defineSpan, i, 0);
-					}
-	
-					if (CurText.IsAt (textOffset, "#else")) {
-						ScanPreProcessorElse (ref i);
-						return true;
-					}
-	
-					if (CurText.IsAt (textOffset, "#if")) {
-						ScanPreProcessorIf (textOffset, ref i);
-						return true;
-					}
-	
-					if (CurText.IsAt (textOffset, "#elif") && spanStack != null && spanStack.Any (span => span is IfBlockSpan)) {
-						ScanPreProcessorElseIf (ref i);
-						return true;
-					}
-				}
-
-				return base.ScanSpan (ref i);
-			}
-			
-			public static Span CreatePreprocessorSpan ()
-			{
-				var result = new Span ();
-				result.TagColor = "Preprocessor";
-				result.Color = "Preprocessor";
-				result.Rule = "String";
-				result.StopAtEol = true;
-				return result;
-			}
-			
-			void PopCurrentIfBlock ()
-			{
-				while (spanStack.Count > 0 && (spanStack.Peek () is IfBlockSpan || spanStack.Peek () is ElseIfBlockSpan || spanStack.Peek () is ElseBlockSpan)) {
-					var poppedSpan = PopSpan ();
-					if (poppedSpan is IfBlockSpan)
-						break;
-				}
-			}
-			
-			protected override bool ScanSpanEnd (Span cur, ref int i)
-			{
-				if (cur is IfBlockSpan || cur is ElseIfBlockSpan || cur is ElseBlockSpan) {
-					int textOffset = i - StartOffset;
-					bool end = CurText.IsAt (textOffset, "#endif");
-					if (end) {
-						FoundSpanEnd (cur, i, 6); // put empty end tag in
-						
-						// if we're in a complex span stack pop it up to the if block
-						if (spanStack.Count > 0) {
-							var prev = spanStack.Peek ();
-							
-							if ((cur is ElseIfBlockSpan || cur is ElseBlockSpan) && (prev is ElseIfBlockSpan || prev is IfBlockSpan))
-								PopCurrentIfBlock ();
-						}
-					}
-					return end;
-				}
-				return base.ScanSpanEnd (cur, ref i);
-			}
-			
-	//		Span preprocessorSpan;
-	//		Rule preprocessorRule;
-			
-			public CSharpSpanParser (CSharpSyntaxMode mode, CloneableStack<Span> spanStack) : base (mode, spanStack)
-			{
-//				foreach (Span span in mode.Spans) {
-//					if (span.Rule == "text.preprocessor") {
-//						preprocessorSpan = span;
-//						preprocessorRule = GetRule (span);
-//					}
-//				}
-			}
-		}
 
 		#region IQuickTaskProvider implementation
 		public event EventHandler TasksUpdated;
