@@ -31,12 +31,12 @@ using System.Collections.Generic;
 using MonoDevelop.Projects;
 using MonoDevelop.CSharp.Project;
 using MonoDevelop.Ide.Tasks;
-using Mono.CSharp;
 using System.Linq;
 using ICSharpCode.NRefactory;
 using MonoDevelop.CSharp.Refactoring.CodeActions;
 using MonoDevelop.Core;
 using ICSharpCode.NRefactory.CSharp.Resolver;
+using ICSharpCode.NRefactory.MonoCSharp;
 
 namespace MonoDevelop.CSharp.Parser
 {
@@ -84,7 +84,20 @@ namespace MonoDevelop.CSharp.Parser
 			result.ParsedFile = pf;
 			result.Add (GetSemanticTags (unit));
 
-			result.CreateRefactoringContext = (doc, token) => MDRefactoringContext.Create (doc, doc.Editor.Caret.Location, token).Result;
+			result.CreateRefactoringContext = delegate (MonoDevelop.Ide.Gui.Document doc, System.Threading.CancellationToken token) {
+				var task = MDRefactoringContext.Create (doc, doc.Editor.Caret.Location, token);
+				try {
+					task.Wait (5000, token);
+				} catch (AggregateException ae) {
+					ae.Flatten ().Handle (aex => aex is OperationCanceledException);
+					return null;
+				} catch (OperationCanceledException) {
+					return null;
+				}
+				if (!task.IsCompleted)
+					return null;
+				return task.Result;
+			};
 			result.CreateRefactoringContextWithEditor = (data, resolver, token) => new MDRefactoringContext ((DotNetProject)project, data, result, (CSharpAstResolver)resolver, TextLocation.Empty, token);
 
 			if (storeAst) {
@@ -367,7 +380,7 @@ namespace MonoDevelop.CSharp.Parser
 			case Tokenizer.PreprocessorDirective.Endregion:
 				if (regions.Count > 0) {
 					var start = regions.Pop ();
-					DomRegion dr = new DomRegion (start.Line, loc.Column, directive.EndLine, directive.EndCol);
+					DomRegion dr = new DomRegion (start.Line, start.Col, directive.EndLine, directive.EndCol);
 					result.Add (new FoldingRegion (start.Arg, dr, FoldType.UserRegion, true));
 				}
 				break;
@@ -418,8 +431,6 @@ namespace MonoDevelop.CSharp.Parser
 		internal static Version ConvertLanguageVersion (LangVersion ver)
 		{
 			switch (ver) {
-			case LangVersion.Default:
-				return new Version (5, 0, 0, 0);
 			case LangVersion.ISO_1:
 				return new Version (1, 0, 0, 0);
 			case LangVersion.ISO_2:
@@ -430,8 +441,12 @@ namespace MonoDevelop.CSharp.Parser
 				return new Version (4, 0, 0, 0);
 			case LangVersion.Version5:
 				return new Version (5, 0, 0, 0);
+			case LangVersion.Version6:
+				return new Version (6, 0, 0, 0);
+			case LangVersion.Default:
+				break;
 			}
-			return new Version (5, 0, 0, 0);;
+			return new Version (6, 0, 0, 0);;
 		}
 	}
 	
