@@ -460,43 +460,16 @@ namespace MonoDevelop.CSharp
 			return cancelRequested ? 0 : exitCode;
 		}
 		
-		// Snatched from our codedom code, with some changes to make it compatible with csc
-		// (the line+column group is optional is csc)
-		static Regex regexError = new Regex (@"^(\s*(?<file>.+[^)])(\((?<line>\d*)(,(?<column>\d*[\+]*))?\))?:\s+)*(?<level>\w+)\s+(?<number>..\d+):\s*(?<message>.*)", RegexOptions.Compiled | RegexOptions.ExplicitCapture);
 		static BuildError CreateErrorFromString (string basePath, string error_string)
 		{
 			// When IncludeDebugInformation is true, prevents the debug symbols stats from braeking this.
-			if (error_string.StartsWith ("WROTE SYMFILE") ||
-			    error_string.StartsWith ("OffsetTable") ||
-			    error_string.StartsWith ("Compilation succeeded") ||
-			    error_string.StartsWith ("Compilation failed"))
+			if (error_string.StartsWith ("WROTE SYMFILE", StringComparison.Ordinal) ||
+			    error_string.StartsWith ("OffsetTable", StringComparison.Ordinal) ||
+			    error_string.StartsWith ("Compilation succeeded", StringComparison.Ordinal) ||
+			    error_string.StartsWith ("Compilation failed", StringComparison.Ordinal))
 				return null;
 			
-			Match match = regexError.Match(error_string);
-			if (!match.Success) 
-				return null;
-			
-			BuildError error = new BuildError ();
-			FilePath filename = match.Result ("${file}");
-			if (filename.IsNullOrEmpty) {
-				filename = FilePath.Empty;
-			} else if (!filename.IsAbsolute && basePath != null) {
-				filename = filename.ToAbsolute (basePath);
-			}
-			error.FileName = filename;
-
-			
-			string line = match.Result ("${line}");
-			error.Line = !string.IsNullOrEmpty (line) ? Int32.Parse (line) : 0;
-			
-			string col = match.Result ("${column}");
-			if (!string.IsNullOrEmpty (col)) 
-				error.Column = col == "255+" ? -1 : Int32.Parse (col);
-			
-			error.IsWarning   = match.Result ("${level}") == "warning";
-			error.ErrorNumber = match.Result ("${number}");
-			error.ErrorText   = match.Result ("${message}");
-			return error;
+			return BuildError.FromMSBuildErrorFormat (error_string);
 		}
 	}
 }
