@@ -30,12 +30,10 @@ using System.Xml;
 
 using MonoDevelop.Projects;
 using MonoDevelop.Core;
-using MonoDevelop.Projects.Dom.Parser;
-using MonoDevelop.Projects.CodeGeneration;
 
 namespace ILAsmBinding
 {
-	public class ILAsmLanguageBinding : IDotNetLanguageBinding
+	class ILAsmLanguageBinding : IDotNetLanguageBinding
 	{
 		public string Language {
 			get {
@@ -49,9 +47,9 @@ namespace ILAsmBinding
 			}
 		}
 		
-		public bool IsSourceCodeFile (string fileName)
+		public bool IsSourceCodeFile (FilePath fileName)
 		{
-			return string.Compare (Path.GetExtension (fileName), ".il", true) == 0;
+			return String.Compare (Path.GetExtension (fileName), ".il", StringComparison.OrdinalIgnoreCase) == 0;
 		}
 		
 		public BuildResult Compile (ProjectItemCollection projectItems, DotNetProjectConfiguration configuration, ConfigurationSelector configSelector, IProgressMonitor monitor)
@@ -78,29 +76,19 @@ namespace ILAsmBinding
 			return null;
 		}
 		
-		public string GetFileName (string baseName)
+		public FilePath GetFileName (FilePath baseName)
 		{
 			return baseName + ".il";
 		}
-			
-		public IParser Parser {
-			get { 
-				return null; 
-			}
-		}
-		
-		public IRefactorer Refactorer {
-			get { 
-				return null; 
-			}
-		}
 		
 		public ClrVersion[] GetSupportedClrVersions ()
 		{
-			return new ClrVersion[] { 
+			return new [] { 
 				ClrVersion.Net_1_1, 
-				ClrVersion.Net_2_0, 
-				ClrVersion.Clr_2_1
+				ClrVersion.Net_2_0,
+				ClrVersion.Clr_2_1,
+				ClrVersion.Net_4_0,
+				ClrVersion.Net_4_5,
 			};
 		}
 	}
