@@ -25,15 +25,15 @@
 // THE SOFTWARE.
 
 using System;
-using System.Drawing;
 using System.Linq;
 using System.Collections.Generic;
-using MonoMac.Foundation;
-using MonoMac.AppKit;
+
+using Foundation;
+using AppKit;
+using CoreGraphics;
 
 using MonoDevelop.Core;
 using MonoDevelop.Ide;
-using MonoDevelop.Ide.Extensions;
 using MonoDevelop.Components.Extensions;
 using MonoDevelop.MacInterop;
 using MonoDevelop.Components;
@@ -47,7 +47,7 @@ namespace MonoDevelop.MacIntegration
 			using (var alert = new NSAlert ()) {
 				alert.Window.Title = data.Title ?? BrandingService.ApplicationName;
 
-				bool stockIcon = false;
+				bool stockIcon;
 				if (data.Message.Icon == MonoDevelop.Ide.Gui.Stock.Error || data.Message.Icon == Gtk.Stock.DialogError) {
 					alert.AlertStyle = NSAlertStyle.Critical;
 					stockIcon = true;
@@ -58,16 +58,10 @@ namespace MonoDevelop.MacIntegration
 					alert.AlertStyle = NSAlertStyle.Informational;
 					stockIcon = data.Message.Icon == MonoDevelop.Ide.Gui.Stock.Information;
 				}
-				
-				//FIXME: use correct size so we don't get horrible scaling?
+
 				if (!stockIcon && !string.IsNullOrEmpty (data.Message.Icon)) {
-					var pix = ImageService.GetIcon (data.Message.Icon, Gtk.IconSize.Dialog).ToPixbuf();
-					byte[] buf = pix.SaveToBuffer ("tiff");
-					unsafe {
-						fixed (byte* b = buf) {
-							alert.Icon = new NSImage (NSData.FromBytes ((IntPtr)b, (uint)buf.Length));
-						}
-					}
+					var img = ImageService.GetIcon (data.Message.Icon, Gtk.IconSize.Dialog);
+					alert.Icon = img.ToNSImage ();
 				} else {
 					//for some reason the NSAlert doesn't pick up the app icon by default
 					alert.Icon = NSApplication.SharedApplication.ApplicationIconImage;
@@ -90,6 +84,7 @@ namespace MonoDevelop.MacIntegration
 					}
 				}
 				
+				var wrappers = new List<AlertButtonWrapper> (buttons.Count);
 				foreach (var button in buttons) {
 					var label = button.Label;
 					if (button.IsStockButton)
@@ -100,7 +95,11 @@ namespace MonoDevelop.MacIntegration
 					if (button == AlertButton.CloseWithoutSave)
 						label = GettextCatalog.GetString ("Don't Save");
 
-					alert.AddButton (label);
+					var nsbutton = alert.AddButton (label);
+					var wrapperButton = new AlertButtonWrapper (nsbutton, data.Message, button, alert);
+					wrappers.Add (wrapperButton);
+					nsbutton.Target = wrapperButton;
+					nsbutton.Action = new ObjCRuntime.Selector ("buttonActivatedAction");
 				}
 				
 				
@@ -111,7 +110,7 @@ namespace MonoDevelop.MacIntegration
 					
 					for (int i = data.Options.Count - 1; i >= 0; i--) {
 						var option = data.Options[i];
-						var button = new NSButton () {
+						var button = new NSButton {
 							Title = option.Text,
 							Tag = i,
 							State = option.Value? NSCellStateValue.On : NSCellStateValue.Off,
@@ -134,8 +133,8 @@ namespace MonoDevelop.MacIntegration
 				
 				// Hack up a slightly wider than normal alert dialog. I don't know how to do this in a nicer way
 				// as the min size constraints are apparently ignored.
-				var frame = ((NSPanel) alert.Window).Frame;
-				((NSPanel) alert.Window).SetFrame (new RectangleF (frame.X, frame.Y, Math.Max (frame.Width, 600), frame.Height), true);
+				var frame = alert.Window.Frame;
+				alert.Window.SetFrame (new CGRect (frame.X, frame.Y, NMath.Max (frame.Width, 600), frame.Height), true);
 				alert.Layout ();
 				
 				bool completed = false;
@@ -150,10 +149,10 @@ namespace MonoDevelop.MacIntegration
 				}
 				
 				if (!data.Message.CancellationToken.IsCancellationRequested) {
-					int result = alert.RunModal () - (int)NSAlertButtonReturn.First;
+					var result = (int)alert.RunModal () - (long)(int)NSAlertButtonReturn.First;
 					completed = true;
 					if (result >= 0 && result < buttons.Count) {
-						data.ResultButton = buttons [result];
+						data.ResultButton = buttons [(int)result];
 					} else {
 						data.ResultButton = null;
 					}
@@ -165,7 +164,7 @@ namespace MonoDevelop.MacIntegration
 				
 				if (optionButtons != null) {
 					foreach (var button in optionButtons) {
-						var option = data.Options[button.Tag];
+						var option = data.Options[(int)button.Tag];
 						data.Message.SetOptionValue (option.Id, button.State != 0);
 					}
 				}
@@ -179,4 +178,29 @@ namespace MonoDevelop.MacIntegration
 			return true;
 		}
 	}
+
+	class AlertButtonWrapper : NSObject
+	{
+		readonly NSButton nsbutton;
+		readonly MessageDescription message;
+		readonly AlertButton alertButton;
+		readonly ObjCRuntime.Selector oldAction;
+		readonly NSAlert alert;
+		public AlertButtonWrapper (NSButton nsbutton, MessageDescription message, AlertButton alertButton, NSAlert alert)
+		{
+			this.nsbutton = nsbutton;
+			this.message = message;
+			this.alertButton = alertButton;
+			this.alert = alert;
+			oldAction = nsbutton.Action;
+		}
+
+		[Export ("buttonActivatedAction")]
+		void ButtonActivatedAction ()
+		{
+			bool close = message.NotifyClicked (alertButton);
+			if (close)
+				nsbutton.SendAction (oldAction, alert);
+		}
+	}
 }
