@@ -309,6 +309,7 @@ namespace MonoDevelop.MacInterop
 		WindowRef = 2003398244, // 'wind'
 		Char = 1413830740, // 'TEXT'
 		UInt32 = 1835100014, // 'magn'
+		UTF8Text = 1970562616, // 'utf8'
 		UnicodeText = 1970567284, // 'utxt'
 		AEList = 1818850164, // 'list'
 		WildCard = 707406378, // '****'
@@ -378,6 +379,7 @@ namespace MonoDevelop.MacInterop
 		RotateWindowsBackward = 1919906914, // 'rotb'
 		RotateFloatingWindowsForward = 1920231031, // 'rtfw'
 		RotateFloatingWindowsBackward = 1920231010, // 'rtfb'
+		QuitAndCloseAllWindows = 1903520631, // 'qukw'
 		
 		//created automatically -- used for inserting before/after the default window list
 		WindowListSeparator = 2003592310, // 'wldv'
@@ -568,7 +570,7 @@ namespace MonoDevelop.MacInterop
 		int value;
 		
 		public int Value {
-			get { return Value; }
+			get { return value; }
 		}
 		
 		public OSType (int value)
