@@ -14,10 +14,10 @@
 // distribute, sublicense, and/or sell copies of the Software, and to
 // permit persons to whom the Software is furnished to do so, subject to
 // the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be
 // included in all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -28,21 +28,20 @@
 //
 
 using System;
-using System.Drawing;
 using System.Collections.Generic;
-using System.Diagnostics;
 using System.IO;
 using System.Linq;
 using System.Text.RegularExpressions;
 
-using MonoMac.AppKit;
-using MonoMac.Foundation;
+using AppKit;
+using Foundation;
+using CoreGraphics;
 
 using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.Instrumentation;
 using MonoDevelop.Components.Commands;
-using MonoDevelop.Ide; 
+using MonoDevelop.Ide;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Commands;
 using MonoDevelop.Ide.Desktop;
@@ -56,19 +55,15 @@ namespace MonoDevelop.MacIntegration
 {
 	public class MacPlatformService : PlatformService
 	{
-		const string monoDownloadUrl = "http://www.go-mono.com/mono-downloads/download.html";
+		const string monoDownloadUrl = "http://www.mono-project.com/download/";
 
 		TimerCounter timer = InstrumentationService.CreateTimerCounter ("Mac Platform Initialization", "Platform Service");
 		TimerCounter mimeTimer = InstrumentationService.CreateTimerCounter ("Mac Mime Database", "Platform Service");
 
 		static bool initedGlobal;
 		bool setupFail, initedApp;
-		
+
 		Lazy<Dictionary<string, string>> mimemap;
-		
-		//this is a BCD value of the form "xxyz", where x = major, y = minor, z = bugfix
-		//eg. 0x1071 = 10.7.1
-		int systemVersion;
 
 		public MacPlatformService ()
 		{
@@ -80,9 +75,12 @@ namespace MonoDevelop.MacIntegration
 			initedGlobal = true;
 
 			timer.BeginTiming ();
-			
-			systemVersion = Carbon.Gestalt ("sysv");
-			
+
+			var dir = Path.GetDirectoryName (typeof(MacPlatformService).Assembly.Location);
+
+			if (ObjCRuntime.Dlfcn.dlopen (Path.Combine (dir, "libxammac.dylib"), 0) == IntPtr.Zero)
+				LoggingService.LogFatalError ("Unable to load libxammac");
+
 			mimemap = new Lazy<Dictionary<string, string>> (LoadMimeMapAsync);
 
 			//make sure the menu app name is correct even when running Mono 2.6 preview, or not running from the .app
@@ -100,12 +98,12 @@ namespace MonoDevelop.MacIntegration
 			//: || Gtk.Global.CheckVersion (major, minor, micro + 1) == null
 			//
 			if (Gtk.Global.CheckVersion (major, minor, micro) != null) {
-				
+
 				LoggingService.LogFatalError (
 					"GTK+ version is incompatible with required version {0}.{1}.{2}.",
 					major, minor, micro
 				);
-				
+
 				var downloadButton = new AlertButton ("Download Mono Framework", null);
 				if (downloadButton == MessageService.GenericAlert (
 					Stock.Error,
@@ -119,7 +117,7 @@ namespace MonoDevelop.MacIntegration
 				{
 					OpenUrl (monoDownloadUrl);
 				}
-				
+
 				Environment.Exit (1);
 			}
 		}
@@ -127,8 +125,8 @@ namespace MonoDevelop.MacIntegration
 		public override Xwt.Toolkit LoadNativeToolkit ()
 		{
 			var path = Path.GetDirectoryName (GetType ().Assembly.Location);
-			System.Reflection.Assembly.LoadFrom (Path.Combine (path, "Xwt.Mac.dll"));
-			var loaded = Xwt.Toolkit.Load (Xwt.ToolkitType.Cocoa);
+			System.Reflection.Assembly.LoadFrom (Path.Combine (path, "Xwt.XamMac.dll"));
+			var loaded = Xwt.Toolkit.Load (Xwt.ToolkitType.XamMac);
 
 			// We require Xwt.Mac to initialize MonoMac before we can execute any code using MonoMac
 			timer.Trace ("Installing App Event Handlers");
@@ -141,7 +139,7 @@ namespace MonoDevelop.MacIntegration
 		protected override string OnGetMimeTypeForUri (string uri)
 		{
 			var ext = Path.GetExtension (uri);
-			string mime = null;
+			string mime;
 			if (ext != null && mimemap.Value.TryGetValue (ext, out mime))
 				return mime;
 			return null;
@@ -151,14 +149,14 @@ namespace MonoDevelop.MacIntegration
 		{
 			OpenUrl (url);
 		}
-		
+
 		internal static void OpenUrl (string url)
 		{
 			Gtk.Application.Invoke (delegate {
 				NSWorkspace.SharedWorkspace.OpenUrl (new NSUrl (url));
 			});
 		}
-		
+
 		public override void OpenFile (string filename)
 		{
 			Gtk.Application.Invoke (delegate {
@@ -169,11 +167,11 @@ namespace MonoDevelop.MacIntegration
 		public override string DefaultMonospaceFont {
 			get { return "Menlo 12"; }
 		}
-		
+
 		public override string Name {
 			get { return "OSX"; }
 		}
-		
+
 		Dictionary<string, string> LoadMimeMapAsync ()
 		{
 			var map = new Dictionary<string, string> ();
@@ -182,7 +180,7 @@ namespace MonoDevelop.MacIntegration
 				LoggingService.LogError ("Apache mime database is missing");
 				return map;
 			}
-			
+
 			mimeTimer.BeginTiming ();
 			try {
 				using (var file = File.OpenRead ("/etc/apache2/mime.types")) {
@@ -192,7 +190,7 @@ namespace MonoDevelop.MacIntegration
 						while ((line = reader.ReadLine ()) != null) {
 							Match m = mime.Match (line);
 							if (m.Success)
-								map ["." + m.Groups [2].Captures [0].Value] = m.Groups [1].Captures [0].Value; 
+								map ["." + m.Groups [2].Captures [0].Value] = m.Groups [1].Captures [0].Value;
 						}
 					}
 				}
@@ -203,35 +201,6 @@ namespace MonoDevelop.MacIntegration
 			return map;
 		}
 
-		public override bool ShowContextMenu (CommandManager commandManager, Gtk.Widget widget, double x, double y, CommandEntrySet entrySet, object initialCommandTarget = null)
-		{
-			Gtk.Application.Invoke (delegate {
-				// Explicitly release the grab because the menu is shown on the mouse position, and the widget doesn't get the mouse release event
-				Gdk.Pointer.Ungrab (Gtk.Global.CurrentEventTime);
-				var menu = new MDMenu (commandManager, entrySet, CommandSource.ContextMenu, initialCommandTarget);
-				var nsview = MacInterop.GtkQuartz.GetView (widget);
-				var toplevel = widget.Toplevel as Gtk.Window;
-				int trans_x, trans_y;
-				widget.TranslateCoordinates (toplevel, (int)x, (int)y, out trans_x, out trans_y);
-
-				// Window coordinates in gtk are the same for cocoa, with the exception of the Y coordinate, that has to be flipped.
-				var pt = new PointF ((float)trans_x, (float)trans_y);
-				int w,h;
-				toplevel.GetSize (out w, out h);
-				pt.Y = h - pt.Y;
-
-				var tmp_event = NSEvent.MouseEvent (NSEventType.LeftMouseDown,
-					pt,
-					0, 0,
-					MacInterop.GtkQuartz.GetWindow (toplevel).WindowNumber,
-					null, 0, 0, 0);
-
-				NSMenu.PopUpContextMenu (menu, tmp_event, nsview);
-			});
-
-			return true;
-		}
-		
 		public override bool SetGlobalMenu (CommandManager commandManager, string commandMenuAddinPath, string appMenuAddinPath)
 		{
 			if (setupFail)
@@ -285,7 +254,7 @@ namespace MonoDevelop.MacIntegration
 				}
 			}
 		}
-		
+
 		void InitApp (CommandManager commandManager)
 		{
 			if (initedApp)
@@ -299,9 +268,9 @@ namespace MonoDevelop.MacIntegration
 			commandManager.GetCommand (HelpCommands.About).Text = GettextCatalog.GetString ("About {0}", BrandingService.ApplicationName);
 			commandManager.GetCommand (MacIntegrationCommands.HideWindow).Text = GettextCatalog.GetString ("Hide {0}", BrandingService.ApplicationName);
 			commandManager.GetCommand (ToolCommands.AddinManager).Text = GettextCatalog.GetString ("Add-in Manager...");
-			
+
 			initedApp = true;
-			
+
 			IdeApp.Workbench.RootWindow.DeleteEvent += HandleDeleteEvent;
 
 			if (MacSystemInformation.OsVersion >= MacSystemInformation.Lion) {
@@ -310,6 +279,49 @@ namespace MonoDevelop.MacIntegration
 					win.CollectionBehavior |= NSWindowCollectionBehavior.FullScreenPrimary;
 				};
 			}
+
+			PatchGtkTheme ();
+		}
+
+		// This will dynamically generate a gtkrc for certain widgets using system control colors.
+		void PatchGtkTheme ()
+		{
+			string color_hex, text_hex;
+
+			if (MonoDevelop.Core.Platform.OSVersion >= MonoDevelop.Core.MacSystemInformation.Yosemite) {
+				NSControlTint tint = NSColor.CurrentControlTint;
+				NSColor text = NSColor.SelectedMenuItemText.UsingColorSpace (NSColorSpace.GenericRGBColorSpace);
+				NSColor color = tint == NSControlTint.Blue ? NSColor.SelectedMenuItem.UsingColorSpace (NSColorSpace.GenericRGBColorSpace) : NSColor.SelectedMenuItem.UsingColorSpace (NSColorSpace.DeviceWhite);
+
+				color_hex = ConvertColorToHex (color);
+				text_hex = ConvertColorToHex (text);
+			} else {
+				color_hex = "#c5d4e0";
+				text_hex = "#000";
+			}
+
+			string gtkrc = String.Format (@"
+				style ""treeview"" = ""default"" {{
+					GtkTreeView::odd-row-color = ""#f5f5f5""
+
+					base[SELECTED] = ""{0}""
+					base[ACTIVE] = ""{0}""
+					text[SELECTED] = ""{1}""
+					text[ACTIVE] = ""{1}""
+					engine ""xamarin"" {{
+						roundness = 0
+						gradient_shades = {{ 1.0, 0.95, 0.95, 0.90 }}
+						glazestyle = 1
+					}}
+				}}
+
+				widget_class ""*.<GtkTreeView>*"" style ""treeview""
+				",
+				color_hex,
+				text_hex
+			);
+
+			Gtk.Rc.ParseString (gtkrc);
 		}
 
 		void GlobalSetup ()
@@ -339,7 +351,7 @@ namespace MonoDevelop.MacIntegration
 					e.UserCancelled = true;
 					e.Handled = true;
 				};
-				
+
 				ApplicationEvents.Reopen += delegate (object sender, ApplicationEventArgs e) {
 					if (IdeApp.Workbench != null && IdeApp.Workbench.RootWindow != null) {
 						IdeApp.Workbench.RootWindow.Deiconify ();
@@ -380,7 +392,7 @@ namespace MonoDevelop.MacIntegration
 								if (!Int32.TryParse (qs ["column"], out column))
 									column = 1;
 
-								return new FileOpenInformation (fileUri.AbsolutePath,
+								return new FileOpenInformation (Uri.UnescapeDataString(fileUri.AbsolutePath),
 									line, column, OpenDocumentOptions.DefaultInternal);
 							} catch (Exception ex) {
 								LoggingService.LogError ("Invalid TextMate URI: " + url, ex);
@@ -408,7 +420,7 @@ namespace MonoDevelop.MacIntegration
 			NSBundle.MainBundle.InfoDictionary ["CFBundleIdentifier"] = new NSString ("com.xamarin.monodevelop");
 
 			FilePath exePath = System.Reflection.Assembly.GetExecutingAssembly ().Location;
-			string iconFile = null;
+			string iconFile;
 			iconFile = BrandingService.GetString ("ApplicationIcon");
 			if (iconFile != null) {
 				iconFile = BrandingService.GetFile (iconFile);
@@ -437,7 +449,7 @@ namespace MonoDevelop.MacIntegration
 			} while ((path = path.ParentDirectory).IsNotNull);
 			return null;
 		}
-		
+
 		[GLib.ConnectBefore]
 		static void HandleDeleteEvent (object o, Gtk.DeleteEventArgs args)
 		{
@@ -447,7 +459,7 @@ namespace MonoDevelop.MacIntegration
 
 		public static Gdk.Pixbuf GetPixbufFromNSImageRep (NSImageRep rep, int width, int height)
 		{
-			var rect = new RectangleF (0, 0, width, height);
+			var rect = new CGRect (0, 0, width, height);
 
 			var bitmap = rep as NSBitmapImageRep;
 			try {
@@ -467,7 +479,7 @@ namespace MonoDevelop.MacIntegration
 
 		public static Gdk.Pixbuf GetPixbufFromNSImage (NSImage icon, int width, int height)
 		{
-			var rect = new RectangleF (0, 0, width, height);
+			var rect = new CGRect (0, 0, width, height);
 
 			var rep = icon.BestRepresentation (rect, null, null);
 			var bitmap = rep as NSBitmapImageRep;
@@ -496,7 +508,7 @@ namespace MonoDevelop.MacIntegration
 				System.Runtime.InteropServices.Marshal.Copy (tiff.Bytes, data, 0, data.Length);
 			}
 
-			int pw = bitmap.PixelsWide, ph = bitmap.PixelsHigh;
+			int pw = (int)bitmap.PixelsWide, ph = (int)bitmap.PixelsHigh;
 			var pixbuf = new Gdk.Pixbuf (data, pw, ph);
 
 			// if one dimension matches, and the other is same or smaller, use as-is
@@ -520,15 +532,15 @@ namespace MonoDevelop.MacIntegration
 
 			return scaled;
 		}
-		
+
 		protected override Xwt.Drawing.Image OnGetIconForFile (string filename)
 		{
 			//this only works on MacOS 10.6.0 and greater
-			if (systemVersion < 0x1060)
+			if (MacSystemInformation.OsVersion < MacSystemInformation.SnowLeopard)
 				return base.OnGetIconForFile (filename);
-			
+
 			NSImage icon = null;
-			
+
 			if (Path.IsPathRooted (filename) && File.Exists (filename)) {
 				icon = NSWorkspace.SharedWorkspace.IconForFile (filename);
 			} else {
@@ -536,20 +548,20 @@ namespace MonoDevelop.MacIntegration
 				if (!string.IsNullOrEmpty (extension))
 					icon = NSWorkspace.SharedWorkspace.IconForFileType (extension);
 			}
-			
+
 			if (icon == null) {
 				return base.OnGetIconForFile (filename);
 			}
-			
+
 			int w, h;
 			if (!Gtk.Icon.SizeLookup (Gtk.IconSize.Menu, out w, out h)) {
 				w = h = 22;
 			}
-				
+
 			var res = GetPixbufFromNSImage (icon, w, h);
 			return res != null ? res.ToXwtImage () : base.OnGetIconForFile (filename);
 		}
-		
+
 		public override IProcessAsyncOperation StartConsoleProcess (string command, string arguments, string workingDirectory,
 		                                                            IDictionary<string, string> environmentVariables,
 		                                                            string title, bool pauseWhenFinished)
@@ -557,7 +569,7 @@ namespace MonoDevelop.MacIntegration
 			return new MacExternalConsoleProcess (command, arguments, workingDirectory, environmentVariables,
 			                                   title, pauseWhenFinished);
 		}
-		
+
 		public override bool CanOpenTerminal {
 			get {
 				return true;
@@ -571,95 +583,97 @@ namespace MonoDevelop.MacIntegration
 				null, null, directory, environmentVariables, title, false, out tabId, out windowId
 			);
 		}
-		
+
 		public override IEnumerable<DesktopApplication> GetApplications (string filename)
 		{
 			//FIXME: we should disambiguate dupliacte apps in different locations and display both
 			//for now, just filter out the duplicates
 			var checkUniqueName = new HashSet<string> ();
 			var checkUniquePath = new HashSet<string> ();
-			
+
 			//FIXME: bundle path is wrong because of how MD is built into an app
 			//var thisPath = NSBundle.MainBundle.BundleUrl.Path;
 			//checkUniquePath.Add (thisPath);
-			
+
 			checkUniqueName.Add ("MonoDevelop");
 			checkUniqueName.Add (BrandingService.ApplicationName);
-			
-			string def = CoreFoundation.GetApplicationUrl (filename, CoreFoundation.LSRolesMask.All);
-			
+
+			string def = MonoDevelop.MacInterop.CoreFoundation.GetApplicationUrl (filename,
+				MonoDevelop.MacInterop.CoreFoundation.LSRolesMask.All);
+
 			var apps = new List<DesktopApplication> ();
-			
-			foreach (var app in CoreFoundation.GetApplicationUrls (filename, CoreFoundation.LSRolesMask.All)) {
+
+			foreach (var app in MonoDevelop.MacInterop.CoreFoundation.GetApplicationUrls (filename,
+				MonoDevelop.MacInterop.CoreFoundation.LSRolesMask.All)) {
 				if (string.IsNullOrEmpty (app) || !checkUniquePath.Add (app))
 					continue;
 				var name = NSFileManager.DefaultManager.DisplayName (app);
 				if (checkUniqueName.Add (name))
 					apps.Add (new MacDesktopApplication (app, name, def != null && def == app));
 			}
-			
+
 			apps.Sort ((DesktopApplication a, DesktopApplication b) => {
 				int r = a.IsDefault.CompareTo (b.IsDefault);
 				if (r != 0)
 					return -r;
 				return a.DisplayName.CompareTo (b.DisplayName);
 			});
-			
+
 			return apps;
 		}
-		
+
 		class MacDesktopApplication : DesktopApplication
 		{
 			public MacDesktopApplication (string app, string name, bool isDefault) : base (app, name, isDefault)
 			{
 			}
-			
+
 			public override void Launch (params string[] files)
 			{
 				foreach (var file in files)
 					NSWorkspace.SharedWorkspace.OpenFile (file, Id);
 			}
 		}
-		
-		public override Gdk.Rectangle GetUsableMonitorGeometry (Gdk.Screen screen, int monitor_id)
+
+		public override Gdk.Rectangle GetUsableMonitorGeometry (Gdk.Screen screen, int monitor)
 		{
-			Gdk.Rectangle ygeometry = screen.GetMonitorGeometry (monitor_id);
+			Gdk.Rectangle ygeometry = screen.GetMonitorGeometry (monitor);
 			Gdk.Rectangle xgeometry = screen.GetMonitorGeometry (0);
-			NSScreen monitor = NSScreen.Screens[monitor_id];
-			RectangleF visible = monitor.VisibleFrame;
-			RectangleF frame = monitor.Frame;
-			
+			NSScreen nss = NSScreen.Screens[monitor];
+			var visible = nss.VisibleFrame;
+			var frame = nss.Frame;
+
 			// Note: Frame and VisibleFrame rectangles are relative to monitor 0, but we need absolute
 			// coordinates.
 			visible.X += xgeometry.X;
 			frame.X += xgeometry.X;
-			
+
 			// VisibleFrame.Y is the height of the Dock if it is at the bottom of the screen, so in order
 			// to get the menu height, we just figure out the difference between the visibleFrame height
 			// and the actual frame height, then subtract the Dock height.
 			//
 			// We need to swap the Y offset with the menu height because our callers expect the Y offset
 			// to be from the top of the screen, not from the bottom of the screen.
-			float x, y, width, height;
-			
+			nfloat x, y, width, height;
+
 			if (visible.Height <= frame.Height) {
-				float dockHeight = visible.Y - frame.Y;
-				float menubarHeight = (frame.Height - visible.Height) - dockHeight;
-				
+				var dockHeight = visible.Y - frame.Y;
+				var menubarHeight = (frame.Height - visible.Height) - dockHeight;
+
 				height = frame.Height - menubarHeight - dockHeight;
 				y = ygeometry.Y + menubarHeight;
 			} else {
 				height = frame.Height;
 				y = ygeometry.Y;
 			}
-			
+
 			// Takes care of the possibility of the Dock being positioned on the left or right edge of the screen.
-			width = Math.Min (visible.Width, frame.Width);
-			x = Math.Max (visible.X, frame.X);
-			
+			width = NMath.Min (visible.Width, frame.Width);
+			x = NMath.Max (visible.X, frame.X);
+
 			return new Gdk.Rectangle ((int) x, (int) y, (int) width, (int) height);
 		}
-		
+
 		public override void GrabDesktopFocus (Gtk.Window window)
 		{
 			window.Present ();
@@ -668,7 +682,7 @@ namespace MonoDevelop.MacIntegration
 
 		static Cairo.Color ConvertColor (NSColor color)
 		{
-			float r, g, b, a;
+			nfloat r, g, b, a;
 			if (color.ColorSpaceName == NSColorSpace.DeviceWhite) {
 				a = 1.0f;
 				r = g = b = color.WhiteComponent;
@@ -678,9 +692,27 @@ namespace MonoDevelop.MacIntegration
 			return new Cairo.Color (r, g, b, a);
 		}
 
+		static string ConvertColorToHex (NSColor color)
+		{
+			nfloat r, g, b, a;
+
+			if (color.ColorSpaceName == NSColorSpace.DeviceWhite) {
+				a = 1.0f;
+				r = g = b = color.WhiteComponent;
+			} else {
+				color.GetRgba (out r, out g, out b, out a);
+			}
+
+			return String.Format ("#{0}{1}{2}",
+				((int)(r * 255)).ToString ("x2"),
+				((int)(g * 255)).ToString ("x2"),
+				((int)(b * 255)).ToString ("x2")
+			);
+		}
+
 		internal static int GetTitleBarHeight ()
 		{
-			var frame = new RectangleF (0, 0, 100, 100);
+			var frame = new CGRect (0, 0, 100, 100);
 			var rect = NSWindow.ContentRectFor (frame, NSWindowStyle.Titled);
 			return (int)(frame.Height - rect.Height);
 		}
@@ -697,12 +729,8 @@ namespace MonoDevelop.MacIntegration
 		internal override void SetMainWindowDecorations (Gtk.Window window)
 		{
 			NSWindow w = GtkQuartz.GetWindow (window);
-			w.IsOpaque = false;
-			
-			var resource = "maintoolbarbg.png";
-			NSImage img = LoadImage (resource);
-			w.BackgroundColor = NSColor.FromPatternImage (img);
-			w.StyleMask |= NSWindowStyle.TexturedBackground;
+			w.IsOpaque = true;
+			w.StyleMask |= NSWindowStyle.UnifiedTitleAndToolbar;
 		}
 
 		internal override void RemoveWindowShadow (Gtk.Window window)
@@ -713,22 +741,20 @@ namespace MonoDevelop.MacIntegration
 			w.HasShadow = false;
 		}
 
-		internal override MainToolbar CreateMainToolbar (Gtk.Window window)
+		internal override IMainToolbarView CreateMainToolbar (Gtk.Window window)
 		{
-			NSWindow w = GtkQuartz.GetWindow (window);
-			w.IsOpaque = false;
-			
-			var resource = "maintoolbarbg.png";
-			NSImage img = LoadImage (resource);
-			var c = NSColor.FromPatternImage (img);
-			w.BackgroundColor = c;
-			w.StyleMask |= NSWindowStyle.TexturedBackground;
+			return new MonoDevelop.MacIntegration.MainToolbar.MainToolbar (window);
+		}
 
-			var result = new MainToolbar () {
-				Background = MonoDevelop.Components.CairoExtensions.LoadImage (typeof (MacPlatformService).Assembly, resource),
-				TitleBarHeight = GetTitleBarHeight ()
-			};
-			return result;
+		internal override void AttachMainToolbar (Gtk.VBox parent, IMainToolbarView toolbar)
+		{
+			var nativeToolbar = (MonoDevelop.MacIntegration.MainToolbar.MainToolbar)toolbar;
+			NSWindow w = GtkQuartz.GetWindow (nativeToolbar.gtkWindow);
+			if (MacSystemInformation.OsVersion >= MacSystemInformation.Yosemite)
+				w.TitleVisibility = NSWindowTitleVisibility.Hidden;
+
+			w.Toolbar = nativeToolbar.widget;
+			nativeToolbar.Initialize ();
 		}
 
 		protected override RecentFiles CreateRecentFilesProvider ()
@@ -754,20 +780,71 @@ namespace MonoDevelop.MacIntegration
 			}
 
 			NSWindow nswin = GtkQuartz.GetWindow (window);
-			if (isFullscreen != ((nswin.StyleMask & NSWindowStyle.FullScreenWindow) != 0)) {
-				//HACK: workaround for MonoMac not allowing null as argument
-				MonoMac.ObjCRuntime.Messaging.void_objc_msgSend_IntPtr (
-					nswin.Handle,
-					MonoMac.ObjCRuntime.Selector.GetHandle ("toggleFullScreen:"),
-					IntPtr.Zero);
-			}
+			if (isFullscreen != ((nswin.StyleMask & NSWindowStyle.FullScreenWindow) != 0))
+				nswin.ToggleFullScreen (null);
 		}
 
 		public override bool IsModalDialogRunning ()
 		{
 			var toplevels = GtkQuartz.GetToplevels ();
 
-			return toplevels.Any (t => t.Key.IsVisible && (t.Value == null || t.Value.Modal) && !t.Key.DebugDescription.StartsWith("<NSStatusBarWindow"));
+			// When we're looking for modal windows that don't belong to GTK, exclude
+			// NSStatusBarWindow (which is visible on Mavericks when we're in fullscreen) and
+			// NSToolbarFullscreenWindow (which is visible on Yosemite in fullscreen).
+			return toplevels.Any (t => t.Key.IsVisible && (t.Value == null || t.Value.Modal) &&
+				!(t.Key.DebugDescription.StartsWith("<NSStatusBarWindow", StringComparison.Ordinal) ||
+					t.Key.DebugDescription.StartsWith ("<NSToolbarFullScreenWindow", StringComparison.Ordinal) ||
+					t.Key.DebugDescription.StartsWith ("<NSCarbonMenuWindow", StringComparison.Ordinal)
+				));
+		}
+
+		public override void AddChildWindow (Gtk.Window parent, Gtk.Window child)
+		{
+			NSWindow w = GtkQuartz.GetWindow (parent);
+			child.Realize ();
+			NSWindow overlay = GtkQuartz.GetWindow (child);
+			overlay.SetExcludedFromWindowsMenu (true);
+			w.AddChildWindow (overlay, NSWindowOrderingMode.Above);
+		}
+
+		public override void PlaceWindow (Gtk.Window window, int x, int y, int width, int height)
+		{
+			if (window.GdkWindow == null)
+				return; // Not yet realized
+
+			NSWindow w = GtkQuartz.GetWindow (window);
+			var dr = FromDesktopRect (new Gdk.Rectangle (x, y, width, height));
+			var r = w.FrameRectFor (dr);
+			w.SetFrame (r, true);
+			base.PlaceWindow (window, x, y, width, height);
+		}
+
+		static CGRect FromDesktopRect (Gdk.Rectangle r)
+		{
+			var desktopBounds = CalcDesktopBounds ();
+			r.Y = desktopBounds.Height - r.Y - r.Height;
+			if (desktopBounds.Y < 0)
+				r.Y += desktopBounds.Y;
+			return new CGRect (desktopBounds.X + r.X, r.Y, r.Width, r.Height);
+		}
+
+		static Gdk.Rectangle CalcDesktopBounds ()
+		{
+			var desktopBounds = new Gdk.Rectangle ();
+			foreach (var s in NSScreen.Screens) {
+				var r = s.Frame;
+				desktopBounds = desktopBounds.Union (new Gdk.Rectangle ((int)r.X, (int)r.Y, (int)r.Width, (int)r.Height));
+			}
+			return desktopBounds;
+		}
+
+		public override void OpenFolder (FilePath folderPath, FilePath[] selectFiles)
+		{
+			if (selectFiles.Length == 0) {
+				System.Diagnostics.Process.Start (folderPath);
+			} else {
+				NSWorkspace.SharedWorkspace.ActivateFileViewer (selectFiles.Select ((f) => NSUrl.FromFilename (f)).ToArray ());
+			}
 		}
 	}
 }
