@@ -158,10 +158,10 @@ namespace MonoDevelop.AssemblyBrowser
 			};
 
 			languageCombobox = Gtk.ComboBox.NewText ();
-			//languageCombobox.AppendText (GettextCatalog.GetString ("Summary"));
+			languageCombobox.AppendText (GettextCatalog.GetString ("Summary"));
 			languageCombobox.AppendText (GettextCatalog.GetString ("IL"));
 			languageCombobox.AppendText (GettextCatalog.GetString ("C#"));
-			languageCombobox.Active = Math.Min (1, PropertyService.Get ("AssemblyBrowser.InspectLanguage", 1));
+			languageCombobox.Active = Math.Min (0, PropertyService.Get ("AssemblyBrowser.Language", 0));
 			languageCombobox.Changed += LanguageComboboxhandleChanged;
 
 			loader = new CecilLoader (true);
@@ -190,6 +190,7 @@ namespace MonoDevelop.AssemblyBrowser
 			TreeView.BorderWidth = 1;
 			TreeView.ShowBorderLine = false;
 			TreeView.Zoom = 1.0;
+
 			treeViewPlaceholder.Add (TreeView);
 
 //			this.descriptionLabel.ModifyFont (Pango.FontDescription.FromString ("Sans 9"));
@@ -338,7 +339,7 @@ namespace MonoDevelop.AssemblyBrowser
 		void LanguageComboboxhandleChanged (object sender, EventArgs e)
 		{
 			this.notebook1.Page = 0;
-			PropertyService.Set ("AssemblyBrowser.InspectLanguage", this.languageCombobox.Active);
+			PropertyService.Set ("AssemblyBrowser.Language", this.languageCombobox.Active);
 			FillInspectLabel ();
 		}
 
@@ -1111,11 +1112,16 @@ namespace MonoDevelop.AssemblyBrowser
 			inspectEditor.Document.ClearFoldSegments ();
 			switch (this.languageCombobox.Active) {
 			case 0:
+				inspectEditor.Options.ShowFoldMargin = true;
+				this.inspectEditor.Document.MimeType = "text/x-csharp";
+				SetReferencedSegments (builder.GetSummary (inspectEditor.GetTextEditorData (), nav, PublicApiOnly));
+				break;
+			case 1:
 				inspectEditor.Options.ShowFoldMargin = true;
 				this.inspectEditor.Document.MimeType = "text/x-ilasm";
 				SetReferencedSegments (builder.Disassemble (inspectEditor.GetTextEditorData (), nav));
 				break;
-			case 1:
+			case 2:
 				inspectEditor.Options.ShowFoldMargin = true;
 				this.inspectEditor.Document.MimeType = "text/x-csharp";
 				SetReferencedSegments (builder.Decompile (inspectEditor.GetTextEditorData (), nav, PublicApiOnly));
@@ -1183,20 +1189,24 @@ namespace MonoDevelop.AssemblyBrowser
 		
 		public void Open (string url, AssemblyLoader currentAssembly = null)
 		{
-			ITreeNavigator nav = SearchMember (url);
-			if (definitions == null) // we've been disposed
-				return;
-			if (nav != null)
-				return;
-			try {
-				if (currentAssembly != null) {
-					OpenFromAssembly (url, currentAssembly);
-				} else {
-					OpenFromAssemblyNames (url);
-				}
-			} catch (Exception e) {
-				LoggingService.LogError ("Error while opening the assembly browser with id:" + url, e); 
-			}
+			Task.WhenAll (this.definitions.Select (d => d.LoadingTask).ToArray ()).ContinueWith (d => {
+				Application.Invoke (delegate {
+					ITreeNavigator nav = SearchMember (url);
+					if (definitions == null) // we've been disposed
+						return;
+					if (nav != null)
+						return;
+					try {
+						if (currentAssembly != null) {
+							OpenFromAssembly (url, currentAssembly);
+						} else {
+							OpenFromAssemblyNames (url);
+						}
+					} catch (Exception e) {
+						LoggingService.LogError ("Error while opening the assembly browser with id:" + url, e);
+					}
+				});
+			});
 		}
 
 		void OpenFromAssembly (string url, AssemblyLoader currentAssembly)
@@ -1219,7 +1229,7 @@ namespace MonoDevelop.AssemblyBrowser
 						loadNext ();
 					return;
 				}
-				var result = AddReferenceByFileName (fileName, true);
+				var result = AddReferenceByFileName (fileName);
 				result.LoadingTask.ContinueWith (t2 => {
 					t2.Wait ();
 					if (definitions == null) // disposed
@@ -1295,6 +1305,9 @@ namespace MonoDevelop.AssemblyBrowser
 				return;
 			
 			ITreeNavigator nav = TreeView.GetRootNode ();
+			if (nav == null)
+				return;
+			
 			do {
 				if (nav.DataItem == cu) {
 					nav.ExpandToNode ();
@@ -1407,12 +1420,12 @@ namespace MonoDevelop.AssemblyBrowser
 		List<AssemblyLoader> definitions = new List<AssemblyLoader> ();
 		List<Project> projects = new List<Project> ();
 		
-		public AssemblyLoader AddReferenceByAssemblyName (AssemblyNameReference reference, bool selectReference = true)
+		public AssemblyLoader AddReferenceByAssemblyName (AssemblyNameReference reference)
 		{
-			return AddReferenceByAssemblyName (reference.Name, selectReference);
+			return AddReferenceByAssemblyName (reference.Name);
 		}
 		
-		public AssemblyLoader AddReferenceByAssemblyName (string assemblyFullName, bool selectReference = true)
+		public AssemblyLoader AddReferenceByAssemblyName (string assemblyFullName)
 		{
 			string assemblyFile = Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyLocation (assemblyFullName, null);
 			if (assemblyFile == null || !System.IO.File.Exists (assemblyFile)) {
@@ -1425,29 +1438,29 @@ namespace MonoDevelop.AssemblyBrowser
 			if (assemblyFile == null || !System.IO.File.Exists (assemblyFile))
 				return null;
 			
-			return AddReferenceByFileName (assemblyFile, selectReference);
+			return AddReferenceByFileName (assemblyFile);
 		}
 		
-		public AssemblyLoader AddReferenceByFileName (string fileName, bool selectReference = true)
+		public AssemblyLoader AddReferenceByFileName (string fileName)
 		{
 			var result = definitions.FirstOrDefault (d => d.FileName == fileName);
 			if (result != null) {
-				// Select the result.
-				if (selectReference) {
-					ITreeNavigator navigator = TreeView.GetNodeAtObject (result);
-					if (navigator != null) {
-						navigator.Selected = true;
-					} else {
-						LoggingService.LogWarning (result + " could not be found.");
-					}
-				}
+//				// Select the result.
+//				if (selectReference) {
+//					ITreeNavigator navigator = TreeView.GetNodeAtObject (result);
+//					if (navigator != null) {
+//						navigator.Selected = true;
+//					} else {
+//						LoggingService.LogWarning (result + " could not be found.");
+//					}
+//				}
 
 				return result;
 			}
 			result = new AssemblyLoader (this, fileName);
 			
 			definitions.Add (result);
-			result.LoadingTask.ContinueWith (delegate {
+			result.LoadingTask = result.LoadingTask.ContinueWith (task => {
 				Application.Invoke (delegate {
 					if (definitions == null)
 						return;
@@ -1458,13 +1471,16 @@ namespace MonoDevelop.AssemblyBrowser
 						} else {
 							builder = TreeView.AddChild (result);
 						}
-						builder.Selected = builder.Expanded = selectReference;
+						TreeIter iter;
+						if (!TreeView.Tree.Selection.GetSelected (out iter))
+							builder.Selected = builder.Expanded = true;
 					} catch (Exception e) {
 						LoggingService.LogError ("Error while adding assembly to the assembly list", e);
 					}
 				});
+				return task.Result;
 			}
-			);
+			                                                     );
 			return result;
 		}
 		
@@ -1477,7 +1493,9 @@ namespace MonoDevelop.AssemblyBrowser
 				// Select the project.
 				if (selectReference) {
 					ITreeNavigator navigator = TreeView.GetNodeAtObject (project);
-					navigator.Selected = true;
+
+					if (navigator != null)
+						navigator.Selected = true;
 				}
 
 				return;
