@@ -27,22 +27,26 @@
 //
 
 using System;
-using MonoDevelop.Projects.Dom;
+using ICSharpCode.NRefactory.TypeSystem;
+using Mono.Cecil;
+
 namespace MonoDevelop.AssemblyBrowser
 {
-	public class BaseTypeFolder
+	class BaseTypeFolder : IDisposable
 	{
-		IType type;
-		
-		public IType Type {
-			get {
-				return type;
-			}
+		public IUnresolvedTypeDefinition Type {
+			get;
+			private set;
 		}
 		
-		public BaseTypeFolder (IType type)
+		public BaseTypeFolder (IUnresolvedTypeDefinition type)
 		{
-			this.type = type;
+			this.Type = type;
+		}
+		
+		public void Dispose ()
+		{
+			Type = null;
 		}
 	}
 }
