@@ -28,19 +28,16 @@
 
 using System;
 using System.IO;
-using System.Collections;
 using System.Collections.Generic;
 using System.Reflection;
 using System.Text;
 using System.Text.RegularExpressions;
 
 using MonoDevelop.Core;
-using MonoDevelop.Core.Gui;
-using MonoDevelop.Core.Execution;
-using MonoDevelop.Core.ProgressMonitoring;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Projects;
+using MonoDevelop.Core.Assemblies;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.Autotools
 {
@@ -58,6 +55,8 @@ namespace MonoDevelop.Autotools
 		[ItemProperty ("ExcludedFiles")]
 		[ProjectPathItemProperty("File", Scope="*")]
 		List<string> excludedFiles = new List<string> ();
+		
+		IAssemblyContext assemblyContext;
 
 		public MakefileData ()
 		{
@@ -65,6 +64,22 @@ namespace MonoDevelop.Autotools
 			buildTargetName = "all";
 			cleanTargetName = "clean";
 			executeTargetName = String.Empty;
+			ParallelProcesses = 1;
+			
+			assemblyContext = GetMonoRuntimeContext ();
+			if (assemblyContext == null)
+				integrationEnabled = false;
+		}
+		
+		internal static IAssemblyContext GetMonoRuntimeContext ()
+		{
+			if (Runtime.SystemAssemblyService.CurrentRuntime.RuntimeId == "Mono")
+				return Runtime.SystemAssemblyService.CurrentRuntime.AssemblyContext;
+			else {
+				foreach (TargetRuntime r in Runtime.SystemAssemblyService.GetTargetRuntimes ("Mono"))
+					return r.AssemblyContext;
+			}
+			return null;
 		}
 
 		public Project OwnerProject {
@@ -82,24 +97,26 @@ namespace MonoDevelop.Autotools
 						((DotNetProject)value).ReferenceAddedToProject += refhandler;
 					}
 
-					ProjectFileEventHandler filehandler = delegate (object sender, ProjectFileEventArgs e) {
+					ProjectFileEventHandler filehandler = delegate (object sender, ProjectFileEventArgs args) {
 						MakefileVar var = null;
-						switch (e.ProjectFile.BuildAction) {
-						case "Compile":
-							var = BuildFilesVar;
-							break;
-						case "Content":
-							var = DeployFilesVar;
-							break;
-						case "EmbeddedResource":
-							var = ResourcesVar;
-							break;
-						case "None":
-							var = OthersVar;
-							break;
+						foreach (ProjectFileEventInfo e in args) {
+							switch (e.ProjectFile.BuildAction) {
+							case "Compile":
+								var = BuildFilesVar;
+								break;
+							case "Content":
+								var = DeployFilesVar;
+								break;
+							case "EmbeddedResource":
+								var = ResourcesVar;
+								break;
+							case "None":
+								var = OthersVar;
+								break;
+							}
+							if (var != null && var.Sync)
+								dirty = true;
 						}
-						if (var != null && var.Sync)
-							dirty = true;
 					};
 
 					value.FileRemovedFromProject += filehandler;
@@ -123,6 +140,10 @@ namespace MonoDevelop.Autotools
 			get { return integrationEnabled; }
 			set { integrationEnabled = value;}
 		}
+
+		public bool SupportsIntegration {
+			get { return IntegrationEnabled && !Platform.IsWindows; }
+		}
 		
 		[ItemProperty (DefaultValue = "")]
 		public string RelativeMakefileName {
@@ -173,6 +194,12 @@ namespace MonoDevelop.Autotools
 			get { return executeTargetName; }
 			set { executeTargetName = value;}
 		}
+		
+		/// <summary>
+		/// The number of parallel build processes to use
+		/// </summary>
+		[ItemProperty (DefaultValue = 1)]
+		public int ParallelProcesses { get; set; }
 
 		//Makefile variables
 		MakefileVar buildFilesVar;
@@ -226,7 +253,7 @@ namespace MonoDevelop.Autotools
 		public bool SyncReferences {
 			get { return syncReferences; }
 			set {
-				GacRefVar.Sync = value;
+				PackageRefVar.Sync = value;
 				AsmRefVar.Sync = value;
 				ProjectRefVar.Sync =value;
 				syncReferences = value;
@@ -251,15 +278,17 @@ namespace MonoDevelop.Autotools
 			get { return ConfiguredPackages != null; }
 		}
 
-		MakefileVar gacRefVar;
-		[ItemProperty]
-		public MakefileVar GacRefVar {
+		MakefileVar packageRefVar;
+		
+		//FIXME: name is quick hack to avoid changing format
+		[ItemProperty ("GacRefVar")]
+		public MakefileVar PackageRefVar {
 			get {
-				if (gacRefVar == null)
-					gacRefVar = new MakefileVar ();
-				return gacRefVar;
+				if (packageRefVar == null)
+					packageRefVar = new MakefileVar ();
+				return packageRefVar;
 			}
-			set { gacRefVar = value; }
+			set { packageRefVar = value; }
 		}
 
 		MakefileVar asmRefVar;
@@ -458,12 +487,13 @@ namespace MonoDevelop.Autotools
 			data.BuildTargetName = this.BuildTargetName;
 			data.CleanTargetName = this.CleanTargetName;
 			data.ExecuteTargetName = this.ExecuteTargetName;
+			data.ParallelProcesses = this.ParallelProcesses;
 			
 			data.BuildFilesVar = new MakefileVar (this.BuildFilesVar);
 			data.DeployFilesVar = new MakefileVar (this.DeployFilesVar);
 			data.ResourcesVar = new MakefileVar (this.ResourcesVar);
 			data.OthersVar = new MakefileVar (this.OthersVar);
-			data.GacRefVar = new MakefileVar (this.GacRefVar);
+			data.PackageRefVar = new MakefileVar (this.PackageRefVar);
 			data.AsmRefVar = new MakefileVar (this.AsmRefVar);
 			data.ProjectRefVar = new MakefileVar (this.ProjectRefVar);
 
@@ -598,7 +628,7 @@ namespace MonoDevelop.Autotools
 					encodeValues [OthersVar.Name] = false;
 					encodeValues [ResourcesVar.Name] = false;
 
-					encodeValues [GacRefVar.Name] = false;
+					encodeValues [PackageRefVar.Name] = false;
 					encodeValues [AsmRefVar.Name] = false;
 					encodeValues [ProjectRefVar.Name] = false;
 				}
@@ -706,34 +736,34 @@ namespace MonoDevelop.Autotools
 				//Do these for DotNetProject only
 				DotNetProject dotnetProject = ownerProject as DotNetProject;
 				if (dotnetProject != null) {
-					GacRefVar.Extra.Clear ();
+					PackageRefVar.Extra.Clear ();
 					AsmRefVar.Extra.Clear ();
 					ProjectRefVar.Extra.Clear ();
 
-					existingGacRefs = new Dictionary<string, ProjectReference> ();
+					existingPackageRefs = new Dictionary<string, ProjectReference> ();
 					requiredPackageVersions = new Dictionary<string,string> ();
-					newGacRefs = new Dictionary<string, ProjectReference> ();
+					newPackageRefs = new Dictionary<string, ProjectReference> ();
 
 					List<ProjectReference> toRemove = new List<ProjectReference> ();
 					foreach (ProjectReference pref in dotnetProject.References) {
-						if (pref.ReferenceType == ReferenceType.Gac) {
-							string [] files = pref.GetReferencedFileNames (ProjectService.DefaultConfiguration);
+						if (pref.ReferenceType == ReferenceType.Package) {
+							string [] files = pref.GetReferencedFileNames (ConfigurationSelector.Default);
 							if (files == null)
 								continue;
 
-							if (pref.ReferenceType == ReferenceType.Gac) {
+							if (pref.ReferenceType == ReferenceType.Package) {
 								// Store the package version required by this reference. We'll use
 								// the same version when trying to match references coming from the makefile
-								SystemAssembly asm = Runtime.SystemAssemblyService.GetAssemblyFromFullName (pref.StoredReference, pref.Package != null ? pref.Package.Name : null);
+								SystemAssembly asm = assemblyContext.GetAssemblyFromFullName (pref.StoredReference, pref.Package != null ? pref.Package.Name : null, dotnetProject.TargetFramework);
 								if (asm != null && asm.Package != null)
 									requiredPackageVersions [asm.Package.Name] = asm.Package.Version;
 							}
 							// this should help normalize paths like /foo//bar/../
 							string fullpath = Path.GetFullPath (files [0]);
-							if (existingGacRefs.ContainsKey (fullpath))
+							if (existingPackageRefs.ContainsKey (fullpath))
 								toRemove.Add (pref);
 							else
-								existingGacRefs [fullpath] = pref;
+								existingPackageRefs [fullpath] = pref;
 						}
 					}
 
@@ -741,12 +771,12 @@ namespace MonoDevelop.Autotools
 					foreach (ProjectReference pref in toRemove)
 						dotnetProject.References.Remove (pref);
 
-					ReadReferences (GacRefVar, ReferenceType.Gac, "Gac References", dotnetProject);
+					ReadReferences (PackageRefVar, ReferenceType.Package, "Package References", dotnetProject);
 
 					// !SaveReferences indicates that previous ref reading failed
-					if (SaveReferences && String.Compare (AsmRefVar.Name, GacRefVar.Name) != 0)
+					if (SaveReferences && String.Compare (AsmRefVar.Name, PackageRefVar.Name) != 0)
 						ReadReferences (AsmRefVar, ReferenceType.Assembly, "Asm References", dotnetProject);
-					if (SaveReferences && (String.Compare (ProjectRefVar.Name, GacRefVar.Name) != 0) && 
+					if (SaveReferences && (String.Compare (ProjectRefVar.Name, PackageRefVar.Name) != 0) && 
 						(String.Compare (ProjectRefVar.Name, AsmRefVar.Name) != 0))
 						ReadReferences (ProjectRefVar, ReferenceType.Project, "Project References", dotnetProject);
 					
@@ -754,12 +784,15 @@ namespace MonoDevelop.Autotools
 					//Required when UpdateProject gets called by ui
 					if (ownerProject.ParentSolution != null)
 						ResolveProjectReferences (ownerProject.ParentSolution.RootFolder, monitor);
+					
+					//only remove unmatched existing refs if everything resolved without errors
+					if (SaveReferences) {
+						foreach (ProjectReference pr in existingPackageRefs.Values)
+							dotnetProject.References.Remove (pr);
+					}
 
-					foreach (ProjectReference pr in existingGacRefs.Values)
-						dotnetProject.References.Remove (pr);
-
-					existingGacRefs.Clear ();
-					newGacRefs.Clear ();
+					existingPackageRefs.Clear ();
+					newPackageRefs.Clear ();
 				}
 			} catch (Exception e) {
 				string msg = GettextCatalog.GetString (
@@ -831,7 +864,7 @@ namespace MonoDevelop.Autotools
 						//FIXME: If it doesn't match, then?
 						fname = fname.Substring (len);
 
-					fname = UnescapeString (fname);
+					fname = FromMakefilePath (fname);
 
 					string resourceId = null;
 					if (buildAction == BuildAction.EmbeddedResource && fname.IndexOf (',') >= 0) {
@@ -895,8 +928,8 @@ namespace MonoDevelop.Autotools
 			}
 		}
 
-		Dictionary<string, ProjectReference> existingGacRefs = null;
-		Dictionary<string, ProjectReference> newGacRefs = null;
+		Dictionary<string, ProjectReference> existingPackageRefs = null;
+		Dictionary<string, ProjectReference> newPackageRefs = null;
 		Dictionary<string, string> requiredPackageVersions = null;
 
 		void ReadReferences (MakefileVar refVar, ReferenceType refType, string id, DotNetProject project)
@@ -967,25 +1000,28 @@ namespace MonoDevelop.Autotools
 				}
 
 				string pkgVarName = rname.Substring (2, rname.Length - 3).Replace ("_LIBS", String.Empty);
-				List<string> pkgNames = ConfiguredPackages.GetNamesFromVarName (pkgVarName);
+				List<PackageContent> pkgNames = ConfiguredPackages.GetPackageContentFromVarName (pkgVarName);
 				if (pkgNames == null) {
 					 LoggingService.LogWarning  ("Package named '{0}' not found in configure.in. Ignoring reference to '{1}'.",
 						pkgVarName, rname);
 					refVar.Extra.Add (reference);
+					SaveReferences = false;
 					return;
 				}
 
 				bool added = false;
-				foreach (string pkgName in pkgNames) {
-					if (ReferencedPackages.Contains (pkgName)) {
+				foreach (PackageContent packageContent in pkgNames) {
+					if (ReferencedPackages.Contains (packageContent.Name)) {
 						added = true;
 						continue;
 					}
 
 					// Add all successfully added packages to ReferencedPackages
-					if (LoadPackageReference (pkgName, project, refVar.Prefix)) {
-						ReferencedPackages.Add (pkgName);
-						added = true;
+					foreach (string referencedName in packageContent.AllReferencedNames) {
+						if (LoadPackageReference (referencedName, project, refVar.Prefix)) {
+							ReferencedPackages.Add (referencedName);
+							added = true;
+						}
 					}
 				}
 
@@ -1030,30 +1066,34 @@ namespace MonoDevelop.Autotools
 				refname = ResolveBuildVars (refname, ref varFound);
 				EncodeValues [refVar.Name] |= varFound;
 
-				//if refname is part of a package then add as gac
-				if (refname.IndexOf (Path.DirectorySeparatorChar) < 0 &&
-					ParseReferenceAsGac (refname, project) != null)
+				string fullpath = Path.GetFullPath (Path.Combine (BaseDirectory, refname));
+				
+				// if refname is part of a package then add as package
+				// but don't do it if the refname exactly matches a file name in the project dir
+				if (refname.IndexOf (Path.DirectorySeparatorChar) < 0 && !File.Exists (fullpath) &&
+					ParseReferenceAsPackage (refname, project) != null)
 					continue;
 				
-				string fullpath = Path.GetFullPath (Path.Combine (BaseDirectory, refname));
-				if (TryGetExistingGacRef (fullpath) != null)
+				if (TryGetExistingPackageRef (fullpath) != null)
 					continue;
 
-				// Check that its a valid assembly
-				string fullname = null;
-				try {
-					fullname = AssemblyName.GetAssemblyName (fullpath).FullName;
-				} catch (FileNotFoundException) {
-				} catch (BadImageFormatException) {
-				}
+				if (refname.IndexOf (Path.DirectorySeparatorChar) < 0) {
+					// Check that its a valid assembly
+					string fullname = null;
+					try {
+						fullname = AssemblyName.GetAssemblyName (fullpath).FullName;
+					} catch (FileNotFoundException) {
+					} catch (BadImageFormatException) {
+					}
 
-				// Valid assembly, From a package, add as Gac
-				SystemPackage pkg = Runtime.SystemAssemblyService.GetPackageFromPath (fullpath);
-				if (fullname != null && pkg != null) {
-					SystemAssembly sa = Runtime.SystemAssemblyService.GetAssemblyFromFullName (fullname, pkg.Name);
-					if (sa != null) {
-						AddNewGacReference (project, sa);
-						continue;
+					// Valid assembly, From a package, add as Package
+					SystemPackage pkg = assemblyContext.GetPackageFromPath (fullpath);
+					if (fullname != null && pkg != null) {
+						SystemAssembly sa = assemblyContext.GetAssemblyFromFullName (fullname, pkg.Name, project.TargetFramework);
+						if (sa != null) {
+							AddNewPackageReference (project, sa);
+							continue;
+						}
 					}
 				}
 
@@ -1068,96 +1108,71 @@ namespace MonoDevelop.Autotools
 			SystemPackage pkg = null;
 			string packageVersion;
 			if (requiredPackageVersions.TryGetValue (pkgName, out packageVersion))
-				pkg = Runtime.SystemAssemblyService.GetPackage (pkgName, packageVersion);
+				pkg = assemblyContext.GetPackage (pkgName, packageVersion);
 			if (pkg == null)
-				pkg = Runtime.SystemAssemblyService.GetPackage (pkgName);
+				pkg = assemblyContext.GetPackage (pkgName);
 			
 			if (pkg == null) {
 				LoggingService.LogWarning ("No package named '{0}' found. Ignoring.", pkgName);
+				//don't sync the unresolved refs from the project back to the makefile or things go horribly wrong
+				SaveReferences = false;
 				return false;
 			}
 
 			foreach (SystemAssembly sa in pkg.Assemblies) {
-				if (TryGetExistingGacRef (sa.Location) != null)
+				if (TryGetExistingPackageRef (sa.Location) != null)
 					continue;
 
 				//Get fullname of the assembly
-				AddNewGacReference (project, sa);
+				AddNewPackageReference (project, sa);
 			}
 
 			return true;
 		}
 
-		// maps "System" -> full name
-		// for assemblies from the GAC (SystemPackage.IsCorePackage)
-		// GetAssemblyNameForVersion should be used to get the exact assembly name
-		// for a ClrVersion
-		static Dictionary<string, string> corePackageAssemblyNames;
-		public static Dictionary<string, string> CorePackageAssemblyNames {
-			get {
-				if (corePackageAssemblyNames == null) {
-					corePackageAssemblyNames = new Dictionary<string, string> ();
-					foreach (SystemPackage pkg in Runtime.SystemAssemblyService.GetPackages ()) {
-						if (pkg.IsCorePackage) {
-							foreach (SystemAssembly asm in pkg.Assemblies)
-								corePackageAssemblyNames [asm.Name] = asm.FullName;
-						}
-					}
-				}
-
-				return corePackageAssemblyNames;
-			}
-		}
-
-		ProjectReference ParseReferenceAsGac (string rname, DotNetProject project)
+		ProjectReference ParseReferenceAsPackage (string rname, DotNetProject project)
 		{
 			string aname = rname;
 			if (rname.EndsWith (".dll", StringComparison.InvariantCultureIgnoreCase))
 				//-r:Mono.Posix.dll
 				aname = rname.Substring (0, rname.Length - 4);
 
-			string fullname;
-			if (!CorePackageAssemblyNames.TryGetValue (aname, out fullname)) {
-				fullname = Runtime.SystemAssemblyService.GetAssemblyFullName (aname);
-
-				//fullname can be null
-				CorePackageAssemblyNames [aname] = fullname;
-			}
+			string fullname = assemblyContext.GetAssemblyFullName (aname, project.TargetFramework);
 			if (fullname == null)
 				return null;
 
-			SystemAssembly asm = Runtime.SystemAssemblyService.GetAssemblyForVersion (fullname, null, project.TargetFramework);
+			SystemAssembly asm = assemblyContext.GetAssemblyForVersion (fullname, null, project.TargetFramework);
 			if (asm == null)
 				return null;
 
-			ProjectReference pref = TryGetExistingGacRef (asm.Location);
+			ProjectReference pref = TryGetExistingPackageRef (asm.Location);
 			if (pref != null)
 				return pref;
 
-			return AddNewGacReference (project, asm);
+			return AddNewPackageReference (project, asm);
 		}
 
-		ProjectReference TryGetExistingGacRef (string fullpath)
+		ProjectReference TryGetExistingPackageRef (string fullpath)
 		{
-			if (existingGacRefs.ContainsKey (fullpath)) {
-				ProjectReference ret = existingGacRefs [fullpath];
-				existingGacRefs.Remove (fullpath);
-				newGacRefs [fullpath] = ret;
+			if (existingPackageRefs.ContainsKey (fullpath)) {
+				ProjectReference ret = existingPackageRefs [fullpath];
+				existingPackageRefs.Remove (fullpath);
+				newPackageRefs [fullpath] = ret;
 
 				return ret;
 			}
 
-			if (newGacRefs.ContainsKey (fullpath))
-				return newGacRefs [fullpath];
+			if (newPackageRefs.ContainsKey (fullpath))
+				return newPackageRefs [fullpath];
 
 			return null;
 		}
 
-		ProjectReference AddNewGacReference (DotNetProject project, SystemAssembly sa)
+		ProjectReference AddNewPackageReference (DotNetProject project, SystemAssembly sa)
 		{
 			ProjectReference pref = new ProjectReference (sa);
 			project.References.Add (pref);
-			newGacRefs [sa.Location] = pref;
+			newPackageRefs [sa.Location] = pref;
 
 			return pref;
 		}
@@ -1165,9 +1180,8 @@ namespace MonoDevelop.Autotools
 		public static void ResolveProjectReferences (SolutionFolder folder, IProgressMonitor monitor)
 		{
 			Dictionary<string, DotNetProject> projects = new Dictionary<string, DotNetProject> ();
-			string defConfig = ProjectService.DefaultConfiguration;
 			foreach (DotNetProject p in folder.GetAllItems<DotNetProject> ()) {
-				string filename = p.GetOutputFileName (defConfig);
+				string filename = p.GetOutputFileName (ConfigurationSelector.Default);
 				// Can be null for Generic projects
 				if (!String.IsNullOrEmpty (filename))
 					projects [filename] = p;
@@ -1188,7 +1202,7 @@ namespace MonoDevelop.Autotools
 							pr.ReferenceType != ReferenceType.Project)
 							continue;
 						
-						string [] files = pr.GetReferencedFileNames (defConfig);
+						string [] files = pr.GetReferencedFileNames (ConfigurationSelector.Default);
 						if (files.Length > 0)
 							asmProjectRefs [files [0]] = pr;
 					}
@@ -1329,11 +1343,11 @@ namespace MonoDevelop.Autotools
 			WriteFiles (ResourcesVar, BuildAction.EmbeddedResource, makeRelative, "Resources");
 
 			if (SyncReferences && SaveReferences) {
-				Makefile.ClearVariableValue (GacRefVar.Name);
+				Makefile.ClearVariableValue (PackageRefVar.Name);
 				Makefile.ClearVariableValue (AsmRefVar.Name);
 				Makefile.ClearVariableValue (ProjectRefVar.Name);
 
-				WriteReferences (GacRefVar, ReferenceType.Gac, makeRelative, "Gac");
+				WriteReferences (PackageRefVar, ReferenceType.Package, makeRelative, "Package");
 				WriteReferences (AsmRefVar, ReferenceType.Assembly, makeRelative, "Assembly");
 				WriteReferences (ProjectRefVar, ReferenceType.Project, makeRelative, "Project");
 			
@@ -1341,17 +1355,17 @@ namespace MonoDevelop.Autotools
 				// but sort only once per distinct var
 				// (Required as we are comparing the full makefile as a string,
 				//  to detect changes!)
-				List<string> list = Makefile.GetListVariable (GacRefVar.Name);
+				List<string> list = Makefile.GetListVariable (PackageRefVar.Name);
 				if (list != null)
 					list.Sort (System.StringComparer.InvariantCulture);
 
-				if (String.Compare (AsmRefVar.Name, GacRefVar.Name) != 0) {
+				if (String.Compare (AsmRefVar.Name, PackageRefVar.Name) != 0) {
 					list = Makefile.GetListVariable (AsmRefVar.Name);
 					if (list != null)
 						list.Sort (System.StringComparer.InvariantCulture);
 				}
 
-				if ((String.Compare (ProjectRefVar.Name, GacRefVar.Name) != 0) && 
+				if ((String.Compare (ProjectRefVar.Name, PackageRefVar.Name) != 0) && 
 					(String.Compare (ProjectRefVar.Name, AsmRefVar.Name) != 0)) {
 					list = Makefile.GetListVariable (ProjectRefVar.Name);
 					if (list != null)
@@ -1391,10 +1405,9 @@ namespace MonoDevelop.Autotools
 						//Files are relative to the Makefile
 						str = GetRelativePath (pf.FilePath);
 					else
-						str = pf.RelativePath;
+						str = pf.FilePath.ToRelative (pf.Project.BaseDirectory);
 
 					string unescapedFileName = Path.GetFileName (str);
-					str = EscapeString (str);
 
 					if (EncodeValues [fileVar.Name]) {
 						if (pf.IsExternalToProject)
@@ -1402,6 +1415,7 @@ namespace MonoDevelop.Autotools
 						else
 							str = EncodeFileName (str, "srcdir", false);
 					}
+					str = ToMakefilePath (str);
 
 					// Emit the resource ID only when it is different from the file name
 					if (pf.BuildAction == BuildAction.EmbeddedResource && pf.ResourceId != null && pf.ResourceId.Length > 0 && pf.ResourceId != unescapedFileName)
@@ -1427,7 +1441,7 @@ namespace MonoDevelop.Autotools
 		bool WriteReferences (MakefileVar refVar, ReferenceType refType, bool makeRelative, string id)
 		{
 			//Reference vars can be shared too, so use existing list
-			//Eg. REF for both Gac and Asm ref
+			//Eg. REF for both Package and Asm ref
 			List<string> references = Makefile.GetListVariable (refVar.Name);
 			if (references == null) {
 				//Var not found, skip
@@ -1450,9 +1464,9 @@ namespace MonoDevelop.Autotools
 
 				string refstr = String.Empty;
 				switch (refType) {
-				case ReferenceType.Gac:
-					//Assemblies coming from packages are always added as Gac
-					refstr = GacRefToString (pr, hasAcSubstPackages, refVar);
+				case ReferenceType.Package:
+					//Assemblies coming from packages are always added as Package
+					refstr = PackageRefToString (pr, hasAcSubstPackages, refVar);
 					if (refstr == null)
 						continue;
 					break;
@@ -1487,15 +1501,15 @@ namespace MonoDevelop.Autotools
 			return true;
 		}
 
-		string GacRefToString (ProjectReference pr, Dictionary<string, bool> hasAcSubstPackages, MakefileVar refVar)
+		string PackageRefToString (ProjectReference pr, Dictionary<string, bool> hasAcSubstPackages, MakefileVar refVar)
 		{
-			//Gac ref can be a full name OR a path!
+			//Package ref can be a full name OR a path!
 			//FIXME: Use GetReferencedFileName and GetPackageFromPath ?
 			string fullname = pr.Reference;
 			SystemPackage pkg = pr.Package;
 			if (pkg == null) {
 				//reference could be a path
-				pkg = Runtime.SystemAssemblyService.GetPackageFromPath (Path.GetFullPath (pr.Reference));
+				pkg = assemblyContext.GetPackageFromPath (Path.GetFullPath (pr.Reference));
 				if (pkg != null) {
 					//Path
 					try {
@@ -1511,7 +1525,7 @@ namespace MonoDevelop.Autotools
 			}
 
 			if (pkg == null)
-				return AsmRefToString (pr.GetReferencedFileNames (ProjectService.DefaultConfiguration) [0], refVar, false);
+				return AsmRefToString (pr.GetReferencedFileNames (ConfigurationSelector.Default) [0], refVar, false);
 
 			// Reference is from a package
 
@@ -1524,7 +1538,7 @@ namespace MonoDevelop.Autotools
 			if (UseAutotools)
 				//Check whether ref'ed in configure.in
 				varname = ConfiguredPackages.GetVarNameFromName (pkg.Name);
-
+			
 			if (varname == null) {
 				//Package not referenced in configure.in
 				//Or not a autotools based project,
@@ -1561,19 +1575,19 @@ namespace MonoDevelop.Autotools
 		{
 			if (EncodeValues [refVar.Name]) {
 				if (!isBuiltAssembly)
-					return EncodeFileName (reference, "top_srcdir", true);
+					return ToMakefilePath (EncodeFileName (reference, "top_srcdir", true));
 				else if (!reference.StartsWith (BaseDirectory))
 					//Reference is external to this project
-					return EncodeFileName (reference, "top_builddir", true);
+					return ToMakefilePath (EncodeFileName (reference, "top_builddir", true));
 			}
 
 			// !external and !encode
-			return GetRelativePath (reference);
+			return ToMakefilePath (GetRelativePath (reference));
 		}
 
 		string ProjectRefToString (ProjectReference pr, MakefileVar refVar)
 		{
-			string [] tmp = pr.GetReferencedFileNames (ProjectService.DefaultConfiguration);
+			string [] tmp = pr.GetReferencedFileNames (ConfigurationSelector.Default);
 			if (tmp == null || tmp.Length == 0)
 				//Reference not found, ignoring
 				return null;
@@ -1581,13 +1595,39 @@ namespace MonoDevelop.Autotools
 			return AsmRefToString (tmp [0], refVar, true);
 		}
 
-		static string EscapeString (string str)
+		public static string GetUnixPath (string path)
+		{
+			if (Path.DirectorySeparatorChar != '/')
+				return path.Replace (Path.DirectorySeparatorChar, '/');
+			else
+				return path;
+		}
+
+		public static string FromUnixPath (string path)
+		{
+			if (Path.DirectorySeparatorChar != '/')
+				return path.Replace (Path.DirectorySeparatorChar, '/');
+			else
+				return path;
+		}
+
+		public static string ToMakefilePath (string str)
+		{
+			return EscapeString (GetUnixPath (str));
+		}
+
+		public static string FromMakefilePath (string str)
+		{
+			return FromUnixPath (UnescapeString (str));
+		}
+
+		public static string EscapeString (string str)
 		{
 			StringBuilder sb = new StringBuilder ();
 			int len = str.Length;
 			for (int i = 0; i < len; i ++) {
 				char c = str [i];
-				if (c == '\\' || c == '#')
+				if (c == '\\' || c == '#' || c == ' ')
 					sb.Append ("\\");
 				
 				sb.Append (c);
@@ -1610,7 +1650,7 @@ namespace MonoDevelop.Autotools
 					continue;
 				}
 				char next = str [i + 1];
-				if (next != '\\' && next != '#')
+				if (next != '\\' && next != '#' && next != ' ')
 					sb.Append ("\\");
 				
 				sb.Append (next);
@@ -1624,7 +1664,7 @@ namespace MonoDevelop.Autotools
 
 	public class ConfiguredPackagesManager
 	{
-		Dictionary<string, List<string>> pkgVarNameToPkgName;
+		Dictionary<string, List<PackageContent>> pkgVarNameToPkgName;
 		Dictionary<string, string> pkgNameToPkgVarName;
 
 		//This dict has entries for all vars that have a
@@ -1646,15 +1686,15 @@ namespace MonoDevelop.Autotools
 			lastWriteTime = finfo.LastWriteTime;
 			ReadPackagesList ();
 		}
-
-		//Gets the pkg-config name from the makefile (or autoconf?) var name
-		public List<string> GetNamesFromVarName (string varname)
+		
+		//Gets the pkg-config PackageContent from the makefile (or autoconf?) var name
+		public List<PackageContent> GetPackageContentFromVarName (string varname)
 		{
 			if (!pkgVarNameToPkgName.ContainsKey (varname)) {
 				LoggingService.LogDebug ("pkg-config variable {0} not found in pkgVarNameToPkgName.", varname);
 				return null;
 			}
-
+			
 			return pkgVarNameToPkgName [varname];
 		}
 
@@ -1673,22 +1713,24 @@ namespace MonoDevelop.Autotools
 		{
 			return varNameAcSubst.ContainsKey (varname);
 		}
-
+		
+		
 		void ReadPackagesList ()
 		{
-			pkgVarNameToPkgName = new Dictionary<string, List<string>> ();
+			pkgVarNameToPkgName = new Dictionary<string, List<PackageContent>> ();
 			pkgNameToPkgVarName = new Dictionary<string, string> ();
 			varNameAcSubst = new Dictionary<string, string> ();
 
 			string content = null;
 			using (StreamReader sr = new StreamReader (fullpath))
 				content = sr.ReadToEnd ();
+			var assemblyContext = MakefileData.GetMonoRuntimeContext ();
 
 			foreach (Match match in PkgCheckModulesRegex.Matches (content)) {
 				if (!match.Success)
 					continue;
 
-				List<string> pkgs = new List<string> ();
+				List<PackageContent> pkgs = new List<PackageContent> ();
 				string pkgId = match.Groups ["pkgId"].Value;
 				if (pkgId.Length > 2 && pkgId [0] == '[' && pkgId [pkgId.Length - 1] == ']')
 					// Remove [] used for quoting
@@ -1702,10 +1744,11 @@ namespace MonoDevelop.Autotools
 					if (s.Length > 2 && s [0] == '[' && s [s.Length - 1] == ']')
 						// Remove [] used for quoting
 						s = s.Substring (1, s.Length - 2);
-					pkgs.Add (s);
+					PackageContent packageContent = new PackageContent (s);
+					pkgs.Add (packageContent);
 					pkgNameToPkgVarName [s] = pkgId;
+					AddRequiredPackages (assemblyContext, pkgId, s, packageContent);
 				}
-
 				pkgVarNameToPkgName [pkgId] = pkgs;
 			}
 			
@@ -1718,10 +1761,10 @@ namespace MonoDevelop.Autotools
 					continue;
 
 				string pkgVarName = s.Replace ("_LIBS", String.Empty);
-				List<string> l = GetNamesFromVarName (pkgVarName);
-
-				if (l != null && l.Count == 1 &&
-					Runtime.SystemAssemblyService.GetPackage (l [0]) != null) {
+				List<PackageContent> l = GetPackageContentFromVarName (pkgVarName);
+				
+				IAssemblyContext r = MakefileData.GetMonoRuntimeContext ();
+				if (l != null && l.Count == 1 && r.GetPackage (l [0].Name) != null) {
 					//PKG_CHECK_MODULES for pkgVarName was found
 					//and it references only a single managed package
 					//
@@ -1733,6 +1776,20 @@ namespace MonoDevelop.Autotools
 			}
 		}
 
+		void AddRequiredPackages (IAssemblyContext assemblyContext, string variableName, string packageName, PackageContent packageContent)
+		{
+			SystemPackage package = assemblyContext.GetPackage (packageName);
+			if (package != null && !string.IsNullOrEmpty (package.Requires)) {
+				foreach (string requiredPackageName in package.Requires.Split (' ')) {
+					if (!pkgNameToPkgVarName.ContainsKey (requiredPackageName)) {
+						pkgNameToPkgVarName[requiredPackageName] = variableName;
+						packageContent.RequiredPackages.Add (requiredPackageName);
+					}
+					AddRequiredPackages (assemblyContext, variableName, requiredPackageName, packageContent);
+				}
+			}
+		}
+		
 		public DateTime LastWriteTime {
 			get { return lastWriteTime; }
 		}
@@ -1756,5 +1813,30 @@ namespace MonoDevelop.Autotools
 			}
 		}
 	}
-
+	
+	public class PackageContent
+	{
+		public string Name { get; set; }
+		
+		System.Collections.Generic.List<string> requiredPackages = new System.Collections.Generic.List<string> ();
+		public List<string> RequiredPackages { 
+			get {
+				return requiredPackages; 
+			}
+		}
+		
+		public IEnumerable<string> AllReferencedNames {
+			get {
+				yield return Name;
+				foreach (string name in requiredPackages) {
+					yield return name;
+				}
+			}
+		}
+		
+		public PackageContent (string name)
+		{
+			this.Name = name;
+		}
+	}
 }
