@@ -27,6 +27,7 @@ using System;
 using NUnit.Framework;
 using Mono.Debugging.Client;
 using System.Collections.Generic;
+using Mono.Debugging.Soft;
 
 namespace MonoDevelop.Debugger.Tests
 {
@@ -437,6 +438,38 @@ namespace MonoDevelop.Debugger.Tests
 			//its looks like pressing F5
 		}
 
+		/// <summary>
+		/// Bug 21510
+		/// </summary>
+		[Test]
+		public void DebuggerStepperBoundaryMethod2ProjectAssembliesOnly ()
+		{
+			InitializeTest ();
+			Session.Options.ProjectAssembliesOnly = true;
+			AddBreakpoint ("f3a22b38-596a-4463-a562-20b342fdec12");
+			AddBreakpoint ("4721f27a-a268-4529-b327-c39f208c08c5");
+			StartTest ("DebuggerStepperBoundaryMethod2");
+			CheckPosition ("f3a22b38-596a-4463-a562-20b342fdec12");
+			StepIn ("4721f27a-a268-4529-b327-c39f208c08c5");
+		}
+
+		/// <summary>
+		/// Bug 21510
+		/// </summary>
+		[Test]
+		public void DebuggerStepperBoundaryMethod2 ()
+		{
+			InitializeTest ();
+			Session.Options.ProjectAssembliesOnly = false;
+			AddBreakpoint ("f3a22b38-596a-4463-a562-20b342fdec12");
+			AddBreakpoint ("4721f27a-a268-4529-b327-c39f208c08c5");
+			StartTest ("DebuggerStepperBoundaryMethod2");
+			CheckPosition ("f3a22b38-596a-4463-a562-20b342fdec12");
+			StepIn ("d110546f-a622-4ec3-9564-1c51bfec28f9", -1);
+			StepIn ("d110546f-a622-4ec3-9564-1c51bfec28f9");
+			StepIn ("4721f27a-a268-4529-b327-c39f208c08c5");
+		}
+
 		/// <summary>
 		/// Bug 3565
 		/// </summary>
@@ -487,12 +520,57 @@ namespace MonoDevelop.Debugger.Tests
 			Continue ("ffde3c82-4310-43d3-93d1-4c39e9cf615e");
 		}
 
+		[Test]
+		public void BreakpointInsideOneLineDelegateNoDisplayClass ()
+		{
+			InitializeTest ();
+			AddBreakpoint ("e0a96c37-577f-43e3-9a20-2cdd8bf7824e");
+			AddBreakpoint ("e72a2fa6-2d95-4f96-b3d0-ba321da3cb55", statement: "Console.WriteLine");
+			StartTest ("BreakpointInsideOneLineDelegateNoDisplayClass");
+			CheckPosition ("e0a96c37-577f-43e3-9a20-2cdd8bf7824e");
+			StepOver ("e72a2fa6-2d95-4f96-b3d0-ba321da3cb55", "Console.WriteLine");
+			StepOut ("3be64647-76c1-455b-a4a7-a21b37383dcb");
+			StepOut ("e0a96c37-577f-43e3-9a20-2cdd8bf7824e");
+		}
+
+		[Test]
+		public void BreakpointInsideOneLineDelegate ()
+		{
+			InitializeTest ();
+			AddBreakpoint ("67ae4cce-22b3-49d8-8221-7e5b26a5e79b");
+			AddBreakpoint ("22af08d6-dafc-47f1-b8d1-bee1526840fd", statement: "button.SetTitle");
+			StartTest ("BreakpointInsideOneLineDelegate");
+			CheckPosition ("67ae4cce-22b3-49d8-8221-7e5b26a5e79b");
+			StepOver ("22af08d6-dafc-47f1-b8d1-bee1526840fd", "button.SetTitle");
+			StepOut ("3be64647-76c1-455b-a4a7-a21b37383dcb");
+			StepOut ("67ae4cce-22b3-49d8-8221-7e5b26a5e79b");
+		}
+
+		[Test]
+		public void BreakpointInsideOneLineDelegateAsync ()
+		{
+			InitializeTest ();
+			AddBreakpoint ("b6a65e9e-5db2-4850-969a-b3747b2459af", statement: "button.SetTitle");
+			AddBreakpoint ("b6a65e9e-5db2-4850-969a-b3747b2459af", 1);
+			StartTest ("BreakpointInsideOneLineDelegateAsync");
+			CheckPosition ("b6a65e9e-5db2-4850-969a-b3747b2459af", 1);
+			StepOver ("b6a65e9e-5db2-4850-969a-b3747b2459af", "button.SetTitle");
+			if (Session is SoftDebuggerSession) {
+				StepOut ("3be64647-76c1-455b-a4a7-a21b37383dcb");
+			} else {
+				StepOut ("3be64647-76c1-455b-a4a7-a21b37383dcb", 1);//Feels like CorDebugger bug
+			}
+			StepOut ("b6a65e9e-5db2-4850-969a-b3747b2459af", 1);
+		}
+
 		/// <summary>
 		/// Bug 2851
 		/// </summary>
 		[Test]
 		public void ForeachEnumerable ()
 		{
+			IgnoreSoftDebugger ("Sdb has some problems when stepping into yeild methods. Have to investigate");
+
 			InitializeTest ();
 			AddBreakpoint ("b73bec88-2c43-4157-8574-ad517730bc74");
 			StartTest ("ForeachEnumerable");
@@ -566,8 +644,9 @@ namespace MonoDevelop.Debugger.Tests
 		[Test]
 		public void PListSchemeTest ()
 		{
+			IgnoreSoftDebugger ("Sdb is reapeating StepIn in StaticConstructor instead of StepOut. Resulting in step stopping at unexpected location.");
+
 			InitializeTest ();
-			Session.Options.ProjectAssembliesOnly = true;
 			AddBreakpoint ("41eb3a30-3b19-4ea5-a7dc-e4c76871f391");
 			StartTest ("PListSchemeTest");
 			CheckPosition ("41eb3a30-3b19-4ea5-a7dc-e4c76871f391");
@@ -575,12 +654,27 @@ namespace MonoDevelop.Debugger.Tests
 		}
 
 		/// <summary>
-		/// Bug 4433
+		/// Bug 4433 StepOverPropertiesAndOperators = true
+		/// </summary>
+		[Test]
+		public void Bug4433StepOverProperties ()
+		{
+			InitializeTest ();
+			Session.Options.StepOverPropertiesAndOperators = true;
+			AddBreakpoint ("a062e69c-e3f7-4fd7-8985-fc7abd5c27d2");
+			StartTest ("Bug4433Test");
+			CheckPosition ("a062e69c-e3f7-4fd7-8985-fc7abd5c27d2");
+			StepIn ("ad9b8803-eef0-438c-bf2b-9156782f4027", -1);
+		}
+
+		/// <summary>
+		/// Bug 4433 StepOverPropertiesAndOperators = false
 		/// </summary>
 		[Test]
 		public void Bug4433 ()
 		{
 			InitializeTest ();
+			Session.Options.StepOverPropertiesAndOperators = false;
 			AddBreakpoint ("a062e69c-e3f7-4fd7-8985-fc7abd5c27d2");
 			StartTest ("Bug4433Test");
 			CheckPosition ("a062e69c-e3f7-4fd7-8985-fc7abd5c27d2");
@@ -779,6 +873,8 @@ namespace MonoDevelop.Debugger.Tests
 		[Test]
 		public void CatchPointTest2 ()
 		{
+			IgnoreSoftDebugger ("I'm having problem testing this because. There is error nonstop happening in framework about CurrentCulture featching.");
+
 			InitializeTest ();
 			AddCatchpoint ("System.Exception", true);
 			StartTest ("Catchpoint2");
@@ -787,6 +883,28 @@ namespace MonoDevelop.Debugger.Tests
 			Assert.IsFalse (Session.ActiveThread.Backtrace.GetFrame (1).IsExternalCode);
 		}
 
+		[Test]
+		public void CatchpointIgnoreExceptionsInNonUserCodeTest ()
+		{
+			//It seems CorDebugger has different definition of what is user code and what is not.
+			IgnoreCorDebugger ("CorDebugger: TODO");
+
+			InitializeTest ();
+			Session.Options.ProjectAssembliesOnly = true;
+			AddBreakpoint ("999b8a83-8c32-4640-a8e1-f74309cda79c");
+			AddCatchpoint ("System.Exception", true);
+			StartTest ("CatchpointIgnoreExceptionsInNonUserCode");
+			CheckPosition ("999b8a83-8c32-4640-a8e1-f74309cda79c");
+
+			InitializeTest ();
+			Session.Options.ProjectAssembliesOnly = false;
+			AddCatchpoint ("System.Exception", true);
+			AddBreakpoint ("999b8a83-8c32-4640-a8e1-f74309cda79c");
+			StartTest ("CatchpointIgnoreExceptionsInNonUserCode");
+			WaitStop (2000);
+			Assert.AreEqual ("3913936e-3f89-4f07-a863-7275aaaa5fc9", Session.ActiveThread.Backtrace.GetFrame (0).GetException ().Message);
+		}
+
 		[Test]
 		public void ConditionalBreakpoints ()
 		{
@@ -803,7 +921,7 @@ namespace MonoDevelop.Debugger.Tests
 			Assert.AreEqual ("2", val.Value);
 			Continue ("3e2e4759-f6d9-4839-98e6-4fa96b227458");
 
-			Assert.Ignore ("TODO: Conditional breakpoints with compare against string or enum is not working");
+			IgnoreCorDebugger ("TODO: Conditional breakpoints with compare against string or enum is not working on CorDebugger");
 
 			InitializeTest ();
 			bp = AddBreakpoint ("033dd01d-6cb4-4e1a-b445-de6d7fa0d2a7");
@@ -960,7 +1078,6 @@ namespace MonoDevelop.Debugger.Tests
 			var debugList = new List<Tuple<int,string,string>> ();
 			debugList.Add (new Tuple<int,string,string> (0, "", "DebugText"));
 			debugList.Add (new Tuple<int, string, string> (3, "SomeCategory", "DebugText2"));
-			debugList.Add (new Tuple<int,string,string> (0, "", ""));
 
 			var unexpectedOutput = new List<string> ();
 			var unexpectedError = new List<string> ();
@@ -968,6 +1085,8 @@ namespace MonoDevelop.Debugger.Tests
 
 			Session.DebugWriter = delegate(int level, string category, string message) {
 				var entry = new Tuple<int,string,string> (level, category, message);
+				if (entry.Equals (new Tuple<int,string,string> (0, "", "")))//Sdb is emitting some empty messages :S 
+					return;
 				if (debugList.Contains (entry)) {
 					debugList.Remove (entry);
 				} else {
@@ -1002,6 +1121,30 @@ namespace MonoDevelop.Debugger.Tests
 			if (unexpectedDebug.Count > 0)
 				Assert.Fail ("Unexcpected Debug list has following items:" + string.Join (",", unexpectedDebug));
 		}
+
+		[Test]
+		public void Bug25358 ()
+		{
+			InitializeTest ();
+			AddBreakpoint ("4b30f826-2ba0-4b53-ab36-85b2cdde1069");
+			StartTest ("TestBug25358");
+			CheckPosition ("4b30f826-2ba0-4b53-ab36-85b2cdde1069");
+			var val = Eval ("e");
+			val = val.GetChildSync ("Message", EvaluationOptions.DefaultOptions);
+			Assert.AreEqual ("\"2b2c4423-accf-4c2c-af31-7d8dcee31c32\"", val.Value);
+		}
+
+		[Test]
+		public void Bug21410 ()
+		{
+			IgnoreSoftDebugger ("Runtime bug.");
+
+			InitializeTest ();
+			AddBreakpoint ("5e6663d0-9088-40ad-914d-0fcc05b2d0d5");
+			StartTest ("TestBug21410");
+			CheckPosition ("5e6663d0-9088-40ad-914d-0fcc05b2d0d5");
+			StepOver ("5e6663d0-9088-40ad-914d-0fcc05b2d0d5", 1);
+		}
 	}
 }
 
