@@ -36,17 +36,15 @@ using System.IO;
 using System.Collections;
 using System.Collections.Generic;
 using System.ComponentModel;
-using System.ComponentModel.Design;
-using System.Drawing.Design;
+using Mono.Addins;
 
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Core;
-using MonoDevelop.Core.Execution;
-using Mono.Addins;
+using MonoDevelop.Core.Assemblies;
 using MonoDevelop.Projects;
 using MonoDevelop.Core.Serialization;
-
 using MonoDevelop.DesignerSupport.Toolbox;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.DesignerSupport
 {
@@ -56,11 +54,13 @@ namespace MonoDevelop.DesignerSupport
 		readonly static string toolboxProviderPath = "/MonoDevelop/DesignerSupport/ToolboxProviders";
 
 		ToolboxConfiguration config;
+		int initializing;
 		List<IToolboxLoader> loaders = new List<IToolboxLoader> ();
 		List<IToolboxDynamicProvider> dynamicProviders  = new List<IToolboxDynamicProvider> ();
 		
 		IToolboxConsumer currentConsumer;
 		ItemToolboxNode selectedItem;
+		IToolboxCustomizer customizer;
 		
 		internal ToolboxService ()
 		{
@@ -76,11 +76,15 @@ namespace MonoDevelop.DesignerSupport
 		}
 		
 		static string ToolboxConfigFile {
-			get { return System.IO.Path.Combine (PropertyService.ConfigPath, "Toolbox.xml"); }
+			get { return UserProfile.Current.LocalConfigDir.Combine ("Toolbox.xml"); }
 		}
 		
 		#region Extension loading
 		
+		public bool Initializing {
+			get { return initializing > 0; }
+		}
+		
 		void OnLoaderExtensionChanged (object s, ExtensionNodeEventArgs args)
 		{
 			if (args.Change == ExtensionChange.Add)
@@ -121,39 +125,23 @@ namespace MonoDevelop.DesignerSupport
 		
 		public void AddUserItems ()
 		{
-			ComponentSelectorDialog dlg = new ComponentSelectorDialog (currentConsumer);
-			try {
+			using (ComponentSelectorDialog dlg = new ComponentSelectorDialog (currentConsumer)) {
 				dlg.Fill ();
-				dlg.Run ();
-			}
-			finally {
-				dlg.Destroy ();
+				MessageService.ShowCustomDialog (dlg);
 			}
 		}
 		
-		void AddUserItems (string fileName)
+		void AddUserItems (IList<ItemToolboxNode> nodes)
 		{			
-			foreach (IToolboxLoader loader in loaders) {
-				//check whether the loader can handle this file extension
-				bool match = false;
-				foreach (string ext in loader.FileTypes)
-					if (fileName.EndsWith (ext))
-						match = true;
-				if (!match)
-					continue;
-				
-				System.Collections.Generic.IList<ItemToolboxNode> loadedItems = loader.Load (fileName);
-				
-				//prevent user from loading the same items again
-				foreach (ItemToolboxNode node in loadedItems) {
-					bool found = false;
-					foreach (ItemToolboxNode n in Configuration.ItemList) {
-						if (node.Equals (n))
-							found = true;
-					}
-					if (!found)
-						Configuration.ItemList.Add (node);
+			//prevent user from loading the same items again
+			foreach (ItemToolboxNode node in nodes) {
+				bool found = false;
+				foreach (ItemToolboxNode n in Configuration.ItemList) {
+					if (node.Equals (n))
+						found = true;
 				}
+				if (!found)
+					Configuration.ItemList.Add (node);
 			}
 		}
 		
@@ -170,18 +158,39 @@ namespace MonoDevelop.DesignerSupport
 			
 			if (!Configuration.LoadedDefaultProviders.Contains (pname)) {
 				Configuration.LoadedDefaultProviders.Add (pname);
-				
-				IEnumerable<ItemToolboxNode> newItems = provider.GetDefaultItems ();
-				if (newItems != null)
-					Configuration.ItemList.AddRange (newItems);
-				
-				IEnumerable<string> files = provider.GetDefaultFiles ();
-				if (files != null) {
-					foreach (string f in files)
-						AddUserItems (f);
-				}
-				SaveConfiguration ();
+				initializing++;
 				OnToolboxContentsChanged ();
+
+				System.Threading.ThreadPool.QueueUserWorkItem (delegate {
+					List<ItemToolboxNode> nodes = new List<ItemToolboxNode> ();
+					try {
+						IEnumerable<ItemToolboxNode> newItems = provider.GetDefaultItems ();
+						if (newItems != null)
+							nodes.AddRange (newItems);
+					} catch (Exception ex) {
+						LoggingService.LogError ("Error getting default items from a IToolboxDefaultProvider", ex);
+					}
+
+					LoaderContext ctx = null;
+					try {
+						IEnumerable<string> files = provider.GetDefaultFiles ();
+						if (files != null) {
+							ctx = new LoaderContext ();
+							foreach (string f in files)
+								nodes.AddRange (GetFileItems (ctx, f));
+						}
+					} finally {
+						if (ctx != null)
+							ctx.Dispose ();
+					}
+
+					DispatchService.GuiDispatch (delegate {
+						AddUserItems (nodes);
+						initializing--;
+						SaveConfiguration ();
+						OnToolboxContentsChanged ();
+					});
+				});
 			}
 		}
 		
@@ -195,35 +204,35 @@ namespace MonoDevelop.DesignerSupport
 			}
 		}
 		
-		internal IList<ItemToolboxNode> GetFileItems (string fileName)
+		internal void Customize (IPadWindow padWindow, IToolboxConfiguration config)
+		{
+			if (customizer != null)
+				customizer.Customize (padWindow, config);
+		}
+		
+		internal IList<ItemToolboxNode> GetFileItems (LoaderContext ctx, string fileName)
 		{
 			// Gets the list of items provided by a file.
-			// It checks every toolbox loader, and loads the file in an external
-			// process if required.
-			
 			List<ItemToolboxNode> items = new List<ItemToolboxNode> ();
-			bool isolatedLoaded = false;
+			
 			foreach (IToolboxLoader loader in loaders) {
+				//check whether the loader can handle this file extension
+				bool match = false;
+				foreach (string ext in loader.FileTypes) {
+					if (fileName.EndsWith (ext)) {
+						match = true;
+						break;
+					}
+				}
+				if (!match)
+					continue;
+				
 				if (!fileName.EndsWith (loader.FileTypes[0]))
 					continue;
 				
 				try {
-					if (loader.ShouldIsolate) {
-						// Load the widgets in an external process. ExternalFileLoader.Load() class will
-						// load the file using all loaders which require isolation.
-						if (isolatedLoaded)
-							continue;
-						isolatedLoaded = true;
-						ExternalFileLoader eloader = (ExternalFileLoader) Runtime.ProcessService.CreateExternalProcessObject (typeof(ExternalFileLoader),true);
-						string s = eloader.LoadFile (fileName);
-						XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
-						ToolboxList list = (ToolboxList) ser.Deserialize (new StringReader (s), typeof(ToolboxList));
-						items.AddRange (list);
-					}
-					else {
-						IList<ItemToolboxNode> loadedItems = loader.Load (fileName);
-						items.AddRange (loadedItems);
-					}
+					IList<ItemToolboxNode> loadedItems = loader.Load (ctx, fileName);
+					items.AddRange (loadedItems);
 				}
 				catch (Exception ex) {
 					// Ignore
@@ -233,30 +242,6 @@ namespace MonoDevelop.DesignerSupport
 			return items;
 		}
 		
-		internal IList<ItemToolboxNode> InternalGetFileItems (string fileName)
-		{
-			// Method called by from the external process to get the items of
-			// a file. Only loaders requiring isolation are used here, since
-			// the other loaders are handled in the main process.
-			
-			List<ItemToolboxNode> items = new List<ItemToolboxNode> ();
-			foreach (IToolboxLoader loader in loaders) {
-				if (!fileName.EndsWith (loader.FileTypes[0]))
-					continue;
-				
-				try {
-					if (loader.ShouldIsolate) {
-						IList<ItemToolboxNode> loadedItems = loader.Load (fileName);
-						items.AddRange (loadedItems);
-					}
-				}
-				catch {
-					// Ignore
-				}
-			}
-			return items;
-		}
-		
 		~ToolboxService ()
 		{
 			AddinManager.RemoveExtensionNodeHandler (toolboxLoaderPath, OnLoaderExtensionChanged);
@@ -396,11 +381,6 @@ namespace MonoDevelop.DesignerSupport
 				OnToolboxUsed (CurrentConsumer, selectedItem);
 			} catch (Exception ex) {
 				MonoDevelop.Core.LoggingService.LogError ("Error dragging toolbox item.", ex);
-				//run this dialog on a timeout so it doesn't block the drag completing
-				GLib.Timeout.Add (100, delegate {
-					MonoDevelop.Core.Gui.MessageService.ShowException (ex, "Error dragging toolbox item.");
-					return false;
-				});
 			}
 		}
 		
@@ -440,9 +420,10 @@ namespace MonoDevelop.DesignerSupport
 			}
 			
 			//only treat active ViewContent as a Toolbox consumer if it implements IToolboxConsumer
-			if (IdeApp.Workbench.ActiveDocument != null) {
-				CurrentConsumer = IdeApp.Workbench.ActiveDocument.ActiveView as IToolboxConsumer;
-				viewProvider    = IdeApp.Workbench.ActiveDocument.ActiveView as IToolboxDynamicProvider;
+			if (IdeApp.Workbench.ActiveDocument != null && IdeApp.Workbench.ActiveDocument.ActiveView != null) {
+				CurrentConsumer = IdeApp.Workbench.ActiveDocument.ActiveView.GetContent<IToolboxConsumer> ();
+				viewProvider    = IdeApp.Workbench.ActiveDocument.ActiveView.GetContent<IToolboxDynamicProvider> ();
+				customizer = IdeApp.Workbench.ActiveDocument.ActiveView.GetContent<IToolboxCustomizer> ();
 				if (viewProvider != null)  {
 					this.dynamicProviders.Add (viewProvider);
 					viewProvider.ItemsChanged += OnProviderItemsChanged;
@@ -451,6 +432,7 @@ namespace MonoDevelop.DesignerSupport
 			} else {
 				CurrentConsumer = null;
 				viewProvider = null;
+				customizer = null;
 			}
 		}
 		
@@ -567,11 +549,11 @@ namespace MonoDevelop.DesignerSupport
 			
 			// Get the list of assemblies that need to be updated
 			
-			Hashtable files = new Hashtable ();
+			HashSet<string> files = new HashSet<string> ();
 			List<ComponentIndexFile> toupdate = new List<ComponentIndexFile> ();
 			List<ComponentIndexFile> todelete = new List<ComponentIndexFile> ();
 			foreach (ComponentIndexFile ia in index.Files) {
-				files [ia.FileName] = ia.FileName;
+				files.Add (ia.FileName);
 				if (!File.Exists (ia.FileName))
 					todelete.Add (ia);
 				if (ia.NeedsUpdate)
@@ -582,14 +564,16 @@ namespace MonoDevelop.DesignerSupport
 			
 			// Look for new assemblies
 			
-			foreach (SystemAssembly asm in Runtime.SystemAssemblyService.GetAssemblies ()) {
-				if (!files.Contains (asm.Location)) {
-					ComponentIndexFile c = new ComponentIndexFile (asm.Location);
-					index.Files.Add (c);
-					toupdate.Add (c);
+			foreach (TargetRuntime runtime in Runtime.SystemAssemblyService.GetTargetRuntimes ()) {
+				foreach (SystemAssembly asm in runtime.AssemblyContext.GetAssemblies ()) {
+					if (files.Add (asm.Location)) {
+						ComponentIndexFile c = new ComponentIndexFile (asm.Location);
+						index.Files.Add (c);
+						toupdate.Add (c);
+					}
+					if (monitor.IsCancelRequested)
+						return index;
 				}
-				if (monitor.IsCancelRequested)
-					return index;
 			}
 			
 			foreach (ComponentIndexFile ia in todelete) {
@@ -598,13 +582,18 @@ namespace MonoDevelop.DesignerSupport
 			
 			if (toupdate.Count > 0) {
 				monitor.BeginTask (GettextCatalog.GetString ("Looking for components..."), toupdate.Count);
-				foreach (ComponentIndexFile ia in toupdate) {
-					ia.Update ();
-					monitor.Step (1);
-					if (monitor.IsCancelRequested)
-						return index;
+				LoaderContext ctx = new LoaderContext ();
+				try {
+					foreach (ComponentIndexFile ia in toupdate) {
+						ia.Update (ctx);
+						monitor.Step (1);
+						if (monitor.IsCancelRequested)
+							return index;
+					}
+				} finally {
+					ctx.Dispose ();
+					monitor.EndTask ();
 				}
-				monitor.EndTask ();
 			}
 			
 			if (toupdate.Count > 0 || todelete.Count > 0)
@@ -622,7 +611,7 @@ namespace MonoDevelop.DesignerSupport
 		List<ComponentIndexFile> files = new List<ComponentIndexFile> ();
 		
 		static string ToolboxIndexFile {
-			get { return Path.Combine (PropertyService.ConfigPath, "ToolboxIndex.xml"); }
+			get { return UserProfile.Current.CacheDir.Combine ("ToolboxIndex.xml"); }
 		}
 		
 		internal static ComponentIndex Load ()
@@ -630,7 +619,7 @@ namespace MonoDevelop.DesignerSupport
 			if (!File.Exists (ToolboxIndexFile))
 				return new ComponentIndex ();
 			
-			XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
+			XmlDataSerializer ser = new XmlDataSerializer (IdeApp.Services.ProjectService.DataContext);
 			try {
 				using (StreamReader sr = new StreamReader (ToolboxIndexFile)) {
 					return (ComponentIndex) ser.Deserialize (sr, typeof(ComponentIndex));
@@ -645,7 +634,7 @@ namespace MonoDevelop.DesignerSupport
 		
 		public void Save ()
 		{
-			XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
+			XmlDataSerializer ser = new XmlDataSerializer (IdeApp.Services.ProjectService.DataContext);
 			try {
 				using (StreamWriter sw = new StreamWriter (ToolboxIndexFile)) {
 					ser.Serialize (sw, this, typeof(ComponentIndex));
@@ -660,7 +649,12 @@ namespace MonoDevelop.DesignerSupport
 		public ComponentIndexFile AddFile (string file)
 		{
 			ComponentIndexFile cf = new ComponentIndexFile (file);
-			cf.Update ();
+			LoaderContext ctx = new LoaderContext ();
+			try {
+				cf.Update (ctx);
+			} finally {
+				ctx.Dispose ();
+			}
 			if (cf.Components.Count == 0)
 				return null;
 			else {
@@ -732,9 +726,9 @@ namespace MonoDevelop.DesignerSupport
 			}
 		}
 
-		public void Update ()
+		public void Update (LoaderContext ctx)
 		{
-			IList<ItemToolboxNode> items = DesignerSupport.Service.ToolboxService.GetFileItems (fileName);
+			IList<ItemToolboxNode> items = DesignerSupport.Service.ToolboxService.GetFileItems (ctx, fileName);
 			
 			location = null;
 			this.timestamp = GetFileTimestamp (fileName);
@@ -742,7 +736,7 @@ namespace MonoDevelop.DesignerSupport
 			
 			// Set the location field only if this is a widget library
 			if (entries.Count > 0 && Runtime.SystemAssemblyService.GetPackageFromPath (fileName) != null)
-				location = Runtime.SystemAssemblyService.GetAssemblyFullName (fileName);
+				location = Runtime.SystemAssemblyService.DefaultAssemblyContext.GetAssemblyFullName (fileName, TargetFramework.Default);
 		}
 		
 		string GetFileTimestamp (string file)
@@ -752,23 +746,6 @@ namespace MonoDevelop.DesignerSupport
 		}
 	}
 	
-	[MonoDevelop.Core.Execution.AddinDependency ("MonoDevelop.DesignerSupport")] 
-	class ExternalFileLoader: RemoteProcessObject
-	{
-		public string LoadFile (string file)
-		{
-			Gtk.Application.Init ();
-			ToolboxService service = new ToolboxService ();
-			XmlDataSerializer ser = new XmlDataSerializer (Services.ProjectService.DataContext);
-			ToolboxList tl = new ToolboxList ();
-			IList<ItemToolboxNode> list = service.InternalGetFileItems (file);
-			tl.AddRange (list);
-			StringWriter sw = new StringWriter ();
-			ser.Serialize (sw, tl);
-			return sw.ToString ();
-		}
-	}
-	
 	public delegate void ToolboxConsumerChangedHandler (object sender, ToolboxConsumerChangedEventArgs e);
 	
 	public class ToolboxConsumerChangedEventArgs
@@ -822,4 +799,5 @@ namespace MonoDevelop.DesignerSupport
 			get { return consumer; }
 		}
 	}
+
 }
