@@ -104,7 +104,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		void UpdateBindings (Stetic.Component obj, IUnresolvedTypeDefinition cls)
 		{
-			if (targetObject == null)
+			if (targetObject == null || cls == null)
 				return;
 
 			// Remove signals for which there isn't a handler in the class
@@ -174,11 +174,11 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			var met = new DefaultUnresolvedMethod (cls, signal.Handler) {
 				Accessibility = Accessibility.Protected,
-				ReturnType = new DefaultUnresolvedTypeDefinition (signal.SignalDescriptor.HandlerReturnTypeName)
+				ReturnType = ReflectionHelper.ParseReflectionName (signal.SignalDescriptor.HandlerReturnTypeName)
 			};
 			foreach (Stetic.ParameterDescriptor pinfo in signal.SignalDescriptor.HandlerParameters)
-				met.Parameters.Add (new DefaultUnresolvedParameter (new DefaultUnresolvedTypeDefinition (pinfo.TypeName), pinfo.Name));
-			var resolvedCls = cls.Resolve (cls.ParsedFile.GetTypeResolveContext (TypeSystemService.GetCompilation (project), cls.Region.Begin)).GetDefinition ();
+				met.Parameters.Add (new DefaultUnresolvedParameter (ReflectionHelper.ParseReflectionName (pinfo.TypeName), pinfo.Name));
+			var resolvedCls = cls.Resolve (project).GetDefinition ();
 			CodeGenerationService.AddNewMember (resolvedCls, cls, met);
 		}
 		
@@ -215,7 +215,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			
 			IEditableTextFile editor = doc.GetContent<IEditableTextFile> ();
 			if (editor != null) {
-				var resolvedCls = cls.Resolve (cls.ParsedFile.GetTypeResolveContext (TypeSystemService.GetCompilation (project), cls.Region.Begin)).GetDefinition ();
+				var resolvedCls = cls.Resolve (project).GetDefinition ();
 				CodeGenerationService.AddNewMember (resolvedCls, cls, GetFieldCode (cls, obj, name));
 			}
 		}
@@ -223,7 +223,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		IUnresolvedField GetFieldCode (IUnresolvedTypeDefinition cls, Stetic.Component obj, string name)
 		{
 			return new DefaultUnresolvedField (cls, name) {
-				ReturnType = new DefaultUnresolvedTypeDefinition (obj.Type.ClassName),
+				ReturnType = ReflectionHelper.ParseReflectionName (obj.Type.ClassName),
 				Accessibility = Accessibility.Protected
 		};
 	}
@@ -269,7 +269,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				cls = matches [0];
 				className = cls.FullName;
 				targetObject.Name = className;
-				gproject.Save (true);
+				gproject.SaveWindow (true, targetObject.Name);
 				return cls;
 			}
 			
@@ -283,7 +283,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 							return null;
 						else {
 							targetObject.Name = className;
-							gproject.Save (true);
+							gproject.SaveWindow (true, targetObject.Name);
 							return gproject.FindClass (className);
 						}
 					}
