@@ -28,6 +28,7 @@
 
 
 using System;
+using System.Collections.Generic;
 using System.ComponentModel;
 
 using MonoDevelop.Core;
@@ -36,7 +37,6 @@ using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide.Commands;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Projects;
-using MonoDevelop.Projects.Dom;
 using MonoDevelop.DesignerSupport.Toolbox; 
 using MonoDevelop.DesignerSupport;
 
@@ -276,7 +276,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				codeBinder.UpdateField (args.Component, args.OldName);
 			}
 			catch (Exception ex) {
-				MessageService.ShowException (ex);
+				LoggingService.LogInternalError (ex);
 			}
 		}
 		
@@ -350,7 +350,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				FileService.MoveFile (oldBuildFile, newBuildFile);
 			}
 			
-			gproject.Save (true);
+			gproject.SaveWindow (true, window.RootWidget.Name);
 		}
 		
 		public override bool IsDirty {
@@ -366,13 +366,13 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		public override void JumpToSignalHandler (Stetic.Signal signal)
 		{
-			IType cls = codeBinder.GetClass ();
+			var cls = codeBinder.GetClass ();
 			if (cls == null)
 				return;
-			foreach (IMethod met in cls.Methods) {
+			foreach (var met in cls.Methods) {
 				if (met.Name == signal.Handler) {
 					ShowPage (0);
-					JumpTo (met.Location.Line, met.Location.Column);
+					JumpTo (met.Region.BeginLine, met.Region.BeginColumn);
 					break;
 				}
 			}
@@ -506,7 +506,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				if (asm == null)
 					return;
 				if (gproject.Project.AssemblyContext.GetPackagesFromFullName (asm).Length > 0) {
-					pref = new ProjectReference (ReferenceType.Gac, asm);
+					pref = new ProjectReference (ReferenceType.Package, asm);
 				} else {
 					asm = gproject.Project.AssemblyContext.GetAssemblyLocation (asm, gproject.Project.TargetFramework);
 					pref = new ProjectReference (ReferenceType.Assembly, asm);
@@ -602,6 +602,11 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			return GuiBuilderDocumentOutline.Instance;
 		}
+		
+		IEnumerable<Gtk.Widget> MonoDevelop.DesignerSupport.IOutlinedDocument.GetToolbarWidgets ()
+		{
+			return null;
+		}
 
 		void MonoDevelop.DesignerSupport.IOutlinedDocument.ReleaseOutlineWidget ()
 		{
