@@ -7,12 +7,12 @@ namespace Stetic.Wrapper {
 		public override void Wrap (object obj, bool initialized)
 		{
 			base.Wrap (obj, initialized);
-			((Gtk.Range)Wrapped).Adjustment.AddNotification (AdjustmentNotifyHandler);
+			NotifyWorkaround.AddNotification (((Gtk.Range)Wrapped).Adjustment, AdjustmentNotifyHandler);
 		}
 		
 		public override void Dispose ()
 		{
-			((Gtk.Range)Wrapped).Adjustment.RemoveNotification (AdjustmentNotifyHandler);
+			NotifyWorkaround.RemoveNotification (((Gtk.Range)Wrapped).Adjustment, AdjustmentNotifyHandler);
 			base.Dispose ();
 		}
 
