@@ -129,16 +129,19 @@ namespace Stetic {
 				}
 			}
 			
-			internal ToolboxItemInfo GetToolboxItem (string name)
+			internal ToolboxItemInfo GetToolboxItem (string name, string asmName)
 			{
 				XmlDocument doc = ObjectsDocument;
 				if (doc != null) {
 					XmlElement elem = (XmlElement) doc.SelectSingleNode ("/objects/object[@type='" + name + "']");
+					if (elem == null)
+						elem = (XmlElement) doc.SelectSingleNode ("/objects/object[@type='" + name + "," + asmName + "']");
 					if (elem != null) {
 						ToolboxItemInfo info = new ToolboxItemInfo (elem.GetAttribute ("base-type"));
 						info.PaletteCategory = elem.GetAttribute ("palette-category");
 						return info;
 					}
+					
 				}
 				return null;
 			}
@@ -283,7 +286,7 @@ namespace Stetic {
 			try {
 				EmbeddedResource res = GetResource (adef, "gui.stetic");
 				if (res != null) {
-					MemoryStream stream = new MemoryStream (res.Data);
+					MemoryStream stream = new MemoryStream (res.GetResourceData ());
 					doc = new XmlDocument ();
 					using (stream)
 						doc.Load (stream);
@@ -326,7 +329,7 @@ namespace Stetic {
 			public string PaletteCategory;
 		}
 
-		ToolboxItemInfo GetToolboxItemInfo (AssemblyResolver resolver, string baseDirectory, AssemblyDefinition asm, TypeDefinition tdef)
+		ToolboxItemInfo GetToolboxItemInfo (AssemblyResolver resolver, string baseDirectory, AssemblyDefinition asm, TypeDefinition tdef, bool checkBaseType)
 		{
 			if (tdef == null)
 				return null;
@@ -335,16 +338,10 @@ namespace Stetic {
 			string category = "General";
 			
 			foreach (CustomAttribute attr in tdef.CustomAttributes) {
-				switch (attr.Constructor.DeclaringType.FullName) {
+				switch (attr.AttributeType.FullName) {
 				case "System.ComponentModel.ToolboxItemAttribute":
-					try {
-						attr.Resolve ();
-					} catch {
-						// Ignore
-						return null;
-					}
-					if (attr.ConstructorParameters.Count > 0) {
-						object param = attr.ConstructorParameters [0];
+					if (attr.ConstructorArguments.Count > 0) {
+						object param = attr.ConstructorArguments [0].Value;
 						if (param == null)
 							return null;
 						else if (param.GetType () == typeof (bool)) {
@@ -352,21 +349,15 @@ namespace Stetic {
 								info = new ToolboxItemInfo ("Gtk.Widget");
 							else 
 								return null;
-						} else if (param.GetType () == typeof (System.Type))
+						} else if (param.GetType () == typeof (TypeReference))
 							info = new ToolboxItemInfo ("Gtk.Widget");
 						else
 							return null;
 					}
 					break;
 				case "System.ComponentModel.CategoryAttribute":
-					try {
-						attr.Resolve ();
-					} catch {
-						// Ignore
-						return null;
-					}
-					if (attr.ConstructorParameters.Count > 0) {
-						object param = attr.ConstructorParameters [0];
+					if (attr.ConstructorArguments.Count > 0) {
+						object param = attr.ConstructorArguments [0].Value;
 						if (param.GetType () == typeof (string))
 							category = (string) param;
 					}
@@ -374,16 +365,15 @@ namespace Stetic {
 				default:
 					continue;
 				}
-
 			}
 
-			if (info == null && tdef.BaseType != null) {
+			if (info == null && checkBaseType && tdef.BaseType != null) {
 				string baseName = tdef.BaseType.FullName;
 
 				foreach (AssemblyNameReference aref in asm.MainModule.AssemblyReferences) {
 					LibraryInfo libInfo = GetInfo (resolver, aref.FullName, baseDirectory);
 					if (libInfo != null && libInfo.HasWidgets) {
-						ToolboxItemInfo binfo = libInfo.GetToolboxItem (baseName);
+						ToolboxItemInfo binfo = libInfo.GetToolboxItem (baseName, aref.Name);
 						if (binfo != null) {
 							info = new ToolboxItemInfo (baseName);
 							category = binfo.PaletteCategory;
@@ -421,7 +411,7 @@ namespace Stetic {
 			return group;
 		}
 		
-		void AddProperty (PropertyDefinition prop, string cat, XmlElement obj)
+		void AddProperty (PropertyDefinition prop, string cat, bool translatable, XmlElement obj)
 		{
 			XmlElement groups = obj ["itemgroups"];
 			if (groups == null) {
@@ -429,8 +419,10 @@ namespace Stetic {
 				obj.AppendChild (groups);
 			}
 			
-			XmlElement group = GetItemGroup (groups, cat, prop.DeclaringType.FullName + " Properties");
+			XmlElement group = GetItemGroup (groups, cat, prop.DeclaringType.Name + " Properties");
 			XmlElement elem = group.OwnerDocument.CreateElement ("property");
+			if (translatable)
+				elem.SetAttribute ("translatable", "yes");
 			elem.SetAttribute ("name", prop.Name);
 			group.AppendChild (elem);
 		}
@@ -451,6 +443,7 @@ namespace Stetic {
 			"System.Double",
 			"System.DateTime",
 			"System.String",
+			"System.String[]",
 			"System.TimeSpan",
 			"Gtk.Adjustment",
 		};
@@ -460,28 +453,36 @@ namespace Stetic {
 			foreach (PropertyDefinition prop in tdef.Properties) {
 				if (prop.GetMethod == null || !prop.GetMethod.IsPublic || prop.SetMethod == null || !prop.SetMethod.IsPublic)
 					continue;
+				
 				else if (Array.IndexOf (supported_types, prop.PropertyType.FullName) < 0)
 					continue;
+				
 				bool browsable = true;
+				bool translatable = false;
 				string category = String.Empty;
 				foreach (CustomAttribute attr in prop.CustomAttributes) {
 					switch (attr.Constructor.DeclaringType.FullName) {
 					case "System.ComponentModel.BrowsableAttribute":
-						attr.Resolve ();
-						if (attr.ConstructorParameters.Count > 0) {
-							object param = attr.ConstructorParameters [0];
+						if (attr.ConstructorArguments.Count > 0) {
+							object param = attr.ConstructorArguments [0].Value;
 							if (param.GetType () == typeof (bool))
 								browsable = (bool) param;
 						}
 						break;
 					case "System.ComponentModel.CategoryAttribute":
-						attr.Resolve ();
-						if (attr.ConstructorParameters.Count > 0) {
-							object param = attr.ConstructorParameters [0];
+						if (attr.ConstructorArguments.Count > 0) {
+							object param = attr.ConstructorArguments [0].Value;
 							if (param.GetType () == typeof (string))
 								category = (string) param;
 						}
 						break;
+					case "System.ComponentModel.LocalizableAttribute":
+						if (attr.ConstructorArguments.Count > 0) {
+							object param = attr.ConstructorArguments [0].Value;
+							if (param.GetType () == typeof (bool))
+								translatable = (bool) param;
+						}
+						break;
 					default:
 						continue;
 					}
@@ -489,7 +490,7 @@ namespace Stetic {
 						break;
 				}
 				if (browsable)
-					AddProperty (prop, category, obj);
+					AddProperty (prop, category, translatable, obj);
 			}
 		}
 	
@@ -501,7 +502,7 @@ namespace Stetic {
 				obj.AppendChild (groups);
 			}
 			
-			XmlElement group = GetItemGroup (groups, cat, ev.DeclaringType.FullName + " Signals");
+			XmlElement group = GetItemGroup (groups, cat, ev.DeclaringType.Name + " Signals");
 			XmlElement elem = group.OwnerDocument.CreateElement ("signal");
 			elem.SetAttribute ("name", ev.Name);
 			group.AppendChild (elem);
@@ -509,7 +510,6 @@ namespace Stetic {
 
 		void AddEvents (TypeDefinition tdef, XmlElement obj)
 		{
-			
 			foreach (EventDefinition ev in tdef.Events) {
 				if (ev.AddMethod == null || !ev.AddMethod.IsPublic)
 					continue;
@@ -518,17 +518,15 @@ namespace Stetic {
 				foreach (CustomAttribute attr in ev.CustomAttributes) {
 					switch (attr.Constructor.DeclaringType.FullName) {
 					case "System.ComponentModel.BrowsableAttribute":
-						attr.Resolve ();
-						if (attr.ConstructorParameters.Count > 0) {
-							object param = attr.ConstructorParameters [0];
+						if (attr.ConstructorArguments.Count > 0) {
+							object param = attr.ConstructorArguments [0].Value;
 							if (param.GetType () == typeof (bool))
 								browsable = (bool) param;
 						}
 						break;
 					case "System.ComponentModel.CategoryAttribute":
-						attr.Resolve ();
-						if (attr.ConstructorParameters.Count > 0) {
-							object param = attr.ConstructorParameters [0];
+						if (attr.ConstructorArguments.Count > 0) {
+							object param = attr.ConstructorArguments [0].Value;
 							if (param.GetType () == typeof (string))
 								category = (string) param;
 						}
@@ -543,31 +541,76 @@ namespace Stetic {
 					AddEvent (ev, category, obj);
 			}
 		}
+
+		void AddObject (TypeDefinition tdef, Dictionary<TypeDefinition,ToolboxItemInfo> localObjects, AssemblyResolver resolver, string basePath, AssemblyDefinition adef)
+		{
+			if (tdef.IsAbstract || !tdef.IsClass) 
+				return;
+
+			ToolboxItemInfo tbinfo = GetToolboxItemInfo (resolver, basePath, adef, tdef, true);
+			if (tbinfo == null)
+				return;
+			
+			localObjects [tdef] = tbinfo;
+
+			foreach (var nestedType in tdef.NestedTypes)
+				AddObject (nestedType, localObjects, resolver, basePath, adef);
+		}
 	
 		void AddObjects (XmlDocument doc, AssemblyResolver resolver, string basePath, AssemblyDefinition adef)
 		{
+			Dictionary<TypeDefinition,ToolboxItemInfo> localObjects = new Dictionary<TypeDefinition, ToolboxItemInfo> ();
+			
 			foreach (TypeDefinition tdef in adef.MainModule.Types) {
-				if (tdef.IsAbstract || !tdef.IsClass) 
-					continue;
-
-				ToolboxItemInfo tbinfo = GetToolboxItemInfo (resolver, basePath, adef, tdef);
-				if (tbinfo == null)
-					continue;
-
+				AddObject (tdef, localObjects, resolver, basePath, adef);
+			}
+			
+			foreach (KeyValuePair<TypeDefinition,ToolboxItemInfo> item in localObjects) {
+				TypeDefinition tdef = item.Key;
+				ToolboxItemInfo tbinfo = item.Value;
 				XmlElement elem = doc.CreateElement ("object");
 				elem.SetAttribute ("type", tdef.FullName);
 				elem.SetAttribute ("allow-children", "false");
-				elem.SetAttribute ("base-type", tbinfo.BaseType);
 				elem.SetAttribute ("palette-category", tbinfo.PaletteCategory);
 				if (tdef.IsNotPublic)
 					elem.SetAttribute ("internal", "true");
 				doc.DocumentElement.AppendChild (elem);
-				AddProperties (tdef, elem);
-				AddEvents (tdef, elem);
+				
+				TypeDefinition curDef = tdef;
+				while (curDef != null && curDef.FullName != tbinfo.BaseType) {
+					if (curDef != tdef && localObjects.ContainsKey (curDef)) {
+						tbinfo.BaseType = curDef.FullName;
+						break;
+					}
+					else if (curDef.Module.Assembly.Name.Name == "gtk-sharp") {
+						tbinfo.BaseType = curDef.FullName;
+						break;
+					}
+					else if (curDef != tdef && GetToolboxItemInfo (resolver, basePath, curDef.Module.Assembly, curDef, false) != null) {
+						tbinfo.BaseType = curDef.FullName;
+						break;
+					}
+					if (curDef.Module.Assembly != adef) {
+						
+						LibraryInfo li = Refresh (resolver, curDef.Module.FullyQualifiedName, basePath);
+						if (li.HasWidgets && li.GetToolboxItem (curDef.FullName, curDef.Module.Assembly.Name.Name) != null) {
+							tbinfo.BaseType = curDef.FullName;
+							break;
+						}
+					}
+					AddProperties (curDef, elem);
+					AddEvents (curDef, elem);
+					if (curDef.BaseType != null && curDef.BaseType.FullName != tbinfo.BaseType)
+						curDef = FindTypeDefinition (resolver, adef, basePath, curDef.BaseType.FullName);
+					else
+						curDef = null;
+				}
+				
+				elem.SetAttribute ("base-type", tbinfo.BaseType);
 			}
 		}
 
-		XmlDocument GetObjectsDoc (AssemblyResolver resolver, AssemblyDefinition adef, string path)
+		XmlDocument GetObjectsDoc (AssemblyResolver resolver, AssemblyDefinition adef, string path, string baseDirectory)
 		{
 			XmlDocument doc = null;
 			bool isMainLib = Path.GetFileName (path) == "libstetic.dll";
@@ -575,7 +618,7 @@ namespace Stetic {
 			try {
 				EmbeddedResource res = GetResource (adef, "objects.xml");
 				if (res != null) {
-					MemoryStream stream = new MemoryStream (res.Data);
+					MemoryStream stream = new MemoryStream (res.GetResourceData ());
 					doc = new XmlDocument ();
 					using (stream)
 						doc.Load (stream);
@@ -584,7 +627,7 @@ namespace Stetic {
 				if (resolver == null)
 					resolver = new AssemblyResolver (null);
 
-				string baseDirectory = Path.GetDirectoryName (path);
+				baseDirectory = baseDirectory ?? Path.GetDirectoryName (path);
 				
 				if (!isMainLib) {
 					// Make sure all referenced assemblies are up to date.
@@ -594,7 +637,7 @@ namespace Stetic {
 				}
 			
 				if (doc == null) {
-					Hashtable visited = new Hashtable ();
+//					Hashtable visited = new Hashtable ();
 					foreach (AssemblyNameReference aref in adef.MainModule.AssemblyReferences) {
 						if (aref.Name != "gtk-sharp") {
 							LibraryInfo info = GetInfo (resolver, aref.FullName, baseDirectory);
@@ -669,8 +712,8 @@ namespace Stetic {
 			info.Timestamp = File.GetLastWriteTime (info.File).ToUniversalTime ();
 			info.Guid = Guid.NewGuid ();
 			Save ();
-			AssemblyDefinition adef = AssemblyFactory.GetAssembly (info.File);
-			XmlDocument objects = GetObjectsDoc (resolver, adef, info.File);
+			AssemblyDefinition adef = AssemblyDefinition.ReadAssembly (info.File);
+			XmlDocument objects = GetObjectsDoc (resolver, adef, info.File, baseDirectory);
 			if (objects != null) {
 				info.ObjectsDocument = objects;
 				XmlDocument gui = GetGuiDoc (adef);
@@ -723,5 +766,32 @@ namespace Stetic {
 			return new LibraryCache ();
 		}
 
+		internal TypeDefinition FindTypeDefinition (AssemblyResolver resolver, AssemblyDefinition assembly, string basePath, string fullName)
+		{
+			TypeDefinition t = FindTypeDefinition (new Hashtable (), resolver, basePath, assembly, fullName);
+			return t;
+		}
+		
+		TypeDefinition FindTypeDefinition (Hashtable visited, AssemblyResolver resolver, string basePath, AssemblyDefinition asm, string fullName)
+		{
+			if (visited.Contains (asm))
+				return null;
+				
+			visited [asm] = asm;
+			
+			TypeDefinition cls = asm.MainModule.GetType (fullName);
+			if (cls != null)
+				return cls;
+			
+			foreach (AssemblyNameReference aref in asm.MainModule.AssemblyReferences) {
+				AssemblyDefinition basm = resolver.Resolve (aref, basePath);
+				if (basm != null) {
+					cls = basm.MainModule.GetType (fullName);
+					if (cls != null)
+						return cls;
+				}
+			}
+			return null;
+		}
 	}
 }
