@@ -277,8 +277,11 @@ namespace Stetic {
 			}
 
 			foreach (WidgetData wd in topLevels) {
-				if (wd.Widget != null)
-					wd.Widget.Destroy ();
+				if (wd.Widget != null) {
+					try {
+						wd.Widget.Destroy ();
+					} catch (Exception e) { Console.WriteLine (e); }
+				}
 			}
 
 			selection = null;
@@ -335,7 +338,7 @@ namespace Stetic {
 					string dir = libElem.GetAttribute ("path");
 					if (dir.Length > 0) {
 						if (basePath != null && !Path.IsPathRooted (dir)) {
-							dir = Path.Combine (basePath, dir);
+							dir = FromOSAgnosticRelPath (Path.Combine (basePath, dir));
 							if (Directory.Exists (dir))
 								dir = Path.GetFullPath (dir);
 						}
@@ -348,7 +351,7 @@ namespace Stetic {
 					string libname = libElem.GetAttribute ("name");
 					if (libname.EndsWith (".dll") || libname.EndsWith (".exe")) {
 						if (basePath != null && !Path.IsPathRooted (libname)) {
-							libname = Path.Combine (basePath, libname);
+							libname = FromOSAgnosticRelPath (Path.Combine (basePath, libname));
 							if (File.Exists (libname))
 								libname = Path.GetFullPath (libname);
 						}
@@ -397,17 +400,28 @@ namespace Stetic {
 			
 			return data.Widget;
 		}
-		
+
 		public void Save (string fileName)
 		{
 			this.fileName = fileName;
 			XmlDocument doc = Write (false);
 			
-			XmlTextWriter writer = null;
+			XmlWriter writer = null;
 			try {
+				//Always normalize line endings to \n so that they remain consistent.
+				//Else, because of the way subdocument are persisted unmodified, we could
+				//end up with mixed line endings.
+				//Also, string arrays are stored in plain text data with \n separators,
+				//and break badly if we have \r\n.
+				var settings = new XmlWriterSettings () {
+					NewLineHandling = NewLineHandling.Replace,
+					Encoding = System.Text.Encoding.UTF8,
+					NewLineChars = "\n",
+					Indent = true,
+				};
+
 				// Write to a temporary file first, just in case something fails
-				writer = new XmlTextWriter (fileName + "~", System.Text.Encoding.UTF8);
-				writer.Formatting = Formatting.Indented;
+				writer = XmlTextWriter.Create (fileName + "~", settings);
 				doc.Save (writer);
 				writer.Close ();
 				
@@ -419,6 +433,18 @@ namespace Stetic {
 			}
 		}
 		
+		//in the cases where a path is relative, we store it using / as a path separator
+		//so the .stetic file doesn't unnecessarily differ between Windows & Linux/Mac
+		string ToOSAgnosticRelPath (string s)
+		{
+			return s.Replace ('\\', '/');
+		}
+
+		string FromOSAgnosticRelPath (string s)
+		{
+			return s.Replace ('/', Path.DirectorySeparatorChar);
+		}
+
 		XmlDocument Write (bool includeUndoInfo)
 		{
 			XmlDocument doc = new XmlDocument ();
@@ -451,7 +477,7 @@ namespace Stetic {
 					foreach (string dir in resolver.Directories) {
 						XmlElement dirElem = doc.CreateElement ("assembly-directory");
 						if (basePath != null)
-							dirElem.SetAttribute ("path", AbsoluteToRelativePath (basePath, dir));
+							dirElem.SetAttribute ("path", ToOSAgnosticRelPath (AbsoluteToRelativePath (basePath, dir)));
 						else
 							dirElem.SetAttribute ("path", dir);
 						toplevel.AppendChild (dirElem);
@@ -463,7 +489,7 @@ namespace Stetic {
 					XmlElement libElem = doc.CreateElement ("widget-library");
 					if (wlib.EndsWith (".dll") || wlib.EndsWith (".exe")) {
 						if (basePath != null)
-							libName = AbsoluteToRelativePath (basePath, wlib);
+							libName = ToOSAgnosticRelPath (AbsoluteToRelativePath (basePath, wlib));
 					}
 
 					libElem.SetAttribute ("name", libName);
@@ -500,7 +526,8 @@ namespace Stetic {
 			}
 			
 			// Remove undo annotations from the xml document
-			CleanUndoData (doc.DocumentElement);
+			if (!includeUndoInfo)
+				CleanUndoData (doc.DocumentElement);
 			
 			return doc;
 		}
@@ -549,6 +576,30 @@ namespace Stetic {
 				throw new InvalidOperationException ("Component not found: " + name);
 			return null;
 		}
+
+		HashSet<string> modifiedWidgets = new HashSet<string> ();
+		bool allModified;
+
+		public void SetWidgetModified (string widgetName)
+		{
+			modifiedWidgets.Add (widgetName);
+		}
+
+		public void SetAllWidgetsModified ()
+		{
+			allModified = true;
+		}
+
+		public void ResetModifiedWidgetFlags ()
+		{
+			allModified = false;
+			modifiedWidgets.Clear ();
+		}
+
+		internal bool IsWidgetModified (string name)
+		{
+			return allModified || modifiedWidgets.Contains (name);
+		}
 		
 		public object AddNewWidget (string type, string name)
 		{
@@ -682,27 +733,35 @@ namespace Stetic {
 		{
 			if (loading) return;
 			
-			if (tempDoc != null) {
-				if (frontend != null)
-					frontend.NotifyProjectReloading ();
-				if (ProjectReloading != null)
-					ProjectReloading (this, EventArgs.Empty);
-				
-				ProjectIconFactory icf = iconFactory;
-				
-				Read (tempDoc);
+			if (tempDoc != null)
+				LoadStatus (tempDoc);
+		}
+		
+		public object SaveStatus ()
+		{
+			return Write (true);
+		}
+		
+		public void LoadStatus (object ob)
+		{
+			if (frontend != null)
+				frontend.NotifyProjectReloading ();
+			if (ProjectReloading != null)
+				ProjectReloading (this, EventArgs.Empty);
+			
+			ProjectIconFactory icf = iconFactory;
+			
+			Read ((XmlDocument)ob);
 
-				// Reuse the same icon factory, since a registry change has no effect to it
-				// and it may be inherited from another project
-				iconFactory = icf;
-				
-				tempDoc = null;
-				if (frontend != null)
-					frontend.NotifyProjectReloaded ();
-				if (ProjectReloaded != null)
-					ProjectReloaded (this, EventArgs.Empty);
-				NotifyComponentTypesChanged ();
-			}
+			// Reuse the same icon factory, since a registry change has no effect to it
+			// and it may be inherited from another project
+			iconFactory = icf;
+			
+			if (frontend != null)
+				frontend.NotifyProjectReloaded ();
+			if (ProjectReloaded != null)
+				ProjectReloaded (this, EventArgs.Empty);
+			NotifyComponentTypesChanged ();
 		}
 		
 		public void Reload ()
@@ -751,6 +810,7 @@ namespace Stetic {
 
 		public void AddWidget (Gtk.Widget widget)
 		{
+			modifiedWidgets.Add (widget.Name);
 			if (!typeof(Gtk.Container).IsInstanceOfType (widget))
 				throw new System.ArgumentException ("widget", "Only containers can be top level widgets");
 			topLevels.Add (new WidgetData (null, null, widget));
@@ -949,7 +1009,7 @@ namespace Stetic {
 			m.Popup ();
 		}
 		
-		internal static string AbsoluteToRelativePath (string baseDirectoryPath, string absPath)
+		static string AbsoluteToRelativePath (string baseDirectoryPath, string absPath)
 		{
 			if (!Path.IsPathRooted (absPath))
 				return absPath;
@@ -1070,4 +1130,17 @@ namespace Stetic {
 			}
 		}
 	}
+
+	public static class EncodingUtility
+	{
+		static System.Text.UTF8Encoding utf8NoBom;
+
+		/// This is so we can write XML files on .NET in a compatible way with Mono,
+		/// since .NET's XmlWriter writes the BOM but Mono's does not.
+		public static System.Text.UTF8Encoding UTF8NoBom {
+			get {
+				return utf8NoBom ?? (utf8NoBom = new System.Text.UTF8Encoding (false));
+			}
+		}
+	}
 }
