@@ -74,11 +74,19 @@ namespace MonoDevelop.PackageManagement.Tests
 		}
 
 		void CreateViewModel (FakeRegisteredPackageRepositories registeredPackageRepositories)
+		{
+			CreateSolution ();
+			CreateViewModel (registeredPackageRepositories, solution);
+		}
+
+		void CreateViewModel (
+			FakeRegisteredPackageRepositories registeredPackageRepositories,
+			FakePackageManagementSolution solution)
 		{
 			taskFactory = new FakeTaskFactory ();
 			var packageViewModelFactory = new FakePackageViewModelFactory ();
 			recentPackageRepository = new FakeRecentPackageRepository ();
-			solution = new FakePackageManagementSolution ();
+
 
 			viewModel = new AvailablePackagesViewModel (
 				solution,
@@ -88,6 +96,11 @@ namespace MonoDevelop.PackageManagement.Tests
 				taskFactory);
 		}
 
+		void CreateSolution ()
+		{
+			solution = new FakePackageManagementSolution ();
+		}
+
 		void CreateExceptionThrowingRegisteredPackageRepositories ()
 		{
 			exceptionThrowingRegisteredPackageRepositories = new ExceptionThrowingRegisteredPackageRepositories ();
@@ -1212,5 +1225,31 @@ namespace MonoDevelop.PackageManagement.Tests
 			};
 			PackageCollectionAssert.AreEqual (expectedPackages, viewModel.PackageViewModels);
 		}
+
+		[Test]
+		public void ReadPackages_TwoSolutionPackagesAndSecondOneThrowsExceptionWhenBeingReturned_ExceptionHandled ()
+		{
+			CreateSolution ();
+			var solutionPackageRepository = new ExceptionThrowingSolutionPackageRepository {
+				ThrowExceptionOnIteration = 1
+			};
+			solution.SolutionPackageRepository = solutionPackageRepository;
+			CreateRegisteredPackageRepositories ();
+			CreateViewModel (registeredPackageRepositories, solution);
+			AddOnePackageSourceToRegisteredSources ();
+			var package1 = new FakePackage ("A", "1.0.0.0");
+			var package2 = new FakePackage ("B", "0.3.0.0");
+			var packages = new [] {
+				package1, package2
+			};
+			registeredPackageRepositories.FakeActiveRepository.FakePackages.AddRange (packages);
+			AddPackageToSolution ("Aa", "1.0.0.0");
+			AddPackageToSolution ("Bb", "1.0.0.0");
+
+			viewModel.ReadPackages ();
+			Assert.DoesNotThrow (() => CompleteReadPackagesTask ());
+
+			PackageCollectionAssert.AreEqual (packages, viewModel.PackageViewModels);
+		}
 	}
 }
