@@ -33,6 +33,7 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Assemblies;
 using MonoDevelop.PackageManagement.Tests.Helpers;
 using MonoDevelop.Projects;
+using MonoDevelop.Projects.Formats.MSBuild;
 using NuGet;
 using NUnit.Framework;
 
@@ -113,6 +114,13 @@ namespace MonoDevelop.PackageManagement.Tests
 			Assert.That (project.ImportsRemoved, Contains.Item (expectedImportRemoved));
 		}
 
+		MSBuildProject CreateMSBuildProject (string xml)
+		{
+			var msbuildProject = new MSBuildProject ();
+			msbuildProject.Document.LoadXml (xml);
+			return msbuildProject;
+		}
+
 		[Test]
 		public void Root_NewInstanceCreated_ReturnsProjectDirectory ()
 		{
@@ -1123,6 +1131,93 @@ namespace MonoDevelop.PackageManagement.Tests
 
 			AssertImportRemoved (@"packages\Foo.0.1\build\Foo.targets");
 		}
+
+		[Test]
+		public void RemoveImport_ImportAlreadyAddedToBottomOfProject_ImportRemovedEventIsFired ()
+		{
+			CreateTestProject (@"d:\projects\MyProject\MyProject\MyProject.csproj");
+			CreateProjectSystem (project);
+			string targetPath = @"d:\projects\MyProject\packages\Foo.0.1\build\Foo.targets".ToNativePath ();
+			projectSystem.AddImport (targetPath, ProjectImportLocation.Bottom);
+			const string expectedImportRemoved = @"..\packages\Foo.0.1\build\Foo.targets";
+			DotNetProjectImportEventArgs eventArgs = null;
+			projectSystem.PackageManagementEvents.ImportRemoved += (sender, e) => {
+				eventArgs = e;
+			};
+
+			projectSystem.RemoveImport (targetPath);
+
+			AssertImportRemoved (expectedImportRemoved);
+			Assert.IsFalse (project.IsProjectBuilderDisposed);
+			Assert.AreEqual (project, eventArgs.Project);
+			Assert.AreEqual (expectedImportRemoved, eventArgs.Import);
+		}
+
+		[Test]
+		public void RemoveImport_ProjectHasNuGetImportTargetAndSingleImportIsBeingRemoved_NuGetImportTargetIsRemoved ()
+		{
+			CreateTestProject (@"d:\projects\MyProject\MyProject.csproj");
+			CreateProjectSystem (project);
+			string targetPath = @"d:\projects\MyProject\packages\Foo.0.1\build\Foo.targets".ToNativePath ();
+			projectSystem.AddImport (targetPath, ProjectImportLocation.Bottom);
+			MSBuildProject msbuildProject = CreateMSBuildProject (
+				"<Project ToolsVersion=\"12.0\" DefaultTargets=\"Build\" xmlns=\"http://schemas.microsoft.com/developer/msbuild/2003\">\r\n" +
+				"  <Target Name=\"EnsureNuGetPackageBuildImports\" BeforeTargets=\"PrepareForBuild\">\r\n" +
+				"    <PropertyGroup>\r\n" +
+				"      <ErrorText>Error.</ErrorText>\r\n" +
+				"    </PropertyGroup>\r\n" +
+				"    <Error Condition=\"!Exists('packages\\Foo.0.1\\build\\Foo.targets')\" Text=\"$([System.String]::Format('$(ErrorText)', 'packages\\Foo.0.1\\build\\Foo.targets'))\" />\r\n" +
+				"  </Target>\r\n" +
+				"</Project>");
+			int targetCountBeforeSave = msbuildProject.Targets.Count ();
+			project.SaveAction = () => {
+				var msbuildExtension = new PackageManagementMSBuildExtension ();
+				msbuildExtension.SaveProject (null, null, msbuildProject);
+			};
+
+			projectSystem.RemoveImport (targetPath);
+
+			AssertImportRemoved (@"packages\Foo.0.1\build\Foo.targets");
+			Assert.AreEqual (1, targetCountBeforeSave);
+			Assert.AreEqual (0, msbuildProject.Targets.Count ());
+		}
+
+		[Test]
+		public void AddReference_AddReferenceToNUnitFramework_ReferenceAddingEventIsFired ()
+		{
+			CreateTestProject ();
+			CreateProjectSystem (project);
+			ProjectReference referenceBeingAdded = null;
+			projectSystem.PackageManagementEvents.ReferenceAdding += (sender, e) => {
+				referenceBeingAdded = e.Reference;
+			};
+
+			string fileName = @"d:\projects\packages\nunit\nunit.framework.dll".ToNativePath ();
+			projectSystem.AddReference (fileName, null);
+
+			Assert.AreEqual (fileName, referenceBeingAdded.HintPath);
+			Assert.IsTrue (referenceBeingAdded.LocalCopy);
+		}
+
+		[Test]
+		public void RemoveReference_RemoveReferenceToNUnitFramework_ReferenceRemovingEventIsFired ()
+		{
+			CreateTestProject ();
+			CreateProjectSystem (project);
+			string fileName = @"d:\projects\packages\nunit\nunit.framework.dll".ToNativePath ();
+			ProjectHelper.AddReference (project, fileName);
+			ProjectReference referenceBeingRemoved = null;
+			bool projectIsSaved = false;
+			projectSystem.PackageManagementEvents.ReferenceRemoving += (sender, e) => {
+				referenceBeingRemoved = e.Reference;
+				projectIsSaved = project.IsSaved;
+			};
+
+			projectSystem.RemoveReference (fileName);
+
+			Assert.AreEqual (fileName, referenceBeingRemoved.HintPath);
+			Assert.IsFalse (projectIsSaved);
+		}
 	}
 }
 
