@@ -26,28 +26,33 @@
 
 using System;
 using System.Collections.Generic;
+using System.IO;
+using System.Linq;
 using ICSharpCode.PackageManagement;
 using NuGet;
 using NUnit.Framework;
 using MonoDevelop.PackageManagement.Tests.Helpers;
+using MonoDevelop.Projects;
 
 namespace MonoDevelop.PackageManagement.Tests
 {
 	[TestFixture]
 	public class UpdatePackageActionTests
 	{
-		UpdatePackageAction action;
+		TestableUpdatePackageAction action;
 		PackageManagementEvents packageManagementEvents;
 		FakePackageManagementProject fakeProject;
 		UpdatePackageHelper updatePackageHelper;
 		FakeFileRemover fileRemover;
+		List<PackageOperationMessage> messagesLogged;
+		FakeFileService fileService;
 
 		void CreateSolution ()
 		{
 			packageManagementEvents = new PackageManagementEvents ();
 			fakeProject = new FakePackageManagementProject ();
 			fileRemover = new FakeFileRemover ();
-			action = new UpdatePackageAction (fakeProject, packageManagementEvents, fileRemover);
+			action = new TestableUpdatePackageAction (fakeProject, packageManagementEvents, fileRemover);
 			updatePackageHelper = new UpdatePackageHelper (action);
 		}
 
@@ -63,6 +68,25 @@ namespace MonoDevelop.PackageManagement.Tests
 			action.Operations = operations;
 		}
 
+		void RecordMessagesLogged ()
+		{
+			messagesLogged = new List<PackageOperationMessage> ();
+			packageManagementEvents.PackageOperationMessageLogged += (sender, e) => messagesLogged.Add (e.Message);
+		}
+
+		void AssertMessageIsLogged (string expectedMessage)
+		{
+			List<string> messages = messagesLogged.Select (m => m.ToString ()).ToList ();
+
+			Assert.That (messages, Contains.Item (expectedMessage));
+		}
+
+		IOpenPackageReadMeMonitor CreateReadMeMonitor (string packageId)
+		{
+			fileService = new FakeFileService (fakeProject.FakeDotNetProject);
+			return new OpenPackageReadMeMonitor (packageId, fakeProject, fileService);
+		}
+
 		[Test]
 		public void Execute_PackageAndRepositoryPassed_PackageIsUpdated ()
 		{
@@ -359,7 +383,7 @@ namespace MonoDevelop.PackageManagement.Tests
 		{
 			CreateSolution ();
 			action.UpdateIfPackageDoesNotExistInProject = false;
-			var expectedPackage = new FakePackage ("Test");
+			var expectedPackage = new FakePackage ("Test", "1.1");
 			action.Package = expectedPackage;
 			fakeProject.FakePackages.Add (new FakePackage ("Test", "1.0"));
 			action.Execute ();
@@ -423,6 +447,210 @@ namespace MonoDevelop.PackageManagement.Tests
 
 			Assert.AreEqual (packageVersion2, fakeProject.PackagePassedToUpdatePackage);
 		}
+
+		[Test]
+		public void Execute_NewerPrereleaseInstalledAndTryToUpdateToOlderStableRelease_UpdateIsNotInstalled ()
+		{
+			CreateSolution ();
+			fakeProject.AddFakePackageToSourceRepository ("MyPackage", "1.2");
+			fakeProject.FakePackages.Add (new FakePackage ("MyPackage", "1.3.0.6275-pre1"));
+			action.PackageId = "MyPackage";
+			fakeProject.Name = "MyProject";
+			action.UpdateIfPackageDoesNotExistInProject = false;
+			RecordMessagesLogged ();
+
+			action.Execute ();
+
+			Assert.IsNull (fakeProject.PackagePassedToUpdatePackage);
+			Assert.IsFalse (fakeProject.IsUpdatePackageCalled);
+			AssertMessageIsLogged ("No updates available for 'MyPackage' in project 'MyProject'.");
+		}
+
+		[Test]
+		public void Execute_NewerPrereleaseInstalledAndTryToUpdateToOlderStableReleaseAndUpdateIfPackageDoesNotExistInProject_UpdateIsNotInstalled ()
+		{
+			CreateSolution ();
+			fakeProject.AddFakePackageToSourceRepository ("MyPackage", "1.2");
+			fakeProject.FakePackages.Add (new FakePackage ("MyPackage", "1.3.0.6275-pre1"));
+			action.PackageId = "MyPackage";
+			fakeProject.Name = "MyProject";
+			action.UpdateIfPackageDoesNotExistInProject = true;
+			RecordMessagesLogged ();
+
+			action.Execute ();
+
+			Assert.IsNull (fakeProject.PackagePassedToUpdatePackage);
+			Assert.IsFalse (fakeProject.IsUpdatePackageCalled);
+			AssertMessageIsLogged ("No updates available for 'MyPackage' in project 'MyProject'.");
+		}
+
+		[Test]
+		public void Execute_ReferenceBeingUpdatedHasLocalCopyTrue_ReferenceAddedHasLocalCopyTrue ()
+		{
+			CreateSolution ();
+			fakeProject.FakePackages.Add (new FakePackage ("Test", "1.0"));
+			action.Package = new FakePackage ("Test", "1.1");
+			var firstReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NewAssembly");
+			var secondReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NUnit.Framework");
+			fakeProject.UpdatePackageAction = (p, a) => {
+				var referenceBeingRemoved = new ProjectReference (ReferenceType.Assembly, "NUnit.Framework") {
+					LocalCopy = true
+				};
+				packageManagementEvents.OnReferenceRemoving (referenceBeingRemoved);
+				packageManagementEvents.OnReferenceAdding (firstReferenceBeingAdded);
+				packageManagementEvents.OnReferenceAdding (secondReferenceBeingAdded);
+			};
+			action.Execute ();
+
+			Assert.IsTrue (firstReferenceBeingAdded.LocalCopy);
+			Assert.IsTrue (secondReferenceBeingAdded.LocalCopy);
+		}
+
+		[Test]
+		public void Execute_ReferenceBeingUpdatedHasLocalCopyTrueButCaseIsDifferent_ReferenceAddedHasLocalCopyTrue ()
+		{
+			CreateSolution ();
+			fakeProject.FakePackages.Add (new FakePackage ("Test", "1.0"));
+			action.Package = new FakePackage ("Test", "1.1");
+			var firstReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NewAssembly");
+			var secondReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NUnit.Framework");
+			fakeProject.UpdatePackageAction = (p, a) => {
+				var referenceBeingRemoved = new ProjectReference (ReferenceType.Assembly, "nunit.framework") {
+					LocalCopy = true
+				};
+				packageManagementEvents.OnReferenceRemoving (referenceBeingRemoved);
+				packageManagementEvents.OnReferenceAdding (firstReferenceBeingAdded);
+				packageManagementEvents.OnReferenceAdding (secondReferenceBeingAdded);
+			};
+			action.Execute ();
+
+			Assert.IsTrue (firstReferenceBeingAdded.LocalCopy);
+			Assert.IsTrue (secondReferenceBeingAdded.LocalCopy);
+		}
+
+		[Test]
+		public void Execute_ReferenceBeingUpdatedHasLocalCopyFalse_ReferenceAddedHasLocalCopyFalse ()
+		{
+			CreateSolution ();
+			fakeProject.FakePackages.Add (new FakePackage ("Test", "1.0"));
+			action.Package = new FakePackage ("Test", "1.1");
+			var firstReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NewAssembly") {
+				LocalCopy = true
+			};
+			var secondReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NUnit.Framework");
+			fakeProject.UpdatePackageAction = (p, a) => {
+				var referenceBeingRemoved = new ProjectReference (ReferenceType.Assembly, "NUnit.Framework") {
+					LocalCopy = false
+				};
+				packageManagementEvents.OnReferenceRemoving (referenceBeingRemoved);
+				packageManagementEvents.OnReferenceAdding (firstReferenceBeingAdded);
+				packageManagementEvents.OnReferenceAdding (secondReferenceBeingAdded);
+			};
+			action.Execute ();
+
+			Assert.IsTrue (firstReferenceBeingAdded.LocalCopy);
+			Assert.IsFalse (secondReferenceBeingAdded.LocalCopy);
+		}
+
+		[Test]
+		public void Execute_ReferenceBeingUpdatedHasLocalCopyFalseButCaseIsDifferent_ReferenceAddedHasLocalCopyFalse ()
+		{
+			CreateSolution ();
+			fakeProject.FakePackages.Add (new FakePackage ("Test", "1.0"));
+			action.Package = new FakePackage ("Test", "1.1");
+			var firstReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NewAssembly") {
+				LocalCopy = true
+			};
+			var secondReferenceBeingAdded = new ProjectReference (ReferenceType.Assembly, "NUnit.Framework");
+			fakeProject.UpdatePackageAction = (p, a) => {
+				var referenceBeingRemoved = new ProjectReference (ReferenceType.Assembly, "nunit.framework") {
+					LocalCopy = false
+				};
+				packageManagementEvents.OnReferenceRemoving (referenceBeingRemoved);
+				packageManagementEvents.OnReferenceAdding (firstReferenceBeingAdded);
+				packageManagementEvents.OnReferenceAdding (secondReferenceBeingAdded);
+			};
+			action.Execute ();
+
+			Assert.IsTrue (firstReferenceBeingAdded.LocalCopy);
+			Assert.IsFalse (secondReferenceBeingAdded.LocalCopy);
+		}
+
+		[Test]
+		public void Execute_PackageUpdatedSuccessfully_OpenPackageReadmeMonitorCreated ()
+		{
+			CreateSolution ();
+			updatePackageHelper.TestPackage.Id = "Test";
+			updatePackageHelper.UpdateTestPackage ();
+
+			Assert.AreEqual ("Test", action.OpenPackageReadMeMonitor.PackageId);
+			Assert.IsTrue (action.OpenPackageReadMeMonitor.IsDisposed);
+		}
+
+		[Test]
+		public void Execute_PackageInstalledSuccessfullyWithReadmeTxt_ReadmeTxtFileIsOpened ()
+		{
+			CreateSolution ();
+			updatePackageHelper.TestPackage.Id = "Test";
+			updatePackageHelper.TestPackage.AddFile ("readme.txt");
+			action.CreateOpenPackageReadMeMonitorAction = packageId => {
+				return CreateReadMeMonitor (packageId);
+			};
+			string installPath = @"d:\projects\myproject\packages\Test.1.0".ToNativePath ();
+			string readmeFileName = Path.Combine (installPath, "readme.txt");
+			fakeProject.UpdatePackageAction = (package, updateAction) => {
+				var eventArgs = new PackageOperationEventArgs (package, null, installPath);
+				fakeProject.FirePackageInstalledEvent (eventArgs);
+				fileService.ExistingFileNames.Add (readmeFileName);
+			};
+			updatePackageHelper.UpdateTestPackage ();
+
+			Assert.IsTrue (fileService.IsOpenFileCalled);
+			Assert.AreEqual (readmeFileName, fileService.FileNamePassedToOpenFile);
+		}
+
+		[Test]
+		public void Execute_PackageWithReadmeTxtIsInstalledButExceptionThrownWhenAddingPackageToProject_ReadmeFileIsNotOpened ()
+		{
+			CreateSolution ();
+			updatePackageHelper.TestPackage.Id = "Test";
+			updatePackageHelper.TestPackage.AddFile ("readme.txt");
+			OpenPackageReadMeMonitor monitor = null;
+			action.CreateOpenPackageReadMeMonitorAction = packageId => {
+				monitor = CreateReadMeMonitor (packageId) as OpenPackageReadMeMonitor;
+				return monitor;
+			};
+			string installPath = @"d:\projects\myproject\packages\Test.1.0".ToNativePath ();
+			string readmeFileName = Path.Combine (installPath, "readme.txt");
+			fakeProject.UpdatePackageAction = (package, updateAction) => {
+				var eventArgs = new PackageOperationEventArgs (package, null, installPath);
+				fakeProject.FirePackageInstalledEvent (eventArgs);
+				fileService.ExistingFileNames.Add (readmeFileName);
+				throw new ApplicationException ();
+			};
+			Assert.Throws<ApplicationException> (() => {
+				updatePackageHelper.UpdateTestPackage ();
+			});
+
+			Assert.IsFalse (fileService.IsOpenFileCalled);
+			Assert.IsTrue (monitor.IsDisposed);
+		}
+
+		[Test]
+		public void Execute_PackagesConfigFileNamedAfterProjectDeletedDuringUpdate_FileServicePackagesConfigFileDeletionIsCancelled ()
+		{
+			CreateSolution ();
+			action.Package = new FakePackage ("Test");
+			string expectedFileName = @"d:\projects\MyProject\packages.MyProject.config".ToNativePath ();
+			bool? fileRemovedResult = null;
+			fakeProject.UpdatePackageAction = (p, a) => {
+				fileRemovedResult = packageManagementEvents.OnFileRemoving (expectedFileName);
+			};
+			action.Execute ();
+
+			Assert.AreEqual (expectedFileName, fileRemover.FileRemoved);
+			Assert.IsFalse (fileRemovedResult.Value);
+		}
 	}
 }
 
