@@ -53,6 +53,9 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 			this.treeviewColors.HeadersVisible = false;
 			this.treeviewColors.Model = colorStore;
 			this.treeviewColors.Selection.Changed += HandleTreeviewColorsSelectionChanged;
+			this.hpaned1.Position = 250;
+			this.SetSizeRequest (1024, 768);
+
 			this.colorbuttonFg.ColorSet += Stylechanged;
 			this.colorbuttonBg.ColorSet += Stylechanged;
 			this.colorbuttonPrimary.ColorSet += Stylechanged;
@@ -114,7 +117,7 @@ namespace MonoDevelop.SourceEditor.OptionPanels
 				colorSheme.Save (fileName);
 				panel.ShowStyles ();
 			} catch (Exception ex) {
-				MessageService.ShowException (ex);
+				LoggingService.LogInternalError (ex);
 			}
 			RefreshAllColors ();
 		}
