@@ -68,7 +68,7 @@ namespace MonoDevelop.SourceEditor
 
 		static ExtensibleTextEditor ()
 		{
-			var icon = Xwt.Drawing.Image.FromResource ("gutter-bookmark-light-15.png");
+			var icon = Xwt.Drawing.Image.FromResource ("gutter-bookmark-15.png");
 
 			BookmarkMarker.DrawBookmarkFunc = delegate(TextEditor editor, Cairo.Context cr, DocumentLine lineSegment, double x, double y, double width, double height) {
 				if (!lineSegment.IsBookmarked)
@@ -88,12 +88,6 @@ namespace MonoDevelop.SourceEditor
 			Initialize (view);
 		}
 
-		void HandleParseOperationFinished (object sender, EventArgs e)
-		{
-			resolveResult = null;
-			resolveRegion = DomRegion.Empty;
-		}
-		
 		public ExtensibleTextEditor (SourceEditorView view)
 		{
 			base.Options = new StyledSourceEditorOptions (view.Project, null);
@@ -118,7 +112,6 @@ namespace MonoDevelop.SourceEditor
 			};
 			
 			Document.TextReplaced += HandleSkipCharsOnReplace;
-			TypeSystemService.ParseOperationFinished += HandleParseOperationFinished;
 
 			UpdateEditMode ();
 			this.DoPopupMenu = ShowPopup;
@@ -196,9 +189,7 @@ namespace MonoDevelop.SourceEditor
 
 		protected override void OnDestroyed ()
 		{
-			TypeSystemService.ParseOperationFinished -= HandleParseOperationFinished;
 			UnregisterAdjustments ();
-			resolveResult = null;
 			Extension = null;
 			ExtensionContext = null;
 			view = null;
@@ -275,8 +266,7 @@ namespace MonoDevelop.SourceEditor
 		
 		void ReportExtensionError (Exception ex) 
 		{
-			MonoDevelop.Core.LoggingService.LogError ("Error in text editor extension chain", ex);
-			MessageService.ShowException (ex, "Error in text editor extension chain");
+			LoggingService.LogInternalError ("Error in text editor extension chain", ex);
 		}
 		
 		IEnumerable<char> TextWithoutCommentsAndStrings {
@@ -510,28 +500,16 @@ namespace MonoDevelop.SourceEditor
 		}
 		
 		int           oldOffset = -1;
-		ResolveResult resolveResult = null;
-		DomRegion     resolveRegion = DomRegion.Empty;
+
 		public ResolveResult GetLanguageItem (int offset, out DomRegion region)
 		{
-			// we'll cache old results.
-			if (offset == oldOffset) {
-				region = this.resolveRegion;
-				return this.resolveResult;
-			}
-			
 			oldOffset = offset;
-			
+			region = DomRegion.Empty;
+
 			if (textEditorResolverProvider != null) {
-				this.resolveResult = textEditorResolverProvider.GetLanguageItem (view.WorkbenchWindow.Document, offset, out region);
-				this.resolveRegion = region;
-			} else {
-				region = DomRegion.Empty;
-				this.resolveResult = null;
-				this.resolveRegion = region;
-			}
-			
-			return this.resolveResult;
+				return textEditorResolverProvider.GetLanguageItem (view.WorkbenchWindow.Document, offset, out region);
+			} 
+			return null;
 		}
 		
 		public CodeTemplateContext GetTemplateContext ()
@@ -556,12 +534,10 @@ namespace MonoDevelop.SourceEditor
 			oldOffset = offset;
 			
 			if (textEditorResolverProvider != null) {
-				this.resolveResult = textEditorResolverProvider.GetLanguageItem (view.WorkbenchWindow.Document, offset, expression);
-			} else {
-				this.resolveResult = null;
+				return textEditorResolverProvider.GetLanguageItem (view.WorkbenchWindow.Document, offset, expression);
 			}
 	
-			return this.resolveResult;
+			return null;
 		}
 
 //		public string GetExpression (int offset)
@@ -609,7 +585,15 @@ namespace MonoDevelop.SourceEditor
 			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet (ctx, menuPath);
 
 			if (Platform.IsMac) {
-				IdeApp.CommandService.ShowContextMenu (this, evt, cset, this);
+				if (evt == null) {
+					int x, y;
+					var pt = LocationToPoint (this.Caret.Location);
+					TranslateCoordinates (Toplevel, pt.X, pt.Y, out x, out y);
+
+					IdeApp.CommandService.ShowContextMenu (this, x, y, cset, this);
+				} else {
+					IdeApp.CommandService.ShowContextMenu (this, evt, cset, this);
+				}
 			} else {
 				Gtk.Menu menu = IdeApp.CommandService.CreateMenu (cset);
 				var imMenu = CreateInputMethodMenuItem (GettextCatalog.GetString ("_Input Methods"));
@@ -623,7 +607,8 @@ namespace MonoDevelop.SourceEditor
 					GtkWorkarounds.ShowContextMenu (menu, this, evt);
 				} else {
 					var pt = LocationToPoint (this.Caret.Location);
-					GtkWorkarounds.ShowContextMenu (menu, this, new Gdk.Rectangle (pt.X, pt.Y, 1, (int)LineHeight));
+
+					GtkWorkarounds.ShowContextMenu (menu, this, (int)pt.X, (int)pt.Y);
 				}
 			}
 		}
@@ -809,7 +794,13 @@ namespace MonoDevelop.SourceEditor
 		{
 			RunAction (DeleteActions.CaretLine);
 		}
-		
+
+		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.DeleteToLineStart)]
+		internal void OnDeleteToLineStart ()
+		{
+			RunAction (DeleteActions.CaretLineToStart);
+		}
+
 		[CommandHandler (MonoDevelop.Ide.Commands.TextEditorCommands.DeleteToLineEnd)]
 		internal void OnDeleteToLineEnd ()
 		{
@@ -1061,8 +1052,19 @@ namespace MonoDevelop.SourceEditor
 				RunAction (Mono.TextEditor.Vi.ViActions.Join);
 			}
 		}
-		
+
+		[CommandHandler (MonoDevelop.Ide.Commands.EditCommands.SortSelectedLines)]
+		void SortSelectedLines ()
+		{
+			RunAction (MiscActions.SortSelectedLines);
+		}
+
+		[CommandUpdateHandler (MonoDevelop.Ide.Commands.EditCommands.SortSelectedLines)]
+		void UpdateSortSelectedLines (CommandInfo ci)
+		{
+			ci.Enabled = GetTextEditorData ().IsMultiLineSelection;
+		}
 #endregion
-		
+	
 	}
 }
