@@ -55,6 +55,7 @@ using System.Text;
 using Mono.Addins;
 using MonoDevelop.Components;
 using Mono.TextEditor.Utils;
+using MonoDevelop.Projects.Policies;
 
 namespace MonoDevelop.SourceEditor
 {	
@@ -614,6 +615,11 @@ namespace MonoDevelop.SourceEditor
 				messageBubbleCache = null;
 			}
 		}
+
+		protected virtual string ProcessSaveText (string text)
+		{
+			return text;
+		}
 		
 		public override void Save (string fileName)
 		{
@@ -651,10 +657,20 @@ namespace MonoDevelop.SourceEditor
 			if (PropertyService.Get ("AutoFormatDocumentOnSave", false)) {
 				try {
 					var formatter = CodeFormatterService.GetFormatter (Document.MimeType);
-					if (formatter != null && formatter.SupportsOnTheFlyFormatting) {
-						using (var undo = TextEditor.OpenUndoGroup ()) {
-							formatter.OnTheFlyFormat (WorkbenchWindow.Document, 0, Document.TextLength);
-							wasEdited = false;
+					if (formatter != null) {
+						var document = WorkbenchWindow.Document;
+						if (formatter.SupportsOnTheFlyFormatting) {
+							using (var undo = TextEditor.OpenUndoGroup ()) {
+								formatter.OnTheFlyFormat (document, 0, Document.TextLength);
+								wasEdited = false;
+							}
+						} else {
+							var text = document.Editor.Text;
+							var policies = document.Project != null ? document.Project.Policies : PolicyService.DefaultPolicies;
+							string formattedText = formatter.FormatText (policies, text);
+							if (formattedText != null && formattedText != text) {
+								document.Editor.Replace (0, text.Length, formattedText);
+							}
 						}
 					}
 				} catch (Exception e) {
@@ -694,7 +710,7 @@ namespace MonoDevelop.SourceEditor
 				try {
 					var writeEncoding = encoding;
 					var writeBom = hadBom;
-					var writeText = Document.Text;
+					var writeText = ProcessSaveText (Document.Text);
 					if (writeEncoding == null) {
 						if (this.encoding != null) {
 							writeEncoding = this.encoding;
@@ -757,6 +773,7 @@ namespace MonoDevelop.SourceEditor
 			string text = null;
 			if (content != null) {
 				text = Mono.TextEditor.Utils.TextFileUtility.GetText (content, out encoding, out hadBom);
+				text = ProcessLoadText (text);
 				Document.Text = text;
 			}
 			this.CreateDocumentParsedHandler ();
@@ -810,8 +827,11 @@ namespace MonoDevelop.SourceEditor
 		}
 
 		MonoDevelop.Ide.Gui.Document ownerDocument;
+		protected MonoDevelop.Ide.Gui.Document OwnerDocument {
+			get { return ownerDocument; }
+		}
 
-		void HandleDocumentParsed (object sender, EventArgs e)
+		protected virtual void HandleDocumentParsed (object sender, EventArgs e)
 		{
 			widget.UpdateParsedDocument (ownerDocument.ParsedDocument);
 		}		
@@ -821,6 +841,11 @@ namespace MonoDevelop.SourceEditor
 			Load (fileName, loadEncoding);
 		}
 
+		protected virtual string ProcessLoadText (string text)
+		{
+			return text;
+		}
+
 		public void Load (string fileName, Encoding loadEncoding, bool reload = false)
 		{
 			// Handle the "reload" case.
@@ -848,6 +873,7 @@ namespace MonoDevelop.SourceEditor
 					encoding = loadEncoding;
 					text = TextFileUtility.ReadAllText (fileName, loadEncoding, out hadBom);
 				}
+				text = ProcessLoadText (text);
 				if (reload) {
 					Document.Replace (0, Document.TextLength, text);
 					Document.DiffTracker.Reset ();
@@ -893,26 +919,26 @@ namespace MonoDevelop.SourceEditor
 			widget.TextEditor.VAdjustment.Value = settings.vAdjustment;
 			widget.TextEditor.HAdjustment.Value = settings.hAdjustment;
 			
-			foreach (var f in widget.TextEditor.Document.FoldSegments) {
-				bool isFolded;
-				if (settings.FoldingStates.TryGetValue (f.Offset, out isFolded))
-					f.IsFolded = isFolded;
-			}
+//			foreach (var f in widget.TextEditor.Document.FoldSegments) {
+//				bool isFolded;
+//				if (settings.FoldingStates.TryGetValue (f.Offset, out isFolded))
+//					f.IsFolded = isFolded;
+//			}
 		}
 		
 		internal void StoreSettings ()
 		{
-			var foldingStates = new Dictionary<int, bool> ();
-			foreach (var f in widget.TextEditor.Document.FoldSegments) {
-				foldingStates [f.Offset] = f.IsFolded;
-			}
+//			var foldingStates = new Dictionary<int, bool> ();
+//			foreach (var f in widget.TextEditor.Document.FoldSegments) {
+//				foldingStates [f.Offset] = f.IsFolded;
+//			}
 			if (string.IsNullOrEmpty (ContentName))
 				return;
 			FileSettingsStore.Store (ContentName, new FileSettingsStore.Settings () {
 				CaretOffset = widget.TextEditor.Caret.Offset,
 				vAdjustment = widget.TextEditor.VAdjustment.Value,
-				hAdjustment = widget.TextEditor.HAdjustment.Value,
-				FoldingStates = foldingStates
+				hAdjustment = widget.TextEditor.HAdjustment.Value//,
+//				FoldingStates = foldingStates
 			});
 		}
 
@@ -1177,7 +1203,7 @@ namespace MonoDevelop.SourceEditor
 		void UpdatePinnedWatches ()
 		{
 			foreach (PinnedWatchInfo wi in pinnedWatches) {
-				widget.TextEditor.Remove (wi.Widget);
+				widget.TextEditor.TextArea.Remove (wi.Widget);
 				wi.Widget.Destroy ();
 			}
 			pinnedWatches.Clear ();
@@ -1213,7 +1239,7 @@ namespace MonoDevelop.SourceEditor
 //			if (w.Value != null)
 //				wi.Marker.AddValue (w.Value);
 
-			widget.TextEditor.AddTopLevelWidget (wi.Widget, w.OffsetX, w.OffsetY);
+			widget.TextEditor.TextArea.AddTopLevelWidget (wi.Widget, w.OffsetX, w.OffsetY);
 			
 //			widget.TextEditor.QueueDraw ();
 		}
@@ -1245,7 +1271,7 @@ namespace MonoDevelop.SourceEditor
 			foreach (PinnedWatchInfo wi in pinnedWatches) {
 				if (wi.Watch == args.Watch) {
 					pinnedWatches.Remove (wi);
-					widget.TextEditor.Remove (wi.Widget);
+					widget.TextEditor.TextArea.Remove (wi.Widget);
 					wi.Widget.Destroy ();
 					break;
 				}
@@ -1257,7 +1283,7 @@ namespace MonoDevelop.SourceEditor
 			foreach (PinnedWatchInfo wi in pinnedWatches) {
 				if (wi.Watch == args.Watch) {
 					wi.Widget.ObjectValue = wi.Watch.Value;
-					widget.TextEditor.MoveTopLevelWidget (wi.Widget, args.Watch.OffsetX, args.Watch.OffsetY);
+					widget.TextEditor.TextArea.MoveTopLevelWidget (wi.Widget, args.Watch.OffsetX, args.Watch.OffsetY);
 					break;
 				}
 			}
@@ -1493,17 +1519,31 @@ namespace MonoDevelop.SourceEditor
 
 		public void SetCaretTo (int line, int column)
 		{
-			this.Document.RunWhenLoaded (() => widget.TextEditor.SetCaretTo (line, column, true));
+			this.Document.RunWhenLoaded (() => {
+				PrepareToSetCaret (line, column);
+				widget.TextEditor.SetCaretTo (line, column, true);
+			});
 		}
 
 		public void SetCaretTo (int line, int column, bool highlight)
 		{
-			this.Document.RunWhenLoaded (() => widget.TextEditor.SetCaretTo (line, column, highlight));
+			this.Document.RunWhenLoaded (() => {
+				PrepareToSetCaret (line, column);
+				widget.TextEditor.SetCaretTo (line, column, highlight);
+			});
 		}
 		
 		public void SetCaretTo (int line, int column, bool highlight, bool centerCaret)
 		{
-			this.Document.RunWhenLoaded (() => widget.TextEditor.SetCaretTo (line, column, highlight, centerCaret));
+			this.Document.RunWhenLoaded (() => {
+				PrepareToSetCaret (line, column);
+				widget.TextEditor.SetCaretTo (line, column, highlight, centerCaret);
+			});
+		}
+
+		protected virtual void PrepareToSetCaret (int line, int column)
+		{
+
 		}
 
 		public void Redo ()
@@ -1655,7 +1695,7 @@ namespace MonoDevelop.SourceEditor
 
 		public void DeleteText (int position, int length)
 		{
-			this.widget.TextEditor.Remove (position, length);
+			this.widget.TextEditor.TextArea.Remove (position, length);
 		}
 		#endregion 
 		
@@ -2606,6 +2646,58 @@ namespace MonoDevelop.SourceEditor
 
 
 
+		#endregion
+
+
+		#region Command handlers
+		[CommandHandler (ScrollbarCommand.Top)]
+		void GotoTop ()
+		{
+			widget.QuickTaskStrip.GotoTop ();
+		}
+
+		[CommandHandler (ScrollbarCommand.Bottom)]
+		void GotoBottom ()
+		{
+			widget.QuickTaskStrip.GotoBottom ();
+		}
+
+		[CommandHandler (ScrollbarCommand.PgUp)]
+		void GotoPgUp ()
+		{
+			widget.QuickTaskStrip.GotoPgUp ();
+		}
+
+		[CommandHandler (ScrollbarCommand.PgDown)]
+		void GotoPgDown ()
+		{
+			widget.QuickTaskStrip.GotoPgDown ();
+		}	
+
+		[CommandUpdateHandler (ScrollbarCommand.ShowTasks)]
+		void UpdateShowMap (CommandInfo info)
+		{
+			widget.QuickTaskStrip.UpdateShowMap (info);
+		}
+
+		[CommandHandler (ScrollbarCommand.ShowTasks)]
+		void ShowMap ()
+		{
+			widget.QuickTaskStrip.ShowMap ();
+		}
+
+		[CommandUpdateHandler (ScrollbarCommand.ShowMinimap)]
+		void UpdateShowFull (CommandInfo info)
+		{
+			widget.QuickTaskStrip.UpdateShowFull (info);
+		}
+
+		[CommandHandler (ScrollbarCommand.ShowMinimap)]
+		void ShowFull ()
+		{
+			widget.QuickTaskStrip.ShowFull ();
+		}
+
 		#endregion
 	}
 } 
