@@ -70,7 +70,14 @@ namespace MonoDevelop.SourceEditor
 				return textEditor.Options;
 			}
 		}
-		
+
+
+		internal QuickTaskStrip QuickTaskStrip { 
+			get {
+				return mainsw.Strip;
+			}
+		} 
+
 		bool isDisposed;
 		
 		ParsedDocument parsedDocument;
@@ -376,6 +383,7 @@ namespace MonoDevelop.SourceEditor
 				if (this.splittedTextEditor == null || !splittedTextEditor.TextArea.HasFocus)
 					OnLostFocus ();
 			};
+			IdeApp.FocusOut += (sender, e) => textEditor.TextArea.HideTooltip (false);
 			mainsw = new DecoratedScrolledWindow (this);
 			mainsw.SetTextEditor (textEditor);
 			
@@ -615,8 +623,16 @@ namespace MonoDevelop.SourceEditor
 		
 		void UpdateErrorUndelines (ParsedDocument parsedDocument)
 		{
-			if (!options.UnderlineErrors || parsedDocument == null)
+			if (!options.UnderlineErrors || parsedDocument == null) {
+				Application.Invoke (delegate {
+					var doc = this.TextEditor != null ? this.TextEditor.Document : null;
+					if (doc == null)
+						return;
+					RemoveErrorUnderlines (doc);
+					UpdateQuickTasks (parsedDocument);
+				});
 				return;
+			}
 				
 			Application.Invoke (delegate {
 				if (!quickTaskProvider.Contains (this))
@@ -628,7 +644,7 @@ namespace MonoDevelop.SourceEditor
 						Document doc = this.TextEditor != null ? this.TextEditor.Document : null;
 						if (doc != null) {
 							RemoveErrorUnderlines (doc);
-							
+
 							// Else we underline the error
 							if (parsedDocument.Errors != null) {
 								foreach (var error in parsedDocument.Errors)
@@ -895,7 +911,7 @@ namespace MonoDevelop.SourceEditor
 		{
 			if (UseIncorrectMarkers || DefaultSourceEditorOptions.Instance.LineEndingConversion == LineEndingConversion.LeaveAsIs)
 				return;
-			ShowIncorretEolMarkers (Document.FileName, multiple);
+			ShowIncorrectEolMarkers (Document.FileName, multiple);
 		}
 
 		internal bool EnsureCorrectEolMarker (string fileName)
@@ -905,9 +921,9 @@ namespace MonoDevelop.SourceEditor
 			if (HasIncorrectEolMarker) {
 				switch (DefaultSourceEditorOptions.Instance.LineEndingConversion) {
 				case LineEndingConversion.Ask:
-					var hasMultipleIncorretEolMarkers = FileRegistry.HasMultipleIncorretEolMarkers;
-					ShowIncorretEolMarkers (fileName, hasMultipleIncorretEolMarkers);
-					if (hasMultipleIncorretEolMarkers) {
+					var hasMultipleIncorrectEolMarkers = FileRegistry.HasMultipleIncorrectEolMarkers;
+					ShowIncorrectEolMarkers (fileName, hasMultipleIncorrectEolMarkers);
+					if (hasMultipleIncorrectEolMarkers) {
 						FileRegistry.UpdateEolMessages ();
 					}
 					return false;
@@ -955,7 +971,7 @@ namespace MonoDevelop.SourceEditor
 
 		OverlayMessageWindow messageOverlayWindow;
 
-		void ShowIncorretEolMarkers (string fileName, bool multiple)
+		void ShowIncorrectEolMarkers (string fileName, bool multiple)
 		{
 			RemoveMessageBar ();
 			messageOverlayWindow = new OverlayMessageWindow ();
@@ -976,15 +992,18 @@ namespace MonoDevelop.SourceEditor
 			hbox.PackStart (label, true, true, 0);
 			var okButton = new Button (Gtk.Stock.Ok);
 			okButton.WidthRequest = 60;
-			hbox.PackEnd (okButton, false, false, 0); 
+
+			// Small amount of vertical padding for the OK button.
+			const int verticalPadding = 2;
+			var vbox = new VBox ();
+			vbox.PackEnd (okButton, true, true, verticalPadding);
+			hbox.PackEnd (vbox, false, false, 0);
 
 			var list = new List<string> ();
 			list.Add (string.Format ("Convert to {0} line endings", GetEolString (textEditor.Options.DefaultEolMarker)));
-			if (multiple)
-				list.Add (string.Format ("Convert all files to {0} line endings", GetEolString (textEditor.Options.DefaultEolMarker)));
+			list.Add (string.Format ("Convert all files to {0} line endings", GetEolString (textEditor.Options.DefaultEolMarker)));
 			list.Add (string.Format ("Keep {0} line endings", GetEolString (DetectedEolMarker)));
-			if (multiple)
-				list.Add (string.Format ("Keep {0} line endings in all files", GetEolString (DetectedEolMarker)));
+			list.Add (string.Format ("Keep {0} line endings in all files", GetEolString (DetectedEolMarker)));
 			var combo = new ComboBox (list.ToArray ());
 			combo.Active = 0;
 			hbox.PackEnd (combo, false, false, 0);
@@ -994,6 +1013,12 @@ namespace MonoDevelop.SourceEditor
 			messageOverlayWindow.Child = container; 
 			messageOverlayWindow.ShowOverlay (this.TextEditor);
 
+			// This is hacky, but it will ensure that our combo appears with with the correct size.
+			GLib.Timeout.Add (100, delegate {
+				combo.QueueResize ();
+				return false;
+			});
+
 			messageOverlayWindow.SizeFunc = () => {
 				return okButton.SizeRequest ().Width +
 					combo.SizeRequest ().Width +
@@ -1008,36 +1033,22 @@ namespace MonoDevelop.SourceEditor
 				RemoveMessageBar ();
 			};
 			okButton.Clicked += delegate {
-				if (multiple) {
-					switch (combo.Active) {
-					case 0:
-						ConvertLineEndings ();
-						view.WorkbenchWindow.ShowNotification = false;
-						view.Save (fileName, view.SourceEncoding);
-						break;
-					case 1:
-						FileRegistry.ConvertLineEndingsInAllFiles ();
-						break;
-					case 2:
-						UseIncorrectMarkers = true;
-						view.WorkbenchWindow.ShowNotification = false;
-						break;
-					case 3:
-						FileRegistry.IgnoreLineEndingsInAllFiles ();
-						break;
-					}
-				} else {
-					switch (combo.Active) {
-					case 0:
-						ConvertLineEndings ();
-						view.WorkbenchWindow.ShowNotification = false;
-						view.Save (fileName, view.SourceEncoding);
-						break;
-					case 1:
-						UseIncorrectMarkers = true;
-						view.WorkbenchWindow.ShowNotification = false;
-						break;
-					}
+				switch (combo.Active) {
+				case 0:
+					ConvertLineEndings ();
+					view.WorkbenchWindow.ShowNotification = false;
+					view.Save (fileName, view.SourceEncoding);
+					break;
+				case 1:
+					FileRegistry.ConvertLineEndingsInAllFiles ();
+					break;
+				case 2:
+					UseIncorrectMarkers = true;
+					view.WorkbenchWindow.ShowNotification = false;
+					break;
+				case 3:
+					FileRegistry.IgnoreLineEndingsInAllFiles ();
+					break;
 				}
 				RemoveMessageBar ();
 			};
@@ -1064,7 +1075,7 @@ namespace MonoDevelop.SourceEditor
 						view.Load (fileName);
 						view.WorkbenchWindow.Document.ReparseDocument ();
 					} catch (Exception ex) {
-						MessageService.ShowException (ex, "Could not remove the autosave file.");
+						LoggingService.LogError ("Could not remove the autosave file.", ex);
 					} finally {
 						RemoveMessageBar ();
 					}
@@ -1083,7 +1094,7 @@ namespace MonoDevelop.SourceEditor
 						view.WorkbenchWindow.Document.ReparseDocument ();
 						view.IsDirty = true;
 					} catch (Exception ex) {
-						MessageService.ShowException (ex, "Could not remove the autosave file.");
+						LoggingService.LogError ("Could not remove the autosave file.", ex);
 					} finally {
 						RemoveMessageBar ();
 					}
@@ -1131,7 +1142,7 @@ namespace MonoDevelop.SourceEditor
 				view.Load (view.ContentName, view.SourceEncoding, true);
 				view.WorkbenchWindow.ShowNotification = false;
 			} catch (Exception ex) {
-				MessageService.ShowException (ex, "Could not reload the file.");
+				MessageService.ShowError ("Could not reload the file.", ex);
 			} finally {
 				RemoveMessageBar ();
 			}
@@ -1482,6 +1493,11 @@ namespace MonoDevelop.SourceEditor
 				if (TextEditor.IsSomethingSelected) {
 					startLine = Document.GetLineByOffset (textEditor.SelectionRange.Offset);
 					endLine = Document.GetLineByOffset (textEditor.SelectionRange.EndOffset);
+
+					// If selection ends at begining of line... This is visible as previous line
+					// is selected, hence we want to select previous line Bug 26287
+					if (endLine.Offset == textEditor.SelectionRange.EndOffset)
+						endLine = endLine.PreviousLine;
 				} else {
 					startLine = endLine = Document.GetLine (textEditor.Caret.Line);
 				}
@@ -1679,17 +1695,17 @@ namespace MonoDevelop.SourceEditor
 		void UpdateQuickTasks (ParsedDocument doc)
 		{
 			tasks.Clear ();
+			if (doc != null) {
+				foreach (var cmt in doc.TagComments) {
+					var newTask = new QuickTask (cmt.Text, cmt.Region.Begin, Severity.Hint);
+					tasks.Add (newTask);
+				}
 			
-			foreach (var cmt in doc.TagComments) {
-				var newTask = new QuickTask (cmt.Text, cmt.Region.Begin, Severity.Hint);
-				tasks.Add (newTask);
+				foreach (var error in doc.Errors) {
+					var newTask = new QuickTask (error.Message, error.Region.Begin, error.ErrorType == ErrorType.Error ? Severity.Error : Severity.Warning);
+					tasks.Add (newTask);
+				}
 			}
-			
-			foreach (var error in doc.Errors) {
-				var newTask = new QuickTask (error.Message, error.Region.Begin, error.ErrorType == ErrorType.Error ? Severity.Error : Severity.Warning);
-				tasks.Add (newTask);
-			}
-			
 			OnTasksUpdated (EventArgs.Empty);
 		}
 		#endregion
