@@ -45,6 +45,7 @@ using Mono.TextEditor;
 using System.Linq;
 using MonoDevelop.Components;
 using MonoDevelop.Ide.Commands;
+using System.Collections.Generic;
 
 namespace MonoDevelop.NUnit
 {
@@ -345,9 +346,11 @@ namespace MonoDevelop.NUnit
 		void OnTestSuiteChanged (object sender, EventArgs e)
 		{
 			if (testService.RootTests.Length > 0) {
+				var s = TreeView.SaveTreeState ();
 				TreeView.Clear ();
 				foreach (UnitTest t in testService.RootTests)
 					TreeView.AddChild (t);
+				TreeView.RestoreTreeState (s);
 			}
 			else {
 				TreeView.Clear ();
@@ -417,6 +420,9 @@ namespace MonoDevelop.NUnit
 				    item,
 				    test.CanRun,
 				    info);
+
+				foreach (var ci in info)
+					ci.Enabled = runningTestOperation == null;
 			}
 		}
 		
@@ -440,9 +446,12 @@ namespace MonoDevelop.NUnit
 				return;
 
 			foreach (var mode in debugModeSet.ExecutionModes) {
-				if (test.CanRun (mode.ExecutionHandler))
-					info.Add (GettextCatalog.GetString ("Debug Test ({0})", mode.Name), mode.Id);
+				if (test.CanRun (mode.ExecutionHandler)) {
+					var ci = info.Add (GettextCatalog.GetString ("Debug Test ({0})", mode.Name), mode.Id);
+					ci.Enabled = runningTestOperation == null;
+				}
 			}
+
 			if (info.Count == 1)
 				info [0].Text = GettextCatalog.GetString ("Debug Test");
 		}
@@ -518,7 +527,7 @@ namespace MonoDevelop.NUnit
 			base.OnSelectionChanged (sender, args);
 			ITreeNavigator nav = TreeView.GetSelectedNode ();
 			if (nav != null) {
-				UnitTest test = (UnitTest) nav.DataItem;
+				UnitTest test = nav.DataItem as UnitTest;
 				if (test != null)
 					FillDetails (test, false);
 			}
