@@ -43,6 +43,8 @@ using MonoDevelop.Ide;
 using System.Text.RegularExpressions;
 using MonoDevelop.Components;
 using MonoDevelop.Ide.Commands;
+using MonoDevelop.Ide.Fonts;
+using MonoDevelop.NUnit.External;
 
 namespace MonoDevelop.NUnit
 {
@@ -103,6 +105,7 @@ namespace MonoDevelop.NUnit
 		public TestResultsPad ()
 		{
 			testService.TestSuiteChanged += new EventHandler (OnTestSuiteChanged);
+			IdeApp.Workspace.WorkspaceItemClosed += OnWorkspaceItemClosed;
 			
 			panel = new VBox ();
 			
@@ -115,7 +118,7 @@ namespace MonoDevelop.NUnit
 			// Failures tree
 			failuresTreeView = new MonoDevelop.Ide.Gui.Components.PadTreeView ();
 			failuresTreeView.HeadersVisible = false;
-			failuresStore = new TreeStore (typeof(Xwt.Drawing.Image), typeof(string), typeof(object), typeof(string), typeof(int));
+			failuresStore = new TreeStore (typeof(Xwt.Drawing.Image), typeof(string), typeof(object), typeof(string), typeof(int), typeof(int));
 			var pr = new CellRendererImage ();
 			CellRendererText tr = new CellRendererText ();
 			TreeViewColumn col = new TreeViewColumn ();
@@ -132,6 +135,7 @@ namespace MonoDevelop.NUnit
 			book.Pack1 (sw, true, true);
 			
 			outputView = new MonoDevelop.Ide.Gui.Components.LogView.LogTextView ();
+			outputView.ModifyFont (FontService.MonospaceFont);
 			outputView.Editable = false;
 			bold = new TextTag ("bold");
 			bold.Weight = Pango.Weight.Bold;
@@ -255,15 +259,29 @@ namespace MonoDevelop.NUnit
 		}
 		
 		public void OnTestSuiteChanged (object sender, EventArgs e)
+		{
+			if (rootTest != null) {
+				rootTest = testService.SearchTest (rootTest.FullName);
+				if (rootTest == null)
+					buttonRun.Sensitive = false;
+			}
+		}
+
+		void OnWorkspaceItemClosed (object sender, EventArgs e)
+		{
+			ClearResults ();
+		}
+
+		void ClearResults ()
 		{
 			if (failuresTreeView.IsRealized)
 				failuresTreeView.ScrollToPoint (0, 0);
 
 			results.Clear ();
-			
+
 			error = null;
 			errorMessage = null;
-			
+
 			failuresStore.Clear ();
 			outputView.Buffer.Clear ();
 			outIters.Clear ();
@@ -271,12 +289,8 @@ namespace MonoDevelop.NUnit
 			progressBar.Text = "";
 			testsRun = 0;
 			resultSummary = new UnitTestResult ();
+			resultLabel.Markup = GetResultsMarkup ();
 			UpdateCounters ();
-			if (rootTest != null) {
-				rootTest = testService.SearchTest (rootTest.FullName);
-				if (rootTest == null)
-					buttonRun.Sensitive = false;
-			}
 		}
 		
 		bool Running {
@@ -362,16 +376,24 @@ namespace MonoDevelop.NUnit
 			errorMessage = message;
 			AddErrorMessage ();
 		}
-		
+		const int ErrorMessage = 1;
+		const int StackTrace = 2;
+
 		public void AddErrorMessage ()
 		{
-			string msg = GettextCatalog.GetString ("Internal error");
-			if (errorMessage != null)
-				msg += ": " + Escape (errorMessage);
+			string msg;
+			if (error is RemoteUnhandledException)
+				msg = Escape (errorMessage);
+			else {
+				msg = GettextCatalog.GetString ("Internal error");
+				if (errorMessage != null)
+					msg += ": " + Escape (errorMessage);
+			}
 
 			var stock = ImageService.GetIcon (Ide.Gui.Stock.Error, Gtk.IconSize.Menu);
 			TreeIter testRow = failuresStore.AppendValues (stock, msg, null, null, 0);
-			failuresStore.AppendValues (testRow, null, Escape (error.GetType ().Name + ": " + error.Message), null);
+			string name = error is RemoteUnhandledException ? ((RemoteUnhandledException)error).RemoteExceptionName : error.GetType ().Name;
+			failuresStore.AppendValues (testRow, null, Escape (name + ": " + error.Message), null);
 			TreeIter row = failuresStore.AppendValues (testRow, null, GettextCatalog.GetString ("Stack Trace"), null, null, 0);
 			AddStackTrace (row, error.StackTrace, null);
 		}
@@ -405,7 +427,7 @@ namespace MonoDevelop.NUnit
 				string fileName;
 				int lineNumber;
 				TryParseLocationFromStackTrace (line, out fileName, out lineNumber);
-				failuresStore.AppendValues (row, null, Escape (line), test, fileName, lineNumber);
+				failuresStore.AppendValues (row, null, Escape (line), test, fileName, lineNumber, StackTrace);
 			}
 		}
 		
@@ -485,8 +507,26 @@ namespace MonoDevelop.NUnit
 				var last = test.GetLastResult ();
 				if (last == null)
 					return;
+
+				Gtk.TreeModel foo;
+				Gtk.TreeIter iter;
+				if (!failuresTreeView.Selection.GetSelected (out foo, out iter))
+					return;
+
+				int type = (int)failuresStore.GetValue (iter, 5);
+
 				var clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
-				clipboard.Text = last.StackTrace;
+				switch (type) {
+				case ErrorMessage:
+					clipboard.Text = last.Message;
+					break;
+				case StackTrace:
+					clipboard.Text = last.StackTrace;
+					break;
+				default:
+					clipboard.Text = last.Message + Environment.NewLine + "Stack trace:" + Environment.NewLine + last.StackTrace;
+					break;
+				}
 			}
 		}
 
@@ -561,7 +601,7 @@ namespace MonoDevelop.NUnit
 			Gtk.TreeIter iter;
 			if (!failuresTreeView.Selection.GetSelected (out foo, out iter))
 				return null;
-				
+
 			UnitTest t = (UnitTest)failuresStore.GetValue (iter, 2);
 			return t;
 		}
@@ -622,13 +662,14 @@ namespace MonoDevelop.NUnit
 					return;
 				string file = test.SourceCodeLocation != null ? test.SourceCodeLocation.FileName + ":" + test.SourceCodeLocation.Line : null;
 				TreeIter testRow = failuresStore.AppendValues (TestStatusIcon.Failure, Escape (test.FullName), test, file);
-				bool hasMessage = result.Message != null && result.Message.Length > 0;
+				bool hasMessage = !string.IsNullOrEmpty (result.Message);
+
 				if (hasMessage)
-					failuresStore.AppendValues (testRow, null, Escape (result.Message), test);
-				if (result.StackTrace != null && result.StackTrace.Length > 0) {
+					failuresStore.AppendValues (testRow, null, "<span font='" + FontService.MonospaceFontName + "'>"+Escape (result.Message) + "</span>", test, null, 0, ErrorMessage);
+				if (!string.IsNullOrEmpty (result.StackTrace)) {
 					TreeIter row = testRow;
 					if (hasMessage)
-						row = failuresStore.AppendValues (testRow, null, GettextCatalog.GetString ("Stack Trace"), test);
+						row = failuresStore.AppendValues (testRow, null, GettextCatalog.GetString ("Stack Trace"), test, null, 0, StackTrace);
 					AddStackTrace (row, result.StackTrace, test);
 				}
 				failuresTreeView.ScrollToCell (failuresStore.GetPath (testRow), null, false, 0, 0);
