@@ -43,8 +43,17 @@ namespace MonoDevelop.NUnit.External
 {
 	public class NUnitTestRunner: MarshalByRefObject
 	{
-		public void Initialize (string nunitPath, string nunitCorePath)
+		public NUnitTestRunner ()
 		{
+		}
+
+		public void PreloadAssemblies (string nunitPath, string nunitCorePath, string nunitCoreInterfacesPath)
+		{
+			// Note: We need to load all nunit.*.dll assemblies before we do *anything* else in this class
+			// This is to ensure that we always load the assemblies from the monodevelop directory and not
+			// from the directory of the assembly under test. For example we wnat to load
+			// /Applications/MonoDevelop/lib/Addins/nunit.framework.dll and not /user/app/foo/bin/debug/nunit.framework.dll
+
 			// In some cases MS.NET can't properly resolve assemblies even if they
 			// are already loaded. For example, when deserializing objects from remoting.
 			AppDomain.CurrentDomain.AssemblyResolve += delegate (object s, ResolveEventArgs args) {
@@ -54,25 +63,41 @@ namespace MonoDevelop.NUnit.External
 				}
 				return null;
 			};
-
+			
 			// Force the loading of the NUnit.Framework assembly.
 			// It's needed since that dll is not located in the test dll directory.
+			Assembly.LoadFrom (nunitCoreInterfacesPath);
 			Assembly.LoadFrom (nunitCorePath);
 			Assembly.LoadFrom (nunitPath);
+		}
 
+		public void Initialize ()
+		{
 			// Initialize ExtensionHost if not already done
 			if ( !CoreExtensions.Host.Initialized )
 				CoreExtensions.Host.InitializeService();
 		}
 		
-		public TestResult Run (EventListener listener, ITestFilter filter, string path, string suiteName, List<string> supportAssemblies)
+		public TestResult Run (EventListener listener, ITestFilter filter, string path, string suiteName, List<string> supportAssemblies, string testRunnerType, string testRunnerAssembly)
 		{
 			InitSupportAssemblies (supportAssemblies);
 			
 			if (filter == null)
 				filter = TestFilter.Empty;
-			
-			RemoteTestRunner tr = new RemoteTestRunner ();
+
+			TestRunner tr;
+			if (!string.IsNullOrEmpty (testRunnerType)) {
+				Type runnerType;
+				if (string.IsNullOrEmpty (testRunnerAssembly))
+					runnerType = Type.GetType (testRunnerType, true);
+				else {
+					var asm = Assembly.LoadFrom (testRunnerAssembly);
+					runnerType = asm.GetType (testRunnerType);
+				}
+				tr = (TestRunner)Activator.CreateInstance (runnerType);
+			} else
+				tr = new RemoteTestRunner ();
+
 			TestPackage package = new TestPackage (path);
 			if (!string.IsNullOrEmpty (suiteName))
 				package.TestName = suiteName;
@@ -94,7 +119,9 @@ namespace MonoDevelop.NUnit.External
 			// The name of inherited tests include the base class name as prefix.
 			// That prefix has to be removed
 			string tname = test.TestName.Name;
-			int i = tname.LastIndexOf ('.');
+			// Find the last index of the dot character that is not a part of the test parameters
+			int j = tname.IndexOf ('(');
+			int i = tname.LastIndexOf ('.', (j == -1) ? (tname.Length - 1) : j);
 			if (i != -1)
 				tname = tname.Substring (i + 1);
 			if (test.FixtureType != null) {
@@ -103,11 +130,6 @@ namespace MonoDevelop.NUnit.External
 			} else if (test.TestType == "ParameterizedTest") {
 				ti.FixtureTypeName = test.Parent.FixtureType.Name;
 				ti.FixtureTypeNamespace = test.Parent.FixtureType.Namespace;
-				Console.WriteLine ("---------------------------------");
-				Console.WriteLine (test.GetType ());
-				Console.WriteLine (tname);
-				Console.WriteLine (test.TestName);
-				Console.WriteLine (test.Description);
 			}
 			ti.Name = tname;
 			ti.TestId = test.TestName.FullName;
@@ -126,6 +148,7 @@ namespace MonoDevelop.NUnit.External
 				for (int n=0; n<test.Tests.Count; n++)
 					ti.Tests [n] = BuildTestInfo ((Test)test.Tests [n]);
 			}
+			ti.IsExplicit = test.RunState == RunState.Explicit;
 			return ti;
 		}
 		
@@ -140,9 +163,8 @@ namespace MonoDevelop.NUnit.External
 		{
 			return null;
 		}
-
 	}
-	
+
 	[Serializable]
 	public class NunitTestInfo
 	{
@@ -151,6 +173,7 @@ namespace MonoDevelop.NUnit.External
 		public string TestId;
 		public string FixtureTypeName;
 		public string FixtureTypeNamespace;
+		public bool IsExplicit;
 		public NunitTestInfo[] Tests;
 	}
 		
