@@ -35,7 +35,6 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using Mono.Addins;
 using MonoDevelop.Projects;
-using NUnit.Core;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Ide;
 
@@ -50,17 +49,18 @@ namespace MonoDevelop.NUnit
 		
 		private NUnitService ()
 		{
-			IdeApp.Workspace.ReferenceAddedToProject += OnWorkspaceChanged;
-			IdeApp.Workspace.ReferenceRemovedFromProject += OnWorkspaceChanged;
 			IdeApp.Workspace.WorkspaceItemOpened += OnWorkspaceChanged;
 			IdeApp.Workspace.WorkspaceItemClosed += OnWorkspaceChanged;
-			IdeApp.Workspace.ItemAddedToSolution += OnWorkspaceChanged;
-			IdeApp.Workspace.ItemRemovedFromSolution += OnWorkspaceChanged;
 			IdeApp.Workspace.ActiveConfigurationChanged += OnWorkspaceChanged;
 
+			IdeApp.Workspace.ItemAddedToSolution += OnItemsChangedInSolution;;
+			IdeApp.Workspace.ItemRemovedFromSolution += OnItemsChangedInSolution;
+			IdeApp.Workspace.ReferenceAddedToProject += OnReferenceChangedInProject;;
+			IdeApp.Workspace.ReferenceRemovedFromProject += OnReferenceChangedInProject;
+
 			Mono.Addins.AddinManager.AddExtensionNodeHandler ("/MonoDevelop/NUnit/TestProviders", OnExtensionChange);
 		}
-		
+
 		public static NUnitService Instance {
 			get {
 				if (instance == null) {
@@ -70,7 +70,7 @@ namespace MonoDevelop.NUnit
 				return instance;
 			}
 		}
-		
+
 		void OnExtensionChange (object s, ExtensionNodeEventArgs args)
 		{
 			if (args.Change == ExtensionChange.Add) {
@@ -107,6 +107,11 @@ namespace MonoDevelop.NUnit
 		}
 		
 		public IAsyncOperation RunTest (UnitTest test, IExecutionHandler context, bool buildOwnerObject)
+		{
+			return RunTest (test, context, buildOwnerObject, true);
+		}
+
+		internal IAsyncOperation RunTest (UnitTest test, IExecutionHandler context, bool buildOwnerObject, bool checkCurrentRunOperation)
 		{
 			string testName = test.FullName;
 			
@@ -146,7 +151,7 @@ namespace MonoDevelop.NUnit
 				}
 			}
 			
-			if (!IdeApp.ProjectOperations.ConfirmExecutionOperation ())
+			if (checkCurrentRunOperation && !IdeApp.ProjectOperations.ConfirmExecutionOperation ())
 				return NullProcessAsyncOperation.Failure;
 			
 			Pad resultsPad = IdeApp.Workbench.GetPad <TestResultsPad>();
@@ -167,10 +172,13 @@ namespace MonoDevelop.NUnit
 					resultsPad.Sticky = false;
 				});
 			};
-			
+
+			OnTestSessionStarting (new TestSessionEventArgs { Session = session, Test = test });
+
 			session.Start ();
-			
-			IdeApp.ProjectOperations.CurrentRunOperation = session;
+
+			if (checkCurrentRunOperation)
+				IdeApp.ProjectOperations.CurrentRunOperation = session;
 			
 			return session;
 		}
@@ -262,7 +270,34 @@ namespace MonoDevelop.NUnit
 		{
 			RebuildTests ();
 		}
-		
+
+		void OnReferenceChangedInProject (object sender, ProjectReferenceEventArgs e)
+		{
+			if (!IsSolutionGroupPresent (e.Project.ParentSolution, rootTests))
+				RebuildTests ();
+		}
+
+		void OnItemsChangedInSolution (object sender, SolutionItemChangeEventArgs e)
+		{
+			if (!IsSolutionGroupPresent (e.Solution, rootTests))
+				RebuildTests ();
+		}
+
+		bool IsSolutionGroupPresent (Solution sol, IEnumerable<UnitTest> tests)
+		{
+			foreach (var t in tests) {
+				var tg = t as SolutionFolderTestGroup;
+				if (tg != null && ((SolutionFolder)tg.OwnerObject).ParentSolution == sol)
+					return true;
+				var g = t as UnitTestGroup;
+				if (g != null && g.HasTests) {
+					if (IsSolutionGroupPresent (sol, g.Tests))
+						return true;
+				}
+			}
+			return false;
+		}
+
 		void RebuildTests ()
 		{
 			if (rootTests != null) {
@@ -302,7 +337,8 @@ namespace MonoDevelop.NUnit
 		{
 			Properties properties = new Properties ();
 			properties.Set ("UnitTest", test);
-			MessageService.ShowCustomDialog (new UnitTestOptionsDialog (IdeApp.Workbench.RootWindow, properties));
+			using (var dlg = new UnitTestOptionsDialog (IdeApp.Workbench.RootWindow, properties))
+				MessageService.ShowCustomDialog (dlg);
 		}
 		
 		void NotifyTestSuiteChanged ()
@@ -333,6 +369,17 @@ namespace MonoDevelop.NUnit
 		}
 
 		public event EventHandler TestSessionCompleted;
+
+		void OnTestSessionStarting (TestSessionEventArgs args)
+		{
+			if (TestSessionStarting != null)
+				TestSessionStarting (this, args);
+		}
+
+		/// <summary>
+		/// Occurs just before a test session is started
+		/// </summary>
+		public event EventHandler<TestSessionEventArgs> TestSessionStarting;
 	}
 	
 
@@ -456,5 +503,11 @@ namespace MonoDevelop.NUnit
 			remove { monitor.CancelRequested -= value; }
 		}
 	}
+
+	public class TestSessionEventArgs: EventArgs
+	{
+		public IAsyncOperation Session { get; set; }
+		public UnitTest Test { get; set; }
+	}
 }
 
