@@ -31,11 +31,12 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.ProgressMonitoring;
 using MonoDevelop.VersionControl;
 using System.Collections.Generic;
+using System.Linq;
 
 namespace MonoDevelop.VersionControl.Tests
 {
 	[TestFixture]
-	abstract class BaseRepoUtilsTest
+	public abstract class BaseRepoUtilsTest
 	{
 		// [Git] Set user and email.
 		protected const string Author = "author";
@@ -56,6 +57,10 @@ namespace MonoDevelop.VersionControl.Tests
 		[TearDown]
 		public virtual void TearDown ()
 		{
+			if (Repo != null) {
+				Repo.Dispose ();
+				Repo = null;
+			}
 			DeleteDirectory (RemotePath);
 			DeleteDirectory (LocalPath);
 			AddedItems.Clear ();
@@ -183,7 +188,7 @@ namespace MonoDevelop.VersionControl.Tests
 
 			// Old cache.
 			vis = Repo.GetDirectoryVersionInfo (LocalPath, false, false);
-			Assert.AreEqual (1 + RepoItemsCount, vis.Length);
+			Assert.AreEqual (1 + RepoItemsCount, vis.Length, "Old DirectoryVersionInfo.");
 
 			// Query.
 			Repo.ClearCachedVersionInfo (LocalPath);
@@ -192,12 +197,12 @@ namespace MonoDevelop.VersionControl.Tests
 
 			// Cached.
 			vis = Repo.GetDirectoryVersionInfo (LocalPath, false, false);
-			Assert.AreEqual (2 + RepoItemsCount, vis.Length);
+			Assert.AreEqual (2 + RepoItemsCount, vis.Length, "New DirectoryVersionInfo.");
 
 			// Wait for result.
 			AddFile ("testfile3", null, true, false);
 			vis = Repo.GetDirectoryVersionInfo (LocalPath, false, true);
-			Assert.AreEqual (4 + RepoItemsCountRecursive, vis.Length);
+			Assert.AreEqual (4 + RepoItemsCountRecursive, vis.Length, "Recursive DirectoryVersionInfo.");
 		}
 
 		[Test]
@@ -240,11 +245,12 @@ namespace MonoDevelop.VersionControl.Tests
 			FilePath second = new FilePath (FileService.CreateTempDirectory () + Path.DirectorySeparatorChar);
 			Checkout (second, RemoteUrl);
 			Repo2 = GetRepo (second, RemoteUrl);
+			ModifyPath (Repo2, ref second);
 			string added = second + "testfile2";
 			File.Create (added).Close ();
 			Repo2.Add (added, false, new NullProgressMonitor ());
-			ChangeSet changes = Repo.CreateChangeSet (Repo.RootPath);
-			changes.AddFile (Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache));
+			ChangeSet changes = Repo2.CreateChangeSet (Repo2.RootPath);
+			changes.AddFile (Repo2.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache));
 			changes.GlobalComment = "test2";
 			Repo2.Commit (changes, new NullProgressMonitor ());
 
@@ -253,19 +259,46 @@ namespace MonoDevelop.VersionControl.Tests
 			Repo.Update (Repo.RootPath, true, new NullProgressMonitor ());
 			Assert.True (File.Exists (LocalPath + "testfile2"));
 
+			Repo2.Dispose ();
 			DeleteDirectory (second);
 		}
 
+		protected virtual void ModifyPath (Repository repo, ref FilePath old)
+		{
+		}
+
 		[Test]
 		// Tests Repository.GetHistory.
 		public void LogIsProper ()
 		{
+			if (!Platform.IsWindows)
+				Assert.Ignore ("Linux/Mac Svn seems to hiccup on symlinks.");
+
 			AddFile ("testfile", null, true, true);
 			AddFile ("testfile2", null, true, true);
-			int index = 0;
-			foreach (Revision rev in Repo.GetHistory (LocalPath + "testfile", null)) {
-				Assert.AreEqual (String.Format ("Commit #{0}", index++), rev.Message);
-			}
+			AddFile ("testfile3", null, true, true);
+
+			CheckLog (Repo);
+		}
+
+		protected abstract void CheckLog (Repository repo);
+
+		[Ignore ("This is failing on Wrench (Windows), and it seems to be choking on symlinks on Mac.")]
+		[TestCase(0)]
+		[TestCase(1)]
+		[TestCase(2)]
+		// Tests Repository.GetHistory with slices.
+		public void LogSinceWorks (int historyId)
+		{
+			AddFile ("testfile", null, true, true);
+			AddFile ("testfile2", null, true, true);
+			AddFile ("testfile3", null, true, true);
+
+			var history = Repo.GetHistory (LocalPath, null);
+			foreach (var rev in Repo.GetHistory (LocalPath, history[historyId]))
+				Assert.AreNotEqual (history [historyId].GetPrevious (), rev, "The revision was found in slice, yet should not be in it.");
+
+			Assert.True (history.Any (r => r == history[historyId]));
 		}
 
 		[Test]
@@ -273,7 +306,7 @@ namespace MonoDevelop.VersionControl.Tests
 		public void DiffIsProper ()
 		{
 			AddFile ("testfile", null, true, true);
-			File.AppendAllText (LocalPath + "testfile", "text" + Environment.NewLine);
+			File.AppendAllText (LocalPath + "testfile", "text");
 
 			TestDiff ();
 		}
@@ -288,12 +321,32 @@ namespace MonoDevelop.VersionControl.Tests
 			AddFile ("testfile", null, true, true);
 			string added = LocalPath + "testfile";
 
+			// Force cache update.
+			Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
+
 			// Revert to head.
 			File.WriteAllText (added, content);
 			Repo.Revert (added, false, new NullProgressMonitor ());
 			Assert.AreEqual (Repo.GetBaseText (added), File.ReadAllText (added));
 		}
 
+		[TestCase (true)]
+		[TestCase (false)]
+		// Tests Repository.Revert
+		public void Reverts2 (bool stage)
+		{
+			AddFile ("init", null, true, true);
+
+			string added = LocalPath + "testfile";
+			AddFile ("testfile", "test", stage, false);
+
+			// Force cache evaluation.
+			Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
+
+			Repo.Revert (added, false, new NullProgressMonitor ());
+			Assert.AreEqual (VersionStatus.Unversioned, Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache).Status);
+		}
+
 		[Test]
 		// Tests Repository.GetRevisionChanges.
 		public void CorrectRevisionChanges ()
@@ -301,7 +354,7 @@ namespace MonoDevelop.VersionControl.Tests
 			AddFile ("testfile", "text", true, true);
 			// TODO: Extend and test each member and more types.
 			foreach (var rev in Repo.GetRevisionChanges (GetHeadRevision ())) {
-				Assert.AreEqual (rev.Action, RevisionAction.Add);
+				Assert.AreEqual (RevisionAction.Add, rev.Action);
 			}
 		}
 
@@ -366,13 +419,13 @@ namespace MonoDevelop.VersionControl.Tests
 		// Tests Repository.MoveDirectory.
 		public virtual void MovesDirectory ()
 		{
-			string srcDir = LocalPath + "test";
-			string dstDir = LocalPath + "test2";
-			string src = srcDir + Path.DirectorySeparatorChar + "testfile";
-			string dst = dstDir + Path.DirectorySeparatorChar + "testfile";
+			string srcDir = LocalPath.Combine ("test");
+			string dstDir = LocalPath.Combine ("test2");
+			string src = Path.Combine (srcDir, "testfile");
+			string dst = Path.Combine (dstDir, "testfile");
 
 			AddDirectory ("test", true, false);
-			AddFile ("test" + Path.DirectorySeparatorChar + "testfile", null, true, true);
+			AddFile (Path.Combine ("test", "testfile"), null, true, true);
 
 			Repo.MoveDirectory (srcDir, dstDir, false, new NullProgressMonitor ());
 			VersionInfo srcVi = Repo.GetVersionInfo (src, VersionInfoQueryFlags.IgnoreCache);
@@ -388,7 +441,7 @@ namespace MonoDevelop.VersionControl.Tests
 			string added;
 			string postFix = keepLocal ? "2" : "";
 			// Versioned file.
-			added = LocalPath + "testfile1" + postFix;
+			added = LocalPath.Combine ("testfile1") + postFix;
 			AddFile ("testfile1" + postFix, null, true, true);
 			Repo.DeleteFile (added, true, new NullProgressMonitor (), keepLocal);
 			vi = Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
@@ -396,7 +449,7 @@ namespace MonoDevelop.VersionControl.Tests
 			Assert.AreEqual (keepLocal, File.Exists (added));
 
 			// Just added file.
-			added = LocalPath + "testfile2" + postFix;
+			added = LocalPath.Combine ("testfile2") + postFix;
 			AddFile ("testfile2" + postFix, null, true, false);
 			Repo.DeleteFile (added, true, new NullProgressMonitor (), keepLocal);
 			vi = Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
@@ -404,7 +457,7 @@ namespace MonoDevelop.VersionControl.Tests
 			Assert.AreEqual (keepLocal, File.Exists (added));
 
 			// Non versioned file.
-			added = LocalPath + "testfile3" + postFix;
+			added = LocalPath.Combine ("testfile3") + postFix;
 			AddFile ("testfile3" + postFix, null, false, false);
 			Repo.DeleteFile (added, true, new NullProgressMonitor (), keepLocal);
 			vi = Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
@@ -412,12 +465,12 @@ namespace MonoDevelop.VersionControl.Tests
 			Assert.AreEqual (keepLocal, File.Exists (added));
 		}
 
-		[Test]
+		[TestCase(false)]
+		[TestCase(true)]
 		// Tests Repository.DeleteFile.
-		public virtual void DeletesFile ()
+		public virtual void DeletesFile (bool keepLocal)
 		{
-			DeleteFileTestHelper (false);
-			DeleteFileTestHelper (true);
+			DeleteFileTestHelper (keepLocal);
 		}
 
 		void DeleteTestDirectoryHelper (bool keepLocal)
@@ -428,10 +481,10 @@ namespace MonoDevelop.VersionControl.Tests
 			string postFix = keepLocal ? "2" : "";
 
 			// Versioned directory.
-			addedDir = LocalPath + "test1" + postFix;
-			added = addedDir + Path.DirectorySeparatorChar + "testfile";
+			addedDir = LocalPath.Combine ("test1") + postFix;
+			added = Path.Combine (addedDir, "testfile");
 			AddDirectory ("test1" + postFix, true, false);
-			AddFile ("test1" + postFix + Path.DirectorySeparatorChar + "testfile", null, true, true);
+			AddFile (Path.Combine ("test1" + postFix, "testfile"), null, true, true);
 
 			Repo.DeleteDirectory (addedDir, true, new NullProgressMonitor (), keepLocal);
 			vi = Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
@@ -439,10 +492,10 @@ namespace MonoDevelop.VersionControl.Tests
 			Assert.AreEqual (keepLocal, File.Exists (added));
 
 			// Just added directory.
-			addedDir = LocalPath + "test2" + postFix;
-			added = addedDir + Path.DirectorySeparatorChar + "testfile";
+			addedDir = LocalPath.Combine ("test2") + postFix;
+			added = Path.Combine (addedDir, "testfile");
 			AddDirectory ("test2" + postFix, true, false);
-			AddFile ("test2" + postFix + Path.DirectorySeparatorChar + "testfile", null, true, false);
+			AddFile (Path.Combine ("test2" + postFix, "testfile"), null, true, false);
 
 			Repo.DeleteDirectory (addedDir, true, new NullProgressMonitor (), keepLocal);
 			vi = Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
@@ -450,10 +503,10 @@ namespace MonoDevelop.VersionControl.Tests
 			Assert.AreEqual (keepLocal, File.Exists (added));
 
 			// Non versioned file.
-			addedDir = LocalPath + "test3" + postFix;
-			added = addedDir + Path.DirectorySeparatorChar + "testfile";
+			addedDir = LocalPath.Combine ("test3") + postFix;
+			added = Path.Combine (addedDir, "testfile");
 			AddDirectory ("test3" + postFix, true, false);
-			AddFile ("test3" + postFix + Path.DirectorySeparatorChar + "testfile", null, false, false);
+			AddFile (Path.Combine ("test3" + postFix, "testfile"), null, false, false);
 
 			Repo.DeleteDirectory (addedDir, true, new NullProgressMonitor (), keepLocal);
 			vi = Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
@@ -524,18 +577,15 @@ namespace MonoDevelop.VersionControl.Tests
 		}
 
 		[Test]
-		[Ignore]
-		// TODO: Fix Subversion for Unix not returning the correct value.
 		// TODO: Fix SvnSharp logic failing to generate correct URL.
-		// TODO: Fix Git TreeWalk failing.
 		// Tests Repository.GetTextAtRevision.
-		public void CorrectTextAtRevision ()
+		public virtual void CorrectTextAtRevision ()
 		{
 			string added = LocalPath + "testfile";
 			AddFile ("testfile", "text1", true, true);
 			File.AppendAllText (added, "text2");
 			CommitFile (added);
-			string text = Repo.GetTextAtRevision (LocalPath, GetHeadRevision ());
+			string text = Repo.GetTextAtRevision (added, GetHeadRevision ());
 			Assert.AreEqual ("text1text2", text);
 		}
 
@@ -543,7 +593,7 @@ namespace MonoDevelop.VersionControl.Tests
 		// Tests Repository.GetAnnotations.
 		public void BlameIsCorrect ()
 		{
-			string added = LocalPath + "testfile";
+			string added = LocalPath.Combine ("testfile");
 			// Initial commit.
 			AddFile ("testfile", "blah" + Environment.NewLine, true, true);
 			// Second commit.
@@ -560,7 +610,7 @@ namespace MonoDevelop.VersionControl.Tests
 			}
 
 			BlameExtraInternals (annotations);
-			
+
 			Assert.False (annotations [2].HasEmail);
 			Assert.IsNotNull (annotations [2].Author);
 			Assert.IsNull (annotations [2].Email);
@@ -570,6 +620,36 @@ namespace MonoDevelop.VersionControl.Tests
 
 		protected abstract void BlameExtraInternals (Annotation [] annotations);
 
+		[Test]
+		// Tests bug #23275
+		public void MoveAndMoveBack ()
+		{
+			string added = LocalPath.Combine ("testfile");
+			string dir = LocalPath.Combine ("testdir");
+			string dirFile = Path.Combine (dir, "testfile");
+			AddFile ("testfile", "test", true, true);
+			AddDirectory ("testdir", true, false);
+			Repo.MoveFile (added, dirFile, true, new NullProgressMonitor ());
+			Repo.MoveFile (dirFile, added, true, new NullProgressMonitor ());
+
+			Assert.AreEqual (VersionStatus.Unversioned, Repo.GetVersionInfo (dirFile, VersionInfoQueryFlags.IgnoreCache).Status);
+			Assert.AreEqual (VersionStatus.Versioned, Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache).Status);
+		}
+
+		[Test]
+		public void RevertingADeleteMakesTheFileVersioned ()
+		{
+			var added = LocalPath.Combine ("testfile");
+			AddFile ("testfile", "test", true, true);
+
+			// Force cache update.
+			Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache);
+
+			Repo.DeleteFile (added, true, new NullProgressMonitor (), false);
+			Repo.Revert (added, false, new NullProgressMonitor ());
+
+			Assert.AreEqual (VersionStatus.Versioned, Repo.GetVersionInfo (added, VersionInfoQueryFlags.IgnoreCache).Status);
+		}
 		#region Util
 
 		protected void Checkout (string path, string url)
@@ -656,4 +736,3 @@ namespace MonoDevelop.VersionControl.Tests
 		#endregion
 	}
 }
-
