@@ -1,21 +1,21 @@
-// 
+//
 // MergeDialog.cs
-//  
+//
 // Author:
 //       Lluis Sanchez Gual <lluis@novell.com>
-// 
+//
 // Copyright (c) 2010 Novell, Inc (http://www.novell.com)
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -29,6 +29,7 @@ using Gtk;
 using MonoDevelop.Core;
 using MonoDevelop.Ide;
 using MonoDevelop.Components;
+using LibGit2Sharp;
 
 namespace MonoDevelop.VersionControl.Git
 {
@@ -38,44 +39,51 @@ namespace MonoDevelop.VersionControl.Git
 		readonly GitRepository repo;
 		string currentSel;
 		string currentType;
+		string currentRemote;
 		readonly bool rebasing;
-		
+
 		public MergeDialog (GitRepository repo, bool rebasing)
 		{
 			this.Build ();
-			
+
+			this.UseNativeContextMenus ();
+
 			this.repo = repo;
 			this.rebasing = rebasing;
-			
+
 			store = new TreeStore (typeof(string), typeof(Xwt.Drawing.Image), typeof (string), typeof(string));
 			tree.Model = store;
-			
-			CellRendererImage crp = new CellRendererImage ();
-			TreeViewColumn col = new TreeViewColumn ();
+
+			var crp = new CellRendererImage ();
+			var col = new TreeViewColumn ();
 			col.PackStart (crp, false);
 			col.AddAttribute (crp, "image", 1);
-			CellRendererText crt = new CellRendererText ();
+			var crt = new CellRendererText ();
 			col.PackStart (crt, true);
 			col.AddAttribute (crt, "text", 2);
 			tree.AppendColumn (col);
-			
+
 			tree.Selection.Changed += HandleTreeSelectionChanged;
-			
+
 			if (rebasing) {
 				labelHeader.Text = GettextCatalog.GetString ("Select the branch to which to rebase:");
 				checkStage.Label = GettextCatalog.GetString ("Stash/unstash local changes before/after rebasing");
 				buttonOk.Label = GettextCatalog.GetString ("Rebase");
 			}
-			
+
 			checkStage.Active = true;
-			
+
 			Fill ();
 		}
-		
+
 		public string SelectedBranch {
 			get { return currentSel; }
 		}
-		
+
+		public string RemoteName {
+			get { return currentRemote; }
+		}
+
 		public bool StageChanges {
 			get { return checkStage.Active; }
 		}
@@ -90,30 +98,37 @@ namespace MonoDevelop.VersionControl.Git
 			if (tree.Selection.GetSelected (out it)) {
 				currentSel = (string) store.GetValue (it, 0);
 				currentType = (string) store.GetValue (it, 3);
+				if (IsRemote) {
+					TreeIter it2;
+					store.IterParent (out it2, it);
+					currentRemote = (string)store.GetValue (it2, 2);
+				} else {
+					currentRemote = null;
+				}
 			}
 			else
 				currentSel = null;
 			UpdateStatus ();
 		}
-		
+
 		void Fill ()
 		{
 			store.Clear ();
-			
+
 			foreach (Branch b in repo.GetBranches ())
-				store.AppendValues (b.Name, ImageService.GetIcon ("vc-branch", IconSize.Menu), b.Name, "branch");
-			
+				store.AppendValues (b.FriendlyName, ImageService.GetIcon ("vc-branch", IconSize.Menu), b.FriendlyName, "branch");
+
 			foreach (string t in repo.GetTags ())
 				store.AppendValues (t, ImageService.GetIcon ("vc-tag", IconSize.Menu), t, "tag");
-			
-			foreach (RemoteSource r in repo.GetRemotes ()) {
+
+			foreach (Remote r in repo.GetRemotes ()) {
 				TreeIter it = store.AppendValues (null, ImageService.GetIcon ("vc-repository", IconSize.Menu), r.Name, null);
 				foreach (string b in repo.GetRemoteBranches (r.Name))
 					store.AppendValues (it, r.Name + "/" + b, ImageService.GetIcon ("vc-branch", IconSize.Menu), b, "remote");
 			}
 			UpdateStatus ();
 		}
-		
+
 		void UpdateStatus ()
 		{
 			if (currentSel != null) {
@@ -143,4 +158,3 @@ namespace MonoDevelop.VersionControl.Git
 		}
 	}
 }
-
