@@ -1,21 +1,21 @@
-// 
+//
 // NewStashDialog.cs
-//  
+//
 // Author:
 //       Lluis Sanchez Gual <lluis@novell.com>
-// 
+//
 // Copyright (c) 2011 Novell, Inc (http://www.novell.com)
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -24,6 +24,8 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+using MonoDevelop.Components;
+
 namespace MonoDevelop.VersionControl.Git
 {
 	partial class NewStashDialog : Gtk.Dialog
@@ -31,11 +33,14 @@ namespace MonoDevelop.VersionControl.Git
 		public NewStashDialog ()
 		{
 			this.Build ();
+
+			this.UseNativeContextMenus ();
+
+			entryComment.Activated += (o, e) => Respond (Gtk.ResponseType.Ok);
 		}
-		
+
 		public string Comment {
 			get { return entryComment.Text; }
 		}
 	}
 }
-
