@@ -59,6 +59,10 @@ namespace MonoDevelop.VersionControl.Subversion
 			get { return true; }
 		}
 
+		public override bool SupportsRevertToRevision {
+			get { return true; }
+		}
+
 		new SubversionVersionControl VersionControlSystem {
 			get { return (SubversionVersionControl)base.VersionControlSystem; }
 		}
@@ -198,10 +202,7 @@ namespace MonoDevelop.VersionControl.Subversion
 		
 		protected override void OnCommit (ChangeSet changeSet, IProgressMonitor monitor)
 		{
-			List<FilePath> list = new List<FilePath> ();
-			foreach (ChangeSetItem it in changeSet.Items)
-				list.Add (it.LocalPath);
-			Svn.Commit (list.ToArray (), changeSet.GlobalComment, monitor);
+			Svn.Commit (changeSet.Items.Select (it => it.LocalPath).ToArray (), changeSet.GlobalComment, monitor);
 		}
 
 		void CreateDirectory (string[] paths, string message, IProgressMonitor monitor)
@@ -220,8 +221,7 @@ namespace MonoDevelop.VersionControl.Subversion
 			// svn will successfully revert the first entry (the directory) and then throw an error when trying to revert the
 			// second and third entries because by reverting the directory the files are implicitly reverted. Try to work around
 			// this issue.
-			Array.Sort<FilePath>(localPaths);
-			Array.Reverse (localPaths);
+			Array.Sort<FilePath> (localPaths, (a, b) => b.CompareTo (a));
 			Svn.Revert (localPaths, recurse, monitor);
 		}
 
@@ -293,10 +293,10 @@ namespace MonoDevelop.VersionControl.Subversion
 		public string Root {
 			get {
 				try {
-					UriBuilder ub = new UriBuilder (Url);
-					ub.Path = string.Empty;
-					ub.Query = string.Empty;
-					return ub.ToString ();
+					return new UriBuilder (Url) {
+						Path = string.Empty,
+						Query = string.Empty
+					}.ToString ();
 				} catch {
 					return string.Empty;
 				}
@@ -331,8 +331,8 @@ namespace MonoDevelop.VersionControl.Subversion
 					Svn.Move (localSrcPath, localDestPath, force, monitor);
 				} else {
 					base.OnMoveFile (localSrcPath, localDestPath, force, monitor);
-					if (!destIsVersioned)
-						Add (localDestPath, false, monitor);
+					Add (localDestPath, false, monitor);
+					DeleteFile (localSrcPath, force, monitor, keepLocal: false);
 				}
 			} else {
 				if (!destIsVersioned && IsVersioned (localSrcPath)) {
@@ -341,6 +341,7 @@ namespace MonoDevelop.VersionControl.Subversion
 				} else
 					base.OnMoveFile (localSrcPath, localDestPath, force, monitor);
 			}
+			ClearCachedVersionInfo (localSrcPath, localDestPath);
 		}
 
 		protected override void OnMoveDirectory (FilePath localSrcPath, FilePath localDestPath, bool force, IProgressMonitor monitor)
@@ -535,14 +536,14 @@ namespace MonoDevelop.VersionControl.Subversion
 		public override DiffInfo[] PathDiff (FilePath baseLocalPath, FilePath[] localPaths, bool remoteDiff)
 		{
 			if (localPaths != null) {
-				ArrayList list = new ArrayList ();
+				var list = new List<DiffInfo> ();
 				foreach (string path in localPaths) {
 					string diff = Svn.GetUnifiedDiff (path, false, remoteDiff);
 					if (string.IsNullOrEmpty (diff))
 						continue;
 					list.AddRange (GenerateUnifiedDiffInfo (diff, baseLocalPath, new FilePath[] { path }));
 				}
-				return (DiffInfo[]) list.ToArray (typeof(DiffInfo));
+				return list.ToArray ();
 			} else {
 				string diff = Svn.GetUnifiedDiff (baseLocalPath, true, remoteDiff);
 				return GenerateUnifiedDiffInfo (diff, baseLocalPath, null);
@@ -553,8 +554,8 @@ namespace MonoDevelop.VersionControl.Subversion
 		{
 			List<Annotation> annotations = new List<Annotation> (Svn.GetAnnotations (this, repositoryPath, SvnRevision.First, SvnRevision.Base));
 			Annotation nextRev = new Annotation (GettextCatalog.GetString ("working copy"), "<uncommitted>", DateTime.MinValue);
-			var baseDocument = new Mono.TextEditor.TextDocument (GetBaseText (repositoryPath));
-			var workingDocument = new Mono.TextEditor.TextDocument (File.ReadAllText (repositoryPath));
+			var baseDocument = Mono.TextEditor.TextDocument.CreateImmutableDocument (GetBaseText (repositoryPath));
+			var workingDocument = Mono.TextEditor.TextDocument.CreateImmutableDocument (File.ReadAllText (repositoryPath));
 			
 			// "SubversionException: blame of the WORKING revision is not supported"
 			foreach (var hunk in baseDocument.Diff (workingDocument)) {
