@@ -33,7 +33,6 @@ using Mono.TextEditor;
 using MonoDevelop.Ide;
 using System.Threading;
 using MonoDevelop.Core;
-using System.Text;
 using MonoDevelop.Components.Commands;
 
 namespace MonoDevelop.VersionControl.Views
@@ -337,35 +336,6 @@ namespace MonoDevelop.VersionControl.Views
 				line = Editor.Document.OffsetToLineNumber (lastFold);
 		}
 
-		internal static string FormatMessage (string msg)
-		{
-			StringBuilder sb = new StringBuilder ();
-			bool wasWs = false;
-			foreach (char ch in msg) {
-				if (ch == ' ' || ch == '\t') {
-					if (!wasWs)
-						sb.Append (' ');
-					wasWs = true;
-					continue;
-				}
-				wasWs = false;
-				sb.Append (ch);
-			}
-			
-			var doc = new TextDocument ();
-			doc.Text = sb.ToString ();
-			for (int i = 1; i <= doc.LineCount; i++) {
-				string text = doc.GetLineText (i).Trim ();
-				int idx = text.IndexOf (':');
-				if (text.StartsWith ("*", StringComparison.Ordinal) && idx >= 0 && idx < text.Length - 1) {
-					int offset = doc.GetLine (i).EndOffsetIncludingDelimiter;
-					msg = text.Substring (idx + 1) + doc.GetTextAt (offset, doc.TextLength - offset);
-					break;
-				}
-			}
-			return msg.TrimStart (' ', '\t');
-		}
-
 		class BlameRenderer : DrawingArea 
 		{
 			static readonly Annotation locallyModified = new Annotation ("", "?", DateTime.MinValue);
@@ -381,22 +351,25 @@ namespace MonoDevelop.VersionControl.Views
 			public BlameRenderer (BlameWidget widget)
 			{
 				this.widget = widget;
-				widget.info.Updated += delegate { QueueDraw (); };
+				widget.info.Updated += OnWidgetChanged;
 				annotations = new List<Annotation> ();
 				UpdateAnnotations ();
 	//			widget.Document.Saved += UpdateAnnotations;
 				document = widget.Editor.Document;
 				document.TextReplacing += EditorDocumentTextReplacing;
 				document.LineChanged += EditorDocumentLineChanged;
-				widget.vScrollBar.ValueChanged += delegate {
-					QueueDraw ();
-				};
+				widget.vScrollBar.ValueChanged += OnWidgetChanged;
 				
 				layout = new Pango.Layout (PangoContext);
 				Events |= EventMask.ButtonPressMask | EventMask.ButtonReleaseMask | EventMask.PointerMotionMask | EventMask.LeaveNotifyMask;
 				OptionsChanged ();
 				Show ();
 			}
+
+			void OnWidgetChanged (object sender, EventArgs e)
+			{
+				QueueDraw ();
+			}
 			
 			public void OptionsChanged ()
 			{
@@ -418,6 +391,11 @@ namespace MonoDevelop.VersionControl.Views
 					layout.Dispose ();
 					layout = null;
 				}
+				if (widget != null && widget.info != null) {
+					widget.info.Updated -= OnWidgetChanged;
+					widget.vScrollBar.ValueChanged -= OnWidgetChanged;
+					widget = null;
+				}
 			}
 			
 			internal double highlightPositon;
@@ -437,7 +415,7 @@ namespace MonoDevelop.VersionControl.Views
 				int startLine = widget.Editor.YToLine (widget.Editor.VAdjustment.Value + evnt.Y);
 				var ann = startLine > 0 && startLine <= annotations.Count ? annotations[startLine - 1] : null;
 				if (ann != null)
-					TooltipText = GetCommitMessage (startLine, true);
+					TooltipText = GetCommitMessage (startLine - 1, true);
 
 				highlightPositon = evnt.Y;
 				if (highlightAnnotation != ann) {
@@ -780,11 +758,11 @@ namespace MonoDevelop.VersionControl.Views
 							line++;
 							widget.JumpOverFoldings (ref line);
 						}
-						
+
 						if (ann != null && line - lineStart > 1) {
 							string msg = GetCommitMessage (lineStart, false);
 							if (!string.IsNullOrEmpty (msg)) {
-								msg = FormatMessage (msg);
+								msg = Revision.FormatMessage (msg);
 
 								layout.SetText (msg);
 								layout.Width = (int)(Allocation.Width * Pango.Scale.PangoScale);
