@@ -42,10 +42,7 @@ namespace MonoDevelop.VersionControl.Views
 			var lw = new LogWidget (info);
 			
 			widget = lw;
-			info.Updated += delegate {
-				lw.History = this.info.History;
-				vinfo   = this.info.VersionInfo;
-			};
+			info.Updated += OnInfoUpdated;
 			lw.History = this.info.History;
 			vinfo   = this.info.VersionInfo;
 		
@@ -53,6 +50,12 @@ namespace MonoDevelop.VersionControl.Views
 				widget.SetToolbar (WorkbenchWindow.GetToolbar (this));
 		}
 
+		void OnInfoUpdated (object sender, EventArgs e)
+		{
+			widget.History = this.info.History;
+			vinfo   = this.info.VersionInfo;
+		}
+
 		[Obsolete]
 		public LogView (string filepath, bool isDirectory, Revision [] history, Repository vc) 
 			: base (Path.GetFileName (filepath) + " Log")
@@ -61,7 +64,7 @@ namespace MonoDevelop.VersionControl.Views
 				this.vinfo = vc.GetVersionInfo (filepath, VersionInfoQueryFlags.IgnoreCache);
 			}
 			catch (Exception ex) {
-				MessageService.ShowException (ex, GettextCatalog.GetString ("Version control command failed."));
+				MessageService.ShowError (GettextCatalog.GetString ("Version control command failed."), ex);
 			}
 			
 			// Widget setup
@@ -100,6 +103,10 @@ namespace MonoDevelop.VersionControl.Views
 				changedpathstore.Dispose ();
 				changedpathstore = null;
 			}
+			if (info != null) {
+				info.Updated -= OnInfoUpdated;
+				info = null;
+			}
 			base.Dispose ();
 		}
 
