@@ -256,7 +256,8 @@ namespace MonoDevelop.VersionControl.Views
 			if (info.Repository.SupportsRevertRevision)
 				toolbar.Add (revertButton);
 
-			toolbar.Add (revertToButton);
+			if (info.Repository.SupportsRevertToRevision)
+				toolbar.Add (revertToButton);
 			toolbar.Add (refreshButton);
 
 			Gtk.HBox a = new Gtk.HBox ();
@@ -374,8 +375,7 @@ namespace MonoDevelop.VersionControl.Views
 						prevRev = rev.GetPrevious ();
 					} catch (Exception e) {
 						Application.Invoke (delegate {
-							LoggingService.LogError ("Error while getting previous revision", e);
-							MessageService.ShowException (e, "Error while getting previous revision.");
+							MessageService.ShowError ("Error while getting previous revision.", e);
 						});
 						return;
 					}
@@ -384,7 +384,7 @@ namespace MonoDevelop.VersionControl.Views
 					if (text == null) {
 						lines = new [] { " Binary files differ" };
 					} else {
-						var changedDocument = new Mono.TextEditor.TextDocument (text);
+						var changedDocument = Mono.TextEditor.TextDocument.CreateImmutableDocument (text);
 						if (prevRev == null) {
 							lines = new string [changedDocument.LineCount];
 							for (int i = 0; i < changedDocument.LineCount; i++) {
@@ -411,7 +411,7 @@ namespace MonoDevelop.VersionControl.Views
 								}
 							}
 
-							var originalDocument = new Mono.TextEditor.TextDocument (prevRevisionText);
+							var originalDocument = Mono.TextEditor.TextDocument.CreateImmutableDocument (prevRevisionText);
 							originalDocument.FileName = "Revision " + prevRev;
 							changedDocument.FileName = "Revision " + rev;
 							lines = Mono.TextEditor.Utils.Diff.GetDiffString (originalDocument, changedDocument).Split ('\n');
@@ -458,6 +458,7 @@ namespace MonoDevelop.VersionControl.Views
 			diffRenderer.Dispose ();
 			messageRenderer.Dispose ();
 			textRenderer.Dispose ();
+			treeviewFiles.Dispose ();
 		}
 		
 		static void DateFunc (Gtk.TreeViewColumn tree_column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
@@ -465,16 +466,18 @@ namespace MonoDevelop.VersionControl.Views
 			CellRendererText renderer = (CellRendererText)cell;
 			var rev = (Revision)model.GetValue (iter, 0);
 			string day;
-			var age = rev.Time - DateTime.Now;
-			if (age.Days == 0) {
+
+			// Grab today's day and the start of tomorrow's day to make Today/Yesterday calculations.
+			var now = DateTime.Now;
+			var age = new DateTime (now.Year, now.Month, now.Day).AddDays(1) - rev.Time;
+			if (age.Days >= 0 && age.Days < 1) { // Check whether it's a commit that's less than a day away. Also discard future commits.
 				day = GettextCatalog.GetString ("Today");
-			} else if (age.Days == 1) {
+			} else if (age.Days < 2) { // Check whether it's a commit from yesterday.
 				day = GettextCatalog.GetString ("Yesterday");
 			} else {
 				day = rev.Time.ToShortDateString ();
 			}
-			string time = rev.Time.ToString ("HH:MM");
-			renderer.Text = day + " " + time;
+			renderer.Text = string.Format ("{0} {1:HH:MM}", day, rev.Time);
 		}	
 		
 		static void GraphFunc (Gtk.TreeViewColumn tree_column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
@@ -495,7 +498,7 @@ namespace MonoDevelop.VersionControl.Views
 			if (string.IsNullOrEmpty (rev.Message)) {
 				renderer.Text = GettextCatalog.GetString ("(No message)");
 			} else {
-				string message = BlameWidget.FormatMessage (rev.Message);
+				string message = Revision.FormatMessage (rev.Message);
 				int idx = message.IndexOf ('\n');
 				if (idx > 0)
 					message = message.Substring (0, idx);
@@ -584,6 +587,9 @@ namespace MonoDevelop.VersionControl.Views
 					var rev = (Revision)logstore.GetValue (iter, 0);
 					if (rev.ToString () == value.ToString ()) {
 						treeviewLog.Selection.SelectIter (iter);
+						TreePath path = logstore.GetPath (iter);
+						treeviewLog.ScrollToCell (path, treeviewLog.Columns[0], true, 0, 0);
+						treeviewLog.SetCursorOnCell (path, treeviewLog.Columns[0], textRenderer, true);
 						return;
 					}
 				} while (treeviewLog.Model.IterNext (ref iter));
