@@ -134,17 +134,18 @@ namespace MonoDevelop.VersionControl.Views
 			widget = main;
 
 			buttonCommit = new Gtk.Button () {
-				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("commit-light-16.png")).ToGtkWidget (),
-				Label = GettextCatalog.GetString ("Commit...")
+				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("commit-16.png")).ToGtkWidget (),
+				Label = GettextCatalog.GetString ("Commit..."),
+				Name = "buttonCommit"
 			};
 			buttonCommit.Image.Show ();
 			buttonRevert = new Gtk.Button () {
-				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("revert-light-16.png")).ToGtkWidget (),
+				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("revert-16.png")).ToGtkWidget (),
 				Label = GettextCatalog.GetString ("Revert")
 			};
 			buttonRevert.Image.Show ();
 			showRemoteStatus = new Gtk.Button () {
-				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("remote-status-light-16.png")).ToGtkWidget (),
+				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("remote-status-16.png")).ToGtkWidget (),
 				Label = GettextCatalog.GetString ("Show Remote Status")
 			};
 			showRemoteStatus.Image.Show ();
@@ -162,14 +163,15 @@ namespace MonoDevelop.VersionControl.Views
 			scroller.VscrollbarPolicy = PolicyType.Automatic;
 			filelist.RowActivated += OnRowActivated;
 			filelist.DiffLineActivated += OnDiffLineActivated;
+			filelist.CommitSelectionToggled += OnCommitSelectionToggled;
 
 			cellToggle = new CellRendererToggle();
-			cellToggle.Toggled += new ToggledHandler(OnCommitToggledHandler);
+			cellToggle.Toggled += new ToggledHandler(OnCommitCellToggled);
 			var crc = new CellRendererImage ();
 			crc.StockId = "vc-comment";
 			colCommit = new TreeViewColumn ();
 			colCommit.Spacing = 2;
-			colCommit.Widget = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("commit-light-16.png")).ToGtkWidget ();
+			colCommit.Widget = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("commit-16.png")).ToGtkWidget ();
 			colCommit.Widget.Show ();
 			colCommit.PackStart (cellToggle, false);
 			colCommit.PackStart (crc, false);
@@ -229,7 +231,7 @@ namespace MonoDevelop.VersionControl.Views
 			HBox labBox = new HBox ();
 			labelCommit = new Gtk.Label (GettextCatalog.GetString ("Commit message:"));
 			labelCommit.Xalign = 0;
-			labBox.PackStart (new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("comment-light-16.png")).ToGtkWidget (), false, false, 0);
+			labBox.PackStart (new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("comment-16.png")).ToGtkWidget (), false, false, 0);
 			labBox.PackStart (labelCommit, true, true, 3);
 
 			commitMessageLabelBox.Add (labBox);
@@ -316,7 +318,7 @@ namespace MonoDevelop.VersionControl.Views
 				toolbar.Add (showRemoteStatus);
 
 			var btnCreatePatch = new Gtk.Button () {
-				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("diff-light-16.png")).ToGtkWidget (),
+				Image = new Xwt.ImageView (Xwt.Drawing.Image.FromResource ("diff-16.png")).ToGtkWidget (),
 				Label = GettextCatalog.GetString ("Create Patch")
 			};
 			btnCreatePatch.Image.Show ();
@@ -409,13 +411,14 @@ namespace MonoDevelop.VersionControl.Views
 				filelist.DoPopupMenu = null;
 				filelist.RowActivated -= OnRowActivated;
 				filelist.DiffLineActivated -= OnDiffLineActivated;
+				filelist.CommitSelectionToggled -= OnCommitSelectionToggled;
 				filelist.TestExpandRow -= OnTestExpandRow;
 				filelist.Selection.Changed -= OnCursorChanged;
 				filelist.Destroy ();
 				filelist = null;
 			}
 			if (cellToggle != null) {
-				cellToggle.Toggled -= OnCommitToggledHandler;
+				cellToggle.Toggled -= OnCommitCellToggled;
 				cellToggle.Destroy ();
 				cellToggle = null;
 			}
@@ -710,11 +713,8 @@ namespace MonoDevelop.VersionControl.Views
 			OnOpen (null, null);
 		}
 
-		void OnCommitToggledHandler(object o, ToggledArgs args) {
-			TreeIter pos;
-			if (!filestore.GetIterFromString (out pos, args.Path))
-				return;
-
+		void ToggleCommitStatus(TreeIter pos)
+		{
 			string localpath = (string) filestore.GetValue (pos, ColFullPath);
 
 			if (changeSet.ContainsFile (localpath)) {
@@ -728,6 +728,20 @@ namespace MonoDevelop.VersionControl.Views
 			UpdateSelectionStatus ();
 		}
 
+		void OnCommitSelectionToggled (object sender, EventArgs e)
+		{
+			filelist.Selection.SelectedForeach((model, path, iter) => ToggleCommitStatus(iter)); 
+		}
+
+		void OnCommitCellToggled(object o, ToggledArgs args) 
+		{
+			TreeIter pos;
+			if (!filestore.GetIterFromString (out pos, args.Path))
+				return;
+
+			ToggleCommitStatus (pos);
+		}
+
 		VersionInfo GetVersionInfo (string file)
 		{
 			foreach (VersionInfo vi in statuses)
@@ -1112,6 +1126,13 @@ namespace MonoDevelop.VersionControl.Views
 		[CommandHandler (MonoDevelop.Ide.Commands.EditCommands.Copy)]
 		protected void OnCopy ()
 		{
+			Gtk.Clipboard clipboard;
+			if (commitText.HasFocus) {
+				clipboard = Gtk.Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
+				commitText.Buffer.CopyClipboard (clipboard);
+				return;
+			}
+
 			StringBuilder sb = new StringBuilder ();
 			TreeIter iter;
 			foreach (var p in filelist.Selection.GetSelectedRows ()) {
@@ -1123,7 +1144,7 @@ namespace MonoDevelop.VersionControl.Views
 					sb.AppendLine (line);
 			}
 
-			var clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
+			clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
 			clipboard.Text = sb.ToString ();
 			clipboard = Clipboard.Get (Gdk.Atom.Intern ("PRIMARY", false));
 			clipboard.Text = sb.ToString ();
@@ -1134,6 +1155,32 @@ namespace MonoDevelop.VersionControl.Views
 	{
 		const Gdk.ModifierType selectionModifiers = Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask;
 
+		protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
+		{
+			if (evnt.Key == Gdk.Key.Right || evnt.Key == Gdk.Key.KP_Right) {
+				Selection.SelectedForeach ((model, path, iter) => ExpandRow (path, true));
+				return true;
+			}
+
+			if (evnt.Key == Gdk.Key.Left || evnt.Key == Gdk.Key.KP_Left) {
+				Selection.SelectedForeach ((model, path, iter) => CollapseRow (path));
+				return true;
+			}
+
+			if (evnt.Key == Gdk.Key.space && CommitSelectionToggled != null) {
+				CommitSelectionToggled (this, EventArgs.Empty);
+				return true;
+			}
+
+			if (evnt.Key == Gdk.Key.Return || evnt.Key == Gdk.Key.KP_Enter) {
+				if (DiffLineActivated != null)
+					DiffLineActivated (this, EventArgs.Empty);
+				return true;
+			}
+
+			return base.OnKeyPressEvent (evnt);
+		}
+
 		protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
 		{
 			bool keepPos = false;
@@ -1228,5 +1275,6 @@ namespace MonoDevelop.VersionControl.Views
 
 		public Action<Gdk.EventButton> DoPopupMenu;
 		public event EventHandler DiffLineActivated;
+		public event EventHandler CommitSelectionToggled;
 	}
 }
