@@ -24,11 +24,12 @@ namespace MonoDevelop.VersionControl
 					dlg.EndCommit (false);
 				} finally {
 					dlg.Destroy ();
+					dlg.Dispose ();
 				}
 				VersionControlService.NotifyAfterCommit (vc, changeSet, false);
 			}
 			catch (Exception ex) {
-					MessageService.ShowException (ex, GettextCatalog.GetString ("Version control command failed."));
+				MessageService.ShowError (GettextCatalog.GetString ("Version control command failed."), ex);
 			}
 		}
 
