@@ -8,12 +8,16 @@ using MonoDevelop.Core.Serialization;
 using MonoDevelop.Core;
 using System.Linq;
 using System.Threading;
+using MonoDevelop.Core.Instrumentation;
+using MonoDevelop.Ide;
 
 namespace MonoDevelop.VersionControl
 {
 	[DataItem (FallbackType=typeof(UnknownRepository))]
 	public abstract class Repository: IDisposable
 	{
+		static Counter Repositories = InstrumentationService.CreateCounter ("VersionControl.RepositoryOpened", "Version Control", id:"VersionControl.RepositoryOpened");
+
 		string name;
 		VersionControlSystem vcs;
 		
@@ -38,6 +42,32 @@ namespace MonoDevelop.VersionControl
 		protected Repository (VersionControlSystem vcs): this ()
 		{
 			VersionControlSystem = vcs;
+			Repositories.SetValue (Repositories.Count + 1, string.Format ("Repository #{0}", Repositories.Count + 1), new Dictionary<string, string> {
+				{ "Type", vcs.Name },
+				{ "Type+Version", string.Format ("{0} {1}", vcs.Name, vcs.Version) },
+			});
+		}
+
+		public override bool Equals (object obj)
+		{
+			var other = obj as Repository;
+			return other != null &&
+				other.RootPath == RootPath &&
+				other.VersionControlSystem == VersionControlSystem &&
+				other.LocationDescription == LocationDescription &&
+				other.Name == Name;
+		}
+
+		public override int GetHashCode ()
+		{
+			int result = 0;
+			result ^= RootPath.GetHashCode ();
+			if (VersionControlSystem != null)
+				result ^= VersionControlSystem.GetHashCode ();
+			if (LocationDescription != null)
+				result ^= LocationDescription.GetHashCode ();
+			result ^= Name.GetHashCode ();
+			return result;
 		}
 		
 		public virtual void CopyConfigurationFrom (Repository other)
@@ -64,9 +94,17 @@ namespace MonoDevelop.VersionControl
 			if (--references == 0)
 				Dispose ();
 		}
-		
+
 		public virtual void Dispose ()
 		{
+			if (!queryRunning)
+				return;
+
+			lock (queryLock) {
+				fileQueryQueue.Clear ();
+				directoryQueryQueue.Clear ();
+				recursiveDirectoryQueryQueue.Clear ();
+			}
 		}
 		
 		// Display name of the repository
@@ -88,14 +126,8 @@ namespace MonoDevelop.VersionControl
 		// Version control system that manages this repository
 		public VersionControlSystem VersionControlSystem {
 			get {
-				if (vcs == null && vcsName != null) {
-					foreach (VersionControlSystem v in VersionControlService.GetVersionControlSystems ()) {
-						if (v.Id == vcsName) {
-							vcs = v;
-							break;
-						}
-					}
-				}
+				if (vcs == null && vcsName != null)
+					vcs = VersionControlService.GetVersionControlSystems ().FirstOrDefault (v => v.Id == vcsName);
 				return vcs;
 			}
 			
@@ -133,13 +165,20 @@ namespace MonoDevelop.VersionControl
 		public virtual bool SupportsRevertRevision {
 			get { return false; }
 		}
+
+		public virtual bool SupportsRevertToRevision {
+			get { return false; }
+		}
 		
 		internal protected virtual VersionControlOperation GetSupportedOperations (VersionInfo vinfo)
 		{
 			VersionControlOperation operations = VersionControlOperation.None;
 			bool exists = !vinfo.LocalPath.IsNullOrEmpty && (File.Exists (vinfo.LocalPath) || Directory.Exists (vinfo.LocalPath));
 			if (vinfo.IsVersioned) {
-				operations = VersionControlOperation.Commit | VersionControlOperation.Update | VersionControlOperation.Log;
+				operations = VersionControlOperation.Commit | VersionControlOperation.Update;
+				if (!vinfo.HasLocalChange (VersionStatus.ScheduledAdd))
+					operations |= VersionControlOperation.Log;
+
 				if (exists) {
 					if (!vinfo.HasLocalChange (VersionStatus.ScheduledDelete))
 						operations |= VersionControlOperation.Remove;
@@ -167,12 +206,14 @@ namespace MonoDevelop.VersionControl
 				LoggingService.LogError ("VersionControl returned {0} items for {1}", infos.Length, localPath);
 				LoggingService.LogError ("The infos were: {0}", string.Join (" ::: ", infos.Select (i => i.LocalPath)));
 			}
-			try {
+			// HACK: This was slowing down the IDE a lot in case in the eventuality of submodules.
+			return infos [0];
+			/*try {
 				return infos.Single ();
 			} catch (InvalidOperationException) {
 				// Workaround for #17216.
 				return infos [0];
-			}
+			}*/
 		}
 		
 		/// <summary>
@@ -216,20 +257,40 @@ namespace MonoDevelop.VersionControl
 			return result;
 		}
 
+		RecursiveDirectoryInfoQuery AcquireLockForQuery (FilePath path, bool getRemoteStatus)
+		{
+			RecursiveDirectoryInfoQuery rq;
+			bool query = false;
+			lock (queryLock) {
+				rq = recursiveDirectoryQueryQueue.FirstOrDefault (q => q.Directory == path);
+				if (rq == null) {
+					query = true;
+					var mre = new ManualResetEvent (false);
+					rq = new RecursiveDirectoryInfoQuery {
+						Directory = path,
+						GetRemoteStatus = getRemoteStatus,
+						ResetEvent = mre,
+						Count = 1,
+					};
+				} else
+					Interlocked.Increment (ref rq.Count);
+			}
+			if (query)
+				AddQuery (rq);
+			return rq;
+		}
+
 		public VersionInfo[] GetDirectoryVersionInfo (FilePath localDirectory, bool getRemoteStatus, bool recursive)
 		{
 			try {
 				if (recursive) {
-					using (var mre = new ManualResetEvent (false)) {
-						var rq = new RecursiveDirectoryInfoQuery {
-							Directory = localDirectory,
-							GetRemoteStatus = getRemoteStatus,
-							ResetEvent = mre,
-						};
-						AddQuery (rq);
-						rq.ResetEvent.WaitOne ();
-						return rq.Result;
-					}
+					RecursiveDirectoryInfoQuery rq = AcquireLockForQuery (localDirectory, getRemoteStatus);
+					rq.ResetEvent.WaitOne ();
+
+					lock (queryLock)
+						if (Interlocked.Decrement (ref rq.Count) == 0)
+							rq.ResetEvent.Dispose ();
+					return rq.Result;
 				}
 
 				var status = infoCache.GetDirectoryStatus (localDirectory);
@@ -274,8 +335,9 @@ namespace MonoDevelop.VersionControl
 
 		class RecursiveDirectoryInfoQuery : DirectoryInfoQuery
 		{
-			public VersionInfo[] Result;
+			public VersionInfo[] Result = new VersionInfo[0];
 			public ManualResetEvent ResetEvent;
+			public int Count;
 		}
 
 		Queue<VersionInfoQuery> fileQueryQueue = new Queue<VersionInfoQuery> ();
@@ -372,6 +434,25 @@ namespace MonoDevelop.VersionControl
 			} catch (Exception ex) {
 				LoggingService.LogError ("Version control status query failed", ex);
 
+				//Release all items in current batch
+				foreach (var item in recursiveDirectoryQueryQueueClone)
+					item.ResetEvent.Set ();
+
+				lock (queryLock) {
+					queryRunning = false;
+						
+					fileQueryQueue.Clear ();
+					filesInQueryQueue.Clear ();
+
+					directoriesInQueryQueue.Clear ();
+					directoryQueryQueue.Clear ();
+
+					recursiveDirectoryQueryQueueClone = recursiveDirectoryQueryQueue.ToArray ();
+					recursiveDirectoriesInQueryQueue.Clear ();
+					recursiveDirectoryQueryQueue.Clear ();
+				}
+
+				//Release newly pending
 				foreach (var item in recursiveDirectoryQueryQueueClone)
 					item.ResetEvent.Set ();
 			}
@@ -654,10 +735,7 @@ namespace MonoDevelop.VersionControl
 		// it can be a list of files to compare.
 		public DiffInfo[] PathDiff (ChangeSet cset, bool remoteDiff)
 		{
-			List<FilePath> paths = new List<FilePath> ();
-			foreach (ChangeSetItem item in cset.Items)
-				paths.Add (item.LocalPath);
-			return PathDiff (cset.BaseLocalPath, paths.ToArray (), remoteDiff);
+			return PathDiff (cset.BaseLocalPath, cset.Items.Select (i => i.LocalPath).ToArray (), remoteDiff);
 		}
 		
 		/// <summary>
@@ -693,7 +771,7 @@ namespace MonoDevelop.VersionControl
 		static protected DiffInfo[] GenerateUnifiedDiffInfo (string diffContent, FilePath basePath, FilePath[] localPaths)
 		{
 			basePath = basePath.FullPath;
-			ArrayList list = new ArrayList ();
+			var list = new List<DiffInfo> ();
 			using (StringReader sr = new StringReader (diffContent)) {
 				string line;
 				StringBuilder content = new StringBuilder ();
@@ -742,7 +820,7 @@ namespace MonoDevelop.VersionControl
 					list.Add (new DiffInfo (basePath, fileName, content.ToString ()));
 				}
 			}
-			return (DiffInfo[]) list.ToArray (typeof(DiffInfo));
+			return list.ToArray ();
 		}
 		
 		/// <summary>
@@ -785,6 +863,11 @@ namespace MonoDevelop.VersionControl
 		}
 
 		protected abstract void OnUnignore (FilePath[] localPath);
+
+		public virtual bool GetFileIsText (FilePath path)
+		{
+			return DesktopService.GetFileIsText (path);
+		}
 	}
 	
 	public class Annotation
@@ -841,27 +924,26 @@ namespace MonoDevelop.VersionControl
 	
 	public class DiffInfo
 	{
-		FilePath fileName;
-		FilePath basePath;
-		string content;
-		
 		public DiffInfo (FilePath basePath, FilePath fileName, string content)
 		{
-			this.basePath = basePath;
-			this.fileName = fileName;
-			this.content = content.Replace ("\r","");
+			BasePath = basePath;
+			FileName = fileName;
+			Content = content.Replace ("\r","");
 		}
 		
 		public FilePath FileName {
-			get { return fileName; }
+			get;
+			private set;
 		}
 		
 		public string Content {
-			get { return content; }
+			get;
+			private set;
 		}
 		
 		public FilePath BasePath {
-			get { return basePath; }
+			get;
+			private set;
 		}
 	}
 
