@@ -1,13 +1,12 @@
 using System;
+using System.Text;
+using Mono.TextEditor;
 
 namespace MonoDevelop.VersionControl
 {
 	public abstract class Revision
 	{
 		Repository repo;
-		DateTime time;
-		string author;
-		string message;
 		string shortMessage;
 		
 		protected Revision (Repository repo)
@@ -18,21 +17,21 @@ namespace MonoDevelop.VersionControl
 		protected Revision (Repository repo, DateTime time, string author, string message)
 		{
 			this.repo = repo;
-			this.time = time;
-			this.author = author;
-			this.message = message;
+			this.Time = time;
+			this.Author = author;
+			this.Message = message;
 		}
 		
 		public abstract Revision GetPrevious ();
 		
 		public DateTime Time {
-			get { return time; }
-			protected set { time = value; }
+			get;
+			protected set;
 		}
 		
 		public string Author {
-			get { return author; }
-			protected set { author = value; }
+			get;
+			protected set;
 		}
 		
 		public string Email { get; set; }
@@ -52,8 +51,8 @@ namespace MonoDevelop.VersionControl
 		}
 		
 		public string Message {
-			get { return message; }
-			protected set { message = value; }
+			get;
+			protected set;
 		}
 		
 		public override int GetHashCode() { return ToString().GetHashCode(); }
@@ -70,35 +69,59 @@ namespace MonoDevelop.VersionControl
 		public virtual string ShortName {
 			get { return Name; }
 		}
+
+		internal static string FormatMessage (string msg)
+		{
+			StringBuilder sb = new StringBuilder ();
+			bool wasWs = false;
+			foreach (char ch in msg) {
+				if (ch == ' ' || ch == '\t') {
+					if (!wasWs)
+						sb.Append (' ');
+					wasWs = true;
+					continue;
+				}
+				wasWs = false;
+				sb.Append (ch);
+			}
+
+			var doc = TextDocument.CreateImmutableDocument (sb.ToString());
+			foreach (var line in doc.Lines) {
+				string text = doc.GetTextAt (line.Offset, line.Length).Trim ();
+				int idx = text.IndexOf (':');
+				if (text.StartsWith ("*", StringComparison.Ordinal) && idx >= 0 && idx < text.Length - 1) {
+					int offset = line.EndOffsetIncludingDelimiter;
+					msg = text.Substring (idx + 1) + doc.GetTextAt (offset, doc.TextLength - offset);
+					break;
+				}
+			}
+			return msg.TrimStart (' ', '\t');
+		}
 	}
 	
 	public class RevisionPath
 	{
-		string path;
-		RevisionAction action;
-		string actionDescription;
-		
 		public RevisionPath (string path, RevisionAction action, string actionDescription)
 		{
-			this.path = path;
-			this.action = action;
-			this.actionDescription = actionDescription;
+			this.Path = path;
+			this.Action = action;
+			this.ActionDescription = actionDescription;
 		}
 		
 		public string Path {
-			get { return path; }
-			set { path = value; }
+			get;
+			set;
 		}
 		
 		public RevisionAction Action {
-			get { return action; }
-			set { action = value; }
+			get;
+			set;
 		}
 		
 		// To use when Action == RevisionAction.Other
 		public string ActionDescription {
-			get { return actionDescription; }
-			set { actionDescription = value; }
+			get;
+			set;
 		}
 	}
 	
