@@ -387,11 +387,6 @@ namespace MonoDevelop.VersionControl
 		[CommandUpdateHandler (Commands.ResolveConflicts)]
 		protected void UpdateResolveConflicts (CommandInfo item)
 		{
-			if (!(CurrentNode.DataItem is UnknownSolutionItem)) {
-				item.Visible = false;
-				return;
-			}
-
 			TestCommand (Commands.ResolveConflicts, item, false);
 		}
 
@@ -480,7 +475,7 @@ namespace MonoDevelop.VersionControl
 				if (test)
 					LoggingService.LogError (ex.ToString ());
 				else
-					MessageService.ShowException (ex, GettextCatalog.GetString ("Version control command failed."));
+					MessageService.ShowError (GettextCatalog.GetString ("Version control command failed."), ex);
 				return TestResult.Disable;
 			}
 			
