@@ -23,6 +23,7 @@ namespace MonoDevelop.VersionControl
 	{
 		static Xwt.Drawing.Image overlay_modified;
 		static Xwt.Drawing.Image overlay_removed;
+		static Xwt.Drawing.Image overlay_renamed;
 		static Xwt.Drawing.Image overlay_conflicted;
 		static Xwt.Drawing.Image overlay_added;
 		internal static Xwt.Drawing.Image overlay_controled;
@@ -55,22 +56,23 @@ namespace MonoDevelop.VersionControl
 		{
 			IdeApp.Initialized += delegate {
 				try {
-					overlay_modified = Xwt.Drawing.Image.FromResource("modified-overlay-light-16.png");
-					overlay_removed = Xwt.Drawing.Image.FromResource("removed-overlay-light-16.png");
-					overlay_conflicted = Xwt.Drawing.Image.FromResource("conflict-overlay-light-16.png");
-					overlay_added = Xwt.Drawing.Image.FromResource("added-overlay-light-16.png");
-					overlay_controled = Xwt.Drawing.Image.FromResource("versioned-overlay-light-16.png");
-					overlay_unversioned = Xwt.Drawing.Image.FromResource("unversioned-overlay-light-16.png");
-					overlay_protected = Xwt.Drawing.Image.FromResource("lock-required-overlay-light-16.png");
-					overlay_unlocked = Xwt.Drawing.Image.FromResource("unlocked-overlay-light-16.png");
-					overlay_locked = Xwt.Drawing.Image.FromResource("locked-overlay-light-16.png");
-					overlay_ignored = Xwt.Drawing.Image.FromResource("ignored-overlay-light-16.png");
+					overlay_modified = Xwt.Drawing.Image.FromResource("modified-overlay-16.png");
+					overlay_removed = Xwt.Drawing.Image.FromResource("removed-overlay-16.png");
+					overlay_renamed = Xwt.Drawing.Image.FromResource("renamed-overlay-16.png");
+					overlay_conflicted = Xwt.Drawing.Image.FromResource("conflict-overlay-16.png");
+					overlay_added = Xwt.Drawing.Image.FromResource("added-overlay-16.png");
+					overlay_controled = Xwt.Drawing.Image.FromResource("versioned-overlay-16.png");
+					overlay_unversioned = Xwt.Drawing.Image.FromResource("unversioned-overlay-16.png");
+					overlay_protected = Xwt.Drawing.Image.FromResource("lock-required-overlay-16.png");
+					overlay_unlocked = Xwt.Drawing.Image.FromResource("unlocked-overlay-16.png");
+					overlay_locked = Xwt.Drawing.Image.FromResource("locked-overlay-16.png");
+					overlay_ignored = Xwt.Drawing.Image.FromResource("ignored-overlay-16.png");
 
 					icon_modified = ImageService.GetIcon ("vc-file-modified", Gtk.IconSize.Menu);
 					icon_removed = ImageService.GetIcon ("vc-file-removed", Gtk.IconSize.Menu);
 					icon_conflicted = ImageService.GetIcon ("vc-file-conflicted", Gtk.IconSize.Menu);
 					icon_added = ImageService.GetIcon ("vc-file-added", Gtk.IconSize.Menu);
-					icon_controled = Xwt.Drawing.Image.FromResource("versioned-overlay-light-16.png");
+					icon_controled = Xwt.Drawing.Image.FromResource("versioned-overlay-16.png");
 				} catch (Exception e) {
 					LoggingService.LogError ("Error while loading icons.", e);
 				}
@@ -122,9 +124,10 @@ namespace MonoDevelop.VersionControl
 			
 			switch (status & VersionStatus.LocalChangesMask) {
 				case VersionStatus.Modified:
-				case VersionStatus.ScheduledReplace:
 				case VersionStatus.ScheduledIgnore:
 					return overlay_modified;
+				case VersionStatus.ScheduledReplace:
+					return overlay_renamed;
 				case VersionStatus.Conflicted:
 					return overlay_conflicted;
 				case VersionStatus.ScheduledAdd:
@@ -207,17 +210,29 @@ namespace MonoDevelop.VersionControl
 			
 			return repo;
 		}
-		
+
 		public static Repository GetRepositoryReference (string path, string id)
 		{
+			VersionControlSystem detectedVCS = null;
+			FilePath bestMatch = FilePath.Null;
+
 			foreach (VersionControlSystem vcs in GetVersionControlSystems ()) {
-				Repository repo = vcs.GetRepositoryReference (path, id);
-				if (repo != null) {
-					repo.VersionControlSystem = vcs;
-					return repo;
+				var newPath = vcs.GetRepositoryPath (path, id);
+				if (!newPath.IsNullOrEmpty) {
+					// Check whether we have no match or if a new match is found with a longer path.
+					// TODO: If the repo root is not the same as the repo reference, ask user for input.
+					// TODO: If we have two version control directories in the same place, ask user for input.
+					if (bestMatch.IsNullOrEmpty) {
+						bestMatch = newPath;
+						detectedVCS = vcs;
+					} else if (bestMatch.CompareTo (newPath) <= 0) {
+						bestMatch = newPath;
+						detectedVCS = vcs;
+					}
 				}
 			}
-			return null;
+			return detectedVCS == null ? null : detectedVCS.GetRepositoryReference (bestMatch, id);
+
 		}
 		
 		internal static void SetCommitComment (string file, string comment, bool save)
@@ -265,13 +280,9 @@ namespace MonoDevelop.VersionControl
 			if (comments != null)
 				return comments;
 
-			ResolveEventHandler localResolve = delegate (object s, ResolveEventArgs args) {
-				foreach (System.Reflection.Assembly asm in AppDomain.CurrentDomain.GetAssemblies ()) {
-					if (asm.GetName ().FullName == args.Name)
-						return asm;
-				}
-				return null;
-			};
+			ResolveEventHandler localResolve = (s, args) =>
+				AppDomain.CurrentDomain.GetAssemblies ()
+					.FirstOrDefault (asm => asm.GetName ().FullName == args.Name);
 
 			string file = CommitMessagesFile;
 			if (File.Exists (file)) {
@@ -285,7 +296,7 @@ namespace MonoDevelop.VersionControl
 				
 					// Remove comments for files that don't exists
 					// Remove comments more than 60 days old
-					
+
 					ArrayList toDelete = new ArrayList ();
 					foreach (DictionaryEntry e in comments) {
 						if (!File.Exists ((string)e.Key))
@@ -372,7 +383,7 @@ namespace MonoDevelop.VersionControl
 				try {
 					PrepareCommit (null, new CommitEventArgs (repo, changeSet, false));
 				} catch (Exception ex) {
-					MessageService.ShowException (ex);
+					LoggingService.LogInternalError (ex);
 				}
 			}
 		}
@@ -390,7 +401,7 @@ namespace MonoDevelop.VersionControl
 				try {
 					BeginCommit (null, new CommitEventArgs (repo, changeSet, false));
 				} catch (Exception ex) {
-					MessageService.ShowException (ex);
+					LoggingService.LogInternalError (ex);
 				}
 			}
 		}
@@ -408,7 +419,7 @@ namespace MonoDevelop.VersionControl
 				try {
 					EndCommit (null, new CommitEventArgs (repo, changeSet, success));
 				} catch (Exception ex) {
-					MessageService.ShowException (ex);
+					LoggingService.LogInternalError (ex);
 					return;
 				}
 			}
@@ -437,6 +448,12 @@ namespace MonoDevelop.VersionControl
 					FileStatusChanged (null, args);
 			}
 		}
+
+		static bool ShouldAddFile (ProjectFileEventInfo info)
+		{
+			const ProjectItemFlags ignoreFlags = ProjectItemFlags.DontPersist | ProjectItemFlags.Hidden;
+			return (info.ProjectFile.Flags & ignoreFlags) != ignoreFlags;
+		}
 		
 		//static void OnFileChanged (object s, ProjectFileEventArgs args)
 		//{
@@ -454,7 +471,8 @@ namespace MonoDevelop.VersionControl
 					Repository repo = GetRepository (repoFiles.Key);
 					if (repo == null)
 						continue;
-					var versionInfos = repo.GetVersionInfo (repoFiles.Select (f => f.ProjectFile.FilePath), VersionInfoQueryFlags.IgnoreCache);
+					var filePaths = repoFiles.Where (ShouldAddFile).Select (f => f.ProjectFile.FilePath);
+					var versionInfos = repo.GetVersionInfo (filePaths, VersionInfoQueryFlags.IgnoreCache);
 					FilePath[] paths = versionInfos.Where (i => i.CanAdd).Select (i => i.LocalPath).ToArray ();
 					if (paths.Length > 0) {
 						if (monitor == null)
@@ -700,18 +718,13 @@ namespace MonoDevelop.VersionControl
 			if (IsGloballyDisabled)
 				return false;
 
-			foreach (VersionControlSystem vcs in GetVersionControlSystems ()) {
-				if (vcs.IsInstalled)
-					return true;
-			}
-
-			return false;
+			return GetVersionControlSystems ().Any (vcs => vcs.IsInstalled);
 		}
 		
 		internal static Repository InternalGetRepositoryReference (string path, string id)
 		{
-			string file = Path.Combine (path, id) + ".mdvcs";
-			if (!File.Exists (file))
+			string file = InternalGetRepositoryPath (path, id);
+			if (file == null)
 				return null;
 			
 			XmlDataSerializer ser = new XmlDataSerializer (dataContext);
@@ -722,6 +735,15 @@ namespace MonoDevelop.VersionControl
 				reader.Close ();
 			}
 		}
+
+		internal static string InternalGetRepositoryPath (string path, string id)
+		{
+			string file = Path.Combine (path, id) + ".mdvcs";
+			if (!File.Exists (file))
+				return null;
+
+			return file;
+		}
 		
 		internal static void InternalStoreRepositoryReference (Repository repo, string path, string id)
 		{
