@@ -1,9 +1,11 @@
 
 using Mono.Addins;
+using Mono.Addins.Description;
 
 [assembly:Addin ("WindowsPlatform", 
     Namespace = "MonoDevelop",
 	Version = MonoDevelop.BuildInfo.Version,
+	Flags = AddinFlags.Hidden,
 	Category = "Platform Support")]
 
 [assembly:AddinName ("MonoDevelop Windows Platform Support")]
