@@ -29,6 +29,8 @@ using System;
 using System.Collections.Generic;
 using System.Diagnostics;
 using System.Text;
+using System.Linq;
+using ICSharpCode.NRefactory;
 
 
 namespace Mono.TextEditor
@@ -239,9 +241,14 @@ namespace Mono.TextEditor
 		{
 			using (var undo = data.OpenUndoGroup ()) {
 				data.EnsureCaretIsNotVirtual ();
+				string indentString;
+				if (data.HasIndentationTracker) {
+					indentString = data.IndentationTracker.GetIndentationString (data.Caret.Location);
+				} else {
+					indentString = data.GetIndentationString (data.Caret.Location);
+				}
 				data.InsertAtCaret (data.EolMarker);
-				var line = data.Document.GetLine (data.Caret.Line);
-				data.InsertAtCaret (data.GetIndentationString (line.EndOffset));
+				data.InsertAtCaret (indentString);
 			}
 		}
 		
@@ -503,5 +510,60 @@ namespace Mono.TextEditor
 				}
 			}
 		}
+
+		public static void SortSelectedLines (TextEditorData data)
+		{
+			var start = data.MainSelection.Start;
+			var end = data.MainSelection.End;
+			var caret = data.Caret.Location;
+
+			int startLine = start.Line;
+			int endLine = end.Line;
+			if (startLine == endLine)
+				return;
+
+			int length = 0;
+			var lines = new string[endLine - startLine + 1];
+			for (int i = startLine; i <= endLine; i++) {
+				//get lines *with* line endings
+				var lineText = data.GetLineText (i, true);
+				lines [i - startLine] = lineText;
+				length += lineText.Length;
+			}
+
+			var linesUnsorted = new string[lines.Length];
+
+			Array.Sort (lines, StringComparer.Ordinal);
+
+			bool changed = false;
+			for (int i = 0; i <= lines.Length; i++) {
+				//can't simply use reference comparison as Array.Sort is not stable
+				if (string.Equals (lines [i], linesUnsorted [i], StringComparison.Ordinal)) {
+					continue;
+				}
+				changed = true;
+				break;
+			}
+			if (!changed) {
+				return;
+			}
+
+
+			var sb = new StringBuilder ();
+			for (int i = 0; i < lines.Length; i++) {
+				sb.Append (lines [i]);
+			}
+
+			var startOffset = data.Document.LocationToOffset (new TextLocation (startLine, 0));
+			data.Replace (startOffset, length, sb.ToString ());
+
+			data.Caret.Location = LimitColumn (data, caret);
+			data.SetSelection (LimitColumn (data, start), LimitColumn (data, end));
+		}
+
+		static DocumentLocation LimitColumn (TextEditorData data, DocumentLocation loc)
+		{
+			return new DocumentLocation (loc.Line, System.Math.Min (loc.Column, data.GetLine (loc.Line).Length + 1));
+		}
 	}
 }
\ No newline at end of file
