@@ -434,13 +434,8 @@ namespace Mono.TextEditor
 			deltaX = deltaY = 0;
 			return false;
 		}
-		
-		/// <summary>Shows a context menu.</summary>
-		/// <param name='menu'>The menu.</param>
-		/// <param name='parent'>The parent widget.</param>
-		/// <param name='evt'>The mouse event. May be null if triggered by keyboard.</param>
-		/// <param name='caret'>The caret/selection position within the parent, if the EventButton is null.</param>
-		public static void ShowContextMenu (Gtk.Menu menu, Gtk.Widget parent, Gdk.EventButton evt, Gdk.Rectangle caret)
+
+		static void ShowContextMenuInternal (Gtk.Menu menu, Gtk.Widget parent, int ix, int iy, Gdk.Rectangle caret, Gdk.Window window, uint time, uint button)
 		{
 			Gtk.MenuPositionFunc posFunc = null;
 
@@ -450,79 +445,108 @@ namespace Mono.TextEditor
 					if (menu.AttachWidget != null)
 						menu.Detach ();
 				};
+
 				posFunc = delegate (Gtk.Menu m, out int x, out int y, out bool pushIn) {
-					Gdk.Window window = evt != null? evt.Window : parent.GdkWindow;
-					window.GetOrigin (out x, out y);
-					var alloc = parent.Allocation;
-					if (evt != null) {
-						x += (int) evt.X;
-						y += (int) evt.Y;
-					} else if (caret.X >= alloc.X && caret.Y >= alloc.Y) {
-						x += caret.X;
-						y += caret.Y + caret.Height;
-					} else {
-						x += alloc.X;
-						y += alloc.Y;
-					}
+					x = ix;
+					y = iy;
+
 					Gtk.Requisition request = m.SizeRequest ();
 					var screen = parent.Screen;
 					Gdk.Rectangle geometry = GetUsableMonitorGeometry (screen, screen.GetMonitorAtPoint (x, y));
-					
+
 					//whether to push or flip menus that would extend offscreen
 					//FIXME: this is the correct behaviour for mac, check other platforms
 					bool flip_left = true;
 					bool flip_up   = false;
-					
+
 					if (x + request.Width > geometry.X + geometry.Width) {
-						if (flip_left) {
+						if (flip_left)
 							x -= request.Width;
-						} else {
+						else
 							x = geometry.X + geometry.Width - request.Width;
-						}
-						
+
 						if (x < geometry.Left)
 							x = geometry.Left;
 					}
-					
+
 					if (y + request.Height > geometry.Y + geometry.Height) {
-						if (flip_up) {
+						if (flip_up)
 							y -= request.Height;
-						} else {
+						else
 							y = geometry.Y + geometry.Height - request.Height;
-						}
-						
+
 						if (y < geometry.Top)
 							y = geometry.Top;
 					}
-					
+
 					pushIn = false;
 				};
 			}
-			
-			uint time;
-			uint button;
-			
-			if (evt == null) {
-				time = Gtk.Global.CurrentEventTime;
-				button = 0;
-			} else {
-				time = evt.Time;
-				button = evt.Button;
-			}
-			
-			//HACK: work around GTK menu issues on mac when passing button to menu.Popup
-			//some menus appear and immediately hide, and submenus don't activate
-			if (Platform.IsMac) {
-				button = 0;
-			}
-			
+
 			menu.Popup (null, null, posFunc, button, time);
 		}
+
+		public static void ShowContextMenu (Gtk.Menu menu, Gtk.Widget parent, Gdk.EventButton evt, Gdk.Rectangle caret)
+		{
+			int x, y;
+			uint time, button;
+
+			var window = evt != null ? evt.Window : parent.GdkWindow;
+
+			if (window == null)
+				return;
+
+			window.GetOrigin (out x, out y);
+
+			if (evt == null) {
+				evt = Global.CurrentEvent as Gdk.EventButton;
+			}
+
+			if (evt != null) {
+				button = evt.Button;
+				time = evt.Time;
+				x += (int)evt.X;
+				y += (int)evt.Y;
+			} else {
+				button = 3;
+				time = 0;
+				x += caret.X;
+				y += caret.Y;
+			}
+
+			ShowContextMenuInternal (menu, parent, x, y, caret, window, time, button);
+		}
+
+		public static void ShowContextMenu (Gtk.Menu menu, Gtk.Widget parent, int ix, int iy, Gdk.Rectangle caret)
+		{
+			int x, y;
+			var window = parent.GdkWindow;
+			var alloc = parent.Allocation;
+
+			window.GetOrigin (out x, out y);
+			x += ix;
+			y += iy;
+
+			if (caret.X >= alloc.X && caret.Y >= alloc.Y) {
+				x += caret.X;
+				y += caret.Y;
+			} else {
+				x += alloc.X;
+				y += alloc.Y;
+			}
+
+			ShowContextMenuInternal (menu, parent, x, y, caret, window, Gtk.Global.CurrentEventTime, 0);
+		}
 		
 		public static void ShowContextMenu (Gtk.Menu menu, Gtk.Widget parent, Gdk.EventButton evt)
 		{
 			ShowContextMenu (menu, parent, evt, Gdk.Rectangle.Zero);
 		}
+
+		public static void ShowContextMenu (Gtk.Menu menu, Gtk.Widget parent, int x, int y)
+		{
+			ShowContextMenu (menu, parent, x, y, Gdk.Rectangle.Zero);
+		}
 		
 		public static void ShowContextMenu (Gtk.Menu menu, Gtk.Widget parent, Gdk.Rectangle caret)
 		{
@@ -1185,6 +1209,13 @@ namespace Mono.TextEditor
 			else
 				return 1d;
 		}
+
+		public static int ConvertToPixelScale (int size)
+		{
+			double scale = GetPixelScale ();
+
+			return (int)(size * scale);
+		}
 		
 		public static Gdk.Pixbuf RenderIcon (this Gtk.IconSet iconset, Gtk.Style style, Gtk.TextDirection direction, Gtk.StateType state, Gtk.IconSize size, Gtk.Widget widget, string detail, double scale)
 		{
