@@ -138,6 +138,27 @@ namespace Mono.TextEditor
 				customTabsToSpaces = value;
 			}
 		}
+
+		bool? customShowRuler;
+		public bool ShowRuler {
+			get {
+				return customShowRuler.HasValue ? customShowRuler.Value : options.ShowRuler;
+			}
+			set {
+				customShowRuler = value;
+			}
+		}
+
+		bool? customHighlightCaretLine;
+		public bool HighlightCaretLine {
+			get {
+				return customHighlightCaretLine.HasValue ? customHighlightCaretLine.Value : options.HighlightCaretLine;
+			}
+			set {
+				customHighlightCaretLine = value;
+			}
+		}
+
 		#region Tooltip providers
 		internal List<TooltipProvider> tooltipProviders = new List<TooltipProvider> ();
 		public IEnumerable<TooltipProvider> TooltipProviders {
@@ -554,7 +575,7 @@ namespace Mono.TextEditor
 			if (!HasIndentationTracker || Options.IndentStyle != IndentStyle.Virtual)
 				return;
 			var line = Document.GetLine (Caret.Line);
-			if (line != null && line.Length > 0 && GetIndentationString (Caret.Location) == Document.GetTextAt (line.Offset, line.Length))
+			if (line != null && line.Length > 0 && GetIndentationString (caret.Line - 1, int.MaxValue) == Document.GetTextAt (line.Offset, line.Length))
 				Remove (line.Offset, line.Length);
 		}
 
