@@ -36,6 +36,7 @@ using Mono.Addins;
 using MonoDevelop.Core.LogReporting;
 using MonoDevelop.Core.Logging;
 using Mono.Unix.Native;
+using System.Text;
 
 namespace MonoDevelop.Core
 {
@@ -94,6 +95,11 @@ namespace MonoDevelop.Core
 					LogError ("Error setting custom log file", e);
 				}
 			}
+			try {
+				PurgeOldLogs ();
+			} catch {
+				LogError ("Could not purge old log files");
+			}
 
 			timestamp = DateTime.Now;
 
@@ -109,11 +115,21 @@ namespace MonoDevelop.Core
 			set { PropertyService.Set (ReportCrashesKey, value); }
 		}
 
+		public static event EventHandler<PropertyChangedEventArgs> ReportCrashesChanged {
+			add { PropertyService.AddPropertyHandler (ReportCrashesKey, value); }
+			remove { PropertyService.RemovePropertyHandler (ReportCrashesKey, value); }
+		}
+
 		public static bool? ReportUsage {
 			get { return PropertyService.Get<bool?> (ReportUsageKey); }
 			set { PropertyService.Set (ReportUsageKey, value); }
 		}
 
+		public static event EventHandler<PropertyChangedEventArgs> ReportUsageChanged {
+			add { PropertyService.AddPropertyHandler (ReportUsageKey, value); }
+			remove { PropertyService.RemovePropertyHandler (ReportUsageKey, value); }
+		}
+
 		/// <summary>
 		/// Creates a session log file with the given identifier.
 		/// </summary>
@@ -162,8 +178,6 @@ namespace MonoDevelop.Core
 		
 		public static void Initialize (bool redirectOutput)
 		{
-			PurgeOldLogs ();
-
 			// Always redirect on windows otherwise we cannot get output at all
 			if (Platform.IsWindows || redirectOutput)
 				RedirectOutputToLogFile ();
@@ -479,20 +493,39 @@ namespace MonoDevelop.Core
 #endregion
 		
 #region convenience methods (string message, Exception ex)
-		
+
+		static string FormatExceptionText (string message, Exception ex)
+		{
+			if (ex == null)
+				return message;
+
+			var exceptionText = new StringBuilder ();
+			exceptionText.AppendLine (message);
+			exceptionText.Append (ex);
+			if (ex.Data.Count > 0) {
+				exceptionText.AppendLine ();
+				exceptionText.Append ("Exception Data:");
+				foreach (DictionaryEntry item in ex.Data) {
+					exceptionText.AppendLine ();
+					exceptionText.AppendFormat ("{0}: {1}", item.Key, item.Value);
+				}
+			}
+			return exceptionText.ToString ();
+		}
+
 		public static void LogDebug (string message, Exception ex)
 		{
-			Log (LogLevel.Debug, message + (ex != null? Environment.NewLine + ex : string.Empty));
+			Log (LogLevel.Debug, FormatExceptionText (message, ex));
 		}
 		
 		public static void LogInfo (string message, Exception ex)
 		{
-			Log (LogLevel.Info, message + (ex != null? Environment.NewLine + ex : string.Empty));
+			Log (LogLevel.Info, FormatExceptionText (message, ex));
 		}
 		
 		public static void LogWarning (string message, Exception ex)
 		{
-			Log (LogLevel.Warn, message + (ex != null? Environment.NewLine + ex : string.Empty));
+			Log (LogLevel.Warn, FormatExceptionText (message, ex));
 		}
 		
 		public static void LogError (string message, Exception ex)
@@ -500,11 +533,17 @@ namespace MonoDevelop.Core
 			LogUserError (message, ex);
 		}
 
+		[Obsolete ("Use LogError")]
 		public static void LogUserError (string message, Exception ex)
 		{
-			Log (LogLevel.Error, message + (ex != null? Environment.NewLine + ex : string.Empty));
+			Log (LogLevel.Error, FormatExceptionText (message, ex));
 		}
 
+		/// <summary>
+		/// Reports that an unexpected error has occurred, but the IDE will continue executing.
+		/// Error information is sent to the crash reporting service
+		/// </summary>
+		/// <param name="ex">Exception</param>
 		public static void LogInternalError (Exception ex)
 		{
 			if (ex != null) {
@@ -514,23 +553,34 @@ namespace MonoDevelop.Core
 			ReportUnhandledException (ex, false, true, "internal");
 		}
 
+		/// <summary>
+		/// Reports that an unexpected error has occurred, but the IDE will continue executing.
+		/// Error information is sent to the crash reporting service
+		/// </summary>
+		/// <param name="message">Error message</param>
+		/// <param name="ex">Exception</param>
 		public static void LogInternalError (string message, Exception ex)
 		{
-			Log (LogLevel.Error, message + (ex != null? Environment.NewLine + ex : string.Empty));
+			Log (LogLevel.Error, FormatExceptionText (message, ex));
 
 			ReportUnhandledException (ex, false, true, "internal");
 		}
 
+		[Obsolete ("Use LogInternalError")]
 		public static void LogCriticalError (string message, Exception ex)
 		{
-			Log (LogLevel.Error, message + (ex != null? Environment.NewLine + ex : string.Empty));
-
-			ReportUnhandledException (ex, false, false, "critical");
+			LogInternalError (message, ex);
 		}
 
+		/// <summary>
+		/// Reports that a fatal error has occurred, and that the IDE will shut down.
+		/// Error information is sent to the crash reporting service
+		/// </summary>
+		/// <param name="message">Error message</param>
+		/// <param name="ex">Exception</param>
 		public static void LogFatalError (string message, Exception ex)
 		{
-			Log (LogLevel.Error, message + (ex != null? Environment.NewLine + ex : string.Empty));
+			Log (LogLevel.Fatal, FormatExceptionText (message, ex));
 
 			ReportUnhandledException (ex, true, false, "fatal");
 		}
