@@ -183,7 +183,7 @@ namespace MonoDevelop.Core
 			//try versioned profiles from most recent to oldest
 			//skip the last in the array, it's the current profile
 			int userProfileMostRecent = UserProfile.ProfileVersions.Length - 2;
-			for (int i = userProfileMostRecent; i >= 0; i--) {
+			for (int i = userProfileMostRecent; i >= 1; i--) {
 				string v = UserProfile.ProfileVersions[i];
 				var p = UserProfile.GetProfile (v);
 				if (File.Exists (p.ConfigDir.Combine (FileName))) {
@@ -194,12 +194,15 @@ namespace MonoDevelop.Core
 			}
 			
 			//try the old unversioned MD <= 2.4 profile
-			var md24 = UserProfile.ForMD24 ();
-			if (File.Exists (md24.ConfigDir.Combine (FileName))) {
-				profile = md24;
-				version = "2.4";
-				return true;
+			if (BrandingService.ProfileDirectoryName == "MonoDevelop") {
+				var md24 = UserProfile.ForMD24 ();
+				if (File.Exists (md24.ConfigDir.Combine (FileName))) {
+					profile = md24;
+					version = "2.4";
+					return true;
+				}
 			}
+
 			return false;
 		}
 		
@@ -230,7 +233,7 @@ namespace MonoDevelop.Core
 		{
 			Debug.Assert (properties != null);
 			var prefsPath = UserProfile.Current.ConfigDir.Combine (FileName);
-			FileService.EnsureDirectoryExists (prefsPath.ParentDirectory);
+			Directory.CreateDirectory (prefsPath.ParentDirectory);
 			properties.Save (prefsPath);
 		}
 		
