@@ -42,6 +42,7 @@ using System.Security.Cryptography.X509Certificates;
 using System.Net.Security;
 using System.Net;
 using MonoDevelop.Core.Web;
+using System.Collections.Generic;
 
 
 namespace MonoDevelop.Core
@@ -205,7 +206,7 @@ namespace MonoDevelop.Core
 		
 		static void OnLoad (object s, AddinEventArgs args)
 		{
-			Counters.AddinsLoaded.Inc ("Add-in loaded: " + args.AddinId);
+			Counters.AddinsLoaded.Inc ("Add-in loaded: " + args.AddinId, new Dictionary<string,string> { { "AddinId", args.AddinId } });
 		}
 		
 		static void OnUnload (object s, AddinEventArgs args)
