@@ -28,16 +28,12 @@
 using System;
 using System.IO;
 using System.Diagnostics;
-using System.Globalization;
 using System.Collections.Generic;
 using System.CodeDom.Compiler;
 using System.Text.RegularExpressions;
-using System.Xml;
 using MonoDevelop.Core;
-using MonoDevelop.Core.Assemblies;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Projects.Extensions;
-using MonoDevelop.Projects.Formats.MSBuild;
 using Microsoft.Build.BuildEngine;
 	
 namespace MonoDevelop.Projects.Formats.MD1
@@ -51,57 +47,22 @@ namespace MonoDevelop.Projects.Formats.MD1
 		DotNetProject Project {
 			get { return (DotNetProject) Item; }
 		}
-		
-		protected override BuildResult OnClean (IProgressMonitor monitor, ConfigurationSelector configuration)
-		{
-			DotNetProject project = Project;
-			DotNetProjectConfiguration conf = (DotNetProjectConfiguration) project.GetConfiguration (configuration);
-			
-			// Delete the generated debug info
-			string file = project.GetOutputFileName (configuration);
-			if (file != null) {
-				string mdb = conf.TargetRuntime.GetAssemblyDebugInfoFile (file);
-				if (File.Exists (mdb))
-					FileService.DeleteFile (mdb);
-			}
 
-			List<string> cultures = new List<string> ();
-			monitor.Log.WriteLine (GettextCatalog.GetString ("Removing all .resources files"));
-			foreach (ProjectFile pfile in project.Files) {
-				if (pfile.BuildAction == BuildAction.EmbeddedResource &&
-					Path.GetExtension (pfile.Name) == ".resx") {
-					string resFilename = Path.ChangeExtension (pfile.Name, ".resources");
-					if (File.Exists (resFilename))
-						FileService.DeleteFile (resFilename);
-				}
-				string culture = GetCulture (pfile.Name);
-				if (culture != null)
-					cultures.Add (culture);
-			}
-
-			if (cultures.Count > 0 && conf != null && project.DefaultNamespace != null) {
-				monitor.Log.WriteLine (GettextCatalog.GetString ("Removing all satellite assemblies"));
-				string outputDir = ((DotNetProjectConfiguration)conf).OutputDirectory;
-				string satelliteAsmName = Path.GetFileNameWithoutExtension (((DotNetProjectConfiguration)conf).OutputAssembly) + ".resources.dll";
-
-				foreach (string culture in cultures) {
-					string path = String.Format ("{0}{3}{1}{3}{2}", outputDir, culture, satelliteAsmName, Path.DirectorySeparatorChar);
-					if (File.Exists (path)) {
-						FileService.DeleteFile (path);
-						string dir = Path.GetDirectoryName (path);
-						if (Directory.GetFiles (dir).Length == 0)
-							FileService.DeleteDirectory (dir);
-					}
-				}
-			}
-			return null;
-		}
-
-		
 		protected override BuildResult OnBuild (IProgressMonitor monitor, ConfigurationSelector configuration)
 		{
+			if (!Project.InternalCheckNeedsBuild (configuration)) {
+				monitor.Log.WriteLine (GettextCatalog.GetString ("Skipping project since output files are up to date"));
+				return new BuildResult ();
+			}
+
 			DotNetProject project = Project;
 			
+			if (!project.TargetRuntime.IsInstalled (project.TargetFramework)) {
+				BuildResult res = new BuildResult ();
+				res.AddError (GettextCatalog.GetString ("Framework '{0}' not installed.", project.TargetFramework.Name));
+				return res;
+			}
+			
 			bool hasBuildableFiles = false;
 			foreach (ProjectFile pf in project.Files) {
 				if (pf.BuildAction == BuildAction.Compile || pf.BuildAction == BuildAction.EmbeddedResource) {
@@ -145,7 +106,7 @@ namespace MonoDevelop.Projects.Formats.MD1
 						refres = new BuildResult ();
 					string msg;
 					if (!pr.IsExactVersion && pr.SpecificVersion) {
-						msg = GettextCatalog.GetString ("Reference '{0}' not found on system. Using '{1} instead.", pr.StoredReference, pr.Reference);
+						msg = GettextCatalog.GetString ("Reference '{0}' not found on system. Using '{1}' instead.", pr.StoredReference, pr.Reference);
 						monitor.ReportWarning (msg);
 						refres.AddWarning (msg);
 					}
@@ -220,14 +181,14 @@ namespace MonoDevelop.Projects.Formats.MD1
 
 				string fname = finfo.Name;
 				string resourceId;
-				CompilerError ce = GetResourceId (env, finfo, ref fname, resgen, out resourceId, monitor);
+				CompilerError ce = GetResourceId (configuration.IntermediateOutputDirectory.Combine (finfo.ResourceId), env, finfo, ref fname, resgen, out resourceId, monitor);
 				if (ce != null) {
 					CompilerResults cr = new CompilerResults (new TempFileCollection ());
 					cr.Errors.Add (ce);
 
 					return new BuildResult (cr, String.Empty);
 				}
-				string culture = GetCulture (finfo.Name);
+				string culture = DotNetProject.GetResourceCulture (finfo.Name);
 				if (culture == null)
 					continue;
 
@@ -247,7 +208,7 @@ namespace MonoDevelop.Projects.Formats.MD1
 				projectItems.Remove (finfo);
 			}
 
-			string al = configuration.TargetFramework.ClrVersion == ClrVersion.Net_2_0 ? "al2" : "al";
+			string al = configuration.TargetRuntime.GetToolPath (configuration.TargetFramework, "al");
 			CompilerError err = GenerateSatelliteAssemblies (resourcesByCulture, configuration.OutputDirectory, al, Path.GetFileName (configuration.OutputAssembly), monitor);
 			if (err != null) {
 				CompilerResults cr = new CompilerResults (new TempFileCollection ());
@@ -259,7 +220,7 @@ namespace MonoDevelop.Projects.Formats.MD1
 			return null;
 		}
 		
-		CompilerError GetResourceId (ExecutionEnvironment env, ProjectFile finfo, ref string fname, string resgen, out string resourceId, IProgressMonitor monitor)
+		CompilerError GetResourceId (FilePath outputFile, ExecutionEnvironment env, ProjectFile finfo, ref string fname, string resgen, out string resourceId, IProgressMonitor monitor)
 		{
 			resourceId = finfo.ResourceId;
 			if (resourceId == null) {
@@ -273,8 +234,8 @@ namespace MonoDevelop.Projects.Formats.MD1
 			if (String.Compare (Path.GetExtension (fname), ".resx", true) != 0)
 				return null;
 
-			if (!IsResgenRequired (fname)) {
-				fname = Path.ChangeExtension (fname, ".resources");
+			if (!IsResgenRequired (fname, outputFile)) {
+				fname = File.Exists (outputFile) ? (string)outputFile : Path.ChangeExtension (fname, ".resources");
 				return null;
 			}
 			
@@ -347,60 +308,14 @@ namespace MonoDevelop.Projects.Formats.MD1
 
 		// true if the resx file or any file referenced
 		// by the resx is newer than the .resources file
-		public static bool IsResgenRequired (string resx_filename)
+		public static bool IsResgenRequired (string resx_filename, string output_filename)
 		{
 			if (String.Compare (Path.GetExtension (resx_filename), ".resx", true) != 0)
 				throw new ArgumentException (".resx file expected", "resx_filename");
 
-			string resources_filename = Path.ChangeExtension (resx_filename, ".resources");
-
-			if (IsFileNewerThan (resx_filename, resources_filename))
-				return true;
-
-			XmlTextReader xr = null;
-			try {
-				// look for
-				// <data type="System.Resources.ResXFileRef, System.Windows.Forms" ..>
-				//   <value>... filename;.. </value>
-				// </data>
-				xr = new XmlTextReader (resx_filename);
-				string basepath = Path.GetDirectoryName (resx_filename);
-				while (xr.Read ()) {
-					if (xr.NodeType != XmlNodeType.Element ||
-						String.Compare (xr.LocalName, "data") != 0)
-						continue;
-
-					string type = xr.GetAttribute ("type");
-					if (String.IsNullOrEmpty (type))
-						continue;
-
-					if (String.Compare (type, "System.Resources.ResXFileRef, System.Windows.Forms") != 0)
-						continue;
-
-					xr.ReadToDescendant ("value");
-					if (xr.NodeType != XmlNodeType.Element)
-						continue;
-
-					string value = xr.ReadElementContentAsString ();
-
-					string [] parts = value.Split (';');
-					if (parts.Length > 0) {
-						string referenced_filename = MSBuildProjectService.FromMSBuildPath (
-								String.Empty, Path.Combine (basepath, parts [0]).Trim ());
-						if (File.Exists (referenced_filename) &&
-							IsFileNewerThan (referenced_filename, resources_filename))
-							return true;
-					}
-				}
-			} catch (XmlException) {
-				// Ignore xml errors, let resgen handle it
-				return true;
-			} finally {
-				if (xr != null)
-					xr.Close ();
-			}
-
-			return false;
+			if (File.Exists (output_filename))
+				return IsFileNewerThan (resx_filename, output_filename);
+			return IsFileNewerThan (resx_filename, Path.ChangeExtension (resx_filename, ".resources"));
 		}
 
 		// true if first is newer than second
@@ -467,43 +382,6 @@ namespace MonoDevelop.Projects.Formats.MD1
 			return null;
 		}
 		
-		//Given a filename like foo.it.resx, get 'it', if its
-		//a valid culture
-		//Note: hand-written as this can get called lotsa times
-		//Note: code duplicated in prj2make/Utils.cs as TrySplitResourceName
-		string GetCulture (string fname)
-		{
-			int last_dot = -1;
-			int culture_dot = -1;
-			int i = fname.Length - 1;
-			while (i >= 0) {
-				if (fname [i] == '.') {
-					last_dot = i;
-					break;
-				}
-				i --;
-			}
-			if (i < 0)
-				return null;
-
-			i--;
-			while (i >= 0) {
-				if (fname [i] == '.') {
-					culture_dot = i;
-					break;
-				}
-				i --;
-			}
-			if (culture_dot < 0)
-				return null;
-
-			string culture = fname.Substring (culture_dot + 1, last_dot - culture_dot - 1);
-			if (!CultureNamesTable.ContainsKey (culture))
-				return null;
-
-			return culture;
-		}
-
 		// Used for parsing "Line 123, position 5" errors from tools
 		// like resgen, xamlg
 		static Regex regexErrorLinePos;
@@ -514,19 +392,6 @@ namespace MonoDevelop.Projects.Formats.MD1
 				return regexErrorLinePos;
 			}
 		}
-
-		static Dictionary<string, string> cultureNamesTable;
-		static Dictionary<string, string> CultureNamesTable {
-			get {
-				if (cultureNamesTable == null) {
-					cultureNamesTable = new Dictionary<string, string> ();
-					foreach (CultureInfo ci in CultureInfo.GetCultures (CultureTypes.AllCultures))
-						cultureNamesTable [ci.Name] = ci.Name;
-				}
-
-				return cultureNamesTable;
-			}
-		}
 	}
 
 	class ProjectParserContext: IExpressionContext
