@@ -498,6 +498,8 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 			extn = fname.Substring (last_dot + 1);
 			return true;
 		}
+
+		static bool runLocal = false;
 		
 		internal static RemoteProjectBuilder GetProjectBuilder (TargetRuntime runtime, string minToolsVersion, string file, string solutionFile)
 		{
@@ -534,6 +536,7 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				//always start the remote process explicitly, even if it's using the current runtime and fx
 				//else it won't pick up the assembly redirects from the builder exe
 				var exe = GetExeLocation (runtime, toolsVersion);
+
 				MonoDevelop.Core.Execution.RemotingService.RegisterRemotingChannel ();
 				var pinfo = new ProcessStartInfo (exe) {
 					UseShellExecute = false,
@@ -544,36 +547,56 @@ namespace MonoDevelop.Projects.Formats.MSBuild
 				runtime.GetToolsExecutionEnvironment ().MergeTo (pinfo);
 				
 				Process p = null;
+
 				try {
-					p = runtime.ExecuteAssembly (pinfo);
-					p.StandardInput.WriteLine (Process.GetCurrentProcess ().Id.ToString ());
-					string responseKey = "[MonoDevelop]";
-					string sref;
-					while (true) {
-						sref = p.StandardError.ReadLine ();
-						if (sref.StartsWith (responseKey, StringComparison.Ordinal)) {
-							sref = sref.Substring (responseKey.Length);
-							break;
-						}
+					IBuildEngine engine;
+					if (!runLocal) {
+						p = runtime.ExecuteAssembly (pinfo);
+
+						// The builder app will write the build engine reference
+						// after reading the process id from the standard input
+						ManualResetEvent ev = new ManualResetEvent (false);
+						string responseKey = "[MonoDevelop]";
+						string sref = null;
+						p.ErrorDataReceived += (sender, e) => {
+							if (e.Data == null)
+								return;
+
+							if (e.Data.StartsWith (responseKey, StringComparison.Ordinal)) {
+								sref = e.Data.Substring (responseKey.Length);
+								ev.Set ();
+							} else
+								Console.WriteLine (e.Data);
+						};
+						p.BeginErrorReadLine ();
+						p.StandardInput.WriteLine (Process.GetCurrentProcess ().Id.ToString ());
+						if (!ev.WaitOne (TimeSpan.FromSeconds (5)))
+							throw new Exception ("MSBuild process could not be started");
+						
+						byte[] data = Convert.FromBase64String (sref);
+						MemoryStream ms = new MemoryStream (data);
+						BinaryFormatter bf = new BinaryFormatter ();
+						engine = (IBuildEngine)bf.Deserialize (ms);
+					} else {
+						var asm = System.Reflection.Assembly.LoadFrom (exe);
+						var t = asm.GetType ("MonoDevelop.Projects.Formats.MSBuild.BuildEngine");
+						engine = (IBuildEngine)Activator.CreateInstance (t);
 					}
-					byte[] data = Convert.FromBase64String (sref);
-					MemoryStream ms = new MemoryStream (data);
-					BinaryFormatter bf = new BinaryFormatter ();
-					var engine = (IBuildEngine)bf.Deserialize (ms);
 					engine.SetCulture (GettextCatalog.UICulture);
 					engine.SetGlobalProperties (GetCoreGlobalProperties (solutionFile));
 					foreach (var gpp in globalPropertyProviders)
-						builder.SetGlobalProperties (gpp.GetGlobalProperties ());
+						engine.SetGlobalProperties (gpp.GetGlobalProperties ());
 					builder = new RemoteBuildEngine (p, engine);
 				} catch {
 					if (p != null) {
 						try {
 							p.Kill ();
-						} catch { }
+						} catch {
+						}
 					}
 					throw;
 				}
-				
+
 				builders [builderKey] = builder;
 				builder.ReferenceCount = 1;
 				builder.Disconnected += delegate {
