@@ -46,15 +46,36 @@ namespace MonoDevelop.Projects
 			base (languageName, projectCreateInfo, projectOptions)
 		{
 		}
+
+		public override System.Collections.Generic.IEnumerable<string> GetProjectTypes ()
+		{
+			yield return "DotNetAssembly";
+			foreach (var pt in base.GetProjectTypes ())
+				yield return pt;
+		}
 		
 		public override bool SupportsFramework (TargetFramework framework)
 		{
 			// DotNetAssemblyProject can only generate assemblies for the regular framework.
 			// Special frameworks such as Moonlight or MonoTouch must subclass DotNetProject directly.
-			if (!framework.IsCompatibleWithFramework (TargetFrameworkMoniker.NET_1_1))
+			if (!framework.CanReferenceAssembliesTargetingFramework (TargetFrameworkMoniker.NET_1_1))
 				return false;
-			else
-				return base.SupportsFramework (framework);
+
+			return base.SupportsFramework (framework);
+		}
+		
+		public override TargetFrameworkMoniker GetDefaultTargetFrameworkForFormat (FileFormat format)
+		{
+			switch (format.Id) {
+			case "MSBuild05":
+				return TargetFrameworkMoniker.NET_2_0;
+			case "MSBuild08":
+				return TargetFrameworkMoniker.NET_2_0;
+			case "MSBuild10":
+			case "MSBuild12":
+				return TargetFrameworkMoniker.NET_4_0;
+			}
+			return Services.ProjectService.DefaultTargetFramework.Id;
 		}
 		
 		protected override string GetDefaultTargetPlatform (ProjectCreateInformation projectCreateInfo)
