@@ -118,6 +118,7 @@ namespace MonoDevelop.Projects
 				string platformSuffix = string.IsNullOrEmpty (platform) ? string.Empty : "|" + platform;
 				DotNetProjectConfiguration configDebug = CreateConfiguration ("Debug" + platformSuffix) as DotNetProjectConfiguration;
 				configDebug.CompilationParameters = languageBinding.CreateCompilationParameters (projectOptions);
+				DefineSymbols (configDebug.CompilationParameters, projectOptions, "DefineConstantsDebug");
 				configDebug.DebugMode = true;
 				configDebug.ExternalConsole = externalConsole;
 				configDebug.PauseConsoleOutput = externalConsole;
@@ -129,6 +130,7 @@ namespace MonoDevelop.Projects
 					XmlElement releaseProjectOptions = (XmlElement)projectOptions.CloneNode (true);
 					releaseProjectOptions.SetAttribute ("Release", "True");
 					configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (releaseProjectOptions);
+					DefineSymbols (configRelease.CompilationParameters, projectOptions, "DefineConstantsRelease");
 				} else {
 					configRelease.CompilationParameters = languageBinding.CreateCompilationParameters (null);
 				}
@@ -162,6 +164,16 @@ namespace MonoDevelop.Projects
 			}
 		}
 
+		void DefineSymbols (ConfigurationParameters pars, XmlElement projectOptions, string attributeName)
+		{
+			if (projectOptions != null) {
+				string symbols = projectOptions.GetAttribute (attributeName);
+				if (!String.IsNullOrEmpty (symbols)) {
+					pars.AddDefineSymbol (symbols);
+				}
+			}
+		}
+
 		TargetFramework GetTargetFrameworkForNewProject (XmlElement projectOptions, TargetFrameworkMoniker defaultMoniker)
 		{
 			if (projectOptions == null)
@@ -459,14 +471,28 @@ namespace MonoDevelop.Projects
 			for (int n=0; n<References.Count; n++) {
 				ProjectReference pr = References [n];
 				if (pr.ReferenceType == ReferenceType.Assembly && DefaultConfiguration != null) {
-					if (pr.GetReferencedFileNames (DefaultConfiguration.Selector).Any (f => f == updatedFile))
+					if (pr.GetReferencedFileNames (DefaultConfiguration.Selector).Any (f => f == updatedFile)) {
+						SetFastBuildCheckDirty ();
 						pr.NotifyStatusChanged ();
+					}
 				} else if (pr.HintPath == updatedFile) {
+					SetFastBuildCheckDirty ();
 					var nr = pr.GetRefreshedReference ();
 					if (nr != null)
 						References [n] = nr;
 				}
 			}
+
+			/* Removed because it is very slow. SetFastBuildCheckDirty() is being called above. It is not the perfect solution but it is good enough
+			 * In the new project model GetReferencedAssemblies is asynchronous, so the code can be uncommented there. 
+ 
+			// If a referenced assembly changes, dirtify the project.
+			foreach (var asm in GetReferencedAssemblies (DefaultConfiguration.Selector))
+				if (asm == updatedFile) {
+					SetFastBuildCheckDirty ();
+					break;
+				}
+				*/
 		}
 
 		internal override void OnFileChanged (object source, MonoDevelop.Core.FileEventArgs e)
@@ -931,6 +957,22 @@ namespace MonoDevelop.Projects
 			return (compileTarget == CompileTarget.Exe || compileTarget == CompileTarget.WinExe) && context.ExecutionHandler.CanExecute (cmd);
 		}
 
+		internal protected override bool OnGetSupportsExecute ()
+		{
+			if (compileTarget == CompileTarget.Exe || compileTarget == CompileTarget.WinExe)
+				return true;
+
+			if (Configurations.OfType<DotNetProjectConfiguration> ().Any (c => c.CustomCommands.Any (cc => cc.Type == CustomCommandType.Execute)))
+				return true;
+
+			// Hack to keep backwards compatibility with the old behavior. For example, a MonoDroid project is a library, but OnGetSupportsExecute
+			// is not overriden, so it depends on DotNetProject to return true by default.
+			if (compileTarget == CompileTarget.Library && GetType () == typeof (DotNetAssemblyProject))
+				return false;
+
+			return true;
+		}
+
 		protected internal override List<FilePath> OnGetItemFiles (bool includeReferencedFiles)
 		{
 			List<FilePath> col = base.OnGetItemFiles (includeReferencedFiles);
@@ -1041,9 +1083,12 @@ namespace MonoDevelop.Projects
 					sb.Append (c);
 				}
 			}
-			if (sb.Length > 0)
+			if (sb.Length > 0) {
+				if (sb[sb.Length - 1] == '.')
+					sb.Remove (sb.Length - 1, 1);
+
 				return sb.ToString ();
-			else
+			} else
 				return null;
 		}
 
@@ -1269,5 +1314,21 @@ namespace MonoDevelop.Projects
 			importsAdded.Clear ();
 			importsRemoved.Clear ();
 		}
+
+		public void RefreshProjectBuilder ()
+		{
+			var handler = ItemHandler as MSBuildProjectHandler;
+			if (handler != null) {
+				handler.RefreshProjectBuilder ();
+			}
+		}
+
+		public void DisposeProjectBuilder ()
+		{
+			var handler = ItemHandler as MSBuildProjectHandler;
+			if (handler != null) {
+				handler.CleanupProjectBuilder ();
+			}
+		}
 	}
 }
