@@ -37,6 +37,9 @@ using MonoDevelop;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Serialization;
 using MonoDevelop.Projects;
+using MonoDevelop.Core.Instrumentation;
+using MonoDevelop.Projects.Formats.MSBuild;
+using System.Runtime.Remoting.Messaging;
 
 
 namespace MonoDevelop.Projects
@@ -52,6 +55,8 @@ namespace MonoDevelop.Projects
 	[ProjectModelDataItem(FallbackType = typeof(UnknownProject))]
 	public abstract class Project : SolutionEntityItem
 	{
+		static Counter ProjectOpenedCounter = InstrumentationService.CreateCounter ("Project Opened", "Project Model", id:"Ide.Project.Open");
+
 		string[] buildActions;
 
 		protected Project ()
@@ -61,7 +66,117 @@ namespace MonoDevelop.Projects
 			Items.Bind (files);
 			DependencyResolutionEnabled = true;
 		}
-		
+
+		public TargetEvaluationResult RunTarget (IProgressMonitor monitor, string target, ConfigurationSelector configuration, TargetEvaluationContext context)
+		{
+			var currentContext = CallContext.GetData ("MonoDevelop.Projects.ProjectOperationContext");
+			var currentResult = CallContext.GetData ("MonoDevelop.Projects.TargetEvaluationResult");
+			try {
+				CallContext.SetData ("MonoDevelop.Projects.ProjectOperationContext", context);
+				CallContext.SetData ("MonoDevelop.Projects.TargetEvaluationResult", null);
+
+				var r = RunTarget (monitor, target, configuration);
+
+				var evalRes = CallContext.GetData ("MonoDevelop.Projects.TargetEvaluationResult") as TargetEvaluationResult;
+				if (evalRes != null) {
+					evalRes.BuildResult = r;
+					return evalRes;
+				}
+				return new TargetEvaluationResult (r);
+			} finally {
+				CallContext.SetData ("MonoDevelop.Projects.ProjectOperationContext", currentContext);
+				CallContext.SetData ("MonoDevelop.Projects.TargetEvaluationResult", currentResult);
+			}
+		}
+
+		internal protected override BuildResult OnRunTarget (IProgressMonitor monitor, string target, ConfigurationSelector configuration)
+		{
+			var currentContext = CallContext.GetData ("MonoDevelop.Projects.ProjectOperationContext") as ProjectOperationContext;
+			ProjectOperationContext newContext = currentContext;
+
+			// Collect last write times for the files generated by this project
+			var fileTimes = new Dictionary<FilePath, DateTime> ();
+			foreach (var f in GetOutputFiles (configuration))
+				fileTimes [f] = File.GetLastWriteTime (f);
+
+			try {
+				if (newContext == null)
+					newContext = new TargetEvaluationContext ();
+				else if (!(newContext is TargetEvaluationContext))
+					newContext = new TargetEvaluationContext (newContext);
+				
+				var res = OnRunTarget (monitor, target, configuration, (TargetEvaluationContext) newContext);
+				CallContext.SetData ("MonoDevelop.Projects.TargetEvaluationResult", res);
+				return res.BuildResult;
+			} finally {
+				if (newContext != currentContext)
+					CallContext.SetData ("MonoDevelop.Projects.ProjectOperationContext", currentContext);
+
+				// If any of the project generated files changes, notify it
+				foreach (var e in fileTimes) {
+					if (File.GetLastWriteTime (e.Key) != e.Value)
+						FileService.NotifyFileChanged (e.Key);
+				}
+			}
+		}
+
+		internal protected virtual TargetEvaluationResult OnRunTarget (IProgressMonitor monitor, string target, ConfigurationSelector configuration, TargetEvaluationContext context)
+		{
+			var currentContext = CallContext.GetData ("MonoDevelop.Projects.ProjectOperationContext") as ProjectOperationContext;
+			CallContext.SetData ("MonoDevelop.Projects.ProjectOperationContext", context);
+
+			try {
+				var r = base.OnRunTarget (monitor, target, configuration);
+				var evalRes = CallContext.GetData ("MonoDevelop.Projects.TargetEvaluationResult") as TargetEvaluationResult;
+				if (evalRes != null)
+					evalRes.BuildResult = r;
+				else
+					evalRes = new TargetEvaluationResult (r);
+				return evalRes;
+			} finally {
+				CallContext.SetData ("MonoDevelop.Projects.ProjectOperationContext", currentContext);
+			}
+		}
+
+		string[] supportedMSBuildTargets;
+
+		internal protected override bool OnGetSupportsTarget (string target)
+		{
+			if (target == ProjectService.BuildTarget || target == ProjectService.CleanTarget)
+				return true;
+			if (supportedMSBuildTargets == null) {
+				var h = ItemHandler as MSBuildProjectHandler;
+				if (h != null)
+					supportedMSBuildTargets = h.GetSupportedTargets ();
+				else
+					supportedMSBuildTargets = new string[0];
+			}
+			return supportedMSBuildTargets.Contains (target);
+		}
+
+		protected override void OnGetProjectEventMetadata (IDictionary<string, string> metadata)
+		{
+			base.OnGetProjectEventMetadata (metadata);
+			var sb = new System.Text.StringBuilder ();
+			var first = true;
+
+			var projectTypes = this.GetProjectTypes ().ToList ();
+			foreach (var p in projectTypes.Where (x => (x != "DotNet") || projectTypes.Count == 1)) {
+				if (!first)
+					sb.Append (", ");
+				sb.Append (p);
+				first = false;
+			}
+			metadata ["ProjectTypes"] = sb.ToString ();
+		}
+
+		protected override void OnEndLoad ()
+		{
+			base.OnEndLoad ();
+
+			ProjectOpenedCounter.Inc (1, null, GetProjectEventMetadata (null));
+		}
+
 		/// <summary>
 		/// Description of the project.
 		/// </summary>
@@ -767,6 +882,7 @@ namespace MonoDevelop.Projects
 			foreach (FileEventInfo fi in e) {
 				ProjectFile file = GetProjectFile (fi.FileName);
 				if (file != null) {
+					SetFastBuildCheckDirty ();
 					try {
 						NotifyFileChangedInProject (file);
 					} catch {
@@ -834,7 +950,7 @@ namespace MonoDevelop.Projects
 						if (!string.IsNullOrEmpty (f.DependsOn))
 							unresolvedDeps.Add (f);
 					}
-					file.DependsOnFile = null;
+					file.DependsOn = null;
 				}
 			}
 			NotifyModified ("Files");
