@@ -30,11 +30,13 @@
 
 using System;
 using System.Xml;
+using System.Linq;
 using System.Collections;
 using System.Collections.Generic;
 using Gtk;
 using Gdk;
 using Xwt.Motion;
+using MonoDevelop.Core;
 
 namespace MonoDevelop.Components.Docking
 {
@@ -96,6 +98,10 @@ namespace MonoDevelop.Components.Docking
 				return !OverlayWidgetVisible;
 			}
 		}
+
+		internal bool UseWindowsForTopLevelFrames {
+			get { return Platform.IsMac; }
+		}
 		
 		/// <summary>
 		/// Compactness level of the gui, from 1 (not compact) to 5 (very compact).
@@ -849,14 +855,46 @@ namespace MonoDevelop.Components.Docking
 			return null;
 		}
 		
-		internal void AddTopLevel (DockFrameTopLevel w, int x, int y)
+		internal void AddTopLevel (DockFrameTopLevel w, int x, int y, int width, int height)
 		{
-			w.Parent = this;
 			w.X = x;
 			w.Y = y;
-			Requisition r = w.SizeRequest ();
-			w.Allocation = new Gdk.Rectangle (Allocation.X + x, Allocation.Y + y, r.Width, r.Height);
-			topLevels.Add (w);
+
+			if (UseWindowsForTopLevelFrames) {
+				var win = new Gtk.Window (Gtk.WindowType.Toplevel);
+				win.SkipTaskbarHint = true;
+				win.Decorated = false;
+				win.TypeHint = Gdk.WindowTypeHint.Toolbar;
+				w.ContainerWindow = win;
+				w.Size = new Size (width, height);
+				win.Add (w);
+				w.Show ();
+				var p = this.GetScreenCoordinates (new Gdk.Point (x, y));
+				win.Opacity = 0.0;
+				win.Move (p.X, p.Y);
+				win.Resize (width, height);
+				win.Show ();
+				Ide.DesktopService.AddChildWindow ((Gtk.Window)Toplevel, win);
+				win.AcceptFocus = true;
+				win.Opacity = 1.0;
+
+				/* When we use real windows for frames, it's possible for pads to be over other
+				 * windows. For some reason simply presenting or raising those dialogs doesn't
+				 * seem to work, so we hide/show them in order to force them above the pad. */
+				var toplevels = Gtk.Window.ListToplevels ().Where (t => t.IsRealized && t.Visible && t.TypeHint == WindowTypeHint.Dialog); // && t.TransientFor != null);
+				foreach (var t in toplevels) {
+					t.Hide ();
+					t.Show ();
+				}
+
+				MonoDevelop.Ide.IdeApp.CommandService.RegisterTopWindow (win);
+			} else {
+				w.Parent = this;
+				w.Size = new Size (width, height);
+				Requisition r = w.SizeRequest ();
+				w.Allocation = new Gdk.Rectangle (Allocation.X + x, Allocation.Y + y, r.Width, r.Height);
+				topLevels.Add (w);
+			}
 		}
 		
 		internal void RemoveTopLevel (DockFrameTopLevel w)
@@ -910,27 +948,29 @@ namespace MonoDevelop.Components.Docking
 			Gdk.Size sBot = GetBarFrameSize (dockBarBottom);
 			Gdk.Size sLeft = GetBarFrameSize (dockBarLeft);
 			Gdk.Size sRgt = GetBarFrameSize (dockBarRight);
-			
-			int x,y;
+
+			int x,y,w,h;
 			if (bar == dockBarLeft || bar == dockBarRight) {
-				aframe.HeightRequest = Allocation.Height - sTop.Height - sBot.Height;
-				aframe.WidthRequest = size;
+				h = Allocation.Height - sTop.Height - sBot.Height;
+				w = size;
 				y = sTop.Height;
 				if (bar == dockBarLeft)
 					x = sLeft.Width;
 				else
 					x = Allocation.Width - size - sRgt.Width;
 			} else {
-				aframe.WidthRequest = Allocation.Width - sLeft.Width - sRgt.Width;
-				aframe.HeightRequest = size;
+				w = Allocation.Width - sLeft.Width - sRgt.Width;
+				h = size;
 				x = sLeft.Width;
 				if (bar == dockBarTop)
 					y = sTop.Height;
 				else
 					y = Allocation.Height - size - sBot.Height;
 			}
-			AddTopLevel (aframe, x, y);
+
+			AddTopLevel (aframe, x, y, w, h);
 			aframe.AnimateShow ();
+
 			return aframe;
 		}
 		
@@ -992,7 +1032,11 @@ namespace MonoDevelop.Components.Docking
 					}
 					parent.Remove (item.TitleTab);
 				}
-				RemoveTopLevel (widget);
+				if (widget.ContainerWindow != null) {
+					widget.ContainerWindow.Destroy ();
+				} else
+					RemoveTopLevel (widget);
+
 				widget.Disposed = true;
 				widget.Destroy ();
 			}
