@@ -60,6 +60,7 @@ namespace MonoDevelop.Components.Extensions
 			get { return FilterSet.DefaultFilter; }
 			set { FilterSet.DefaultFilter = value; }
 		}
+		public bool ShowHidden { get; set; }
 	}	
 			
 	/// <summary>
@@ -163,6 +164,16 @@ namespace MonoDevelop.Components.Extensions
 			get { return data.DefaultFilter; }
 			set { data.DefaultFilter = value; }
 		}
+
+		/// <summary>
+		/// Gets or sets whether the file dialog will show hidden files and folders.
+		/// show hidden.
+		/// </summary>
+		/// <value><c>true</c> if hidden files are shown; otherwise, <c>false</c>.</value>
+		public bool ShowHidden {
+			get { return data.ShowHidden; }
+			set { data.ShowHidden = value; }
+		}
 		
 		#region File filter utilities
 		
@@ -313,6 +324,7 @@ namespace MonoDevelop.Components.Extensions
 				return result == (int) Gtk.ResponseType.Ok;
 			} finally {
 				fdiag.Destroy ();
+				fdiag.Dispose ();
 			}
 		}
 	}
