@@ -27,12 +27,9 @@
 //
 
 using System;
-using System.Reflection;
-using System.Collections;
 using System.ComponentModel;
-using MonoDevelop.Components.PropertyGrid.PropertyEditors;
-using Gtk;
 using Gdk;
+using Gtk;
 using Mono.TextEditor;
 
 namespace MonoDevelop.Components.PropertyGrid
@@ -40,20 +37,23 @@ namespace MonoDevelop.Components.PropertyGrid
 	public class PropertyEditorCell
 	{
 		Pango.Layout layout;
-		PropertyDescriptor property; 
-		object obj;
-		Gtk.Widget container;
+		ITypeDescriptorContext context;
+		Widget container;
 		EditorManager editorManager;
 		
 		public object Instance {
-			get { return obj; }
+			get { return context.Instance; }
 		}
 		
 		public PropertyDescriptor Property {
-			get { return property; }
+			get { return context.PropertyDescriptor; }
+		}
+
+		protected ITypeDescriptorContext Context {
+			get { return context; }
 		}
 		
-		public Gtk.Widget Container {
+		public Widget Container {
 			get { return container; }
 		}
 		
@@ -61,7 +61,7 @@ namespace MonoDevelop.Components.PropertyGrid
 			get { return editorManager; }
 		}
 		
-		internal void Initialize (Widget container, EditorManager editorManager, PropertyDescriptor property, object obj)
+		internal void Initialize (Widget container, EditorManager editorManager, ITypeDescriptorContext context)
 		{
 			this.container = container;
 			this.editorManager = editorManager;
@@ -72,25 +72,24 @@ namespace MonoDevelop.Components.PropertyGrid
 			Pango.FontDescription des = container.Style.FontDescription.Copy();
 			layout.FontDescription = des;
 			
-			this.property = property;
-			this.obj = obj;
+			this.context = context;
 			Initialize ();
 		}
 
-		public EditSession StartEditing (Gdk.Rectangle cell_area, StateType state)
+		public EditSession StartEditing (Rectangle cellArea, StateType state)
 		{
-			IPropertyEditor ed = CreateEditor (cell_area, state);
+			IPropertyEditor ed = CreateEditor (cellArea, state);
 			if (ed == null)
 				return null;
-			return new EditSession (container, obj, property, ed);
+			return new EditSession (container, context, ed);
 		}
 		
 		protected virtual string GetValueText ()
 		{
-			if (obj == null) return "";
-			object val = property.GetValue (obj);
-			if (val == null) return "";
-			else return property.Converter.ConvertToString (val);
+			var val = Value;
+			if (val != null)
+				return Property.Converter.ConvertToString (context, val);
+			return "";
 		}
 		
 		protected virtual string GetValueMarkup ()
@@ -107,16 +106,15 @@ namespace MonoDevelop.Components.PropertyGrid
 			if (i == -1)
 				return s;
 			
-			s = s.TrimStart ('\n',' ','\t');
+			s = s.TrimStart ('\n', ' ', '\t');
 			i = s.IndexOf ('\n');
 			if (i != -1)
 				return s.Substring (0, i) + "...";
-			else
-				return s;
+			return s;
 		}
 		
 		public object Value {
-			get { return obj != null ? property.GetValue (obj) : null; }
+			get { return Instance != null ? Property.GetValue (Instance) : null; }
 		}
 		
 		protected virtual void Initialize ()
@@ -133,7 +131,7 @@ namespace MonoDevelop.Components.PropertyGrid
 			layout.GetPixelSize (out width, out height);
 		}
 
-		public virtual void Render (Gdk.Drawable window, Cairo.Context ctx, Gdk.Rectangle bounds, StateType state)
+		public virtual void Render (Drawable window, Cairo.Context ctx, Rectangle bounds, StateType state)
 		{
 			int w, h;
 			layout.GetPixelSize (out w, out h);
@@ -146,17 +144,17 @@ namespace MonoDevelop.Components.PropertyGrid
 			ctx.Restore ();
 		}
 		
-		protected virtual IPropertyEditor CreateEditor (Gdk.Rectangle cell_area, StateType state)
+		protected virtual IPropertyEditor CreateEditor (Rectangle cellArea, StateType state)
 		{
-			if (DialogueEdit && (!property.IsReadOnly || EditsReadOnlyObject)) {
-				return new PropertyDialogueEditor (this);
+			if (DialogueEdit && (!Property.IsReadOnly || EditsReadOnlyObject)) {
+				return new PropertyDialogueEditor (this, context);
 			}
 			else {
-				Type editorType = editorManager.GetEditorType (property);
+				Type editorType = editorManager.GetEditorType (context);
 				if (editorType == null)
 					return null;
 				
-				IPropertyEditor editor = Activator.CreateInstance (editorType) as IPropertyEditor;
+				var editor = Activator.CreateInstance (editorType) as IPropertyEditor;
 				if (editor == null)
 					throw new Exception ("The property editor '" + editorType + "' must implement the interface IPropertyEditor");
 				return editor;
@@ -183,18 +181,18 @@ namespace MonoDevelop.Components.PropertyGrid
 		public virtual void LaunchDialogue ()
 		{
 			if (DialogueEdit)
-				throw new NotImplementedException();
+				throw new NotImplementedException ();
 		}
 	}
 	
 	
-	class DefaultPropertyEditor: Gtk.Entry, IPropertyEditor
+	class DefaultPropertyEditor: Entry, IPropertyEditor
 	{
 		PropertyDescriptor property;
 		
 		public void Initialize (EditSession session)
 		{
-			this.property = session.Property;
+			property = session.Property;
 		}
 		
 		public object Value {
@@ -221,24 +219,22 @@ namespace MonoDevelop.Components.PropertyGrid
 	
 	public class EditSession : ITypeDescriptorContext
 	{
-		PropertyDescriptor property; 
-		object obj;
-		Gtk.Widget container;
+		Widget container;
 		IPropertyEditor currentEditor;
 		bool syncing;
+		readonly ITypeDescriptorContext context;
 		
 		public event EventHandler Changed;
 		
-		public EditSession (Gtk.Widget container, object instance, PropertyDescriptor property, IPropertyEditor currentEditor)
+		internal EditSession (Widget container, ITypeDescriptorContext context, IPropertyEditor currentEditor)
 		{
-			this.property = property;
-			this.obj = instance;
+			this.context = context;
 			this.container = container;
 			this.currentEditor = currentEditor;
 			
 			currentEditor.Initialize (this);
-			if (instance != null)
-				currentEditor.Value = property.GetValue (instance);
+			if (Instance != null)
+				currentEditor.Value = context.PropertyDescriptor.GetValue (Instance);
 			
 			currentEditor.ValueChanged += OnValueChanged;
 		}
@@ -249,18 +245,18 @@ namespace MonoDevelop.Components.PropertyGrid
 		}
 		
 		public object Instance {
-			get { return obj; }
+			get { return context.Instance; }
 		}
 		
 		public PropertyDescriptor Property {
-			get { return property; }
+			get { return context.PropertyDescriptor; }
 		}
 		
 		PropertyDescriptor ITypeDescriptorContext.PropertyDescriptor {
-			get { return property; }
+			get { return context.PropertyDescriptor; }
 		}
 		
-		public Gtk.Widget Container {
+		public Widget Container {
 			get { return container; }
 		}
 		
@@ -272,8 +268,8 @@ namespace MonoDevelop.Components.PropertyGrid
 		{
 			if (!syncing) {
 				syncing = true;
-				if (!property.IsReadOnly) {
-					property.SetValue (obj, currentEditor.Value);
+				if (!context.PropertyDescriptor.IsReadOnly) {
+					context.PropertyDescriptor.SetValue (context.Instance, currentEditor.Value);
 					if (Changed != null)
 						Changed (s, a);
 				}
@@ -301,58 +297,53 @@ namespace MonoDevelop.Components.PropertyGrid
 		{
 			if (!syncing) {
 				syncing = true;
-				currentEditor.Value = property.GetValue (obj);
+				currentEditor.Value = context.PropertyDescriptor.GetValue (context.Instance);
 				syncing = false;
 			}
 		}
-		
-		#region FIXME Unimplemented ITypeDescriptorContext and IServiceProvider members
-		
+
 		object IServiceProvider.GetService (Type serviceType)
 		{
-			return null;
+			return context.GetService (serviceType);
 		}
 		
 		void ITypeDescriptorContext.OnComponentChanged ()
 		{
+			context.OnComponentChanged ();
 		}
 		
 		bool ITypeDescriptorContext.OnComponentChanging ()
 		{
-			return true;
+			return context.OnComponentChanging ();
 		}
 		
-		IContainer ITypeDescriptorContext.Container { get { return null; } }
-		
-		#endregion
+		IContainer ITypeDescriptorContext.Container { get { return context.Container; } }
 	}
 	
-	class CellRendererWidget: Gtk.DrawingArea
+	class CellRendererWidget: DrawingArea
 	{
-		PropertyEditorCell cell;
-		object obj;
-		PropertyDescriptor property;
-		EditorManager em;
+		readonly PropertyEditorCell cell;
+		readonly ITypeDescriptorContext context;
+		readonly EditorManager em;
 		
-		public CellRendererWidget (PropertyEditorCell cell)
+		public CellRendererWidget (PropertyEditorCell cell, ITypeDescriptorContext context)
 		{
 			this.cell = cell;
-			this.obj = cell.Instance;
-			this.property = cell.Property;
+			this.context = context;
 			em = cell.EditorManager;
-			this.ModifyBg (Gtk.StateType.Normal, this.Style.White);
+			this.ModifyBg (StateType.Normal, this.Style.White);
 		}
 		
 		protected override bool OnExposeEvent (EventExpose evnt)
 		{
 			bool res = base.OnExposeEvent (evnt);
-			cell.Initialize (this, em, property, obj);
+			cell.Initialize (this, em, context);
 			
-			Gdk.Rectangle rect = Allocation;
+			Rectangle rect = Allocation;
 			rect.Inflate (-3, 0);// Add some margin
 
-			using (Cairo.Context ctx = Gdk.CairoHelper.Create (this.GdkWindow)) {
-				cell.Render (this.GdkWindow, ctx, rect, StateType.Normal);
+			using (Cairo.Context ctx = CairoHelper.Create (GdkWindow)) {
+				cell.Render (GdkWindow, ctx, rect, StateType.Normal);
 			}
 			return res;
 		}
@@ -362,19 +353,19 @@ namespace MonoDevelop.Components.PropertyGrid
 	{
 		PropertyEditorCell cell;
 		
-		public PropertyDialogueEditor (PropertyEditorCell cell)
+		public PropertyDialogueEditor (PropertyEditorCell cell, ITypeDescriptorContext context)
 		{
 			this.cell = cell;
 			Spacing = 3;
-			PackStart (new CellRendererWidget (cell), true, true, 0);
+			PackStart (new CellRendererWidget (cell, context), true, true, 0);
 			Label buttonLabel = new Label ();
 			buttonLabel.UseMarkup = true;
 			buttonLabel.Xpad = 0; buttonLabel.Ypad = 0;
 			buttonLabel.Markup = "<span size=\"small\">...</span>";
 			Button dialogueButton = new Button (buttonLabel);
-			dialogueButton.Clicked += new EventHandler (DialogueButtonClicked);
+			dialogueButton.Clicked += DialogueButtonClicked;
 			PackStart (dialogueButton, false, false, 0);
-			this.ModifyBg (Gtk.StateType.Normal, this.Style.White);
+			this.ModifyBg (StateType.Normal, this.Style.White);
 			ShowAll ();
 		}
 		
