@@ -90,16 +90,21 @@ namespace MonoDevelop.Components
 
 		public new string Path {
 			get {
-				FilePath path = pathEntry.Text;
-				if (path.IsAbsolute || path.IsNullOrEmpty || string.IsNullOrEmpty (DefaultPath))
-					return path;
-				return path.ToAbsolute (DefaultPath);
+				try {
+					FilePath path = pathEntry.Text;
+					if (path.IsAbsolute || path.IsNullOrEmpty || string.IsNullOrEmpty (DefaultPath))
+						return path;
+					return path.ToAbsolute (DefaultPath);
+				} catch {
+					// If path conversion fails for some reason, return the plain text
+					return pathEntry.Text;
+				}
 			}
 			set {
 				loading = true;
 				if (!string.IsNullOrEmpty (value) && displayAsRelativePath)
 					value = ((FilePath)value).ToRelative (DefaultPath);
-				pathEntry.Text = value;
+				pathEntry.Text = value ?? "";
 				loading = false;
 			}
 		}
@@ -126,8 +131,11 @@ namespace MonoDevelop.Components
 				startIn = DefaultPath;
 
 			string path = ShowBrowseDialog (BrowserTitle, startIn);
-			if (path != null)
-				pathEntry.Text = path;
+			if (path != null) {
+				Path = path;
+				// Path setter suppresses change events so fire the event ourselves
+				OnTextChanged (null, null);
+			}
 		}
 		
 		void OnTextChanged (object o, EventArgs args)
