@@ -104,19 +104,21 @@ namespace MonoDevelop.Ide.CodeTemplates
 					                                AlertButton.Cancel, AlertButton.Remove) == AlertButton.Remove) {
 						templates.Remove (template);
 						templateStore.Remove (ref selected);
+						templatesToRemove.Add (template);
 					}
 				}
 			}
 		}
 		List<CodeTemplate> templatesToSave = new List<CodeTemplate> ();
+		List<CodeTemplate> templatesToRemove = new List<CodeTemplate> ();
 		void ButtonEditClicked (object sender, EventArgs e)
 		{
 			var template = GetSelectedTemplate ();
 			if (template != null) {
 				templatesToSave.Add (template);
-				var editDialog = new EditTemplateDialog (template, false);
-				if (MessageService.ShowCustomDialog (editDialog, this.Toplevel as Gtk.Window) == (int)ResponseType.Ok)
-					templatesToSave.Add (template);
+				using (var editDialog = new EditTemplateDialog (template, false))
+					if (MessageService.ShowCustomDialog (editDialog, this.Toplevel as Gtk.Window) == (int)ResponseType.Ok)
+						templatesToSave.Add (template);
 				HandleChanged (this, EventArgs.Empty);
 			}
 		}
@@ -132,18 +134,21 @@ namespace MonoDevelop.Ide.CodeTemplates
 		void ButtonAddClicked (object sender, EventArgs e)
 		{
 			var newTemplate = new CodeTemplate ();
-			var editDialog = new EditTemplateDialog (newTemplate, true);
-			if (MessageService.ShowCustomDialog (editDialog, this.Toplevel as Gtk.Window) == (int)ResponseType.Ok) {
-				InsertTemplate (newTemplate);
-				templates.Add (newTemplate);
-				templatesToSave.Add (newTemplate);
+			using (var editDialog = new EditTemplateDialog (newTemplate, true)) {
+				if (MessageService.ShowCustomDialog (editDialog, this.Toplevel as Gtk.Window) == (int)ResponseType.Ok) {
+					InsertTemplate (newTemplate);
+					templates.Add (newTemplate);
+					templatesToSave.Add (newTemplate);
+				}
 			}
 		}
 		
 		public void Store ()
 		{
-			templatesToSave.ForEach (template => CodeTemplateService.SaveTemplate (template));
+			templatesToSave.ForEach (CodeTemplateService.SaveTemplate);
 			templatesToSave.Clear ();
+			templatesToRemove.ForEach (CodeTemplateService.DeleteTemplate);
+			templatesToRemove.Clear ();
 			CodeTemplateService.Templates = templates;
 		}
 		
