@@ -30,7 +30,7 @@
 //
 //
 
-
+using System;
 using System.Threading;
 using MonoDevelop.Core;
 using MonoDevelop.Components.Commands;
@@ -41,6 +41,8 @@ using MonoDevelop.Core.Execution;
 using MonoDevelop.Ide.Execution;
 using CustomCommand = MonoDevelop.Projects.CustomCommand;
 using System.Linq;
+using MonoDevelop.Ide.Projects;
+using MonoDevelop.Projects.Policies;
 
 namespace MonoDevelop.Ide.Commands
 {
@@ -61,6 +63,8 @@ namespace MonoDevelop.Ide.Commands
 		AddNewFiles,
 		AddFiles,
 		NewFolder,
+		AddFilesFromFolder,
+		AddExistingFolder,
 		IncludeToProject,
 		BuildSolution,
 		Build,
@@ -78,10 +82,12 @@ namespace MonoDevelop.Ide.Commands
 		ConfigurationSelector,
 		CustomCommandList,
 		Reload,
-		ExportProject,
+		ExportSolution,
 		SpecificAssemblyVersion,
 		SelectActiveConfiguration,
-		SelectActiveRuntime
+		SelectActiveRuntime,
+		EditSolutionItem,
+		Unload
 	}
 
 	internal class SolutionOptionsHandler : CommandHandler
@@ -151,13 +157,13 @@ namespace MonoDevelop.Ide.Commands
 	{
 		protected override void Update (CommandInfo info)
 		{
-			info.Enabled = ((IdeApp.Workspace.IsOpen) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
+			info.Enabled = ((IdeApp.ProjectOperations.CurrentSelectedSolution != null) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
 		}
 
 		protected override void Run ()
 		{
 			//Build solution
-			IdeApp.ProjectOperations.Build (IdeApp.Workspace);
+			IdeApp.ProjectOperations.Build (IdeApp.ProjectOperations.CurrentSelectedSolution);
 		}
 	}
 
@@ -167,9 +173,9 @@ namespace MonoDevelop.Ide.Commands
 		{
 			if (IdeApp.Workspace.IsOpen) {
 				IBuildTarget buildTarget = IdeApp.ProjectOperations.CurrentSelectedBuildTarget;
-				info.Enabled = ((buildTarget != null) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
+				info.Enabled = ((buildTarget != null) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted)) && buildTarget.SupportsTarget (ProjectService.BuildTarget);
 				if (buildTarget != null) {
-					info.Text = GettextCatalog.GetString ("B_uild {0}", buildTarget.Name);
+					info.Text = GettextCatalog.GetString ("B_uild {0}", buildTarget.Name.Replace ("_","__"));
 					if (buildTarget is SolutionFolder)
 						info.Description = GettextCatalog.GetString ("Build solution {0}", buildTarget.Name);
 					else if (buildTarget is Project)
@@ -181,7 +187,7 @@ namespace MonoDevelop.Ide.Commands
 			else {
 				info.Enabled = ((IdeApp.Workbench.ActiveDocument != null) && (IdeApp.Workbench.ActiveDocument.IsBuildTarget) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
 				if (IdeApp.Workbench.ActiveDocument != null) {
-					info.Text = GettextCatalog.GetString ("B_uild {0}", Path.GetFileName (IdeApp.Workbench.ActiveDocument.Name));
+					info.Text = GettextCatalog.GetString ("B_uild {0}", Path.GetFileName (IdeApp.Workbench.ActiveDocument.Name).Replace ("_","__"));
                     info.Description = GettextCatalog.GetString ("Build {0}", Path.GetFileName (IdeApp.Workbench.ActiveDocument.Name));
 				}
 			}
@@ -203,13 +209,13 @@ namespace MonoDevelop.Ide.Commands
 	{
 		protected override void Update (CommandInfo info)
 		{
-			info.Enabled = ((IdeApp.Workspace.IsOpen) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
+			info.Enabled = ((IdeApp.ProjectOperations.CurrentSelectedSolution != null) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
 		}
 
 		protected override void Run ()
 		{
 			//Build solution
-			IdeApp.ProjectOperations.Rebuild (IdeApp.Workspace);
+			IdeApp.ProjectOperations.Rebuild (IdeApp.ProjectOperations.CurrentSelectedSolution);
 		}
 	}
 
@@ -219,16 +225,16 @@ namespace MonoDevelop.Ide.Commands
 		{
 			if (IdeApp.Workspace.IsOpen) {
 				IBuildTarget buildTarget = IdeApp.ProjectOperations.CurrentSelectedBuildTarget;
-				info.Enabled = ((buildTarget != null) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
+				info.Enabled = ((buildTarget != null) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted)) && buildTarget.SupportsTarget (ProjectService.BuildTarget);
 				if (buildTarget != null) {
-					info.Text = GettextCatalog.GetString ("R_ebuild {0}", IdeApp.ProjectOperations.CurrentSelectedBuildTarget.Name);
+					info.Text = GettextCatalog.GetString ("R_ebuild {0}", IdeApp.ProjectOperations.CurrentSelectedBuildTarget.Name.Replace ("_","__"));
 					info.Description = GettextCatalog.GetString ("Rebuild {0}", IdeApp.ProjectOperations.CurrentSelectedBuildTarget.Name);
 				}
 			}
 			else {
 				info.Enabled = ((IdeApp.Workbench.ActiveDocument != null) && (IdeApp.Workbench.ActiveDocument.IsBuildTarget) && (IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted));
 				if (info.Enabled) {
-					info.Text = GettextCatalog.GetString ("R_ebuild {0}", IdeApp.Workbench.ActiveDocument.FileName);
+					info.Text = GettextCatalog.GetString ("R_ebuild {0}", IdeApp.Workbench.ActiveDocument.FileName.FileName.Replace ("_","__"));
 					info.Description = GettextCatalog.GetString ("Rebuild {0}", IdeApp.Workbench.ActiveDocument.FileName);
 				}
 			}
@@ -251,17 +257,26 @@ namespace MonoDevelop.Ide.Commands
 		protected override void Update (CommandInfo info)
 		{
 			if ((IdeApp.Workspace.IsOpen) && (!IdeApp.ProjectOperations.CurrentRunOperation.IsCompleted))
-				info.Text = GettextCatalog.GetString ("_Run again");
+				info.Text = GettextCatalog.GetString ("Restart Without Debugging");
 			else
-				info.Text = GettextCatalog.GetString ("_Run");
+				info.Text = GettextCatalog.GetString ("Start Without Debugging");
 
 			info.Enabled = CanRun (Runtime.ProcessService.DefaultExecutionHandler);
 		}
 		
+		static IBuildTarget GetRunTarget ()
+		{
+			return IdeApp.ProjectOperations.CurrentSelectedSolution != null && IdeApp.ProjectOperations.CurrentSelectedSolution.StartupItem != null ? 
+				IdeApp.ProjectOperations.CurrentSelectedSolution.StartupItem : 
+					IdeApp.ProjectOperations.CurrentSelectedBuildTarget;
+		}
+		
 		public static bool CanRun (IExecutionHandler executionHandler)
 		{
-            if (IdeApp.Workspace.IsOpen)
-                return (IdeApp.ProjectOperations.CanExecute (IdeApp.Workspace, executionHandler)) && !(IdeApp.ProjectOperations.CurrentSelectedItem is Workspace);
+            if (IdeApp.Workspace.IsOpen) {
+				var target = GetRunTarget ();
+				return target != null && IdeApp.ProjectOperations.CanExecute (target, executionHandler);
+			}
             else
                 return (IdeApp.Workbench.ActiveDocument != null) && (IdeApp.Workbench.ActiveDocument.CanRun (executionHandler));
 		}
@@ -285,19 +300,15 @@ namespace MonoDevelop.Ide.Commands
                             IdeApp.Workbench.ActiveDocument.Run (executionHandler);
                     };
                 }
+				return;
             }
-            else {
-                if (!IdeApp.Preferences.BuildBeforeExecuting)
-                    IdeApp.ProjectOperations.Execute (IdeApp.Workspace, executionHandler);
-                else {
-                    IAsyncOperation asyncOperation = IdeApp.ProjectOperations.Build (IdeApp.Workspace);
-                    asyncOperation.Completed += delegate {
-                        if ((asyncOperation.Success) || (IdeApp.Preferences.RunWithWarnings && asyncOperation.SuccessWithWarnings))
-                            IdeApp.ProjectOperations.Execute (IdeApp.Workspace, executionHandler);
-                    };
-                }
 
-            }
+			var target = GetRunTarget ();
+			var op = IdeApp.ProjectOperations.CheckAndBuildForExecute (target);
+			op.Completed += delegate {
+				if (op.Success)
+					IdeApp.ProjectOperations.Execute (target, executionHandler);
+			};
         }
 
 		protected override void Run ()
@@ -338,16 +349,12 @@ namespace MonoDevelop.Ide.Commands
 
 		protected override void Run ()
 		{
-			if (!IdeApp.Preferences.BuildBeforeExecuting)
-				IdeApp.ProjectOperations.Execute (IdeApp.ProjectOperations.CurrentSelectedBuildTarget);
-			else {
-				IAsyncOperation asyncOperation = IdeApp.ProjectOperations.Build (IdeApp.ProjectOperations.CurrentSelectedBuildTarget);
-				asyncOperation.Completed += delegate
-				{
-					if ((asyncOperation.Success) || (IdeApp.Preferences.RunWithWarnings && asyncOperation.SuccessWithWarnings))
-                        IdeApp.ProjectOperations.Execute (IdeApp.ProjectOperations.CurrentSelectedBuildTarget);
-				};
-			}
+			var target = IdeApp.ProjectOperations.CurrentSelectedBuildTarget;
+			var op = IdeApp.ProjectOperations.CheckAndBuildForExecute (target);
+			op.Completed += delegate {
+				if (op.Success)
+					IdeApp.ProjectOperations.Execute (target);
+			};
 		}
 	}
 
@@ -372,17 +379,12 @@ namespace MonoDevelop.Ide.Commands
 			IBuildTarget target = IdeApp.ProjectOperations.CurrentSelectedBuildTarget;
 			if (h == null || !IdeApp.ProjectOperations.CurrentRunOperation.IsCompleted)
 				return;
-			
-			if (!IdeApp.Preferences.BuildBeforeExecuting)
-				IdeApp.ProjectOperations.Execute (target, h);
-			else {
-				IAsyncOperation asyncOperation = IdeApp.ProjectOperations.Build (target);
-				asyncOperation.Completed += delegate
-				{
-					if ((asyncOperation.Success) || (IdeApp.Preferences.RunWithWarnings && asyncOperation.SuccessWithWarnings))
-						IdeApp.ProjectOperations.Execute (target, h);
-				};
-			}
+
+			var op = IdeApp.ProjectOperations.CheckAndBuildForExecute (target);
+			op.Completed += delegate {
+				if (op.Success)
+					IdeApp.ProjectOperations.Execute (target, h);
+			};
 		}
 	}
 
@@ -393,7 +395,8 @@ namespace MonoDevelop.Ide.Commands
 			if (IdeApp.ProjectOperations.CurrentSelectedBuildTarget == null)
 				info.Enabled = false;
 			else {
-				info.Text = GettextCatalog.GetString ("C_lean {0}", IdeApp.ProjectOperations.CurrentSelectedBuildTarget.Name);
+				info.Enabled = IdeApp.ProjectOperations.CurrentSelectedBuildTarget.SupportsTarget (ProjectService.BuildTarget);
+				info.Text = GettextCatalog.GetString ("C_lean {0}", IdeApp.ProjectOperations.CurrentSelectedBuildTarget.Name.Replace ("_","__"));
 				info.Description = GettextCatalog.GetString ("Clean {0}", IdeApp.ProjectOperations.CurrentSelectedBuildTarget.Name);
 			}
 		}
@@ -408,13 +411,12 @@ namespace MonoDevelop.Ide.Commands
 	{
 		protected override void Update (CommandInfo info)
 		{
-			if (!IdeApp.Workspace.IsOpen)
-				info.Enabled = false;
+			info.Enabled = IdeApp.ProjectOperations.CurrentSelectedSolution != null;
 		}
 
 		protected override void Run ()
 		{
-			IdeApp.ProjectOperations.Clean (IdeApp.Workspace);
+			IdeApp.ProjectOperations.Clean (IdeApp.ProjectOperations.CurrentSelectedSolution);
 		}
 	}
 
@@ -464,7 +466,11 @@ namespace MonoDevelop.Ide.Commands
 			Thread t = new Thread (
 				delegate () {
 					using (monitor) {
-						cmd.Execute (monitor, ce, IdeApp.Workspace.ActiveConfiguration);
+						try {
+							cmd.Execute (monitor, ce, IdeApp.Workspace.ActiveConfiguration);
+						} catch (Exception ex) {
+							monitor.ReportError (GettextCatalog.GetString ("Command execution failed"), ex);
+						}
 					}
 				}
 			);
@@ -473,17 +479,25 @@ namespace MonoDevelop.Ide.Commands
 		}
 	}
 
-	internal class ExportProjectHandler : CommandHandler
+	internal class ExportSolutionHandler : CommandHandler
 	{
 		protected override void Update (CommandInfo info)
 		{
-			if (!(IdeApp.ProjectOperations.CurrentSelectedItem is WorkspaceItem) && !(IdeApp.ProjectOperations.CurrentSelectedItem is SolutionEntityItem))
+			// FIXME: Once we fix Workspaces to offer Visual Studio formats (instead of the deprecated MonoDevelop 1.0 format), we can allow exporting of Workspaces as well.
+			if (!(IdeApp.ProjectOperations.CurrentSelectedItem is Solution) && !(IdeApp.ProjectOperations.CurrentSelectedItem is SolutionEntityItem))
 				info.Enabled = false;
 		}
 
 		protected override void Run ()
 		{
-			IdeApp.ProjectOperations.Export (((IWorkspaceObject)IdeApp.ProjectOperations.CurrentSelectedItem), null);
+			WorkspaceItem workspace;
+			
+			if (!(IdeApp.ProjectOperations.CurrentSelectedItem is WorkspaceItem))
+				workspace = ((SolutionEntityItem) IdeApp.ProjectOperations.CurrentSelectedItem).ParentSolution;
+			else
+				workspace = (WorkspaceItem) IdeApp.ProjectOperations.CurrentSelectedItem;
+			
+			IdeApp.ProjectOperations.Export (workspace, null);
 		}
 	}
 	
@@ -524,4 +538,76 @@ namespace MonoDevelop.Ide.Commands
 			IdeApp.Workspace.ActiveRuntime = (MonoDevelop.Core.Assemblies.TargetRuntime) dataItem;
 		}
 	}
+	
+	class ApplyPolicyHandler: CommandHandler
+	{
+		protected override void Update (CommandInfo info)
+		{
+			info.Enabled = IdeApp.ProjectOperations.CurrentSelectedSolutionItem != null || IdeApp.ProjectOperations.CurrentSelectedSolution != null;
+		}
+		
+		protected override void Run ()
+		{
+			using (ApplyPolicyDialog dlg = new ApplyPolicyDialog ((IPolicyProvider)IdeApp.ProjectOperations.CurrentSelectedSolutionItem ?? (IPolicyProvider)IdeApp.ProjectOperations.CurrentSelectedSolution))
+				MessageService.ShowCustomDialog (dlg);
+		}
+	}
+	
+	class ExportPolicyHandler: CommandHandler
+	{
+		protected override void Update (CommandInfo info)
+		{
+			info.Enabled = IdeApp.ProjectOperations.CurrentSelectedSolutionItem != null || IdeApp.ProjectOperations.CurrentSelectedSolution != null;
+		}
+		
+		protected override void Run ()
+		{
+			using (ExportProjectPolicyDialog dlg = new ExportProjectPolicyDialog ((IPolicyProvider)IdeApp.ProjectOperations.CurrentSelectedSolutionItem ?? (IPolicyProvider)IdeApp.ProjectOperations.CurrentSelectedSolution))
+				MessageService.ShowCustomDialog (dlg);
+		}
+	}
+
+	internal class RunCodeAnalysisSolutionHandler : CommandHandler
+	{
+		protected override void Update (CommandInfo info)
+		{
+			//TODO: Roslyn, switch back to SupportsTarget check
+			//info.Enabled = (IdeApp.ProjectOperations.CurrentSelectedSolution != null) &&
+			//	(IdeApp.ProjectOperations.CurrentBuildOperation.IsCompleted) &&
+			//	IdeApp.ProjectOperations.CurrentSelectedSolution.GetAllProjects ().Any (p => p.SupportsTarget ("RunCodeAnalysis"));
+			info.Enabled = true;
+		}
+
+		protected override void Run ()
+		{
+			var context = new ProjectOperationContext ();
+			context.GlobalProperties.Add ("RunCodeAnalysisOnce", "true");
+			IdeApp.ProjectOperations.Rebuild (IdeApp.ProjectOperations.CurrentSelectedSolution, context);
+		}
+	}
+
+	internal class RunCodeAnalysisProjectHandler : CommandHandler
+	{
+		protected override void Update (CommandInfo info)
+		{
+			if (IdeApp.Workspace.IsOpen) {
+				var project = IdeApp.ProjectOperations.CurrentSelectedProject;
+				if (project != null) {
+					//TODO: Roslyn, switch back to SupportsTarget check
+					info.Enabled = true;//project.SupportsTarget ("RunCodeAnalysis");
+					info.Text = GettextCatalog.GetString ("Run Code Analysis on {0}", project.Name.Replace ("_","__"));
+					return;
+				}
+			}
+			info.Text = GettextCatalog.GetString ("Run Code Analysis on Project");
+			info.Enabled = false;
+		}
+
+		protected override void Run ()
+		{
+			var context = new ProjectOperationContext ();
+			context.GlobalProperties.Add ("RunCodeAnalysisOnce", "true");
+			IdeApp.ProjectOperations.Rebuild (IdeApp.ProjectOperations.CurrentSelectedProject, context);
+		}
+	}
 }
