@@ -43,7 +43,19 @@ namespace MonoDevelop.Ide.FindInFiles
 			get;
 			set;
 		}
-		
+
+		public virtual PathMode PathMode {
+			get {
+				var workspace = IdeApp.Workspace;
+				var solutions = workspace != null ? workspace.GetAllSolutions () : null;
+
+				if (solutions != null && solutions.Count == 1)
+					return PathMode.Relative;
+
+				return PathMode.Absolute;
+			}
+		}
+
 		public abstract int GetTotalWork (FilterOptions filterOptions);
 		public abstract IEnumerable<FileProvider> GetFiles (IProgressMonitor monitor, FilterOptions filterOptions);
 		public abstract string GetDescription (FilterOptions filterOptions, string pattern, string replacePattern);
@@ -51,6 +63,10 @@ namespace MonoDevelop.Ide.FindInFiles
 
 	public class DocumentScope : Scope
 	{
+		public override PathMode PathMode {
+			get { return PathMode.Hidden; }
+		}
+
 		public override int GetTotalWork (FilterOptions filterOptions)
 		{
 			return 1;
@@ -73,6 +89,10 @@ namespace MonoDevelop.Ide.FindInFiles
 
 	public class SelectionScope : Scope
 	{
+		public override PathMode PathMode {
+			get { return PathMode.Hidden; }
+		}
+
 		public override int GetTotalWork (FilterOptions filterOptions)
 		{
 			return 1;
@@ -114,18 +134,20 @@ namespace MonoDevelop.Ide.FindInFiles
 							continue;
 						if (alreadyVisited.Contains (file.FullPath))
 							continue;
-						alreadyVisited.Add (file.FileName);
+						alreadyVisited.Add (file.FullPath);
 						yield return new FileProvider (file.FullPath);
 					}
 				}
 				foreach (Project project in IdeApp.Workspace.GetAllProjects ()) {
 					monitor.Log.WriteLine (GettextCatalog.GetString ("Looking in project '{0}'", project.Name));
 					foreach (ProjectFile file in project.Files.Where (f => filterOptions.NameMatches (f.Name) && File.Exists (f.Name))) {
+						if ((file.Flags & ProjectItemFlags.Hidden) == ProjectItemFlags.Hidden)
+							continue;
 						if (!IncludeBinaryFiles && !DesktopService.GetFileIsText (file.FilePath))
 							continue;
-						if (alreadyVisited.Contains (file.Name))
+						if (alreadyVisited.Contains (file.FilePath.FullPath))
 							continue;
-						alreadyVisited.Add (file.Name);
+						alreadyVisited.Add (file.FilePath.FullPath);
 						yield return new FileProvider (file.Name, project);
 					}
 				}
@@ -163,12 +185,13 @@ namespace MonoDevelop.Ide.FindInFiles
 				monitor.Log.WriteLine (GettextCatalog.GetString ("Looking in project '{0}'", project.Name));
 				var alreadyVisited = new HashSet<string> ();
 				foreach (ProjectFile file in project.Files.Where (f => filterOptions.NameMatches (f.Name) && File.Exists (f.Name))) {
+					if ((file.Flags & ProjectItemFlags.Hidden) == ProjectItemFlags.Hidden)
+						continue;
 					if (!IncludeBinaryFiles && !DesktopService.GetFileIsText (file.Name))
 						continue;
-
-					if (alreadyVisited.Contains (file.Name))
+					if (alreadyVisited.Contains (file.FilePath.FullPath))
 						continue;
-					alreadyVisited.Add (file.Name);
+					alreadyVisited.Add (file.FilePath.FullPath);
 					yield return new FileProvider (file.Name, project);
 				}
 			}
@@ -212,7 +235,11 @@ namespace MonoDevelop.Ide.FindInFiles
 	{
 		readonly string path;
 		readonly bool recurse;
-		
+
+		public override PathMode PathMode {
+			get { return PathMode.Absolute; }
+		}
+
 		public bool IncludeHiddenFiles {
 			get;
 			set;
