@@ -238,6 +238,9 @@ namespace MonoDevelop.Ide.Gui.Components
 
 		void SetIconCellData (Gtk.TreeViewColumn col, Gtk.CellRenderer renderer, Gtk.TreeModel model, Gtk.TreeIter it)
 		{
+			if (model == null)
+				return;
+
 			var info = (NodeInfo)model.GetValue (it, NodeInfoColumn);
 			var cell = (ZoomableCellRendererPixbuf)renderer;
 
@@ -252,6 +255,9 @@ namespace MonoDevelop.Ide.Gui.Components
 
 		void SetTextCellData (Gtk.TreeViewColumn col, Gtk.CellRenderer renderer, Gtk.TreeModel model, Gtk.TreeIter it)
 		{
+			if (model == null)
+				return;
+
 			var info = (NodeInfo)model.GetValue (it, NodeInfoColumn);
 			var cell = (CustomCellRendererText)renderer;
 
@@ -411,11 +417,8 @@ namespace MonoDevelop.Ide.Gui.Components
 				text_render.Pushed = true;
 				args.RetVal = true;
 				var entryset = BuildEntrySet ();
-				var menu = IdeApp.CommandService.CreateMenu (entryset, this);
-				if (menu != null) {
-					menu.Hidden += HandleMenuHidden;
-					GtkWorkarounds.ShowContextMenu (menu, tree, text_render.PopupAllocation);
-				}
+
+				IdeApp.CommandService.ShowContextMenu (tree, args.Event, entryset, this, HandleMenuHidden);
 			}
 		}
 
@@ -494,7 +497,9 @@ namespace MonoDevelop.Ide.Gui.Components
 
 		void HandleMenuHidden (object sender, EventArgs e)
 		{
-			((Gtk.Menu)sender).Hidden -= HandleMenuHidden;
+			if (sender is Gtk.Menu) {
+				((Gtk.Menu)sender).Hidden -= HandleMenuHidden;
+			}
 			text_render.Pushed = false;
 			QueueDraw ();
 		}
@@ -1279,8 +1284,7 @@ namespace MonoDevelop.Ide.Gui.Components
 								handler.SetCurrentNode (nav);
 								handler.RenameItem (e.NewText);
 							} catch (Exception ex) {
-								MessageService.ShowException (ex);
-								LoggingService.LogError (ex.ToString ());
+								LoggingService.LogInternalError (ex);
 							}
 							nav.MoveToPosition (pos);
 						}
@@ -1297,7 +1301,7 @@ namespace MonoDevelop.Ide.Gui.Components
 				builder.Update ();
 			}
 			catch (Exception ex) {
-				MessageService.ShowException (ex, "The item could not be renamed");
+				LoggingService.LogInternalError ("The item could not be renamed", ex);
 			}
 		}
 
@@ -1323,19 +1327,24 @@ namespace MonoDevelop.Ide.Gui.Components
 			if (root == null)
 				return null;
 
-			var state = root.SaveState ();
+			var rootState = NodeState.CreateRoot ();
+			List<NodeState> children = new List<NodeState> ();
+			rootState.ChildrenState = children;
 
 			var s = new Dictionary<string, bool> ();
 			foreach (TreePadOption opt in options) {
 				bool val;
 				if (globalOptions.TryGetValue (opt.Id, out val) && val != opt.DefaultValue)
-					s[opt.Id] = val;
-			}
-			if (s.Count != 0) {
-				state.Options = s;
+					s [opt.Id] = val;
 			}
+			if (s.Count != 0)
+				rootState.Options = s;
 
-			return state;
+			do {
+				rootState.ChildrenState.Add (root.SaveState ());
+			} while (root.MoveNext ());
+
+			return rootState;
 		}
 
 		public void RestoreTreeState (NodeState state)
@@ -1347,7 +1356,21 @@ namespace MonoDevelop.Ide.Gui.Components
 			if (nav == null)
 				return;
 
-			nav.RestoreState (state);
+			if (state.IsRoot) {
+				if (state.ChildrenState != null) {
+					var pos = nav.CurrentPosition;
+					foreach (NodeState ces in state.ChildrenState) {
+						do {
+							if (nav.NodeName == ces.NodeName) {
+								nav.RestoreState (ces);
+								break;
+							}
+						} while (nav.MoveNext ());
+						nav.MoveToPosition (pos);
+					}
+				}
+			} else
+				nav.RestoreState (state);
 
 			globalOptions = new TreeOptions ();
 			foreach (TreePadOption opt in options) {
@@ -1762,7 +1785,16 @@ namespace MonoDevelop.Ide.Gui.Components
 		{
 			var entryset = BuildEntrySet () ?? new CommandEntrySet ();
 
-			IdeApp.CommandService.ShowContextMenu (this, evt, entryset, this);
+			if (evt == null) {
+				var paths = tree.Selection.GetSelectedRows ();
+				if (paths != null) {
+					var area = tree.GetCellArea (paths [0], tree.Columns [0]);
+
+					IdeApp.CommandService.ShowContextMenu (this, area.Left, area.Top, entryset, this);
+				}
+			} else {
+				IdeApp.CommandService.ShowContextMenu (this, evt, entryset, this);
+			}
 		}
 
 		protected CommandEntrySet BuildEntrySet ()
@@ -2248,9 +2280,9 @@ namespace MonoDevelop.Ide.Gui.Components
 
 			static CustomCellRendererText ()
 			{
-				popupIcon = Xwt.Drawing.Image.FromResource ("tree-popup-button-light.png");
-				popupIconDown = Xwt.Drawing.Image.FromResource ("tree-popup-button-down-light.png");
-				popupIconHover = Xwt.Drawing.Image.FromResource ("tree-popup-button-hover-light.png");
+				popupIcon = Xwt.Drawing.Image.FromResource ("tree-popup-button.png");
+				popupIconDown = Xwt.Drawing.Image.FromResource ("tree-popup-button-down.png");
+				popupIconHover = Xwt.Drawing.Image.FromResource ("tree-popup-button-hover.png");
 			}
 
 			[GLib.Property ("text-markup")]
