@@ -29,6 +29,7 @@ using System;
 using System.Collections.Generic;
 using MonoDevelop.Core;
 using System.Collections;
+using MonoDevelop.Components;
 
 namespace MonoDevelop.Ide.Gui.Components
 {
@@ -91,7 +92,7 @@ namespace MonoDevelop.Ide.Gui.Components
 	
 			public ITreeOptions Options {
 				get {
-					return (node != null) ? node.Options : navigator.Options;
+					return tree.globalOptions;
 				}
 			}
 	
@@ -348,7 +349,12 @@ namespace MonoDevelop.Ide.Gui.Components
 					node = null;
 				}
 			}
-	
+		
+			public void ScrollToNode ()
+			{
+				tree.ScrollToCell (tree.store.GetPath (node.NodeIter));
+			}
+
 			public bool FindChild (object dataObject)
 			{
 				return FindChild (dataObject, false);
@@ -413,7 +419,7 @@ namespace MonoDevelop.Ide.Gui.Components
 				if (!citPath.Up ())
 					return false;
 
-				if (citPath == pitPath)
+				if (citPath.Equals (pitPath))
 					return true;
 
 				return recursive && pitPath.IsAncestor (citPath);
@@ -498,6 +504,14 @@ namespace MonoDevelop.Ide.Gui.Components
 			{
 				node.Filled = true;
 				NodePosition pos = CurrentPosition;
+				foreach (NodeBuilder builder in node.BuilderChain) {
+					try {
+						builder.PrepareChildNodes (node.DataItem);
+					} catch (Exception ex) {
+						LoggingService.LogError (ex.ToString ());
+					}
+					MoveToPosition (pos);
+				}
 				foreach (NodeBuilder builder in node.BuilderChain) {
 					try {
 						builder.BuildChildNodes (this, node.DataItem);
@@ -548,7 +562,6 @@ namespace MonoDevelop.Ide.Gui.Components
 				}
 				TreeNode n = CreateNode (dataObject);
 				if (n != null) {
-					n.Options = node.Options;
 					if (node.Children == null)
 						node.Children = new List<TreeNode> ();
 					node.Children.Add (n);
@@ -582,7 +595,6 @@ namespace MonoDevelop.Ide.Gui.Components
 				n.Filled = navigator.Filled;
 				n.Name = navigator.NodeName;
 				n.NodeIter = navigator.CurrentPosition._iter;
-				n.Options = navigator.Options;
 				n.Selected = navigator.Selected;
 				n.TypeNodeBuilder = navigator.TypeNodeBuilder;
 				tstore.RegisterNode (n);
@@ -647,19 +659,13 @@ namespace MonoDevelop.Ide.Gui.Components
 
 			void UpdateNode (TreeNode n, NodeBuilder[] chain, NodeAttributes ats, object dataObject)
 			{
-				string text;
-				Gdk.Pixbuf icon;
-				Gdk.Pixbuf closedIcon;
-				TreeBuilder.GetNodeInfo (tree, this, chain, dataObject, out text, out icon, out closedIcon);
+				var ni = new NodeInfo ();
+				n.NodeInfo = TreeBuilder.GetNodeInfo (ni, tree, this, chain, dataObject);
 
-				n.Text = text;
-				n.Icon = icon;
-				n.ClosedIcon = closedIcon;
-				
 				if (chain != null && chain.Length > 0)
 					n.Name = ((TypeNodeBuilder)chain[0]).GetNodeName (this, n.DataItem);
 				else
-					n.Name = n.Text;
+					n.Name = n.NodeInfo.Label;
 				
 				n.Modified = true;
 			}
@@ -667,14 +673,11 @@ namespace MonoDevelop.Ide.Gui.Components
 
 		internal class TreeNode
 		{
-			public string Text;
-			public Gdk.Pixbuf Icon;
-			public Gdk.Pixbuf ClosedIcon;
-			
+			public NodeInfo NodeInfo;
+
 			public bool Selected;
 			public bool Filled;
 			public bool Expanded;
-			public ITreeOptions Options;
 			public TypeNodeBuilder TypeNodeBuilder;
 			public TreeNode Parent;
 			public object DataItem;
@@ -751,8 +754,10 @@ namespace MonoDevelop.Ide.Gui.Components
 					foreach (TreeNode cn in node.Children)
 						RemoveNode (cn);
 				}
-				if (!node.HasIter) {
-					List<TreeNode> list = objects [node.DataItem];
+				
+				List<TreeNode> list = null;
+				
+				if (!node.HasIter && objects.TryGetValue (node.DataItem, out list)) {
 					list.Remove (node);
 					if (list.Count == 0) {
 						objects.Remove (node.DataItem);
@@ -830,15 +835,11 @@ namespace MonoDevelop.Ide.Gui.Components
 						LoggingService.LogError ("Found invalid iter for node. " + node.DataItem);
 						return;
 					}
-					tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.TextColumn, node.Text);
-					if (node.Icon != null)
-						tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.OpenIconColumn, node.Icon);
-					if (node.ClosedIcon != null)
-						tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.ClosedIconColumn, node.ClosedIcon);
+					tree.Store.SetValue (node.NodeIter, ExtensibleTreeView.NodeInfoColumn, node.NodeInfo);
 				}
 				if (node.Children != null) {
 					foreach (TreeNode cn in node.Children) {
-						Gtk.TreeIter it = tree.Store.AppendValues (node.NodeIter, cn.Text, cn.Icon, cn.ClosedIcon, cn.DataItem, cn.BuilderChain, cn.Filled);
+						Gtk.TreeIter it = tree.Store.AppendValues (node.NodeIter, cn.NodeInfo, cn.DataItem, cn.BuilderChain, cn.Filled, false);
 						if (!cn.Filled)
 							tree.Store.AppendNode (it);	// Dummy node
 						// The OnNodeAdded event was already fired when the node was added. There is no need to fire it again.
@@ -907,5 +908,10 @@ namespace MonoDevelop.Ide.Gui.Components
 				while (tree.store.IterNext (ref child));
 			}
 		}
+	
+		public void ScrollToCell (Gtk.TreePath treePath)
+		{
+			Tree.ScrollToCell (treePath, null, true, 0, 0);
+		}
 	}
 }
