@@ -42,10 +42,10 @@ using System.Linq;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 {
-	public class ProjectFolderNodeBuilder: FolderNodeBuilder
+	class ProjectFolderNodeBuilder: FolderNodeBuilder
 	{
-		Gdk.Pixbuf folderOpenIcon;
-		Gdk.Pixbuf folderClosedIcon;
+		Xwt.Drawing.Image folderOpenIcon;
+		Xwt.Drawing.Image folderClosedIcon;
 		
 		EventHandler<FileCopyEventArgs> fileRenamedHandler;
 		EventHandler<FileEventArgs> fileRemovedHandler;
@@ -119,15 +119,15 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 		}
 	
-		public override void BuildNode (ITreeBuilder treeBuilder, object dataObject, ref string label, ref Gdk.Pixbuf icon, ref Gdk.Pixbuf closedIcon)
+		public override void BuildNode (ITreeBuilder treeBuilder, object dataObject, NodeInfo nodeInfo)
 		{
-			base.BuildNode (treeBuilder, dataObject, ref label, ref icon, ref closedIcon);
+			base.BuildNode (treeBuilder, dataObject, nodeInfo);
 
 			ProjectFolder folder = (ProjectFolder) dataObject;
 
-			label = GLib.Markup.EscapeText (folder.Name);
-			icon = folderOpenIcon;
-			closedIcon = folderClosedIcon;
+			nodeInfo.Label = GLib.Markup.EscapeText (folder.Name);
+			nodeInfo.Icon = folderOpenIcon;
+			nodeInfo.ClosedIcon = folderClosedIcon;
 		}
 		
 		public override object GetParentObject (object dataObject)
@@ -137,8 +137,13 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		}
 	}
 	
-	public class ProjectFolderCommandHandler: FolderCommandHandler
+	class ProjectFolderCommandHandler: FolderCommandHandler
 	{
+		public override void ActivateItem ()
+		{
+			CurrentNode.Expanded = !CurrentNode.Expanded;
+		}
+
 		public override string GetFolderPath (object dataObject)
 		{
 			return ((ProjectFolder)dataObject).Path;
@@ -154,17 +159,27 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				try {
 					if (!FileService.IsValidPath (newFoldername)) {
 						MessageService.ShowWarning (GettextCatalog.GetString ("The name you have chosen contains illegal characters. Please choose a different name."));
-					} else if (File.Exists (newFoldername) || Directory.Exists (newFoldername)) {
+						return;
+					} 
+					if (File.Exists (newFoldername)) {
 						MessageService.ShowWarning (GettextCatalog.GetString ("File or directory name is already in use. Please choose a different one."));
-					} else {
-						FileService.RenameDirectory (oldFoldername, newName);
-						if (folder.Project != null)
-							IdeApp.ProjectOperations.Save (folder.Project);
+						return;
 					}
+					// Don't use Directory.Exists because we want to check for the exact case in case-insensitive file systems
+					var di = Directory.GetDirectories (Path.GetDirectoryName (newFoldername), Path.GetFileName (newFoldername)).FirstOrDefault ();
+					if (di != null) {
+						MessageService.ShowWarning (GettextCatalog.GetString ("File or directory name is already in use. Please choose a different one."));
+						return;
+					}
+
+					FileService.RenameDirectory (oldFoldername, newName);
+					if (folder.Project != null)
+						IdeApp.ProjectOperations.Save (folder.Project);
+
 				} catch (System.ArgumentException) { // new file name with wildcard (*, ?) characters in it
 					MessageService.ShowWarning (GettextCatalog.GetString ("The name you have chosen contains illegal characters. Please choose a different name."));
 				} catch (System.IO.IOException ex) {
-					MessageService.ShowException (ex, GettextCatalog.GetString ("There was an error renaming the directory."));
+					MessageService.ShowError (GettextCatalog.GetString ("There was an error renaming the directory."), ex.Message, ex);
 				}
 			}
 		}
@@ -186,50 +201,68 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			question.Buttons.Add (AlertButton.Cancel);
 			question.Buttons.Add (AlertButton.Delete);
 			question.Buttons.Add (removeButton);
+
+			var deleteOnlyQuestion = new QuestionMessage () {
+				AllowApplyToAll = folders.Count > 1,
+				SecondaryText = GettextCatalog.GetString ("The directory and any files it contains will be permanently removed from your hard disk. ")
+			};
+			deleteOnlyQuestion.Buttons.Add (AlertButton.Cancel);
+			deleteOnlyQuestion.Buttons.Add (AlertButton.Delete);
 			
 			foreach (var folder in folders) {
 				var project = folder.Project;
-				
+
+				AlertButton result;
+
+				if (project == null) {
+					deleteOnlyQuestion.Text = GettextCatalog.GetString ("Are you sure you want to remove directory {0}?", folder.Name);
+					result = MessageService.AskQuestion (deleteOnlyQuestion);
+					if (result == AlertButton.Delete) {
+						DeleteFolder (folder);
+						continue;
+					} else
+						break;
+				}
+
 				var folderRelativePath = folder.Path.ToRelative (project.BaseDirectory);
 				var files = project.Files.GetFilesInVirtualPath (folderRelativePath).ToList ();
 				var folderPf = project.Files.GetFileWithVirtualPath (folderRelativePath);
-				
+				bool isProjectFolder = files.Count == 0 && folderPf == null;
+
 				//if the parent directory has already been removed, there may be nothing to do
-				if (files.Count == 0 && folderPf == null)
-					continue;
-				
-				question.Text = GettextCatalog.GetString ("Are you sure you want to remove directory {0} from project {1}?",
-					folder.Name, project.Name);
-				var result = MessageService.AskQuestion (question);
-				if (result != removeButton && result != AlertButton.Delete) 
-					break;
-				
-				projects.Add (project);
-				
-				//remove the files and link files in the directory
-				foreach (var f in files)
-					project.Files.Remove (f);
-				
-				// also remove the folder's own ProjectFile, if it exists 
-				// FIXME: it probably was already in the files list
-				if (folderPf != null)
-					project.Files.Remove (folderPf);
+				if (isProjectFolder) {
+					deleteOnlyQuestion.Text = GettextCatalog.GetString ("Are you sure you want to remove directory {0}?", folder.Name);
+					result = MessageService.AskQuestion (deleteOnlyQuestion);
+					if (result != AlertButton.Delete) 
+						break;
+				}
+				else {
+					question.Text = GettextCatalog.GetString ("Are you sure you want to remove directory {0} from project {1}?",
+						folder.Name, project.Name);
+					result = MessageService.AskQuestion (question);
+					if (result != removeButton && result != AlertButton.Delete) 
+						break;
+					
+					projects.Add (project);
+					
+					//remove the files and link files in the directory
+					foreach (var f in files)
+						project.Files.Remove (f);
+					
+					// also remove the folder's own ProjectFile, if it exists 
+					// FIXME: it probably was already in the files list
+					if (folderPf != null)
+						project.Files.Remove (folderPf);
+				}
 				
 				if (result == AlertButton.Delete) {
-					try {
-						if (Directory.Exists (folder.Path))
-							// FileService events should remove remaining files from the project
-							FileService.DeleteDirectory (folder.Path);
-					} catch (Exception ex) {
-						MessageService.ShowError (GettextCatalog.GetString (
-							"The folder {0} could not be deleted from disk: {1}", folder.Path, ex.Message));
-					}
+					DeleteFolder (folder);
 				} else {
 					//explictly remove the node from the tree, since it currently only tracks real folder deletions
 					folder.Remove ();
 				}
 				
-				if (folder.Path.ParentDirectory != project.BaseDirectory) {
+				if (isProjectFolder && folder.Path.ParentDirectory != project.BaseDirectory) {
 					// If it's the last item in the parent folder, make sure we keep a reference to the parent 
 					// folder, so it is not deleted from the tree.
 					var inParentFolder = project.Files.GetFilesInVirtualPath (folderRelativePath.ParentDirectory);
@@ -242,7 +275,19 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			}
 			IdeApp.ProjectOperations.Save (projects);
 		}
-		
+
+		static void DeleteFolder (ProjectFolder folder)
+		{
+			try {
+				if (Directory.Exists (folder.Path))
+					// FileService events should remove remaining files from the project
+					FileService.DeleteDirectory (folder.Path);
+			}
+			catch (Exception ex) {
+				MessageService.ShowError (GettextCatalog.GetString ("The folder {0} could not be deleted from disk: {1}", folder.Path, ex.Message));
+			}
+		}
+
 		[CommandUpdateHandler (EditCommands.Delete)]
 		public void UpdateRemoveItem (CommandInfo info)
 		{
