@@ -50,7 +50,8 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		ProjectFileRenamedEventHandler fileRenamedHandler;
 		ProjectFileEventHandler filePropertyChangedHandler;
 		SolutionItemModifiedEventHandler projectChanged;
-		
+		EventHandler<FileEventArgs> deletedHandler;
+
 		public override Type NodeDataType {
 			get { return typeof(Project); }
 		}
@@ -66,13 +67,14 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			filePropertyChangedHandler = (ProjectFileEventHandler) DispatchService.GuiDispatch (new ProjectFileEventHandler (OnFilePropertyChanged));
 			fileRenamedHandler = (ProjectFileRenamedEventHandler) DispatchService.GuiDispatch (new ProjectFileRenamedEventHandler (OnRenameFile));
 			projectChanged = (SolutionItemModifiedEventHandler) DispatchService.GuiDispatch (new SolutionItemModifiedEventHandler (OnProjectModified));
-			
+			deletedHandler = (EventHandler<FileEventArgs>) DispatchService.GuiDispatch (new EventHandler<FileEventArgs> (OnSystemFileDeleted));
+
 			IdeApp.Workspace.FileAddedToProject += fileAddedHandler;
 			IdeApp.Workspace.FileRemovedFromProject += fileRemovedHandler;
 			IdeApp.Workspace.FileRenamedInProject += fileRenamedHandler;
 			IdeApp.Workspace.FilePropertyChangedInProject += filePropertyChangedHandler;
-			
 			IdeApp.Workspace.ActiveConfigurationChanged += IdeAppWorkspaceActiveConfigurationChanged;
+			FileService.FileRemoved += deletedHandler;
 		}
 
 		public override void Dispose ()
@@ -82,6 +84,7 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			IdeApp.Workspace.FileRenamedInProject -= fileRenamedHandler;
 			IdeApp.Workspace.FilePropertyChangedInProject -= filePropertyChangedHandler;
 			IdeApp.Workspace.ActiveConfigurationChanged -= IdeAppWorkspaceActiveConfigurationChanged;
+			FileService.FileRemoved -= deletedHandler;
 		}
 
 		public override void OnNodeAdded (object dataObject)
@@ -200,8 +203,28 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 				RemoveFile (e.ProjectFile, e.Project);
 		}
 		
+		void OnSystemFileDeleted (object sender, FileEventArgs args)
+		{
+			if (!args.Any (f => f.IsDirectory))
+				return;
+
+			// When a folder is deleted, we need to remove all references in the tree, for all projects
+			ITreeBuilder tb = Context.GetTreeBuilder ();
+			var dirs = args.Where (d => d.IsDirectory).Select (d => d.FileName).ToArray ();
+
+			foreach (var p in IdeApp.Workspace.GetAllProjects ()) {
+				foreach (var dir in dirs) {
+					if (tb.MoveToObject (new ProjectFolder (dir, p)) && tb.MoveToParent ())
+						tb.UpdateAll ();
+				}
+			}
+		}
+
 		void AddFile (ProjectFile file, Project project)
 		{
+			if (!file.Visible || file.Flags.HasFlag (ProjectItemFlags.Hidden))
+				return;
+			
 			ITreeBuilder tb = Context.GetTreeBuilder ();
 			
 			if (file.DependsOnFile != null) {
@@ -341,12 +364,8 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		{
 			foreach (Project p in IdeApp.Workspace.GetAllProjects ()) {
 				ITreeBuilder tb = Context.GetTreeBuilder (p);
-				if (tb != null) {
+				if (tb != null)
 					tb.Update ();
-					SolutionConfiguration conf = p.ParentSolution.GetConfiguration (IdeApp.Workspace.ActiveConfiguration);
-					if (conf == null || !conf.BuildEnabledForItem (p))
-						tb.Expanded = false;
-				}
 			}
 		}
 		
@@ -375,16 +394,23 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		public void UpdateSetAsStartupProject (CommandInfo ci)
 		{
 			Project project = (Project) CurrentNode.DataItem;
-			ci.Visible = project.CanExecute (new ExecutionContext (Runtime.ProcessService.DefaultExecutionHandler, null, IdeApp.Workspace.ActiveExecutionTarget), IdeApp.Workspace.ActiveConfiguration);
+
+			// This should check for SupportsExecute only, but we keep the call to CanExecute in order to
+			// be backwards compatible with the old behavior.
+			// TODO NPM: Remove the CanExecute call
+			ci.Visible = project.SupportsExecute () || project.CanExecute (new ExecutionContext (Runtime.ProcessService.DefaultExecutionHandler, null, IdeApp.Workspace.ActiveExecutionTarget), IdeApp.Workspace.ActiveConfiguration);
 		}
 
 		[CommandHandler (ProjectCommands.SetAsStartupProject)]
 		public void SetAsStartupProject ()
 		{
 			Project project = CurrentNode.DataItem as Project;
-			project.ParentSolution.SingleStartup = true;
 			project.ParentSolution.StartupItem = project;
-			IdeApp.ProjectOperations.Save (project.ParentSolution);
+			if (!project.ParentSolution.SingleStartup) {
+				project.ParentSolution.SingleStartup = true;
+				IdeApp.ProjectOperations.Save (project.ParentSolution);
+			} else
+				project.ParentSolution.SaveUserProperties ();
 		}
 		
 		public override void DeleteItem ()
