@@ -44,8 +44,7 @@ using MonoDevelop.Components.Docking;
 
 namespace MonoDevelop.Ide.Gui.Pads
 {
-	
-	public class TaskListPad : IPadContent
+	class TaskListPad : IPadContent
 	{
 		Widget control;
 		ITaskListView activeView;
@@ -97,7 +96,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			}
 			catch (Exception e) // no codons loaded
 			{
-				LoggingService.LogError ("Loading of Task List Views failed: {0}", e.ToString ());
+				LoggingService.LogError ("Loading of Tasks Views failed: {0}", e.ToString ());
 			}
 			
 			switcherCombo = new ComboBox (switcherComboList);
