@@ -355,7 +355,7 @@ namespace MonoDevelop.Ide.Gui
 					try {
 						FileService.RequestFileEdit (Window.ViewContent.ContentName, true);
 					} catch (Exception ex) {
-						MessageService.ShowException (ex, GettextCatalog.GetString ("The file could not be saved."));
+						MessageService.ShowError (GettextCatalog.GetString ("The file could not be saved."), ex.Message, ex);
 					}
 					
 					FileAttributes attr = FileAttributes.ReadOnly | FileAttributes.Directory | FileAttributes.Offline | FileAttributes.System;
@@ -381,7 +381,9 @@ namespace MonoDevelop.Ide.Gui
 				if (doc != null && doc.ParsedFile != null) {
 					string fileName = Window.ViewContent.ContentName;
 					try {
-						doc.ParsedFile.LastWriteTime = File.GetLastWriteTimeUtc (fileName);
+						// filename could be null if the user cancelled SaveAs and this is a new & unsaved file
+						if (fileName != null)
+							doc.ParsedFile.LastWriteTime = File.GetLastWriteTimeUtc (fileName);
 					} catch (Exception e) {
 						doc.ParsedFile.LastWriteTime = DateTime.UtcNow;
 						LoggingService.LogWarning ("Exception while getting the write time from " + fileName, e); 
@@ -768,9 +770,7 @@ namespace MonoDevelop.Ide.Gui
 				var wrapper = TypeSystemService.GetProjectContentWrapper (project);
 				wrapper.Loaded += HandleInLoadChanged;
 				currentWrapper = wrapper;
-				RunWhenLoaded (delegate {
-					currentWrapper.RequestLoad ();
-				});
+				currentWrapper.RequestLoad ();
 			}
 			StartReparseThread ();
 		}
@@ -902,7 +902,7 @@ namespace MonoDevelop.Ide.Gui
 		internal object ExtendedCommandTargetChain {
 			get {
 				// Only go through the text editor chain, if the text editor is selected as subview
-				if (Window != null && Window.ActiveViewContent == Window.ViewContent)
+				if (Window != null && Window.ActiveViewContent.GetContent (typeof(IExtensibleTextEditor)) != null)
 					return editorExtension;
 				return null;
 			}
@@ -914,7 +914,7 @@ namespace MonoDevelop.Ide.Gui
 				window.ViewContent.Project = null;
 		}
 		
-		void OnDocumentParsed (EventArgs e)
+		protected virtual void OnDocumentParsed (EventArgs e)
 		{
 			EventHandler handler = this.DocumentParsed;
 			if (handler != null)
@@ -978,7 +978,8 @@ namespace MonoDevelop.Ide.Gui
 		/// </summary>
 		public void DisableAutoScroll ()
 		{
-			Mono.TextEditor.Utils.FileSettingsStore.Remove (FileName);
+			if (IsFile)
+				Mono.TextEditor.Utils.FileSettingsStore.Remove (FileName);
 		}
 	}
 	
