@@ -92,7 +92,7 @@ namespace MonoDevelop.Ide
 			
 			if (FeedbackService.Enabled) {
 				CustomFrame fr = new CustomFrame (0, 0, 1, 0);
-				var px = Xwt.Drawing.Image.FromResource ("feedback-light-16.png");
+				var px = Xwt.Drawing.Image.FromResource ("feedback-16.png");
 				HBox b = new HBox (false, 3);
 				b.PackStart (new Xwt.ImageView (px).ToGtkWidget ());
 				b.PackStart (new Gtk.Label ("Feedback"));
