@@ -264,7 +264,7 @@ namespace MonoDevelop.Ide.Gui
 			workbench.Toolbar.HideCommandBar (barId);
 		}
 
-		internal MonoDevelop.Components.MainToolbar.MainToolbar Toolbar {
+		internal MonoDevelop.Components.MainToolbar.MainToolbarController Toolbar {
 			get {
 				return workbench.Toolbar;
 			}
@@ -467,7 +467,6 @@ namespace MonoDevelop.Ide.Gui
 					Counters.OpenDocumentTimer.Trace ("Look for open document");
 					foreach (Document doc in Documents) {
 						IBaseViewContent vcFound = null;
-						int vcIndex = 0;
 
 						//search all ViewContents to see if they can "re-use" this filename
 						if (doc.Window.ViewContent.CanReuseView (info.FileName))
@@ -497,7 +496,6 @@ namespace MonoDevelop.Ide.Gui
 							
 							if (info.Options.HasFlag (OpenDocumentOptions.BringToFront)) {
 								doc.Select ();
-								doc.Window.SwitchView (vcIndex);
 								doc.Window.SelectWindow ();
 								NavigationHistoryService.LogActiveDocument ();
 							}
@@ -632,6 +630,7 @@ namespace MonoDevelop.Ide.Gui
 				}
 			} finally {
 				ops.Destroy ();
+				ops.Dispose ();
 			}
 		}
 		
@@ -654,6 +653,7 @@ namespace MonoDevelop.Ide.Gui
 				MessageService.RunCustomDialog (ops, parentWindow);
 			} finally {
 				ops.Destroy ();
+				ops.Dispose ();
 			}
 		}
 		
@@ -763,23 +763,8 @@ namespace MonoDevelop.Ide.Gui
 				    GettextCatalog.GetString ("If you don't save, all changes will be permanently lost."),
 				    AlertButton.CloseWithoutSave, AlertButton.Cancel, window.ViewContent.IsUntitled ? AlertButton.SaveAs : AlertButton.Save);
 				if (result == AlertButton.Save || result == AlertButton.SaveAs) {
-					if (window.ViewContent.ContentName == null) {
-						FindDocument (window).Save ();
-						args.Cancel = window.ViewContent.IsDirty;
-					} else {
-						try {
-							if (window.ViewContent.IsFile)
-								window.ViewContent.Save (window.ViewContent.ContentName);
-							else
-								window.ViewContent.Save ();
-							args.Cancel |= window.ViewContent.IsDirty;
-
-						}
-						catch (Exception ex) {
-							args.Cancel = true;
-							MessageService.ShowException (ex, GettextCatalog.GetString ("The document could not be saved."));
-						}
-					}
+					FindDocument (window).Save ();
+					args.Cancel = window.ViewContent.IsDirty;
 					if (args.Cancel)
 						FindDocument (window).Select ();
 				} else {
@@ -841,12 +826,8 @@ namespace MonoDevelop.Ide.Gui
 				monitor.ReportError (GettextCatalog.GetString ("Invalid file name"), null);
 				return;
 			}
-			
-			if (origName.StartsWith ("file://", StringComparison.Ordinal))
-				fileName = new Uri (origName).LocalPath;
-			else
-				fileName = origName;
-			
+
+			fileName = openFileInfo.FileName;
 			if (!origName.StartsWith ("http://", StringComparison.Ordinal))
 				fileName = fileName.FullPath;
 			
@@ -1000,7 +981,7 @@ namespace MonoDevelop.Ide.Gui
 			var dp = new DocumentUserPrefs ();
 			dp.FileName = FileService.AbsoluteToRelativePath (args.Item.BaseDirectory, document.FileName);
 			if (document.Editor != null) {
-				dp.Line = document.Editor.Caret.Column;
+				dp.Line = document.Editor.Caret.Line;
 				dp.Column = document.Editor.Caret.Column;
 			}
 			return dp;
@@ -1148,13 +1129,13 @@ namespace MonoDevelop.Ide.Gui
 
 		List<FileData> fileStatus;
 		object fileStatusLock = new object ();
-		
+		// http://msdn.microsoft.com/en-us/library/system.io.file.getlastwritetimeutc(v=vs.110).aspx
+		static DateTime NonExistentFile = new DateTime(1601, 1, 1);
 		internal void SaveFileStatus ()
 		{
-			fileStatus = new List<FileData> ();
-			
 //			DateTime t = DateTime.Now;
 			List<FilePath> files = new List<FilePath> (GetKnownFiles ());
+			fileStatus = new List<FileData> (files.Count);
 //			Console.WriteLine ("SaveFileStatus(0) " + (DateTime.Now - t).TotalMilliseconds + "ms " + files.Count);
 			
 			ThreadPool.QueueUserWorkItem (delegate {
@@ -1162,8 +1143,8 @@ namespace MonoDevelop.Ide.Gui
 				lock (fileStatusLock) {
 					foreach (FilePath file in files) {
 						try {
-							FileInfo fi = new FileInfo (file);
-							FileData fd = new FileData (file, fi.Exists ? fi.LastWriteTimeUtc : DateTime.MinValue);
+							DateTime ft = File.GetLastWriteTimeUtc (file);
+							FileData fd = new FileData (file, ft != NonExistentFile ? ft : DateTime.MinValue);
 							fileStatus.Add (fd);
 						} catch {
 							// Ignore
@@ -1184,12 +1165,12 @@ namespace MonoDevelop.Ide.Gui
 //					DateTime t = DateTime.Now;
 					if (fileStatus == null)
 						return;
-					List<FilePath> modified = new List<FilePath> ();
+					List<FilePath> modified = new List<FilePath> (fileStatus.Count);
 					foreach (FileData fd in fileStatus) {
 						try {
-							FileInfo fi = new FileInfo (fd.File);
-							if (fi.Exists) {
-								if (fi.LastWriteTimeUtc != fd.TimeUtc)
+							DateTime ft = File.GetLastWriteTimeUtc (fd.File);
+							if (ft != NonExistentFile) {
+								if (ft != fd.TimeUtc)
 									modified.Add (fd.File);
 							} else if (fd.TimeUtc != DateTime.MinValue) {
 								FileService.NotifyFileRemoved (fd.File);
@@ -1287,7 +1268,7 @@ namespace MonoDevelop.Ide.Gui
 				return fileName;
 			}
 			set {
-				fileName = ResolveSymbolicLink (value.CanonicalPath);
+				fileName = value.CanonicalPath.ResolveLinks ();
 				if (fileName.IsNullOrEmpty)
 					LoggingService.LogError ("FileName == null\n" + Environment.StackTrace);
 			}
@@ -1339,36 +1320,6 @@ namespace MonoDevelop.Ide.Gui
 				this.Options &= ~OpenDocumentOptions.BringToFront;
 			}
 		}
-
-		static FilePath ResolveSymbolicLink (FilePath fileName)
-		{
-			if (fileName.IsEmpty)
-				return fileName;
-			try {
-				var alreadyVisted = new HashSet<FilePath> ();
-				while (true) {
-					if (alreadyVisted.Contains (fileName)) {
-						LoggingService.LogError ("Cyclic links detected: " + fileName);
-						return FilePath.Empty;
-					}
-					alreadyVisted.Add (fileName);
-					var linkInfo = new Mono.Unix.UnixSymbolicLinkInfo (fileName);
-					if (linkInfo.IsSymbolicLink && linkInfo.HasContents) {
-						FilePath contentsPath = linkInfo.ContentsPath;
-						if (contentsPath.IsAbsolute) {
-							fileName = linkInfo.ContentsPath;
-						} else {
-							fileName = fileName.ParentDirectory.Combine (contentsPath);
-						}
-						fileName = fileName.CanonicalPath;
-						continue;
-					}
-					return ResolveSymbolicLink (fileName.ParentDirectory).Combine (fileName.FileName).CanonicalPath;
-				}
-			} catch (Exception) {
-				return fileName;
-			}
-		}
 	}
 	
 	class LoadFileWrapper
