@@ -413,8 +413,8 @@ namespace MonoDevelop.Ide.Projects.OptionPanels
 			warning.SetAlignment (0.0f, 0.5f);
 			warning.Show ();
 
-			infoImage = new Image (GetType ().Assembly, "warning-light-16.png");
-			warningImage = new Image (GetType ().Assembly, "error-light-16.png");
+			infoImage = new Image (GetType ().Assembly, "warning-16.png");
+			warningImage = new Image (GetType ().Assembly, "error-16.png");
 
 			warningHBox = new HBox (false, 6);
 			warningHBox.PackStart (infoImage, false, false, 0);
