@@ -59,6 +59,7 @@ namespace MonoDevelop.Ide.Tasks
 
 		MonoDevelop.Ide.Gui.Components.PadTreeView view;
 		ListStore store;
+		TreeModelSort sortModel;
 
 		Gdk.Color highPrioColor, normalPrioColor, lowPrioColor;
 
@@ -98,7 +99,9 @@ namespace MonoDevelop.Ide.Tasks
 				typeof (Gdk.Color),  // foreground color
 				typeof (int));       // font weight
 
-			view = new MonoDevelop.Ide.Gui.Components.PadTreeView (store);
+			sortModel = new TreeModelSort (store);
+
+			view = new MonoDevelop.Ide.Gui.Components.PadTreeView (sortModel);
 			view.RulesHint = true;
 			view.SearchColumn = (int)Columns.Description;
 			view.DoPopupMenu = (evt) => IdeApp.CommandService.ShowContextMenu (view, evt, CreateMenu ());
@@ -109,22 +112,16 @@ namespace MonoDevelop.Ide.Tasks
 			col.Clickable = false;
 
 			col = view.AppendColumn (GettextCatalog.GetString ("Description"), view.TextRenderer, "text", Columns.Description, "foreground-gdk", Columns.Foreground, "weight", Columns.Bold);
-			col.Clickable = true;
 			col.SortColumnId = (int)Columns.Description;
 			col.Resizable = true;
-			col.Clicked += Resort;
 
 			col = view.AppendColumn (GettextCatalog.GetString ("File"), view.TextRenderer, "text", Columns.File, "foreground-gdk", Columns.Foreground, "weight", Columns.Bold);
-			col.Clickable = true;
 			col.SortColumnId = (int)Columns.File;
 			col.Resizable = true;
-			col.Clicked += Resort;
 
 			col = view.AppendColumn (GettextCatalog.GetString ("Path"), view.TextRenderer, "text", Columns.Path, "foreground-gdk", Columns.Foreground, "weight", Columns.Bold);
-			col.Clickable = true;
 			col.SortColumnId = (int)Columns.Path;
 			col.Resizable = true;
-			col.Clicked += Resort;
 
 			LoadColumnsVisibility ();
 			
@@ -217,17 +214,20 @@ namespace MonoDevelop.Ide.Tasks
 				}
 			}
 		}
+
+		void HandleSolutionItemAdded (object sender, SolutionItemChangeEventArgs e)
+		{
+			var newProject = e.SolutionItem as Project;
+			if (newProject == null)
+				return;
+			UpdateCommentTagsForProject (e.Solution, newProject);
+		}
 		
 		void LoadSolutionContents (Solution sln)
 		{
 			loadedSlns.Add (sln);
 			System.Threading.ThreadPool.QueueUserWorkItem (delegate {
-				sln.SolutionItemAdded += delegate(object sender, SolutionItemChangeEventArgs e) {
-					var newProject = e.SolutionItem as Project;
-					if (newProject == null)
-						return;
-					UpdateCommentTagsForProject (sln, newProject);
-				};
+				sln.SolutionItemAdded += HandleSolutionItemAdded;
 
 				// Load all tags that are stored in pidb files
 				foreach (Project p in sln.GetAllProjects ()) {
@@ -247,8 +247,10 @@ namespace MonoDevelop.Ide.Tasks
 		
 		void OnWorkspaceItemUnloaded (object sender, WorkspaceItemEventArgs e)
 		{
-			foreach (var sln in e.Item.GetAllSolutions ())
-				loadedSlns.Remove (sln);
+			foreach (var sln in e.Item.GetAllSolutions ()) {
+				if (loadedSlns.Remove (sln))
+					sln.SolutionItemAdded -= HandleSolutionItemAdded;
+			}
 			comments.RemoveItemTasks (e.Item, true);
 		}		
 
@@ -469,9 +471,9 @@ namespace MonoDevelop.Ide.Tasks
 			Task task = SelectedTask;
 			if (task != null) {
 				clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
-				clipboard.Text = task.ToString ();
+				clipboard.Text = task.Description;
 				clipboard = Clipboard.Get (Gdk.Atom.Intern ("PRIMARY", false));
-				clipboard.Text = task.ToString ();
+				clipboard.Text = task.Description;
 			}
 		}
 
@@ -535,31 +537,6 @@ namespace MonoDevelop.Ide.Tasks
 			}
 		}
 
-		void Resort (object sender, EventArgs args)
-		{
-			TreeViewColumn col = (TreeViewColumn)sender;
-			foreach (TreeViewColumn c in view.Columns)
-			{
-				if (c != col) c.SortIndicator = false;
-			}
-			col.SortOrder = ReverseSortOrder (col);
-			col.SortIndicator = true;
-			store.SetSortColumnId (col.SortColumnId, col.SortOrder);
-		}
-		
-		static SortType ReverseSortOrder (TreeViewColumn col)
-		{
-			if (col.SortIndicator) {
-				if (col.SortOrder == SortType.Ascending)
-					return SortType.Descending;
-				else
-					return SortType.Ascending;
-			} else
-			{
-				return SortType.Ascending;
-			}
-		}
-		
 		Gdk.Color GetColorByPriority (TaskPriority prio)
 		{
 			switch (prio)
