@@ -27,14 +27,10 @@
 //
 
 using System;
-using System.IO;
-using System.Collections;
 using Gtk;
 
 using MonoDevelop.Core;
-using MonoDevelop.Projects;
 using MonoDevelop.Ide;
-using MonoDevelop.Ide.Gui;
 using MonoDevelop.Components;
 
 namespace MonoDevelop.Ide.Tasks
@@ -53,10 +49,12 @@ namespace MonoDevelop.Ide.Tasks
 		}
 		
 		Button newButton;
+		Button copyButton;
 		Button delButton;
 
 		MonoDevelop.Ide.Gui.Components.PadTreeView view;
 		ListStore store;
+		TreeModelSort sortModel;
 		CellRendererText cellRendDesc;
 		
 		Gdk.Color highPrioColor, normalPrioColor, lowPrioColor;
@@ -79,14 +77,13 @@ namespace MonoDevelop.Ide.Tasks
 				typeof (Task),	 // user task
 				typeof (Gdk.Color),  // foreground color
 				typeof (int));		 // font style
-			
-			view = new MonoDevelop.Ide.Gui.Components.PadTreeView (store);
+
+			sortModel = new TreeModelSort (store);
+
+			view = new MonoDevelop.Ide.Gui.Components.PadTreeView (sortModel);
 			view.RulesHint = true;
 			view.SearchColumn = (int)Columns.Description;
 			view.Selection.Changed += new EventHandler (SelectionChanged);
-			view.PopupMenu += new PopupMenuHandler (OnUserPopupMenu);
-			view.ButtonPressEvent += new ButtonPressEventHandler (OnUserButtonPressed);
-			
 			TreeViewColumn col;
 			
 			CellRendererComboBox cellRendPriority = new CellRendererComboBox ();
@@ -94,35 +91,39 @@ namespace MonoDevelop.Ide.Tasks
 			cellRendPriority.Editable = true;
 			cellRendPriority.Changed += new ComboSelectionChangedHandler (UserTaskPriorityEdited);
 			col = view.AppendColumn (GettextCatalog.GetString ("Priority"), cellRendPriority, "text", Columns.Priority, "foreground-gdk", Columns.Foreground, "weight", Columns.Bold);
-			col.Clickable = true;
 			col.Resizable = true;
-			TreeIterCompareFunc sortFunc = new TreeIterCompareFunc (PrioirtySortFunc);
-			store.SetSortFunc ((int)Columns.Priority, sortFunc);
-			col.Clicked += new EventHandler (UserTaskPriorityResort);
+			col.SortColumnId = (int)Columns.Priority;
 			
 			CellRendererToggle cellRendCompleted = new CellRendererToggle ();
 			cellRendCompleted.Toggled += new ToggledHandler (UserTaskCompletedToggled);
 			cellRendCompleted.Activatable = true;
 			col = view.AppendColumn (String.Empty, cellRendCompleted, "active", Columns.Completed);
-			col.Clickable = true;
-			col.Clicked += new EventHandler (UserTaskCompletedResort);
-			
+
 			cellRendDesc = view.TextRenderer;
 			cellRendDesc.Editable = true;
 			cellRendDesc.Edited += new EditedHandler (UserTaskDescEdited);
 			col = view.AppendColumn (GettextCatalog.GetString ("Description"), cellRendDesc, "text", Columns.Description, "strikethrough", Columns.Completed, "foreground-gdk", Columns.Foreground, "weight", Columns.Bold);
-			col.Clickable = true;
 			col.Resizable = true;
-			col.Clicked += new EventHandler (UserTaskDescResort);
+			col.SortColumnId = (int)Columns.Description;
 			
 			newButton = new Button ();
-			newButton.Image = new Gtk.Image (Gtk.Stock.New, IconSize.Button);
 			newButton.Label = GettextCatalog.GetString ("New Task");
-			newButton.ImagePosition = PositionType.Left;
+			newButton.Image = new Gtk.Image (Gtk.Stock.New, IconSize.Button);
+			newButton.Image.Show ();
 			newButton.Clicked += new EventHandler (NewUserTaskClicked); 
 			newButton.TooltipText = GettextCatalog.GetString ("Create New Task");
+
+			copyButton = new Button ();
+			copyButton.Label = GettextCatalog.GetString ("Copy Task");
+			copyButton.Image = new Gtk.Image (Gtk.Stock.Copy, IconSize.Button);
+			copyButton.Image.Show ();
+			copyButton.Clicked += CopyUserTaskClicked;
+			copyButton.TooltipText = GettextCatalog.GetString ("Copy Task Description");
 			
-			delButton = new Button (new Gtk.Image (Gtk.Stock.Delete, IconSize.Button));
+			delButton = new Button ();
+			delButton.Label = GettextCatalog.GetString ("Delete Task");
+			delButton.Image = new Gtk.Image (Gtk.Stock.Delete, IconSize.Button);
+			delButton.Image.Show ();
 			delButton.Clicked += new EventHandler (DeleteUserTaskClicked); 
 			delButton.TooltipText = GettextCatalog.GetString ("Delete Task");
 
@@ -158,9 +159,12 @@ namespace MonoDevelop.Ide.Tasks
 		{
 			if (updating)
 				return;
+
+			if (view.IsRealized)
+				view.ScrollToPoint (0, 0);
+
 			store.Clear ();
-			foreach (Task task in TaskService.UserTasks)
-			{
+			foreach (Task task in TaskService.UserTasks) {
 				store.AppendValues (GettextCatalog.GetString (Enum.GetName (typeof (TaskPriority), task.Priority)), task.Completed, task.Description, task, GetColorByPriority (task.Priority), task.Completed ? (int)Pango.Weight.Light : (int)Pango.Weight.Bold);
 			}
 			ValidateButtons ();
@@ -206,7 +210,7 @@ namespace MonoDevelop.Ide.Tasks
 		void ValidateButtons ()
 		{
 			newButton.Sensitive = solutionLoaded;
-			delButton.Sensitive = solutionLoaded && view.Selection.CountSelectedRows () > 0;
+			delButton.Sensitive = copyButton.Sensitive = solutionLoaded && view.Selection.CountSelectedRows () > 0;
 		}
 		
 		void NewUserTaskClicked (object obj, EventArgs e)
@@ -223,6 +227,25 @@ namespace MonoDevelop.Ide.Tasks
 			view.SetCursorOnCell (path, view.Columns[(int)Columns.Description], cellRendDesc, true);
 			TaskService.SaveUserTasks (task.WorkspaceObject);
 		}
+
+		void CopyUserTaskClicked (object o, EventArgs args)
+		{
+			Task task;
+			TreeModel model;
+			TreeIter iter;
+
+			if (view.Selection.GetSelected (out model, out iter))
+			{
+				task = (Task) model.GetValue (iter, (int)Columns.UserTask);
+			}
+			else return; // no one selected
+
+			clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
+			clipboard.Text = task.Description;
+			clipboard = Clipboard.Get (Gdk.Atom.Intern ("PRIMARY", false));
+			clipboard.Text = task.Description;
+		}
+
 		
 		void DeleteUserTaskClicked (object obj, EventArgs e)
 		{
@@ -262,28 +285,6 @@ namespace MonoDevelop.Ide.Tasks
 			}
 		}
 		
-		int PrioirtySortFunc (TreeModel model, TreeIter iter1, TreeIter iter2)
-		{
-			TaskPriority prio1 = (TaskPriority) Enum.Parse (typeof (TaskPriority), (string)model.GetValue (iter1, (int)Columns.Priority));
-			TaskPriority prio2 = (TaskPriority) Enum.Parse (typeof (TaskPriority), (string)model.GetValue (iter2, (int)Columns.Priority));
-
-			if (prio1 == prio2)
-				return 0;
-			return prio1 < prio2 ? 1 : -1;
-		}
-		
-		void UserTaskPriorityResort (object sender, EventArgs args)
-		{
-			TreeViewColumn col = view.Columns[(int)Columns.Priority];
-			foreach (TreeViewColumn c in view.Columns)
-			{
-				if (c != col) c.SortIndicator = false;
-			}
-			col.SortOrder = ReverseSortOrder (col);
-			col.SortIndicator = true;
-			store.SetSortColumnId ((int)Columns.Priority, col.SortOrder);
-		}
-		
 		void UserTaskCompletedToggled (object o, ToggledArgs args)
 		{
 			Gtk.TreeIter iter;
@@ -297,18 +298,6 @@ namespace MonoDevelop.Ide.Tasks
 			}
 		}
 		
-		void UserTaskCompletedResort (object sender, EventArgs args)
-		{
-			TreeViewColumn col = view.Columns[(int)Columns.Completed];
-			foreach (TreeViewColumn c in view.Columns)
-			{
-				if (c != col) c.SortIndicator = false;
-			}
-			col.SortOrder = ReverseSortOrder (col);
-			col.SortIndicator = true;
-			store.SetSortColumnId ((int)Columns.Completed, col.SortOrder);
-		}
-		
 		void UserTaskDescEdited (object o, EditedArgs args)
 		{
 			Gtk.TreeIter iter;
@@ -320,18 +309,6 @@ namespace MonoDevelop.Ide.Tasks
 			}
 		}
 		
-		void UserTaskDescResort (object sender, EventArgs args)
-		{
-			TreeViewColumn col = view.Columns[(int)Columns.Description];
-			foreach (TreeViewColumn c in view.Columns)
-			{
-				if (c != col) c.SortIndicator = false;
-			}
-			col.SortOrder = ReverseSortOrder (col);
-			col.SortIndicator = true;
-			store.SetSortColumnId ((int)Columns.Description, col.SortOrder);
-		}
-		
 		Gdk.Color GetColorByPriority (TaskPriority prio)
 		{
 			switch (prio)
@@ -344,48 +321,7 @@ namespace MonoDevelop.Ide.Tasks
 					return lowPrioColor;
 			}
 		}
-		
-		[GLib.ConnectBefore]
-		void OnUserButtonPressed (object o, ButtonPressEventArgs args)
-		{
-			if (args.Event.Button == 3)
-				ShowUserPopup ();
-		}
-		
-		void OnUserPopupMenu (object o, PopupMenuArgs args)
-		{
-			ShowUserPopup ();
-		}
 
-		void ShowUserPopup ()
-		{
-			Menu menu = new Menu ();
-			menu.AccelGroup = new AccelGroup ();
-			ImageMenuItem copy = new ImageMenuItem (Gtk.Stock.Copy, menu.AccelGroup);
-			copy.Activated += new EventHandler (OnUserTaskCopied);
-			menu.Append (copy);
-			menu.Popup (null, null, null, 3, Gtk.Global.CurrentEventTime);
-			menu.ShowAll ();
-		}
-
-		void OnUserTaskCopied (object o, EventArgs args)
-		{
-			Task task;
-			TreeModel model;
-			TreeIter iter;
-
-			if (view.Selection.GetSelected (out model, out iter))
-			{
-				task = (Task) model.GetValue (iter, (int)Columns.UserTask);
-			}
-			else return; // no one selected
-
-			clipboard = Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", false));
-			clipboard.Text = task.ToString ();
-			clipboard = Clipboard.Get (Gdk.Atom.Intern ("PRIMARY", false));
-			clipboard.Text = task.ToString ();
-		}
-		
 		static SortType ReverseSortOrder (TreeViewColumn col)
 		{
 			if (col.SortIndicator) {
@@ -420,7 +356,7 @@ namespace MonoDevelop.Ide.Tasks
 		
 		#region ITaskListView members
 		TreeView ITaskListView.Content { get { return view; } }
-		Widget[] ITaskListView.ToolBarItems { get { return new Widget[] { newButton, delButton }; } }
+		Widget[] ITaskListView.ToolBarItems { get { return new Widget[] { newButton, delButton, copyButton }; } }
 		#endregion
 	}
 }
