@@ -117,7 +117,12 @@ namespace MonoDevelop.Ide.Templates
 				}
             }
 
-            workspaceItem.Name = StringParserService.Parse (name, new string[,] { {"ProjectName", projectCreateInformation.SolutionName} });
+			var substitution = new string[,] { { "ProjectName", projectCreateInformation.SolutionName } };
+            workspaceItem.Name = StringParserService.Parse (name, substitution);
+			string newStartupProjectName = startupProject;
+			if (newStartupProjectName != null) {
+				newStartupProjectName = StringParserService.Parse (startupProject, substitution);
+			}
 
             workspaceItem.SetLocation (projectCreateInformation.SolutionPath, workspaceItem.Name);
 
@@ -144,9 +149,10 @@ namespace MonoDevelop.Ide.Templates
                 for ( int i = 0; i < entryDescriptors.Count; i++ ) {
                     ProjectCreateInformation entryProjectCI;
                     var entry = entryDescriptors[i] as ICustomProjectCIEntry;
-                    if (entry != null)
-	                    entryProjectCI = entry.CreateProjectCI (localProjectCI);
-                    else
+					if (entry != null) {
+						entryProjectCI = entry.CreateProjectCI (localProjectCI);
+						entryProjectCI = new ProjectTemplateCreateInformation (entryProjectCI, localProjectCI.ProjectName);
+					} else
 	                    entryProjectCI = localProjectCI;
 
 					var solutionItemDesc = entryDescriptors[i];
@@ -171,10 +177,10 @@ namespace MonoDevelop.Ide.Templates
                     }
 
 					if ((info is Project) && (solutionItemDesc is ProjectDescriptor)) {
-						workspaceItemCreatedInfo.AddPackageReferenceForCreatedProject ((Project)info, (ProjectDescriptor)solutionItemDesc);
+						workspaceItemCreatedInfo.AddPackageReferenceForCreatedProject ((Project)info, (ProjectDescriptor)solutionItemDesc, projectCreateInformation);
 					}
                     solution.RootFolder.Items.Add (info);
-					if (startupProject == info.Name)
+					if (newStartupProjectName == info.Name)
 						solution.StartupItem = info;
                 }
             }
