@@ -38,6 +38,7 @@ using ICSharpCode.NRefactory;
 using MonoDevelop.Ide.TypeSystem;
 using MonoDevelop.Ide;
 using ICSharpCode.NRefactory.CSharp.TypeSystem;
+using System.Threading.Tasks;
 
 namespace MonoDevelop.Ide.TypeSystem
 {
@@ -66,8 +67,10 @@ namespace MonoDevelop.Ide.TypeSystem
 			if (newMember is CodeMemberMethod)
 				options.BracingStyle = "C";
 			generator.GenerateCodeFromMember (newMember, sw, options);
-			
-			suitableInsertionPoint.Insert (data, sw.ToString ());
+
+			var code = sw.ToString ();
+			if (!string.IsNullOrEmpty (code))
+				suitableInsertionPoint.Insert (data, code);
 			if (!isOpen) {
 				try {
 					File.WriteAllText (type.Region.FileName, data.Text);
@@ -104,7 +107,95 @@ namespace MonoDevelop.Ide.TypeSystem
 				}
 			}
 		}
+
+		public static Task<bool> InsertMemberWithCursor (
+			string operation, ITypeDefinition parentType, IUnresolvedTypeDefinition part,
+			IUnresolvedMember newMember, bool implementExplicit = false)
+		{
+			var tcs = new TaskCompletionSource<bool>();
+			if (parentType == null)
+				return tcs.Task;
+			part = part ?? parentType.Parts.FirstOrDefault ();
+			if (part == null)
+				return tcs.Task;
+			var loadedDocument = IdeApp.Workbench.OpenDocument (part.Region.FileName);
+			loadedDocument.RunWhenLoaded (delegate {
+				var editor = loadedDocument.Editor;
+				var loc = part.Region.Begin;
+				var parsedDocument = loadedDocument.UpdateParseDocument ();
+				var declaringType = parsedDocument.GetInnermostTypeDefinition (loc);
+				var mode = new InsertionCursorEditMode (
+					editor.Parent,
+					CodeGenerationService.GetInsertionPoints (loadedDocument, declaringType));
+				if (mode.InsertionPoints.Count == 0) {
+					MessageService.ShowError (
+						GettextCatalog.GetString ("No valid insertion point can be found in type '{0}'.", declaringType.Name)
+						);
+					return;
+				}
+				var suitableInsertionPoint = GetSuitableInsertionPoint (mode.InsertionPoints, part, newMember);
+				if (suitableInsertionPoint != null)
+					mode.CurIndex = mode.InsertionPoints.IndexOf (suitableInsertionPoint);
+				else
+					mode.CurIndex = 0;
+
+				var helpWindow = new Mono.TextEditor.PopupWindow.InsertionCursorLayoutModeHelpWindow () {
+					TitleText = operation
+				};
+				mode.HelpWindow = helpWindow;
+
+				mode.StartMode ();
+				mode.Exited += delegate(object s, InsertionCursorEventArgs iCArgs) {
+					if (!iCArgs.Success) {
+						tcs.SetResult (false);
+						return;
+					}
+					var generator = CreateCodeGenerator (editor, parentType.Compilation);
+					generator.IndentLevel = CalculateBodyIndentLevel (declaringType);
+					var generatedCode = generator.CreateMemberImplementation (parentType, part, newMember, implementExplicit);
+					mode.InsertionPoints[mode.CurIndex].Insert (editor, generatedCode.Code);
+					tcs.SetResult (true);
+				};
+			});
+
+			return tcs.Task;
+		}
 		
+		public static Task<bool> InsertMember (
+			ITypeDefinition parentType, IUnresolvedTypeDefinition part,
+			IUnresolvedMember newMember, bool implementExplicit = false)
+		{
+			var tcs = new TaskCompletionSource<bool>();
+			if (parentType == null)
+				return tcs.Task;
+			part = part ?? parentType.Parts.FirstOrDefault ();
+			if (part == null)
+				return tcs.Task;
+
+			var loadedDocument = IdeApp.Workbench.OpenDocument (part.Region.FileName);
+			loadedDocument.RunWhenLoaded (delegate {
+				var editor = loadedDocument.Editor;
+				var loc = part.Region.Begin;
+				var parsedDocument = loadedDocument.UpdateParseDocument ();
+				var declaringType = parsedDocument.GetInnermostTypeDefinition (loc);
+				var insertionPoints = CodeGenerationService.GetInsertionPoints (loadedDocument, declaringType);
+				if (insertionPoints.Count == 0) {
+					MessageService.ShowError (
+						GettextCatalog.GetString ("No valid insertion point can be found in type '{0}'.", declaringType.Name)
+						);
+					return;
+				}
+				var suitableInsertionPoint = GetSuitableInsertionPoint (insertionPoints, part, newMember) ?? insertionPoints.First ();
+
+				var generator = CreateCodeGenerator (editor, parentType.Compilation);
+				generator.IndentLevel = CalculateBodyIndentLevel (declaringType);
+				var generatedCode = generator.CreateMemberImplementation (parentType, part, newMember, implementExplicit);
+				suitableInsertionPoint.Insert (editor, generatedCode.Code);
+			});
+
+			return tcs.Task;
+		}
+
 		public static int CalculateBodyIndentLevel (IUnresolvedTypeDefinition declaringType)
 		{
 			if (declaringType == null)
@@ -114,7 +205,7 @@ namespace MonoDevelop.Ide.TypeSystem
 				indentLevel++;
 				declaringType = declaringType.DeclaringTypeDefinition;
 			}
-			var file = declaringType.ParsedFile as CSharpParsedFile;
+			var file = declaringType.UnresolvedFile as CSharpUnresolvedFile;
 			if (file == null)
 				return indentLevel;
 			var scope = file.GetUsingScope (declaringType.Region.Begin);
@@ -247,7 +338,9 @@ namespace MonoDevelop.Ide.TypeSystem
 				int col = type.BodyRegion.EndColumn - 1;
 				var line = data.GetLine (type.BodyRegion.EndLine);
 				if (line != null) {
-					while (col > 1 && char.IsWhiteSpace (data.GetCharAt (line.Offset + col - 2)))
+					var lineOffset = line.Offset;
+					col = Math.Min (line.Length, col);
+					while (lineOffset + col - 2 >= 0 && col > 1 && char.IsWhiteSpace (data.GetCharAt (lineOffset + col - 2)))
 						col--;
 				}
 				result.Add (new InsertionPoint (new DocumentLocation (type.BodyRegion.EndLine, col), insertLine, NewLineInsertion.Eol));
@@ -260,6 +353,8 @@ namespace MonoDevelop.Ide.TypeSystem
 				result.Add (new InsertionPoint (new DocumentLocation (region.Region.EndLine + 1, 1), NewLineInsertion.Eol, NewLineInsertion.Eol));
 			}
 			result.Sort ((left, right) => left.Location.CompareTo (right.Location));
+//			foreach (var res in result)
+//				Console.WriteLine (res);
 			return result;
 		}
 
@@ -311,34 +406,45 @@ namespace MonoDevelop.Ide.TypeSystem
 			if (nextLine == null) // check for 1 line case.
 				return new InsertionPoint (new DocumentLocation (line, column + 1), NewLineInsertion.BlankLine, NewLineInsertion.BlankLine);
 			
-			for (int i = nextLine.Offset; i < nextLine.Offset + nextLine.Length; i++) {
+			for (int i = nextLine.Offset; i < nextLine.EndOffset; i++) {
 				char ch = doc.GetCharAt (i);
 				if (!char.IsWhiteSpace (ch)) {
 					// case2: next line contains non ws chars.
 					return new InsertionPoint (new DocumentLocation (line + 1, 1), NewLineInsertion.Eol, NewLineInsertion.BlankLine);
 				}
 			}
-			// case3: whitespace line
+
+			DocumentLine nextLine2 = doc.GetLine (line + 2);
+			if (nextLine2 != null) {
+				for (int i = nextLine2.Offset; i < nextLine2.EndOffset; i++) {
+					char ch = doc.GetCharAt (i);
+					if (!char.IsWhiteSpace (ch)) {
+						// case3: one blank line
+						return new InsertionPoint (new DocumentLocation (line + 1, 1), NewLineInsertion.Eol, NewLineInsertion.Eol);
+					}
+				}
+			}
+			// case4: more than 1 blank line
 			return new InsertionPoint (new DocumentLocation (line + 1, 1), NewLineInsertion.Eol, NewLineInsertion.None);
 		}
 		
 		static InsertionPoint GetSuitableInsertionPoint (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls, IUnresolvedMember member)
 		{
 			var mainPart = cls;
-			switch (member.EntityType) {
-			case EntityType.Field:
+			switch (member.SymbolKind) {
+			case SymbolKind.Field:
 				return GetNewFieldPosition (points, mainPart);
-			case EntityType.Method:
-			case EntityType.Constructor:
-			case EntityType.Destructor:
-			case EntityType.Operator:
+			case SymbolKind.Method:
+			case SymbolKind.Constructor:
+			case SymbolKind.Destructor:
+			case SymbolKind.Operator:
 				return GetNewMethodPosition (points, mainPart);
-			case EntityType.Event:
+			case SymbolKind.Event:
 				return GetNewEventPosition (points, mainPart);
-			case EntityType.Property:
+			case SymbolKind.Property:
 				return GetNewPropertyPosition (points, mainPart);
 			}
-			throw new InvalidOperationException ("Invalid member type: " + member.EntityType);
+			throw new InvalidOperationException ("Invalid member type: " + member.SymbolKind);
 		}
 		
 		static InsertionPoint GetSuitableInsertionPoint (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls, CodeTypeMember mem)
@@ -357,38 +463,34 @@ namespace MonoDevelop.Ide.TypeSystem
 		
 		static InsertionPoint GetNewFieldPosition (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls)
 		{
-			throw new NotImplementedException ();
-//			if (cls.GetDefinition ().Fields.Count  == 0) 
-//				return points.FirstOrDefault ();
-//			var lastField = cls.Fields.Last ();
-//			return points.FirstOrDefault (p => p.Location.Convert () > lastField.Location);
+			if (!cls.Fields.Any ()) 
+				return points.FirstOrDefault ();
+			var lastField = cls.Fields.Last ();
+			return points.FirstOrDefault (p => p.Location.Convert () > lastField.Region.Begin);
 		}
 		
 		static InsertionPoint GetNewMethodPosition (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls)
 		{
-			throw new NotImplementedException ();
-//			if (cls.MethodCount + cls.ConstructorCount == 0) 
-//				return GetNewPropertyPosition (points, cls);
-//			var lastMember = cls.Members.Last ();
-//			return points.FirstOrDefault (p => p.Location.Convert () > lastMember.Location);
+			if (!cls.Methods.Any ()) 
+				return GetNewPropertyPosition (points, cls);
+			var lastMember = cls.Members.Last ();
+			return points.FirstOrDefault (p => p.Location.Convert () > lastMember.Region.Begin);
 		}
 		
 		static InsertionPoint GetNewPropertyPosition (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls)
 		{
-			throw new NotImplementedException ();
-//			if (cls.PropertyCount == 0)
-//			IProperty lastProperty = cls.Properties.Last ();
-//				return GetNewFieldPosition (points, cls);
-//			return points.FirstOrDefault (p => p.Location.Convert () > lastProperty.Location);
+			if (!cls.Properties.Any ())
+				return GetNewFieldPosition (points, cls);
+			var lastProperty = cls.Properties.Last ();
+			return points.FirstOrDefault (p => p.Location.Convert () > lastProperty.Region.Begin);
 		}
 		
 		static InsertionPoint GetNewEventPosition (IEnumerable<InsertionPoint> points, IUnresolvedTypeDefinition cls)
 		{
-			throw new NotImplementedException ();
-//			if (cls.EventCount == 0)
-//				return GetNewMethodPosition (points, cls);
-//			IEvent lastEvent = cls.Events.Last ();
-//			return points.FirstOrDefault (p => p.Location.Convert () > lastEvent.Location);
+			if (!cls.Events.Any ())
+				return GetNewMethodPosition (points, cls);
+			var lastEvent = cls.Events.Last ();
+			return points.FirstOrDefault (p => p.Location.Convert () > lastEvent.Region.Begin);
 		}
 		#endregion
 		
@@ -405,8 +507,12 @@ namespace MonoDevelop.Ide.TypeSystem
 			
 			var type = new CodeTypeDeclaration ("temp");
 			type.CustomAttributes.Add (attr);
-			
-			var provider = ((DotNetProject)cls.GetSourceProject ()).LanguageBinding.GetCodeDomProvider ();
+			Project project;
+			if (!cls.TryGetSourceProject (out project)) {
+				LoggingService.LogError ("Error can't get source project for:" + cls.FullName); 
+			}
+
+			var provider = ((DotNetProject)project).LanguageBinding.GetCodeDomProvider ();
 			var sw = new StringWriter ();
 			provider.GenerateCodeFromType (type, sw, new CodeGeneratorOptions ());
 			string code = sw.ToString ();
