@@ -48,7 +48,7 @@ using System.Text;
 using ICSharpCode.NRefactory.Completion;
 using System.Diagnostics;
 using MonoDevelop.Projects.SharedAssetsProjects;
-using Mono.CSharp.Nullable;
+using MonoDevelop.Ide.Templates;
 
 namespace MonoDevelop.Ide.TypeSystem
 {
@@ -197,13 +197,13 @@ namespace MonoDevelop.Ide.TypeSystem
 			});
 
 			AddinManager.AddExtensionNodeHandler ("/MonoDevelop/TypeSystem/OutputTracking", delegate (object sender, ExtensionNodeEventArgs args) {
-				var projectType = ((TypeSystemOutputTrackingNode)args.ExtensionNode).ProjectType;
+				var node = (TypeSystemOutputTrackingNode)args.ExtensionNode;
 				switch (args.Change) {
 				case ExtensionChange.Add:
-					outputTrackedProjects.Add (projectType);
+					outputTrackedProjects.Add (node);
 					break;
 				case ExtensionChange.Remove:
-					outputTrackedProjects.Remove (projectType);
+					outputTrackedProjects.Remove (node);
 					break;
 				}
 			});
@@ -260,13 +260,23 @@ namespace MonoDevelop.Ide.TypeSystem
 			}
 		}
 
-		static readonly List<string> outputTrackedProjects = new List<string> ();
+		static readonly List<TypeSystemOutputTrackingNode> outputTrackedProjects = new List<TypeSystemOutputTrackingNode> ();
+
+		static bool IsOutputTracked (DotNetProject project)
+		{
+			foreach (var projectType in project.GetProjectTypes ()) {
+				if (outputTrackedProjects.Any (otp => otp.ProjectType != null && string.Equals (otp.ProjectType, projectType, StringComparison.OrdinalIgnoreCase))) {
+					return true;
+				}
+			}
+			return outputTrackedProjects.Any (otp => otp.LanguageName != null && string.Equals (otp.LanguageName, project.LanguageName, StringComparison.OrdinalIgnoreCase));
+		}
 
 		static void CheckProjectOutput (DotNetProject project, bool autoUpdate)
 		{
 			if (project == null)
 				throw new ArgumentNullException ("project");
-			if (project.GetProjectTypes ().Any (p => outputTrackedProjects.Contains (p, StringComparer.OrdinalIgnoreCase))) {
+			if (IsOutputTracked (project)) {
 				var fileName = project.GetOutputFileName (IdeApp.Workspace.ActiveConfiguration);
 
 				var wrapper = GetProjectContentWrapper (project);
@@ -393,9 +403,11 @@ namespace MonoDevelop.Ide.TypeSystem
 						var file = pcnt._content.GetFile (fileName);
 						if (file != null) {
 							var newResult = parser.Parse (false, fileName, new StringReader (content), pcnt.Project);
-							pcnt.UpdateContent (c => c.AddOrUpdateFiles (newResult.ParsedFile));
-							pcnt.InformFileRemoved (new ParsedFileEventArgs (file));
-							pcnt.InformFileAdded (new ParsedFileEventArgs (newResult.ParsedFile));
+							if ((newResult.Flags & ParsedDocumentFlags.NonSerializable) != ParsedDocumentFlags.NonSerializable) {
+								pcnt.UpdateContent (c => c.AddOrUpdateFiles (newResult.ParsedFile));
+								pcnt.InformFileRemoved (new ParsedFileEventArgs (file));
+								pcnt.InformFileAdded (new ParsedFileEventArgs (newResult.ParsedFile));
+							}
 						}
 					}
 				}
@@ -1033,11 +1045,14 @@ namespace MonoDevelop.Ide.TypeSystem
 			public void RunWhenLoaded (Action<IProjectContent> act)
 			{
 				lock (updateContentLock) {
-					if (_content is LazyProjectLoader) {
-						if (loadActions != null) {
-							loadActions.Add (act);
+					var lazyProjectLoader = _content as LazyProjectLoader;
+					if (loadActions != null) {
+						lock (loadActions) {
+							if (lazyProjectLoader != null && !lazyProjectLoader.ContextTask.IsCompleted) {
+								loadActions.Add (act);
+								return;
+							}
 						}
-						return;
 					}
 				}
 				act (Content);
@@ -1069,22 +1084,29 @@ namespace MonoDevelop.Ide.TypeSystem
 			{
 				if (loadActions == null)
 					return;
-				var actions = loadActions.ToArray ();
-				loadActions = null;
+				Action<IProjectContent>[] actions;
+				lock (loadActions) {
+					actions = loadActions.ToArray ();
+					loadActions = null;
+				}
 				foreach (var action in actions)
 					action (Content);
 			}
 
 			public void UpdateContent (Func<IProjectContent, IProjectContent> updateFunc)
 			{
+				LazyProjectLoader lazyProjectLoader;
 				lock (updateContentLock) {
-					var lazyProjectLoader = Content as LazyProjectLoader;
+					lazyProjectLoader = _content as LazyProjectLoader;
 					if (lazyProjectLoader != null) {
 						lazyProjectLoader.ContextTask.Wait ();
 					}
-					Content = updateFunc (Content);
+					_content = updateFunc (_content);
 					ClearCachedCompilations ();
 					WasChanged = true;
+					if (lazyProjectLoader != null && !(_content is LazyProjectLoader)) {
+						RunLoadActions ();
+					}
 				}
 			}
 
@@ -1107,10 +1129,11 @@ namespace MonoDevelop.Ide.TypeSystem
 			public bool WasChanged;
 			[NonSerialized]
 			ICompilation compilation;
+			object compilationContentLock = new object ();
 
 			public ICompilation Compilation {
 				get {
-					lock (updateContentLock) {
+					lock (compilationContentLock) {
 						if (compilation == null) {
 							compilation = Content.CreateCompilation ();
 						}
@@ -1224,7 +1247,6 @@ namespace MonoDevelop.Ide.TypeSystem
 					throw new ArgumentNullException ("project");
 				this.Project = project;
 				var lazyProjectLoader = new LazyProjectLoader (this);
-				lazyProjectLoader.ContextTask.ContinueWith (delegate { RunLoadActions (); }, TaskContinuationOptions.OnlyOnRanToCompletion);
 				this.Content = lazyProjectLoader;
 			}
 
@@ -1244,18 +1266,29 @@ namespace MonoDevelop.Ide.TypeSystem
 						return contextTask;
 					}
 				}
-
+				object contentLock = new object ();
+				IProjectContent contentWithReferences;
 				public IProjectContent Content {
 					get {
-						if (References != null)
-							return contextTask.Result.AddAssemblyReferences (References); 
-						return contextTask.Result;
+						lock (contentLock) {
+							if (References != null) {
+								return contentWithReferences ?? (contentWithReferences = contextTask.Result.AddAssemblyReferences (References)); 
+							}
+							return contextTask.Result;
+						}
 					}
 				}
 
+				List<IAssemblyReference> references;
 				public List<IAssemblyReference> References {
-					get;
-					set;
+					get {
+						return references;
+					}
+					set {
+						lock (contentLock) {
+							references = value;
+						}
+					}
 				}
 
 				public LazyProjectLoader (ProjectContentWrapper wrapper)
@@ -1514,11 +1547,9 @@ namespace MonoDevelop.Ide.TypeSystem
 				return newReferencedAssemblies;
 			}
 
-			object assemblyReconnectLock = new object();
-
 			public void EnsureReferencesAreLoaded ()
 			{
-				lock (assemblyReconnectLock) {
+				lock (projectContentLock) {
 					if (referencesConnected)
 						return;
 					compilation = null;
@@ -1590,7 +1621,7 @@ namespace MonoDevelop.Ide.TypeSystem
 					}
 				}
 			}
-			object reconnectLock = new object();
+			static readonly object reconnectLock = new object();
 			public void ReconnectAssemblyReferences ()
 			{
 				var netProject = Project as DotNetProject;
@@ -1614,7 +1645,13 @@ namespace MonoDevelop.Ide.TypeSystem
 				foreach (var asm in referencedAssemblies) {
 					asm.Loaded -= HandleReferencedProjectInLoadChange;
 				}
+				foreach (var wrapper in referencedWrappers) {
+					wrapper.Loaded -= HandleReferencedProjectInLoadChange;
+				}
 				loadActions = null;
+				foreach (var wrapper in referencedWrappers) {
+					wrapper.Loaded -= HandleReferencedProjectInLoadChange;
+				}
 				referencedWrappers.Clear ();
 				referencedAssemblies.Clear ();
 				Loaded = null;
@@ -2022,6 +2059,11 @@ namespace MonoDevelop.Ide.TypeSystem
 			}
 
 			#endregion
+
+			public override string ToString ()
+			{
+				return string.Format ("[UnresolvedAssemblyProxy: FileName={0}]", FileName);
+			}
 		}
 
 		internal class LazyAssemblyLoader : IUnresolvedAssembly
@@ -2644,7 +2686,7 @@ namespace MonoDevelop.Ide.TypeSystem
 							return;
 						parsedFiles.Add (Tuple.Create (parsedDocument, Context._content.GetFile (fileName))); 
 					}
-					Context.UpdateContent (c => c.AddOrUpdateFiles (parsedFiles.Select (p => p.Item1.ParsedFile)));
+					Context.UpdateContent (c => c.AddOrUpdateFiles (parsedFiles.Where (f => (f.Item1.Flags & ParsedDocumentFlags.NonSerializable) != ParsedDocumentFlags.NonSerializable).Select (p => p.Item1.ParsedFile)));
 					foreach (var file in parsedFiles) {
 						if (token.IsCancellationRequested)
 							return;
@@ -2803,8 +2845,10 @@ namespace MonoDevelop.Ide.TypeSystem
 			if (token.IsCancellationRequested) {
 				return;
 			}
+//			Console.WriteLine ("add modified file check for :" + project.Name);
 			content.RunWhenLoaded (delegate(IProjectContent cnt) {
 				try {
+//					Console.WriteLine ("check for " + project.Name);
 					content.BeginLoadOperation ();
 					var modifiedFiles = new List<ProjectFile> ();
 					var oldFileNewFile = new List<Tuple<ProjectFile, IUnresolvedFile>> ();
