@@ -60,6 +60,7 @@ namespace MonoDevelop.Ide
 				platformService = new DefaultPlatformService ();
 				LoggingService.LogFatalError ("A platform service implementation has not been found.");
 			}
+			PlatformService.Initialize ();
 			if (PlatformService.CanOpenTerminal)
 				Runtime.ProcessService.SetExternalConsoleHandler (PlatformService.StartConsoleProcess);
 			
@@ -87,6 +88,21 @@ namespace MonoDevelop.Ide
 			}
 		}
 
+		public static void SetGlobalProgress (double progress)
+		{
+			platformService.SetGlobalProgressBar (progress);
+		}
+
+		public static void ShowGlobalProgressIndeterminate ()
+		{
+			platformService.ShowGlobalProgressBarIndeterminate ();
+		}
+
+		public static void ShowGlobalProgressError ()
+		{
+			platformService.ShowGlobalProgressBarError ();
+		}
+
 		public static IEnumerable<DesktopApplication> GetApplications (string filename)
 		{
 			return PlatformService.GetApplications (filename);
@@ -118,9 +134,9 @@ namespace MonoDevelop.Ide
 			PlatformService.OpenFile (filename);
 		}
 
-		public static void OpenFolder (FilePath folderPath)
+		public static void OpenFolder (FilePath folderPath, params FilePath[] selectFiles)
 		{
-			PlatformService.OpenFolder (folderPath);
+			PlatformService.OpenFolder (folderPath, selectFiles);
 		}
 
 		public static string GetMimeTypeForUri (string uri)
@@ -196,13 +212,7 @@ namespace MonoDevelop.Ide
 			return PlatformService.GetIconForType (mimeType).WithSize (size);
 		}
 
-		public static bool ShowContextMenu (MonoDevelop.Components.Commands.CommandManager commandManager,
-			Gtk.Widget widget, double x, double y, MonoDevelop.Components.Commands.CommandEntrySet entrySet, object initialCommandTarget = null)
-		{
-			return PlatformService.ShowContextMenu (commandManager, widget, x, y, entrySet, initialCommandTarget);
-		}
-
-		public static bool SetGlobalMenu (MonoDevelop.Components.Commands.CommandManager commandManager,
+		internal static bool SetGlobalMenu (MonoDevelop.Components.Commands.CommandManager commandManager,
 			string commandMenuAddinPath, string appMenuAddinPath)
 		{
 			return PlatformService.SetGlobalMenu (commandManager, commandMenuAddinPath, appMenuAddinPath);
@@ -303,9 +313,15 @@ namespace MonoDevelop.Ide
 			PlatformService.SetMainWindowDecorations (window);
 		}
 
-		internal static MainToolbar CreateMainToolbar (Gtk.Window window)
+		internal static MainToolbarController CreateMainToolbar (Gtk.Window window)
 		{
-			return PlatformService.CreateMainToolbar (window);
+			return new MainToolbarController (PlatformService.CreateMainToolbar (window));
+		}
+
+		internal static void AttachMainToolbar (Gtk.VBox parent, MainToolbarController toolbar)
+		{
+			PlatformService.AttachMainToolbar (parent, toolbar.ToolbarView);
+			toolbar.Initialize ();
 		}
 
 		public static bool GetIsFullscreen (Gtk.Window window)
@@ -322,5 +338,20 @@ namespace MonoDevelop.Ide
 		{
 			return PlatformService.IsModalDialogRunning ();
 		}
+
+		internal static void AddChildWindow (Gtk.Window parent, Gtk.Window child)
+		{
+			PlatformService.AddChildWindow (parent, child);
+		}
+
+		internal static void RemoveChildWindow (Gtk.Window parent, Gtk.Window child)
+		{
+			PlatformService.RemoveChildWindow (parent, child);
+		}
+
+		internal static void PlaceWindow (Gtk.Window window, int x, int y, int width, int height)
+		{
+			PlatformService.PlaceWindow (window, x, y, width, height);
+		}
 	}
 }
