@@ -35,8 +35,36 @@ using MonoDevelop.Core;
 using MonoDevelop.Components.Extensions;
 using MonoDevelop.Ide.Gui;
 
+#if MAC
+using AppKit;
+using MonoDevelop.Components.Mac;
+#endif
+
 namespace MonoDevelop.Ide
 {
+	public class AlertButtonEventArgs : EventArgs
+	{
+		public AlertButton Button {
+			get;
+			private set;
+		}
+
+		public bool CloseDialog {
+			get;
+			set;
+		}
+
+		public AlertButtonEventArgs (AlertButton button, bool closeDialog)
+		{
+			Button = button;
+			CloseDialog = closeDialog;
+		}
+
+		public AlertButtonEventArgs (AlertButton button) : this (button, true)
+		{
+		}
+	}
+
 	public class AlertButton 
 	{
 		public static AlertButton Ok      = new AlertButton (Gtk.Stock.Ok, true);
@@ -151,18 +179,55 @@ namespace MonoDevelop.Ide
 		{
 			ShowError ((Window)null, primaryText);
 		}
-		public static void ShowError (Window parent, string primaryText)
+
+		public static void ShowError (string primaryText, Exception ex)
 		{
-			ShowError (parent, primaryText, null);
+			ShowError ((Window)null, primaryText, null, ex);
 		}
+
 		public static void ShowError (string primaryText, string secondaryText)
 		{
-			ShowError ((Window)null, primaryText, secondaryText);
+			ShowError ((Window)null, primaryText, secondaryText, null);
+		}
+
+		public static void ShowError (string primaryText, string secondaryText, Exception ex)
+		{
+			ShowError ((Window)null, primaryText, secondaryText, ex);
+		}
+
+		public static void ShowError (Window parent, string primaryText)
+		{
+			ShowError (parent, primaryText, null, null);
 		}
 
 		public static void ShowError (Window parent, string primaryText, string secondaryText)
 		{
-			GenericAlert (parent, MonoDevelop.Ide.Gui.Stock.Error, primaryText, secondaryText, AlertButton.Ok);
+			ShowError (parent, primaryText, secondaryText, null);
+		}
+
+		public static void ShowError (Window parent, string primaryText, string secondaryText, Exception ex)
+		{
+			ShowError (parent, primaryText, secondaryText, ex, true, AlertButton.Ok);
+		}
+
+		internal static AlertButton ShowError (Window parent, string primaryText, string secondaryText, Exception ex, bool logError, params AlertButton[] buttons)
+		{
+			if (logError) {
+				string msg = string.IsNullOrEmpty (secondaryText) ? primaryText : primaryText + ". " + secondaryText;
+				LoggingService.LogError (msg, ex);
+			}
+
+			if (string.IsNullOrEmpty (secondaryText) && (ex != null))
+				secondaryText = ex.Message;
+
+			return GenericAlert (parent, MonoDevelop.Ide.Gui.Stock.Error, primaryText, secondaryText, buttons);
+		}
+
+		internal static void ShowFatalError (string primaryText, string secondaryText, Exception ex)
+		{
+			string msg = string.IsNullOrEmpty (secondaryText) ? primaryText : primaryText + ". " + secondaryText;
+			LoggingService.LogFatalError (msg, ex);
+			GenericAlert (null, MonoDevelop.Ide.Gui.Stock.Error, primaryText, secondaryText, AlertButton.Ok);
 		}
 		#endregion
 		
@@ -301,10 +366,38 @@ namespace MonoDevelop.Ide
 			if (dialog.Title == null)
 				dialog.Title = BrandingService.ApplicationName;
 
-			PlaceDialog (dialog, parent);
+			#if MAC
+			DispatchService.GuiSyncDispatch (() => {
+				// If there is a native NSWindow model window running, we need
+				// to show the new dialog over that window.
+				if (NSApplication.SharedApplication.ModalWindow != null)
+					dialog.Shown += HandleShown;
+				else
+					PlaceDialog (dialog, parent);
+			});
+			#endif
 			return Mono.TextEditor.GtkWorkarounds.RunDialogWithNotification (dialog);
 		}
-		
+
+		#if MAC
+		static void HandleShown (object sender, EventArgs e)
+		{
+			var dialog = (Gtk.Window)sender;
+			var nsdialog = GtkMacInterop.GetNSWindow (dialog);
+
+			// Make the GTK window modal WRT the current modal NSWindow
+			var s = NSApplication.SharedApplication.BeginModalSession (nsdialog);
+
+			EventHandler unrealizer = null;
+			unrealizer = delegate {
+				NSApplication.SharedApplication.EndModalSession (s);
+				dialog.Unrealized -= unrealizer;
+			};
+			dialog.Unrealized += unrealizer;
+			dialog.Shown -= HandleShown;
+		}
+		#endif
+
 		/// <summary>
 		/// Gets a default parent for modal dialogs.
 		/// </summary>
@@ -514,6 +607,17 @@ namespace MonoDevelop.Ide
 		public bool AllowApplyToAll { get; set; }
 		public int DefaultButton { get; set; }
 		public CancellationToken CancellationToken { get; private set; }
+		public bool UseMarkup { get; set; }
+
+		public event EventHandler<AlertButtonEventArgs> AlertButtonClicked;
+
+		internal bool NotifyClicked (AlertButton button)
+		{
+			var args = new AlertButtonEventArgs (button);
+			if (AlertButtonClicked != null)
+				AlertButtonClicked (this, args);
+			return args.CloseDialog;
+		}
 		
 		public void AddOption (string id, string text, bool setByDefault)
 		{
